import functools

import manim as m

from manta.color_theme.color_theme_ABC import ColorThemeABC
from manta.font_style.fontABC import FontABC

SYMBOLS_UNICODE = {
    'POWER SYMBOL': 9211,  # ⏻ U+23FB
    'POWER ON-OFF SYMBOL': 9212,  # ⏼ U+23FC
    'POWER ON SYMBOL': 9213,  # ⏽ U+23FD
    'POWER SLEEP SYMBOL': 9214,  # ⏾ U+23FE
    'uni2630': 9776,  # ☰ U+2630
    'heart': 9829,  # ♥ U+2665
    # 'zap': 9889,  # ⚡ U+26A1 # NOTE: this one does somehow not work with manim
    'uni276C': 10092,  # ❬ U+276C
    'uni276D': 10093,  # ❭ U+276D
    'uni276E': 10094,  # ❮ U+276E
    'uni276F': 10095,  # ❯ U+276F
    'uni2770': 10096,  # ❰ U+2770
    'uni2771': 10097,  # ❱ U+2771
    'HEAVY CIRCLE': 11096,  # ⭘ U+2B58
    'CLEAN_CODE': 57344,  #  U+E000
    'POMODORO_DONE': 57345,  #  U+E001
    'POMODORO_ESTIMATED': 57346,  #  U+E002
    'POMODORO_TICKING': 57347,  #  U+E003
    'POMODORO_SQUASHED': 57348,  #  U+E004
    'SHORT_PAUSE': 57349,  #  U+E005
    'LONG_PAUSE': 57350,  #  U+E006
    'AWAY': 57351,  #  U+E007
    'PAIR_PROGRAMMING': 57352,  #  U+E008
    'INTERNAL_INTERRUPTION': 57353,  #  U+E009
    'EXTERNAL_INTERRUPTION': 57354,  #  U+E00A
    'uniE0A0': 57504,  #  U+E0A0
    'uniE0A1': 57505,  #  U+E0A1
    'uniE0A2': 57506,  #  U+E0A2
    'uniE0A3': 57507,  #  U+E0A3
    'uniE0B0': 57520,  #  U+E0B0
    'uniE0B1': 57521,  #  U+E0B1
    'uniE0B2': 57522,  #  U+E0B2
    'uniE0B3': 57523,  #  U+E0B3
    'uniE0B4': 57524,  #  U+E0B4
    'uniE0B5': 57525,  #  U+E0B5
    'uniE0B6': 57526,  #  U+E0B6
    'uniE0B7': 57527,  #  U+E0B7
    'uniE0B8': 57528,  #  U+E0B8
    'uniE0B8': 57528,  #  U+E0B8
    'uniE0B9': 57529,  #  U+E0B9
    'uniE0BA': 57530,  #  U+E0BA
    'uniE0BB': 57531,  #  U+E0BB
    'uniE0BC': 57532,  #  U+E0BC
    'uniE0BD': 57533,  #  U+E0BD
    'uniE0BE': 57534,  #  U+E0BE
    'uniE0BF': 57535,  #  U+E0BF
    'uniE0C0': 57536,  #  U+E0C0
    'uniE0C1': 57537,  #  U+E0C1
    'uniE0C2': 57538,  #  U+E0C2
    'uniE0C3': 57539,  #  U+E0C3
    'uniE0C4': 57540,  #  U+E0C4
    'uniE0C5': 57541,  #  U+E0C5
    'uniE0C6': 57542,  #  U+E0C6
    'uniE0C7': 57543,  #  U+E0C7
    'uniE0C8': 57544,  #  U+E0C8
    'uniE0CA': 57546,  #  U+E0CA
    'uniE0CC': 57548,  #  U+E0CC
    'uniE0CD': 57549,  #  U+E0CD
    'uniE0CE': 57550,  #  U+E0CE
    'uniE0CF': 57551,  #  U+E0CF
    'uniE0D0': 57552,  #  U+E0D0
    'uniE0D1': 57553,  #  U+E0D1
    'uniE0D2': 57554,  #  U+E0D2
    'uniE0D4': 57556,  #  U+E0D4
    'uniE0D6': 57558,  #  U+E0D6
    'uniE0D7': 57559,  #  U+E0D7
    'smaller': 57856,  #  U+E200
    'snowing': 57857,  #  U+E201
    'soda': 57858,  #  U+E202
    'sofa': 57859,  #  U+E203
    'soup': 57860,  #  U+E204
    'spermatozoon': 57861,  #  U+E205
    'spin-double': 57862,  #  U+E206
    'stomach': 57863,  #  U+E207
    'storm': 57864,  #  U+E208
    'telescope': 57865,  #  U+E209
    'thermometer': 57866,  #  U+E20A
    'thermometer-high': 57867,  #  U+E20B
    'thermometer-low': 57868,  #  U+E20C
    'thin-close': 57869,  #  U+E20D
    'toilet': 57870,  #  U+E20E
    'tools': 57871,  #  U+E20F
    'tooth': 57872,  #  U+E210
    'uterus': 57873,  #  U+E211
    'w3c': 57874,  #  U+E212
    'walking': 57875,  #  U+E213
    'virus': 57876,  #  U+E214
    'telegram-circle': 57877,  #  U+E215
    'slash': 57878,  #  U+E216
    'telegram': 57879,  #  U+E217
    'shirt': 57880,  #  U+E218
    'tacos': 57881,  #  U+E219
    'sushi': 57882,  #  U+E21A
    'triangle-ruler': 57883,  #  U+E21B
    'tree': 57884,  #  U+E21C
    'sun-cloud': 57885,  #  U+E21D
    'ruby-o': 57886,  #  U+E21E
    'ruler': 57887,  #  U+E21F
    'umbrella': 57888,  #  U+E220
    'medicine': 57889,  #  U+E221
    'microscope': 57890,  #  U+E222
    'milk-bottle': 57891,  #  U+E223
    'minimize': 57892,  #  U+E224
    'molecule': 57893,  #  U+E225
    'moon-cloud': 57894,  #  U+E226
    'mushroom': 57895,  #  U+E227
    'mustache': 57896,  #  U+E228
    'mysql': 57897,  #  U+E229
    'nintendo': 57898,  #  U+E22A
    'palette-color': 57899,  #  U+E22B
    'pi': 57900,  #  U+E22C
    'pizza': 57901,  #  U+E22D
    'planet': 57902,  #  U+E22E
    'plant': 57903,  #  U+E22F
    'playstation': 57904,  #  U+E230
    'poison': 57905,  #  U+E231
    'popcorn': 57906,  #  U+E232
    'popsicle': 57907,  #  U+E233
    'pulse': 57908,  #  U+E234
    'python': 57909,  #  U+E235
    'quora-circle': 57910,  #  U+E236
    'quora-square': 57911,  #  U+E237
    'radioactive': 57912,  #  U+E238
    'raining': 57913,  #  U+E239
    'real-heart': 57914,  #  U+E23A
    'refrigerator': 57915,  #  U+E23B
    'restore': 57916,  #  U+E23C
    'ring': 57917,  #  U+E23D
    'ruby': 57918,  #  U+E23E
    'fingerprint': 57919,  #  U+E23F
    'floppy': 57920,  #  U+E240
    'footprint': 57921,  #  U+E241
    'freecodecamp': 57922,  #  U+E242
    'galaxy': 57923,  #  U+E243
    'galery': 57924,  #  U+E244
    'glass': 57925,  #  U+E245
    'google-drive': 57926,  #  U+E246
    'google-play': 57927,  #  U+E247
    'gps': 57928,  #  U+E248
    'grav': 57929,  #  U+E249
    'guitar': 57930,  #  U+E24A
    'gut': 57931,  #  U+E24B
    'halter': 57932,  #  U+E24C
    'hamburger': 57933,  #  U+E24D
    'hat': 57934,  #  U+E24E
    'hexagon': 57935,  #  U+E24F
    'high-heel': 57936,  #  U+E250
    'hotdog': 57937,  #  U+E251
    'ice-cream': 57938,  #  U+E252
    'id-card': 57939,  #  U+E253
    'imdb': 57940,  #  U+E254
    'infinity': 57941,  #  U+E255
    'java': 57942,  #  U+E256
    'layers': 57943,  #  U+E257
    'lips': 57944,  #  U+E258
    'lipstick': 57945,  #  U+E259
    'liver': 57946,  #  U+E25A
    'lung': 57947,  #  U+E25B
    'makeup-brushes': 57948,  #  U+E25C
    'maximize': 57949,  #  U+E25D
    'wallet': 57950,  #  U+E25E
    'chess-horse': 57951,  #  U+E25F
    'chess-king': 57952,  #  U+E260
    'chess-pawn': 57953,  #  U+E261
    'chess-queen': 57954,  #  U+E262
    'chess-tower': 57955,  #  U+E263
    'chesse': 57956,  #  U+E264
    'chilli': 57957,  #  U+E265
    'chip': 57958,  #  U+E266
    'cicling': 57959,  #  U+E267
    'cloud': 57960,  #  U+E268
    'cockroach': 57961,  #  U+E269
    'coffe-beans': 57962,  #  U+E26A
    'coins': 57963,  #  U+E26B
    'comb': 57964,  #  U+E26C
    'comet': 57965,  #  U+E26D
    'crown': 57966,  #  U+E26E
    'cup-coffe': 57967,  #  U+E26F
    'dice': 57968,  #  U+E270
    'disco': 57969,  #  U+E271
    'dna': 57970,  #  U+E272
    'donut': 57971,  #  U+E273
    'dress': 57972,  #  U+E274
    'drop': 57973,  #  U+E275
    'ello': 57974,  #  U+E276
    'envelope-open': 57975,  #  U+E277
    'envelope-open-o': 57976,  #  U+E278
    'equal': 57977,  #  U+E279
    'equal-bigger': 57978,  #  U+E27A
    'feedly': 57979,  #  U+E27B
    'file-export': 57980,  #  U+E27C
    'file-import': 57981,  #  U+E27D
    'wind': 57982,  #  U+E27E
    'atom': 57983,  #  U+E27F
    'bacteria': 57984,  #  U+E280
    'banana': 57985,  #  U+E281
    'bath': 57986,  #  U+E282
    'bed': 57987,  #  U+E283
    'benzene': 57988,  #  U+E284
    'bigger': 57989,  #  U+E285
    'biohazard': 57990,  #  U+E286
    'blogger-circle': 57991,  #  U+E287
    'blogger-square': 57992,  #  U+E288
    'bones': 57993,  #  U+E289
    'book-open': 57994,  #  U+E28A
    'book-open-o': 57995,  #  U+E28B
    'brain': 57996,  #  U+E28C
    'bread': 57997,  #  U+E28D
    'butterfly': 57998,  #  U+E28E
    'carot': 57999,  #  U+E28F
    'cc-by': 58000,  #  U+E290
    'cc-cc': 58001,  #  U+E291
    'cc-nc': 58002,  #  U+E292
    'cc-nc-eu': 58003,  #  U+E293
    'cc-nc-jp': 58004,  #  U+E294
    'cc-nd': 58005,  #  U+E295
    'cc-remix': 58006,  #  U+E296
    'cc-sa': 58007,  #  U+E297
    'cc-share': 58008,  #  U+E298
    'cc-zero': 58009,  #  U+E299
    'checklist-o': 58010,  #  U+E29A
    'cherry': 58011,  #  U+E29B
    'chess-bishop': 58012,  #  U+E29C
    'xbox': 58013,  #  U+E29D
    'apple-fruit': 58014,  #  U+E29E
    'chicken-thigh': 58015,  #  U+E29F
    'gift-card': 58016,  #  U+E2A0
    'injection': 58017,  #  U+E2A1
    'isle': 58018,  #  U+E2A2
    'lollipop': 58019,  #  U+E2A3
    'loyalty-card': 58020,  #  U+E2A4
    'meat': 58021,  #  U+E2A5
    'mountains': 58022,  #  U+E2A6
    'orange': 58023,  #  U+E2A7
    'peach': 58024,  #  U+E2A8
    'pear': 58025,  #  U+E2A9
    'uniF000': 58112,  #  U+E300
    'uniF001': 58113,  #  U+E301
    'uniF002': 58114,  #  U+E302
    'uniF003': 58115,  #  U+E303
    'uniF004': 58116,  #  U+E304
    'uniF005': 58117,  #  U+E305
    'uniF006': 58118,  #  U+E306
    'uniF007': 58119,  #  U+E307
    'uniF008': 58120,  #  U+E308
    'uniF009': 58121,  #  U+E309
    'uniF00A': 58122,  #  U+E30A
    'uniF00B': 58123,  #  U+E30B
    'uniF00C': 58124,  #  U+E30C
    'uniF00D': 58125,  #  U+E30D
    'uniF00E': 58126,  #  U+E30E
    'uniF010': 58127,  #  U+E30F
    'uniF011': 58128,  #  U+E310
    'uniF012': 58129,  #  U+E311
    'uniF013': 58130,  #  U+E312
    'uniF014': 58131,  #  U+E313
    'uniF015': 58132,  #  U+E314
    'uniF016': 58133,  #  U+E315
    'uniF017': 58134,  #  U+E316
    'uniF018': 58135,  #  U+E317
    'uniF019': 58136,  #  U+E318
    'uniF01A': 58137,  #  U+E319
    'uniF01B': 58138,  #  U+E31A
    'uniF01C': 58139,  #  U+E31B
    'uniF01D': 58140,  #  U+E31C
    'uniF01E': 58141,  #  U+E31D
    'uniF021': 58142,  #  U+E31E
    'uniF022': 58143,  #  U+E31F
    'uniF023': 58144,  #  U+E320
    'uniF024': 58145,  #  U+E321
    'uniF025': 58146,  #  U+E322
    'uniF026': 58147,  #  U+E323
    'uniF027': 58148,  #  U+E324
    'uniF028': 58149,  #  U+E325
    'uniF029': 58150,  #  U+E326
    'uniF02A': 58151,  #  U+E327
    'uniF02B': 58152,  #  U+E328
    'uniF02C': 58153,  #  U+E329
    'uniF02D': 58154,  #  U+E32A
    'uniF02E': 58155,  #  U+E32B
    'uniF02F': 58156,  #  U+E32C
    'uniF030': 58157,  #  U+E32D
    'uniF031': 58158,  #  U+E32E
    'uniF032': 58159,  #  U+E32F
    'uniF033': 58160,  #  U+E330
    'uniF034': 58161,  #  U+E331
    'uniF035': 58162,  #  U+E332
    'uniF036': 58163,  #  U+E333
    'uniF037': 58164,  #  U+E334
    'uniF038': 58165,  #  U+E335
    'uniF039': 58166,  #  U+E336
    'uniF03A': 58167,  #  U+E337
    'uniF03B': 58168,  #  U+E338
    'uniF03C': 58169,  #  U+E339
    'uniF03D': 58170,  #  U+E33A
    'uniF03E': 58171,  #  U+E33B
    'uniF040': 58172,  #  U+E33C
    'uniF041': 58173,  #  U+E33D
    'uniF042': 58174,  #  U+E33E
    'uniF043': 58175,  #  U+E33F
    'uniF044': 58176,  #  U+E340
    'uniF045': 58177,  #  U+E341
    'uniF046': 58178,  #  U+E342
    'uniF047': 58179,  #  U+E343
    'uniF048': 58180,  #  U+E344
    'uniF049': 58181,  #  U+E345
    'uniF04A': 58182,  #  U+E346
    'uniF04B': 58183,  #  U+E347
    'uniF04C': 58184,  #  U+E348
    'uniF04D': 58185,  #  U+E349
    'uniF04E': 58186,  #  U+E34A
    'uniF050': 58187,  #  U+E34B
    'uniF051': 58188,  #  U+E34C
    'uniF052': 58189,  #  U+E34D
    'uniF053': 58190,  #  U+E34E
    'uniF054': 58191,  #  U+E34F
    'uniF055': 58192,  #  U+E350
    'uniF056': 58193,  #  U+E351
    'uniF057': 58194,  #  U+E352
    'uniF058': 58195,  #  U+E353
    'uniF059': 58196,  #  U+E354
    'uniF05A': 58197,  #  U+E355
    'uniF05B': 58198,  #  U+E356
    'uniF05C': 58199,  #  U+E357
    'uniF05D': 58200,  #  U+E358
    'uniF05E': 58201,  #  U+E359
    'uniF060': 58202,  #  U+E35A
    'uniF061': 58203,  #  U+E35B
    'uniF062': 58204,  #  U+E35C
    'uniF063': 58205,  #  U+E35D
    'uniF064': 58206,  #  U+E35E
    'uniF065': 58207,  #  U+E35F
    'uniF066': 58208,  #  U+E360
    'uniF067': 58209,  #  U+E361
    'uniF068': 58210,  #  U+E362
    'uniF069': 58211,  #  U+E363
    'uniF06A': 58212,  #  U+E364
    'uniF06B': 58213,  #  U+E365
    'uniF06C': 58214,  #  U+E366
    'uniF06D': 58215,  #  U+E367
    'uniF06E': 58216,  #  U+E368
    'uniF070': 58217,  #  U+E369
    'uniF071': 58218,  #  U+E36A
    'uniF072': 58219,  #  U+E36B
    'uniF073': 58220,  #  U+E36C
    'uniF074': 58221,  #  U+E36D
    'uniF075': 58222,  #  U+E36E
    'uniF076': 58223,  #  U+E36F
    'uniF077': 58224,  #  U+E370
    'uniF078': 58225,  #  U+E371
    'uniF079': 58226,  #  U+E372
    'uniF07A': 58227,  #  U+E373
    'uniF07B': 58228,  #  U+E374
    'uniF07C': 58229,  #  U+E375
    'uniF07D': 58230,  #  U+E376
    'uniF07E': 58231,  #  U+E377
    'uniF080': 58232,  #  U+E378
    'uniF081': 58233,  #  U+E379
    'uniF082': 58234,  #  U+E37A
    'uniF083': 58235,  #  U+E37B
    'uniF084': 58236,  #  U+E37C
    'uniF085': 58237,  #  U+E37D
    'uniF086': 58238,  #  U+E37E
    'uniF087': 58239,  #  U+E37F
    'uniF088': 58240,  #  U+E380
    'uniF089': 58241,  #  U+E381
    'uniF08A': 58242,  #  U+E382
    'uniF08B': 58243,  #  U+E383
    'uniF08C': 58244,  #  U+E384
    'uniF08D': 58245,  #  U+E385
    'uniF08E': 58246,  #  U+E386
    'uniF08F': 58247,  #  U+E387
    'uniF090': 58248,  #  U+E388
    'uniF091': 58249,  #  U+E389
    'uniF092': 58250,  #  U+E38A
    'uniF093': 58251,  #  U+E38B
    'uniF094': 58252,  #  U+E38C
    'uniF095': 58253,  #  U+E38D
    'uniF096': 58254,  #  U+E38E
    'uniF097': 58255,  #  U+E38F
    'uniF098': 58256,  #  U+E390
    'uniF099': 58257,  #  U+E391
    'uniF09A': 58258,  #  U+E392
    'uniF09B': 58259,  #  U+E393
    'uniF09C': 58260,  #  U+E394
    'uniF09D': 58261,  #  U+E395
    'uniF09E': 58262,  #  U+E396
    'uniF09F': 58263,  #  U+E397
    'uniF0A0': 58264,  #  U+E398
    'uniF0A1': 58265,  #  U+E399
    'uniF0A2': 58266,  #  U+E39A
    'uniF0A3': 58267,  #  U+E39B
    'uniF0A4': 58268,  #  U+E39C
    'uniF0A5': 58269,  #  U+E39D
    'uniF0A6': 58270,  #  U+E39E
    'uniF0A7': 58271,  #  U+E39F
    'uniF0A8': 58272,  #  U+E3A0
    'uniF0A9': 58273,  #  U+E3A1
    'uniF0AA': 58274,  #  U+E3A2
    'uniF0AB': 58275,  #  U+E3A3
    'uniF0AC': 58276,  #  U+E3A4
    'uniF0AD': 58277,  #  U+E3A5
    'uniF0AE': 58278,  #  U+E3A6
    'uniF0AF': 58279,  #  U+E3A7
    'uniF0B0': 58280,  #  U+E3A8
    'uniF0B1': 58281,  #  U+E3A9
    'uniF0B2': 58282,  #  U+E3AA
    'uniF0B3': 58283,  #  U+E3AB
    'uniF0B4': 58284,  #  U+E3AC
    'uniF0B5': 58285,  #  U+E3AD
    'uniF0B6': 58286,  #  U+E3AE
    'uniF0B7': 58287,  #  U+E3AF
    'uniF0B8': 58288,  #  U+E3B0
    'uniF0B9': 58289,  #  U+E3B1
    'uniF0BA': 58290,  #  U+E3B2
    'uniF0BB': 58291,  #  U+E3B3
    'uniF0BC': 58292,  #  U+E3B4
    'uniF0BD': 58293,  #  U+E3B5
    'uniF0BE': 58294,  #  U+E3B6
    'uniF0BF': 58295,  #  U+E3B7
    'uniF0C0': 58296,  #  U+E3B8
    'uniF0C1': 58297,  #  U+E3B9
    'uniF0C2': 58298,  #  U+E3BA
    'uniF0C3': 58299,  #  U+E3BB
    'uniF0C4': 58300,  #  U+E3BC
    'uniF0C5': 58301,  #  U+E3BD
    'uniF0C6': 58302,  #  U+E3BE
    'uniF0C7': 58303,  #  U+E3BF
    'uniF0C8': 58304,  #  U+E3C0
    'uniF0C9': 58305,  #  U+E3C1
    'uniF0CA': 58306,  #  U+E3C2
    'uniF0CB': 58307,  #  U+E3C3
    'uniF0CC': 58308,  #  U+E3C4
    'uniF0CD': 58309,  #  U+E3C5
    'uniF0CE': 58310,  #  U+E3C6
    'uniF0CF': 58311,  #  U+E3C7
    'uniF0D0': 58312,  #  U+E3C8
    'uniF0D1': 58313,  #  U+E3C9
    'uniF0D2': 58314,  #  U+E3CA
    'uniF0D3': 58315,  #  U+E3CB
    'uniF0D4': 58316,  #  U+E3CC
    'uniF0D5': 58317,  #  U+E3CD
    'uniF0D6': 58318,  #  U+E3CE
    'uniF0D7': 58319,  #  U+E3CF
    'uniF0D8': 58320,  #  U+E3D0
    'uniF0D9': 58321,  #  U+E3D1
    'uniF0DA': 58322,  #  U+E3D2
    'uniF0DB': 58323,  #  U+E3D3
    'uniF0DC': 58324,  #  U+E3D4
    'uniF0DD': 58325,  #  U+E3D5
    'uniF0DE': 58326,  #  U+E3D6
    'uniF0DF': 58327,  #  U+E3D7
    'uniF0E0': 58328,  #  U+E3D8
    'uniF0E1': 58329,  #  U+E3D9
    'uniF0E2': 58330,  #  U+E3DA
    'uniF0E3': 58331,  #  U+E3DB
    'uniF0E4': 58332,  #  U+E3DC
    'uniF0E5': 58333,  #  U+E3DD
    'uniF0E6': 58334,  #  U+E3DE
    'uniF0E7': 58335,  #  U+E3DF
    'uniF0E8': 58336,  #  U+E3E0
    'uniF0E9': 58337,  #  U+E3E1
    'uniF0EA': 58338,  #  U+E3E2
    'uniF0EB': 58339,  #  U+E3E3
    'i_custom_folder_npm': 58874,  #  U+E5FA
    'i_custom_folder_git': 58875,  #  U+E5FB
    'i_custom_folder_config': 58876,  #  U+E5FC
    'i_custom_folder_github': 58877,  #  U+E5FD
    'i_custom_folder_open': 58878,  #  U+E5FE
    'i_custom_folder': 58879,  #  U+E5FF
    'i_seti_stylus': 58880,  #  U+E600
    'i_seti_project': 58881,  #  U+E601
    'i_custom_play_arrow': 58882,  #  U+E602
    'i_seti_sass': 58883,  #  U+E603
    'i_seti_rails': 58884,  #  U+E604
    'i_seti_ruby': 58885,  #  U+E605
    'i_seti_python': 58886,  #  U+E606
    'i_seti_heroku': 58887,  #  U+E607
    'i_seti_php': 58888,  #  U+E608
    'i_seti_markdown': 58889,  #  U+E609
    'i_seti_license': 58890,  #  U+E60A
    'i_seti_json': 58891,  #  U+E60B
    'i_seti_javascript': 58892,  #  U+E60C
    'i_seti_image': 58893,  #  U+E60D
    'i_seti_html': 58894,  #  U+E60E
    'i_seti_mustache': 58895,  #  U+E60F
    'i_seti_gulp': 58896,  #  U+E610
    'i_seti_grunt': 58897,  #  U+E611
    'i_custom_default': 58898,  #  U+E612
    'i_seti_folder': 58899,  #  U+E613
    'i_seti_css': 58900,  #  U+E614
    'i_seti_config': 58901,  #  U+E615
    'i_seti_npm': 58902,  #  U+E616
    'i_custom_home': 58903,  #  U+E617
    'i_seti_ejs': 58904,  #  U+E618
    'i_seti_xml': 58905,  #  U+E619
    'i_seti_bower': 58906,  #  U+E61A
    'i_seti_coffee': 58907,  #  U+E61B
    'i_seti_twig': 58908,  #  U+E61C
    'i_custom_cpp': 58909,  #  U+E61D
    'i_custom_c': 58910,  #  U+E61E
    'i_seti_haskell': 58911,  #  U+E61F
    'i_seti_lua': 58912,  #  U+E620
    'i_indent_line': 58913,  #  U+E621
    'i_seti_karma': 58914,  #  U+E622
    'i_seti_favicon': 58915,  #  U+E623
    'i_seti_julia': 58916,  #  U+E624
    'i_seti_react': 58917,  #  U+E625
    'i_custom_go': 58918,  #  U+E626
    'i_seti_go': 58919,  #  U+E627
    'i_seti_typescript': 58920,  #  U+E628
    'i_custom_msdos': 58921,  #  U+E629
    'i_custom_windows': 58922,  #  U+E62A
    'i_custom_vim': 58923,  #  U+E62B
    'i_custom_elm': 58924,  #  U+E62C
    'i_custom_elixir': 58925,  #  U+E62D
    'i_custom_electron': 58926,  #  U+E62E
    'i_custom_crystal': 58927,  #  U+E62F
    'i_custom_purescript': 58928,  #  U+E630
    'i_seti_puppet': 58929,  #  U+E631
    'i_custom_emacs': 58930,  #  U+E632
    'i_custom_orgmode': 58931,  #  U+E633
    'i_custom_kotlin': 58932,  #  U+E634
    'i_seti_apple': 58933,  #  U+E635
    'i_seti_argdown': 58934,  #  U+E636
    'i_seti_asm': 58935,  #  U+E637
    'i_seti_audio': 58936,  #  U+E638
    'i_seti_babel': 58937,  #  U+E639
    'i_seti_bazel': 58938,  #  U+E63A
    'i_seti_bicep': 58939,  #  U+E63B
    'i_seti_bsl': 58940,  #  U+E63C
    'i_seti_cake_php': 58941,  #  U+E63D
    'i_seti_cake': 58942,  #  U+E63E
    'i_seti_checkbox': 58943,  #  U+E63F
    'i_seti_checkbox_unchecked': 58944,  #  U+E640
    'i_seti_clock': 58945,  #  U+E641
    'i_seti_clojure': 58946,  #  U+E642
    'i_seti_code_climate': 58947,  #  U+E643
    'i_seti_code_search': 58948,  #  U+E644
    'i_seti_coldfusion': 58949,  #  U+E645
    'i_seti_cpp': 58950,  #  U+E646
    'i_seti_crystal_embedded': 58951,  #  U+E647
    'i_seti_c_sharp': 58952,  #  U+E648
    'i_seti_c': 58953,  #  U+E649
    'i_seti_csv': 58954,  #  U+E64A
    'i_seti_cu': 58955,  #  U+E64B
    'i_seti_dart': 58956,  #  U+E64C
    'i_seti_db': 58957,  #  U+E64D
    'i_seti_default': 58958,  #  U+E64E
    'i_seti_deprecation_cop': 58959,  #  U+E64F
    'i_seti_docker': 58960,  #  U+E650
    'i_seti_d': 58961,  #  U+E651
    'i_seti_editorconfig': 58962,  #  U+E652
    'i_seti_elixir_script': 58963,  #  U+E653
    'i_seti_error': 58964,  #  U+E654
    'i_seti_eslint': 58965,  #  U+E655
    'i_seti_ethereum': 58966,  #  U+E656
    'i_seti_firebase': 58967,  #  U+E657
    'i_seti_firefox': 58968,  #  U+E658
    'i_seti_font': 58969,  #  U+E659
    'i_seti_f_sharp': 58970,  #  U+E65A
    'i_seti_github': 58971,  #  U+E65B
    'i_seti_gitlab': 58972,  #  U+E65C
    'i_seti_git': 58973,  #  U+E65D
    'i_seti_go2': 58974,  #  U+E65E
    'i_seti_godot': 58975,  #  U+E65F
    'i_seti_gradle': 58976,  #  U+E660
    'i_seti_grails': 58977,  #  U+E661
    'i_seti_graphql': 58978,  #  U+E662
    'i_seti_hacklang': 58979,  #  U+E663
    'i_seti_haml': 58980,  #  U+E664
    'i_seti_happenings': 58981,  #  U+E665
    'i_seti_haxe': 58982,  #  U+E666
    'i_seti_hex': 58983,  #  U+E667
    'i_seti_ignored': 58984,  #  U+E668
    'i_seti_illustrator': 58985,  #  U+E669
    'i_seti_info': 58986,  #  U+E66A
    'i_seti_ionic': 58987,  #  U+E66B
    'i_seti_jade': 58988,  #  U+E66C
    'i_seti_java': 58989,  #  U+E66D
    'i_seti_jenkins': 58990,  #  U+E66E
    'i_seti_jinja': 58991,  #  U+E66F
    'i_seti_liquid': 58992,  #  U+E670
    'i_seti_livescript': 58993,  #  U+E671
    'i_seti_lock': 58994,  #  U+E672
    'i_seti_makefile': 58995,  #  U+E673
    'i_seti_maven': 58996,  #  U+E674
    'i_seti_mdo': 58997,  #  U+E675
    'i_seti_new_file': 58998,  #  U+E676
    'i_seti_nim': 58999,  #  U+E677
    'i_seti_notebook': 59000,  #  U+E678
    'i_seti_nunjucks': 59001,  #  U+E679
    'i_seti_ocaml': 59002,  #  U+E67A
    'i_seti_odata': 59003,  #  U+E67B
    'i_seti_pddl': 59004,  #  U+E67C
    'i_seti_pdf': 59005,  #  U+E67D
    'i_seti_perl': 59006,  #  U+E67E
    'i_seti_photoshop': 59007,  #  U+E67F
    'i_seti_pipeline': 59008,  #  U+E680
    'i_seti_plan': 59009,  #  U+E681
    'i_seti_platformio': 59010,  #  U+E682
    'i_seti_powershell': 59011,  #  U+E683
    'i_seti_prisma': 59012,  #  U+E684
    'i_seti_prolog': 59013,  #  U+E685
    'i_seti_pug': 59014,  #  U+E686
    'i_seti_reasonml': 59015,  #  U+E687
    'i_seti_rescript': 59016,  #  U+E688
    'i_seti_rollup': 59017,  #  U+E689
    'i_seti_r': 59018,  #  U+E68A
    'i_seti_rust': 59019,  #  U+E68B
    'i_seti_salesforce': 59020,  #  U+E68C
    'i_seti_sbt': 59021,  #  U+E68D
    'i_seti_scala': 59022,  #  U+E68E
    'i_seti_search': 59023,  #  U+E68F
    'i_seti_settings': 59024,  #  U+E690
    'i_seti_shell': 59025,  #  U+E691
    'i_seti_slim': 59026,  #  U+E692
    'i_seti_smarty': 59027,  #  U+E693
    'i_seti_spring': 59028,  #  U+E694
    'i_seti_stylelint': 59029,  #  U+E695
    'i_seti_sublime': 59030,  #  U+E696
    'i_seti_svelte': 59031,  #  U+E697
    'i_seti_svg': 59032,  #  U+E698
    'i_seti_swift': 59033,  #  U+E699
    'i_seti_terraform': 59034,  #  U+E69A
    'i_seti_tex': 59035,  #  U+E69B
    'i_seti_todo': 59036,  #  U+E69C
    'i_seti_tsconfig': 59037,  #  U+E69D
    'i_seti_vala': 59038,  #  U+E69E
    'i_seti_video': 59039,  #  U+E69F
    'i_seti_vue': 59040,  #  U+E6A0
    'i_seti_wasm': 59041,  #  U+E6A1
    'i_seti_wat': 59042,  #  U+E6A2
    'i_seti_webpack': 59043,  #  U+E6A3
    'i_seti_wgt': 59044,  #  U+E6A4
    'i_seti_word': 59045,  #  U+E6A5
    'i_seti_xls': 59046,  #  U+E6A6
    'i_seti_yarn': 59047,  #  U+E6A7
    'i_seti_yml': 59048,  #  U+E6A8
    'i_seti_zig': 59049,  #  U+E6A9
    'i_seti_zip': 59050,  #  U+E6AA
    'i_custom_asm': 59051,  #  U+E6AB
    'i_custom_v_lang': 59052,  #  U+E6AC
    'i_custom_folder_oct': 59053,  #  U+E6AD
    'i_custom_neovim': 59054,  #  U+E6AE
    'i_custom_fennel': 59055,  #  U+E6AF
    'i_custom_common_lisp': 59056,  #  U+E6B0
    'i_custom_scheme': 59057,  #  U+E6B1
    'i_custom_toml': 59058,  #  U+E6B2
    'i_custom_astro': 59059,  #  U+E6B3
    'i_custom_prettier': 59060,  #  U+E6B4
    'i_custom_ada': 59061,  #  U+E6B5
    'uniE600': 59136,  #  U+E700
    'uniE601': 59137,  #  U+E701
    'uniE602': 59138,  #  U+E702
    'uniE603': 59139,  #  U+E703
    'uniE604': 59140,  #  U+E704
    'uniE605': 59141,  #  U+E705
    'uniE606': 59142,  #  U+E706
    'uniE607': 59143,  #  U+E707
    'uniE608': 59144,  #  U+E708
    'uniE609': 59145,  #  U+E709
    'uniE60A': 59146,  #  U+E70A
    'uniE60B': 59147,  #  U+E70B
    'uniE60C': 59148,  #  U+E70C
    'uniE60D': 59149,  #  U+E70D
    'uniE60E': 59150,  #  U+E70E
    'uniE60F': 59151,  #  U+E70F
    'uniE610': 59152,  #  U+E710
    'uniE611': 59153,  #  U+E711
    'uniE612': 59154,  #  U+E712
    'uniE613': 59155,  #  U+E713
    'uniE614': 59156,  #  U+E714
    'uniE615': 59157,  #  U+E715
    'uniE616': 59158,  #  U+E716
    'uniE617': 59159,  #  U+E717
    'uniE618': 59160,  #  U+E718
    'uniE619': 59161,  #  U+E719
    'uniE61A': 59162,  #  U+E71A
    'uniE61B': 59163,  #  U+E71B
    'uniE61C': 59164,  #  U+E71C
    'uniE61D': 59165,  #  U+E71D
    'uniE61E': 59166,  #  U+E71E
    'uniE61F': 59167,  #  U+E71F
    'uniE620': 59168,  #  U+E720
    'uniE621': 59169,  #  U+E721
    'uniE622': 59170,  #  U+E722
    'uniE623': 59171,  #  U+E723
    'uniE624': 59172,  #  U+E724
    'uniE625': 59173,  #  U+E725
    'uniE626': 59174,  #  U+E726
    'uniE627': 59175,  #  U+E727
    'uniE628': 59176,  #  U+E728
    'uniE629': 59177,  #  U+E729
    'uniE62A': 59178,  #  U+E72A
    'uniE62B': 59179,  #  U+E72B
    'uniE62C': 59180,  #  U+E72C
    'uniE62D': 59181,  #  U+E72D
    'uniE62E': 59182,  #  U+E72E
    'uniE62F': 59183,  #  U+E72F
    'uniE630': 59184,  #  U+E730
    'uniE631': 59185,  #  U+E731
    'uniE632': 59186,  #  U+E732
    'uniE633': 59187,  #  U+E733
    'uniE634': 59188,  #  U+E734
    'uniE635': 59189,  #  U+E735
    'uniE636': 59190,  #  U+E736
    'uniE637': 59191,  #  U+E737
    'uniE638': 59192,  #  U+E738
    'uniE639': 59193,  #  U+E739
    'uniE63A': 59194,  #  U+E73A
    'uniE63B': 59195,  #  U+E73B
    'uniE63C': 59196,  #  U+E73C
    'uniE63D': 59197,  #  U+E73D
    'uniE63E': 59198,  #  U+E73E
    'uniE63F': 59199,  #  U+E73F
    'uniE640': 59200,  #  U+E740
    'uniE641': 59201,  #  U+E741
    'uniE642': 59202,  #  U+E742
    'uniE643': 59203,  #  U+E743
    'uniE644': 59204,  #  U+E744
    'uniE645': 59205,  #  U+E745
    'uniE646': 59206,  #  U+E746
    'uniE647': 59207,  #  U+E747
    'uniE648': 59208,  #  U+E748
    'uniE649': 59209,  #  U+E749
    'uniE64A': 59210,  #  U+E74A
    'uniE64B': 59211,  #  U+E74B
    'uniE64C': 59212,  #  U+E74C
    'uniE64D': 59213,  #  U+E74D
    'uniE64E': 59214,  #  U+E74E
    'uniE64F': 59215,  #  U+E74F
    'uniE650': 59216,  #  U+E750
    'uniE651': 59217,  #  U+E751
    'uniE652': 59218,  #  U+E752
    'uniE653': 59219,  #  U+E753
    'uniE654': 59220,  #  U+E754
    'uniE655': 59221,  #  U+E755
    'uniE656': 59222,  #  U+E756
    'uniE657': 59223,  #  U+E757
    'uniE658': 59224,  #  U+E758
    'uniE659': 59225,  #  U+E759
    'uniE65A': 59226,  #  U+E75A
    'uniE65B': 59227,  #  U+E75B
    'uniE65C': 59228,  #  U+E75C
    'uniE65D': 59229,  #  U+E75D
    'uniE65E': 59230,  #  U+E75E
    'uniE65F': 59231,  #  U+E75F
    'uniE660': 59232,  #  U+E760
    'uniE661': 59233,  #  U+E761
    'uniE662': 59234,  #  U+E762
    'uniE663': 59235,  #  U+E763
    'uniE664': 59236,  #  U+E764
    'uniE665': 59237,  #  U+E765
    'uniE666': 59238,  #  U+E766
    'uniE667': 59239,  #  U+E767
    'uniE668': 59240,  #  U+E768
    'uniE669': 59241,  #  U+E769
    'uniE66A': 59242,  #  U+E76A
    'uniE66B': 59243,  #  U+E76B
    'uniE66C': 59244,  #  U+E76C
    'uniE66D': 59245,  #  U+E76D
    'uniE66E': 59246,  #  U+E76E
    'uniE66F': 59247,  #  U+E76F
    'uniE670': 59248,  #  U+E770
    'uniE671': 59249,  #  U+E771
    'uniE672': 59250,  #  U+E772
    'uniE673': 59251,  #  U+E773
    'uniE674': 59252,  #  U+E774
    'uniE675': 59253,  #  U+E775
    'uniE676': 59254,  #  U+E776
    'uniE677': 59255,  #  U+E777
    'uniE678': 59256,  #  U+E778
    'uniE679': 59257,  #  U+E779
    'uniE67A': 59258,  #  U+E77A
    'uniE67B': 59259,  #  U+E77B
    'uniE67C': 59260,  #  U+E77C
    'uniE67D': 59261,  #  U+E77D
    'uniE67E': 59262,  #  U+E77E
    'uniE67F': 59263,  #  U+E77F
    'uniE680': 59264,  #  U+E780
    'uniE681': 59265,  #  U+E781
    'uniE682': 59266,  #  U+E782
    'uniE683': 59267,  #  U+E783
    'uniE684': 59268,  #  U+E784
    'uniE685': 59269,  #  U+E785
    'uniE686': 59270,  #  U+E786
    'uniE687': 59271,  #  U+E787
    'uniE688': 59272,  #  U+E788
    'uniE689': 59273,  #  U+E789
    'uniE68A': 59274,  #  U+E78A
    'uniE68B': 59275,  #  U+E78B
    'uniE68C': 59276,  #  U+E78C
    'uniE68D': 59277,  #  U+E78D
    'uniE68E': 59278,  #  U+E78E
    'uniE68F': 59279,  #  U+E78F
    'uniE690': 59280,  #  U+E790
    'uniE691': 59281,  #  U+E791
    'uniE692': 59282,  #  U+E792
    'uniE693': 59283,  #  U+E793
    'uniE694': 59284,  #  U+E794
    'uniE695': 59285,  #  U+E795
    'uniE696': 59286,  #  U+E796
    'uniE697': 59287,  #  U+E797
    'uniE698': 59288,  #  U+E798
    'uniE699': 59289,  #  U+E799
    'uniE69A': 59290,  #  U+E79A
    'uniE69B': 59291,  #  U+E79B
    'uniE69C': 59292,  #  U+E79C
    'uniE69D': 59293,  #  U+E79D
    'uniE69E': 59294,  #  U+E79E
    'uniE69F': 59295,  #  U+E79F
    'uniE6A0': 59296,  #  U+E7A0
    'uniE6A1': 59297,  #  U+E7A1
    'uniE6A2': 59298,  #  U+E7A2
    'uniE6A3': 59299,  #  U+E7A3
    'uniE6A4': 59300,  #  U+E7A4
    'uniE6A5': 59301,  #  U+E7A5
    'uniE6A6': 59302,  #  U+E7A6
    'uniE6A7': 59303,  #  U+E7A7
    'uniE6A8': 59304,  #  U+E7A8
    'uniE6A9': 59305,  #  U+E7A9
    'uniE6AA': 59306,  #  U+E7AA
    'uniE6AB': 59307,  #  U+E7AB
    'uniE6AC': 59308,  #  U+E7AC
    'uniE6AD': 59309,  #  U+E7AD
    'uniE6AE': 59310,  #  U+E7AE
    'uniE6AF': 59311,  #  U+E7AF
    'uniE6B0': 59312,  #  U+E7B0
    'uniE6B1': 59313,  #  U+E7B1
    'uniE6B2': 59314,  #  U+E7B2
    'uniE6B3': 59315,  #  U+E7B3
    'uniE6B4': 59316,  #  U+E7B4
    'uniE6B5': 59317,  #  U+E7B5
    'uniE6B6': 59318,  #  U+E7B6
    'uniE6B7': 59319,  #  U+E7B7
    'uniE6B8': 59320,  #  U+E7B8
    'uniE6B9': 59321,  #  U+E7B9
    'uniE6BA': 59322,  #  U+E7BA
    'uniE6BB': 59323,  #  U+E7BB
    'uniE6BC': 59324,  #  U+E7BC
    'uniE6BD': 59325,  #  U+E7BD
    'uniE6BE': 59326,  #  U+E7BE
    'uniE6BF': 59327,  #  U+E7BF
    'uniE6C0': 59328,  #  U+E7C0
    'uniE6C1': 59329,  #  U+E7C1
    'uniE6C2': 59330,  #  U+E7C2
    'uniE6C3': 59331,  #  U+E7C3
    'uniE6C4': 59332,  #  U+E7C4
    'uniE6C5': 59333,  #  U+E7C5
    'add': 60000,  #  U+EA60
    'lightbulb': 60001,  #  U+EA61
    'repo': 60002,  #  U+EA62
    'repo-forked': 60003,  #  U+EA63
    'git-pull-request': 60004,  #  U+EA64
    'record-keys': 60005,  #  U+EA65
    'tag': 60006,  #  U+EA66
    'person': 60007,  #  U+EA67
    'source-control': 60008,  #  U+EA68
    'mirror': 60009,  #  U+EA69
    'star-empty': 60010,  #  U+EA6A
    'comment': 60011,  #  U+EA6B
    'warning': 60012,  #  U+EA6C
    'search': 60013,  #  U+EA6D
    'sign-out': 60014,  #  U+EA6E
    'sign-in': 60015,  #  U+EA6F
    'eye': 60016,  #  U+EA70
    'circle-filled': 60017,  #  U+EA71
    'primitive-square': 60018,  #  U+EA72
    'edit': 60019,  #  U+EA73
    'info': 60020,  #  U+EA74
    'lock': 60021,  #  U+EA75
    'close': 60022,  #  U+EA76
    'sync': 60023,  #  U+EA77
    'desktop-download': 60024,  #  U+EA78
    'beaker': 60025,  #  U+EA79
    'vm': 60026,  #  U+EA7A
    'file': 60027,  #  U+EA7B
    'ellipsis': 60028,  #  U+EA7C
    'reply': 60029,  #  U+EA7D
    'organization': 60030,  #  U+EA7E
    'new-file': 60031,  #  U+EA7F
    'new-folder': 60032,  #  U+EA80
    'trash': 60033,  #  U+EA81
    'history': 60034,  #  U+EA82
    'folder': 60035,  #  U+EA83
    'github': 60036,  #  U+EA84
    'terminal': 60037,  #  U+EA85
    'symbol-event': 60038,  #  U+EA86
    'error': 60039,  #  U+EA87
    'symbol-variable': 60040,  #  U+EA88
    'symbol-array': 60042,  #  U+EA8A
    'symbol-namespace': 60043,  #  U+EA8B
    'symbol-method': 60044,  #  U+EA8C
    'symbol-boolean': 60047,  #  U+EA8F
    'symbol-numeric': 60048,  #  U+EA90
    'symbol-structure': 60049,  #  U+EA91
    'symbol-parameter': 60050,  #  U+EA92
    'symbol-key': 60051,  #  U+EA93
    'go-to-file': 60052,  #  U+EA94
    'symbol-enum': 60053,  #  U+EA95
    'symbol-ruler': 60054,  #  U+EA96
    'activate-breakpoints': 60055,  #  U+EA97
    'archive': 60056,  #  U+EA98
    'arrow-both': 60057,  #  U+EA99
    'arrow-down': 60058,  #  U+EA9A
    'arrow-left': 60059,  #  U+EA9B
    'arrow-right': 60060,  #  U+EA9C
    'arrow-small-down': 60061,  #  U+EA9D
    'arrow-small-left': 60062,  #  U+EA9E
    'arrow-small-right': 60063,  #  U+EA9F
    'arrow-small-up': 60064,  #  U+EAA0
    'arrow-up': 60065,  #  U+EAA1
    'bell': 60066,  #  U+EAA2
    'bold': 60067,  #  U+EAA3
    'book': 60068,  #  U+EAA4
    'bookmark': 60069,  #  U+EAA5
    'debug-breakpoint-conditional-unverified': 60070,  #  U+EAA6
    'debug-breakpoint-conditional': 60071,  #  U+EAA7
    'debug-breakpoint-data-unverified': 60072,  #  U+EAA8
    'debug-breakpoint-data': 60073,  #  U+EAA9
    'debug-breakpoint-log-unverified': 60074,  #  U+EAAA
    'debug-breakpoint-log': 60075,  #  U+EAAB
    'briefcase': 60076,  #  U+EAAC
    'broadcast': 60077,  #  U+EAAD
    'browser': 60078,  #  U+EAAE
    'bug': 60079,  #  U+EAAF
    'calendar': 60080,  #  U+EAB0
    'case-sensitive': 60081,  #  U+EAB1
    'check': 60082,  #  U+EAB2
    'checklist': 60083,  #  U+EAB3
    'chevron-down': 60084,  #  U+EAB4
    'chevron-left': 60085,  #  U+EAB5
    'chevron-right': 60086,  #  U+EAB6
    'chevron-up': 60087,  #  U+EAB7
    'chrome-close': 60088,  #  U+EAB8
    'chrome-maximize': 60089,  #  U+EAB9
    'chrome-minimize': 60090,  #  U+EABA
    'chrome-restore': 60091,  #  U+EABB
    'circle': 60092,  #  U+EABC
    'circle-slash': 60093,  #  U+EABD
    'circuit-board': 60094,  #  U+EABE
    'clear-all': 60095,  #  U+EABF
    'clippy': 60096,  #  U+EAC0
    'close-all': 60097,  #  U+EAC1
    'cloud-download': 60098,  #  U+EAC2
    'cloud-upload': 60099,  #  U+EAC3
    'code': 60100,  #  U+EAC4
    'collapse-all': 60101,  #  U+EAC5
    'color-mode': 60102,  #  U+EAC6
    'comment-discussion': 60103,  #  U+EAC7
    'credit-card': 60105,  #  U+EAC9
    'dash': 60108,  #  U+EACC
    'dashboard': 60109,  #  U+EACD
    'database': 60110,  #  U+EACE
    'debug-continue': 60111,  #  U+EACF
    'debug-disconnect': 60112,  #  U+EAD0
    'debug-pause': 60113,  #  U+EAD1
    'debug-restart': 60114,  #  U+EAD2
    'debug-start': 60115,  #  U+EAD3
    'debug-step-into': 60116,  #  U+EAD4
    'debug-step-out': 60117,  #  U+EAD5
    'debug-step-over': 60118,  #  U+EAD6
    'debug-stop': 60119,  #  U+EAD7
    'debug': 60120,  #  U+EAD8
    'device-camera-video': 60121,  #  U+EAD9
    'device-camera': 60122,  #  U+EADA
    'device-mobile': 60123,  #  U+EADB
    'diff-added': 60124,  #  U+EADC
    'diff-ignored': 60125,  #  U+EADD
    'diff-modified': 60126,  #  U+EADE
    'diff-removed': 60127,  #  U+EADF
    'diff-renamed': 60128,  #  U+EAE0
    'diff': 60129,  #  U+EAE1
    'discard': 60130,  #  U+EAE2
    'editor-layout': 60131,  #  U+EAE3
    'empty-window': 60132,  #  U+EAE4
    'exclude': 60133,  #  U+EAE5
    'extensions': 60134,  #  U+EAE6
    'eye-closed': 60135,  #  U+EAE7
    'file-binary': 60136,  #  U+EAE8
    'file-code': 60137,  #  U+EAE9
    'file-media': 60138,  #  U+EAEA
    'file-pdf': 60139,  #  U+EAEB
    'file-submodule': 60140,  #  U+EAEC
    'file-symlink-directory': 60141,  #  U+EAED
    'file-symlink-file': 60142,  #  U+EAEE
    'file-zip': 60143,  #  U+EAEF
    'files': 60144,  #  U+EAF0
    'filter': 60145,  #  U+EAF1
    'flame': 60146,  #  U+EAF2
    'fold-down': 60147,  #  U+EAF3
    'fold-up': 60148,  #  U+EAF4
    'fold': 60149,  #  U+EAF5
    'folder-active': 60150,  #  U+EAF6
    'folder-opened': 60151,  #  U+EAF7
    'gear': 60152,  #  U+EAF8
    'gift': 60153,  #  U+EAF9
    'gist-secret': 60154,  #  U+EAFA
    'file-code#1': 60155,  #  U+EAFB
    'git-commit': 60156,  #  U+EAFC
    'git-compare': 60157,  #  U+EAFD
    'git-merge': 60158,  #  U+EAFE
    'github-action': 60159,  #  U+EAFF
    'github-alt': 60160,  #  U+EB00
    'globe': 60161,  #  U+EB01
    'grabber': 60162,  #  U+EB02
    'graph': 60163,  #  U+EB03
    'gripper': 60164,  #  U+EB04
    'heart#1': 60165,  #  U+EB05
    'home': 60166,  #  U+EB06
    'horizontal-rule': 60167,  #  U+EB07
    'hubot': 60168,  #  U+EB08
    'inbox': 60169,  #  U+EB09
    'issue-reopened': 60171,  #  U+EB0B
    'issues': 60172,  #  U+EB0C
    'italic': 60173,  #  U+EB0D
    'jersey': 60174,  #  U+EB0E
    'json': 60175,  #  U+EB0F
    'kebab-vertical': 60176,  #  U+EB10
    'key': 60177,  #  U+EB11
    'law': 60178,  #  U+EB12
    'lightbulb-autofix': 60179,  #  U+EB13
    'link-external': 60180,  #  U+EB14
    'link': 60181,  #  U+EB15
    'list-ordered': 60182,  #  U+EB16
    'list-unordered': 60183,  #  U+EB17
    'live-share': 60184,  #  U+EB18
    'loading': 60185,  #  U+EB19
    'location': 60186,  #  U+EB1A
    'mail-read': 60187,  #  U+EB1B
    'mail': 60188,  #  U+EB1C
    'markdown': 60189,  #  U+EB1D
    'megaphone': 60190,  #  U+EB1E
    'mention': 60191,  #  U+EB1F
    'milestone': 60192,  #  U+EB20
    'mortar-board': 60193,  #  U+EB21
    'move': 60194,  #  U+EB22
    'multiple-windows': 60195,  #  U+EB23
    'mute': 60196,  #  U+EB24
    'no-newline': 60197,  #  U+EB25
    'note': 60198,  #  U+EB26
    'octoface': 60199,  #  U+EB27
    'open-preview': 60200,  #  U+EB28
    'package': 60201,  #  U+EB29
    'paintcan': 60202,  #  U+EB2A
    'pin': 60203,  #  U+EB2B
    'play': 60204,  #  U+EB2C
    'plug': 60205,  #  U+EB2D
    'preserve-case': 60206,  #  U+EB2E
    'preview': 60207,  #  U+EB2F
    'project': 60208,  #  U+EB30
    'pulse#1': 60209,  #  U+EB31
    'question': 60210,  #  U+EB32
    'quote': 60211,  #  U+EB33
    'radio-tower': 60212,  #  U+EB34
    'reactions': 60213,  #  U+EB35
    'references': 60214,  #  U+EB36
    'refresh': 60215,  #  U+EB37
    'regex': 60216,  #  U+EB38
    'remote-explorer': 60217,  #  U+EB39
    'remote': 60218,  #  U+EB3A
    'remove': 60219,  #  U+EB3B
    'replace-all': 60220,  #  U+EB3C
    'replace': 60221,  #  U+EB3D
    'repo-clone': 60222,  #  U+EB3E
    'repo-force-push': 60223,  #  U+EB3F
    'repo-pull': 60224,  #  U+EB40
    'repo-push': 60225,  #  U+EB41
    'report': 60226,  #  U+EB42
    'request-changes': 60227,  #  U+EB43
    'rocket': 60228,  #  U+EB44
    'root-folder-opened': 60229,  #  U+EB45
    'root-folder': 60230,  #  U+EB46
    'rss': 60231,  #  U+EB47
    'ruby#1': 60232,  #  U+EB48
    'save-all': 60233,  #  U+EB49
    'save-as': 60234,  #  U+EB4A
    'save': 60235,  #  U+EB4B
    'screen-full': 60236,  #  U+EB4C
    'screen-normal': 60237,  #  U+EB4D
    'search-stop': 60238,  #  U+EB4E
    'server': 60240,  #  U+EB50
    'settings-gear': 60241,  #  U+EB51
    'settings': 60242,  #  U+EB52
    'shield': 60243,  #  U+EB53
    'smiley': 60244,  #  U+EB54
    'sort-precedence': 60245,  #  U+EB55
    'split-horizontal': 60246,  #  U+EB56
    'split-vertical': 60247,  #  U+EB57
    'squirrel': 60248,  #  U+EB58
    'star-full': 60249,  #  U+EB59
    'star-half': 60250,  #  U+EB5A
    'symbol-class': 60251,  #  U+EB5B
    'symbol-color': 60252,  #  U+EB5C
    'symbol-constant': 60253,  #  U+EB5D
    'symbol-enum-member': 60254,  #  U+EB5E
    'symbol-field': 60255,  #  U+EB5F
    'symbol-file': 60256,  #  U+EB60
    'symbol-interface': 60257,  #  U+EB61
    'symbol-keyword': 60258,  #  U+EB62
    'symbol-misc': 60259,  #  U+EB63
    'symbol-operator': 60260,  #  U+EB64
    'symbol-property': 60261,  #  U+EB65
    'symbol-snippet': 60262,  #  U+EB66
    'tasklist': 60263,  #  U+EB67
    'telescope#1': 60264,  #  U+EB68
    'text-size': 60265,  #  U+EB69
    'three-bars': 60266,  #  U+EB6A
    'thumbsdown': 60267,  #  U+EB6B
    'thumbsup': 60268,  #  U+EB6C
    'tools#1': 60269,  #  U+EB6D
    'triangle-down': 60270,  #  U+EB6E
    'triangle-left': 60271,  #  U+EB6F
    'triangle-right': 60272,  #  U+EB70
    'triangle-up': 60273,  #  U+EB71
    'twitter': 60274,  #  U+EB72
    'unfold': 60275,  #  U+EB73
    'unlock': 60276,  #  U+EB74
    'unmute': 60277,  #  U+EB75
    'unverified': 60278,  #  U+EB76
    'verified': 60279,  #  U+EB77
    'versions': 60280,  #  U+EB78
    'vm-active': 60281,  #  U+EB79
    'vm-outline': 60282,  #  U+EB7A
    'vm-running': 60283,  #  U+EB7B
    'watch': 60284,  #  U+EB7C
    'whitespace': 60285,  #  U+EB7D
    'whole-word': 60286,  #  U+EB7E
    'window': 60287,  #  U+EB7F
    'word-wrap': 60288,  #  U+EB80
    'zoom-in': 60289,  #  U+EB81
    'zoom-out': 60290,  #  U+EB82
    'list-filter': 60291,  #  U+EB83
    'list-flat': 60292,  #  U+EB84
    'list-selection': 60293,  #  U+EB85
    'list-tree': 60294,  #  U+EB86
    'debug-breakpoint-function-unverified': 60295,  #  U+EB87
    'debug-breakpoint-function': 60296,  #  U+EB88
    'debug-stackframe-active': 60297,  #  U+EB89
    'circle-small-filled': 60298,  #  U+EB8A
    'debug-stackframe': 60299,  #  U+EB8B
    'debug-breakpoint-unsupported': 60300,  #  U+EB8C
    'symbol-string': 60301,  #  U+EB8D
    'debug-reverse-continue': 60302,  #  U+EB8E
    'debug-step-back': 60303,  #  U+EB8F
    'debug-restart-frame': 60304,  #  U+EB90
    'debug-alt': 60305,  #  U+EB91
    'call-incoming': 60306,  #  U+EB92
    'call-outgoing': 60307,  #  U+EB93
    'menu': 60308,  #  U+EB94
    'expand-all': 60309,  #  U+EB95
    'feedback': 60310,  #  U+EB96
    'group-by-ref-type': 60311,  #  U+EB97
    'ungroup-by-ref-type': 60312,  #  U+EB98
    'account': 60313,  #  U+EB99
    'bell-dot': 60314,  #  U+EB9A
    'debug-console': 60315,  #  U+EB9B
    'library': 60316,  #  U+EB9C
    'output': 60317,  #  U+EB9D
    'run-all': 60318,  #  U+EB9E
    'sync-ignored': 60319,  #  U+EB9F
    'pinned': 60320,  #  U+EBA0
    'github-inverted': 60321,  #  U+EBA1
    'server-process': 60322,  #  U+EBA2
    'server-environment': 60323,  #  U+EBA3
    'pass': 60324,  #  U+EBA4
    'stop-circle': 60325,  #  U+EBA5
    'play-circle': 60326,  #  U+EBA6
    'record': 60327,  #  U+EBA7
    'debug-alt-small': 60328,  #  U+EBA8
    'vm-connect': 60329,  #  U+EBA9
    'cloud#1': 60330,  #  U+EBAA
    'merge': 60331,  #  U+EBAB
    'export': 60332,  #  U+EBAC
    'graph-left': 60333,  #  U+EBAD
    'magnet': 60334,  #  U+EBAE
    'notebook': 60335,  #  U+EBAF
    'redo': 60336,  #  U+EBB0
    'check-all': 60337,  #  U+EBB1
    'pinned-dirty': 60338,  #  U+EBB2
    'pass-filled': 60339,  #  U+EBB3
    'circle-large-filled': 60340,  #  U+EBB4
    'circle-large': 60341,  #  U+EBB5
    'combine': 60342,  #  U+EBB6
    'table': 60343,  #  U+EBB7
    'variable-group': 60344,  #  U+EBB8
    'type-hierarchy': 60345,  #  U+EBB9
    'type-hierarchy-sub': 60346,  #  U+EBBA
    'type-hierarchy-super': 60347,  #  U+EBBB
    'source-branch-plus': 60348,  #  U+EBBC
    'run-above': 60349,  #  U+EBBD
    'run-below': 60350,  #  U+EBBE
    'notebook-template': 60351,  #  U+EBBF
    'debug-rerun': 60352,  #  U+EBC0
    'workspace-trusted': 60353,  #  U+EBC1
    'workspace-untrusted': 60354,  #  U+EBC2
    'workspace-unknown': 60355,  #  U+EBC3
    'terminal-cmd': 60356,  #  U+EBC4
    'terminal-debian': 60357,  #  U+EBC5
    'terminal-linux': 60358,  #  U+EBC6
    'terminal-powershell': 60359,  #  U+EBC7
    'terminal-tmux': 60360,  #  U+EBC8
    'terminal-ubuntu': 60361,  #  U+EBC9
    'terminal-bash': 60362,  #  U+EBCA
    'arrow-swap': 60363,  #  U+EBCB
    'copy': 60364,  #  U+EBCC
    'person-add': 60365,  #  U+EBCD
    'filter-filled': 60366,  #  U+EBCE
    'wand': 60367,  #  U+EBCF
    'debug-line-by-line': 60368,  #  U+EBD0
    'inspect': 60369,  #  U+EBD1
    'layers#1': 60370,  #  U+EBD2
    'layers-dot': 60371,  #  U+EBD3
    'layers-active': 60372,  #  U+EBD4
    'compass': 60373,  #  U+EBD5
    'compass-dot': 60374,  #  U+EBD6
    'compass-active': 60375,  #  U+EBD7
    'azure': 60376,  #  U+EBD8
    'issue-draft': 60377,  #  U+EBD9
    'git-pull-request-closed': 60378,  #  U+EBDA
    'git-pull-request-draft': 60379,  #  U+EBDB
    'debug-all': 60380,  #  U+EBDC
    'debug-coverage': 60381,  #  U+EBDD
    'run-errors': 60382,  #  U+EBDE
    'folder-library': 60383,  #  U+EBDF
    'debug-continue-small': 60384,  #  U+EBE0
    'beaker-stop': 60385,  #  U+EBE1
    'graph-line': 60386,  #  U+EBE2
    'graph-scatter': 60387,  #  U+EBE3
    'pie-chart': 60388,  #  U+EBE4
    'bracket-dot': 60389,  #  U+EBE5
    'bracket-error': 60390,  #  U+EBE6
    'lock-small': 60391,  #  U+EBE7
    'azure-devops': 60392,  #  U+EBE8
    'verified-filled': 60393,  #  U+EBE9
    'newline': 60394,  #  U+EBEA
    'layout': 60395,  #  U+EBEB
    'layout-activitybar-left': 60396,  #  U+EBEC
    'layout-activitybar-right': 60397,  #  U+EBED
    'layout-panel-left': 60398,  #  U+EBEE
    'layout-panel-center': 60399,  #  U+EBEF
    'layout-panel-justify': 60400,  #  U+EBF0
    'layout-panel-right': 60401,  #  U+EBF1
    'layout-panel': 60402,  #  U+EBF2
    'layout-sidebar-left': 60403,  #  U+EBF3
    'layout-sidebar-right': 60404,  #  U+EBF4
    'layout-statusbar': 60405,  #  U+EBF5
    'layout-menubar': 60406,  #  U+EBF6
    'layout-centered': 60407,  #  U+EBF7
    'target': 60408,  #  U+EBF8
    'indent': 60409,  #  U+EBF9
    'record-small': 60410,  #  U+EBFA
    'error-small': 60411,  #  U+EBFB
    'arrow-circle-down': 60412,  #  U+EBFC
    'arrow-circle-left': 60413,  #  U+EBFD
    'arrow-circle-right': 60414,  #  U+EBFE
    'arrow-circle-up': 60415,  #  U+EBFF
    'layout-sidebar-right-off': 60416,  #  U+EC00
    'layout-panel-off': 60417,  #  U+EC01
    'layout-sidebar-left-off': 60418,  #  U+EC02
    'blank': 60419,  #  U+EC03
    'heart-filled': 60420,  #  U+EC04
    'map': 60421,  #  U+EC05
    'map-filled': 60422,  #  U+EC06
    'circle-small': 60423,  #  U+EC07
    'bell-slash': 60424,  #  U+EC08
    'bell-slash-dot': 60425,  #  U+EC09
    'comment-unresolved': 60426,  #  U+EC0A
    'git-pull-request-go-to-changes': 60427,  #  U+EC0B
    'git-pull-request-new-changes': 60428,  #  U+EC0C
    'search-fuzzy': 60429,  #  U+EC0D
    'comment-draft': 60430,  #  U+EC0E
    'send': 60431,  #  U+EC0F
    'sparkle': 60432,  #  U+EC10
    'insert': 60433,  #  U+EC11
    'mic': 60434,  #  U+EC12
    'thumbsdown-filled': 60435,  #  U+EC13
    'thumbsup-filled': 60436,  #  U+EC14
    'coffee': 60437,  #  U+EC15
    'snake': 60438,  #  U+EC16
    'game': 60439,  #  U+EC17
    'vr': 60440,  #  U+EC18
    'chip#1': 60441,  #  U+EC19
    'piano': 60442,  #  U+EC1A
    'music': 60443,  #  U+EC1B
    'mic-filled': 60444,  #  U+EC1C
    'git-fetch': 60445,  #  U+EC1D
    'copilot': 60446,  #  U+EC1E
    'location_dot': 60672,  #  U+ED00
    'medapps': 60673,  #  U+ED01
    'medrt': 60674,  #  U+ED02
    'microphone_lines': 60675,  #  U+ED03
    'microsoft': 60676,  #  U+ED04
    'mix': 60677,  #  U+ED05
    'mizuni': 60678,  #  U+ED06
    'mobile_button': 60679,  #  U+ED07
    'mobile': 60680,  #  U+ED08
    'mobile_screen': 60681,  #  U+ED09
    'monero': 60682,  #  U+ED0A
    'money_bill_1': 60683,  #  U+ED0B
    'napster': 60684,  #  U+ED0C
    'node_js': 60685,  #  U+ED0D
    'npm': 60686,  #  U+ED0E
    'ns8': 60687,  #  U+ED0F
    'nutritionix': 60688,  #  U+ED10
    'page4': 60689,  #  U+ED11
    'palfed': 60690,  #  U+ED12
    'patreon': 60691,  #  U+ED13
    'periscope': 60692,  #  U+ED14
    'phabricator': 60693,  #  U+ED15
    'phoenix_framework': 60694,  #  U+ED16
    'phone_slash': 60695,  #  U+ED17
    'playstation#1': 60696,  #  U+ED18
    'image_portrait': 60697,  #  U+ED19
    'pushed': 60698,  #  U+ED1A
    'python#1': 60699,  #  U+ED1B
    'red_river': 60700,  #  U+ED1C
    'wpressr': 60701,  #  U+ED1D
    'replyd': 60702,  #  U+ED1E
    'resolving': 60703,  #  U+ED1F
    'rocketchat': 60704,  #  U+ED20
    'rockrms': 60705,  #  U+ED21
    'schlix': 60706,  #  U+ED22
    'searchengin': 60707,  #  U+ED23
    'servicestack': 60708,  #  U+ED24
    'shield_halved': 60709,  #  U+ED25
    'sistrix': 60710,  #  U+ED26
    'speakap': 60711,  #  U+ED27
    'staylinked': 60712,  #  U+ED28
    'steam_symbol': 60713,  #  U+ED29
    'sticker_mule': 60714,  #  U+ED2A
    'studiovinari': 60715,  #  U+ED2B
    'supple': 60716,  #  U+ED2C
    'tablet_button': 60717,  #  U+ED2D
    'tablet': 60718,  #  U+ED2E
    'gauge_high': 60719,  #  U+ED2F
    'ticket_simple': 60720,  #  U+ED30
    'uber': 60721,  #  U+ED31
    'uikit': 60722,  #  U+ED32
    'uniregistry': 60723,  #  U+ED33
    'untappd': 60724,  #  U+ED34
    'user_large': 60725,  #  U+ED35
    'ussunnah': 60726,  #  U+ED36
    'vaadin': 60727,  #  U+ED37
    'viber': 60728,  #  U+ED38
    'vimeo': 60729,  #  U+ED39
    'vnv': 60730,  #  U+ED3A
    'square_whatsapp': 60731,  #  U+ED3B
    'whmcs': 60732,  #  U+ED3C
    'wordpress_simple': 60733,  #  U+ED3D
    'xbox#1': 60734,  #  U+ED3E
    'yandex': 60735,  #  U+ED3F
    'yandex_international': 60736,  #  U+ED40
    'apple_pay': 60737,  #  U+ED41
    'cc_apple_pay': 60738,  #  U+ED42
    'fly': 60739,  #  U+ED43
    'node': 60740,  #  U+ED44
    'osi': 60741,  #  U+ED45
    'react': 60742,  #  U+ED46
    'autoprefixer': 60743,  #  U+ED47
    'less': 60744,  #  U+ED48
    'sass': 60745,  #  U+ED49
    'vuejs': 60746,  #  U+ED4A
    'angular': 60747,  #  U+ED4B
    'aviato': 60748,  #  U+ED4C
    'down_left_and_up_right_to_center': 60749,  #  U+ED4D
    'ember': 60750,  #  U+ED4E
    'up_right_and_down_left_from_center': 60751,  #  U+ED4F
    'gitter': 60752,  #  U+ED50
    'hooli': 60753,  #  U+ED51
    'strava': 60754,  #  U+ED52
    'stripe': 60755,  #  U+ED53
    'stripe_s': 60756,  #  U+ED54
    'typo3': 60757,  #  U+ED55
    'amazon_pay': 60758,  #  U+ED56
    'cc_amazon_pay': 60759,  #  U+ED57
    'ethereum': 60760,  #  U+ED58
    'korvue': 60761,  #  U+ED59
    'elementor': 60762,  #  U+ED5A
    'baseball_bat_ball': 60763,  #  U+ED5B
    'baseball': 60764,  #  U+ED5C
    'basketball': 60765,  #  U+ED5D
    'bowling_ball': 60766,  #  U+ED5E
    'chess': 60767,  #  U+ED5F
    'chess_bishop': 60768,  #  U+ED60
    'chess_board': 60769,  #  U+ED61
    'chess_king': 60770,  #  U+ED62
    'chess_knight': 60771,  #  U+ED63
    'chess_pawn': 60772,  #  U+ED64
    'chess_queen': 60773,  #  U+ED65
    'chess_rook': 60774,  #  U+ED66
    'dumbbell': 60775,  #  U+ED67
    'flipboard': 60776,  #  U+ED68
    'football': 60777,  #  U+ED69
    'golf_ball_tee': 60778,  #  U+ED6A
    'hips': 60779,  #  U+ED6B
    'hockey_puck': 60780,  #  U+ED6C
    'php': 60781,  #  U+ED6D
    'broom_ball': 60782,  #  U+ED6E
    'quinscape': 60783,  #  U+ED6F
    'square_full': 60784,  #  U+ED70
    'table_tennis_paddle_ball': 60785,  #  U+ED71
    'volleyball': 60786,  #  U+ED72
    'hand_dots': 60787,  #  U+ED73
    'bandage': 60788,  #  U+ED74
    'box': 60789,  #  U+ED75
    'boxes_stacked': 60790,  #  U+ED76
    'briefcase_medical': 60791,  #  U+ED77
    'fire_flame_simple': 60792,  #  U+ED78
    'capsules': 60793,  #  U+ED79
    'clipboard_check': 60794,  #  U+ED7A
    'clipboard_list': 60795,  #  U+ED7B
    'person_dots_from_line': 60796,  #  U+ED7C
    'dna#1': 60797,  #  U+ED7D
    'dolly': 60798,  #  U+ED7E
    'cart_flatbed': 60799,  #  U+ED7F
    'file_medical': 60800,  #  U+ED80
    'file_waveform': 60801,  #  U+ED81
    'kit_medical': 60802,  #  U+ED82
    'circle_h': 60803,  #  U+ED83
    'id_card_clip': 60804,  #  U+ED84
    'notes_medical': 60805,  #  U+ED85
    'pallet': 60806,  #  U+ED86
    'pills': 60807,  #  U+ED87
    'prescription_bottle': 60808,  #  U+ED88
    'prescription_bottle_medical': 60809,  #  U+ED89
    'bed_pulse': 60810,  #  U+ED8A
    'truck_fast': 60811,  #  U+ED8B
    'smoking': 60812,  #  U+ED8C
    'syringe': 60813,  #  U+ED8D
    'tablets': 60814,  #  U+ED8E
    'thermometer_alt': 60815,  #  U+ED8F
    'vial': 60816,  #  U+ED90
    'vials': 60817,  #  U+ED91
    'warehouse': 60818,  #  U+ED92
    'weight_scale': 60819,  #  U+ED93
    'x_ray': 60820,  #  U+ED94
    'box_open': 60821,  #  U+ED95
    'comment_slash': 60822,  #  U+ED96
    'couch': 60823,  #  U+ED97
    'circle_dollar_to_slot': 60824,  #  U+ED98
    'dove': 60825,  #  U+ED99
    'hand_holding': 60826,  #  U+ED9A
    'hand_holding_heart': 60827,  #  U+ED9B
    'hand_holding_dollar': 60828,  #  U+ED9C
    'hand_holding_droplet': 60829,  #  U+ED9D
    'hands_holding': 60830,  #  U+ED9E
    'handshake_angle': 60831,  #  U+ED9F
    'handshake_simple': 60832,  #  U+EDA0
    'parachute_box': 60833,  #  U+EDA1
    'people_carry_box': 60834,  #  U+EDA2
    'piggy_bank': 60835,  #  U+EDA3
    'readme': 60836,  #  U+EDA4
    'ribbon': 60837,  #  U+EDA5
    'route': 60838,  #  U+EDA6
    'seedling': 60839,  #  U+EDA7
    'sign_hanging': 60840,  #  U+EDA8
    'face_smile_wink': 60841,  #  U+EDA9
    'tape': 60842,  #  U+EDAA
    'truck_ramp_box': 60843,  #  U+EDAB
    'truck_moving': 60844,  #  U+EDAC
    'video_slash': 60845,  #  U+EDAD
    'wine_glass': 60846,  #  U+EDAE
    'java#1': 60847,  #  U+EDAF
    'pied_piper_hat': 60848,  #  U+EDB0
    'creative_commons_by': 60849,  #  U+EDB1
    'creative_commons_nc': 60850,  #  U+EDB2
    'creative_commons_nc_eu': 60851,  #  U+EDB3
    'creative_commons_nc_jp': 60852,  #  U+EDB4
    'creative_commons_nd': 60853,  #  U+EDB5
    'creative_commons_pd': 60854,  #  U+EDB6
    'creative_commons_pd_alt': 60855,  #  U+EDB7
    'creative_commons_remix': 60856,  #  U+EDB8
    'creative_commons_sa': 60857,  #  U+EDB9
    'creative_commons_sampling': 60858,  #  U+EDBA
    'creative_commons_sampling_plus': 60859,  #  U+EDBB
    'creative_commons_share': 60860,  #  U+EDBC
    'creative_commons_zero': 60861,  #  U+EDBD
    'ebay': 60862,  #  U+EDBE
    'keybase': 60863,  #  U+EDBF
    'mastodon': 60864,  #  U+EDC0
    'r_project': 60865,  #  U+EDC1
    'researchgate': 60866,  #  U+EDC2
    'teamspeak': 60867,  #  U+EDC3
    'user_large_slash': 60868,  #  U+EDC4
    'user_astronaut': 60869,  #  U+EDC5
    'user_check': 60870,  #  U+EDC6
    'user_clock': 60871,  #  U+EDC7
    'user_gear': 60872,  #  U+EDC8
    'user_pen': 60873,  #  U+EDC9
    'user_group': 60874,  #  U+EDCA
    'user_graduate': 60875,  #  U+EDCB
    'user_lock': 60876,  #  U+EDCC
    'user_minus': 60877,  #  U+EDCD
    'user_ninja': 60878,  #  U+EDCE
    'user_shield': 60879,  #  U+EDCF
    'user_slash': 60880,  #  U+EDD0
    'user_tag': 60881,  #  U+EDD1
    'user_tie': 60882,  #  U+EDD2
    'users_gear': 60883,  #  U+EDD3
    'first_order_alt': 60884,  #  U+EDD4
    'fulcrum': 60885,  #  U+EDD5
    'galactic_republic': 60886,  #  U+EDD6
    'galactic_senate': 60887,  #  U+EDD7
    'jedi_order': 60888,  #  U+EDD8
    'mandalorian': 60889,  #  U+EDD9
    'old_republic': 60890,  #  U+EDDA
    'phoenix_squadron': 60891,  #  U+EDDB
    'sith': 60892,  #  U+EDDC
    'trade_federation': 60893,  #  U+EDDD
    'wolf_pack_battalion': 60894,  #  U+EDDE
    'scale_unbalanced': 60895,  #  U+EDDF
    'scale_unbalanced_flip': 60896,  #  U+EDE0
    'blender': 60897,  #  U+EDE1
    'book_open': 60898,  #  U+EDE2
    'tower_broadcast': 60899,  #  U+EDE3
    'broom': 60900,  #  U+EDE4
    'chalkboard': 60901,  #  U+EDE5
    'chalkboard_user': 60902,  #  U+EDE6
    'church': 60903,  #  U+EDE7
    'coins#1': 60904,  #  U+EDE8
    'compact_disc': 60905,  #  U+EDE9
    'crow': 60906,  #  U+EDEA
    'crown#1': 60907,  #  U+EDEB
    'dice#1': 60908,  #  U+EDEC
    'dice_five': 60909,  #  U+EDED
    'dice_four': 60910,  #  U+EDEE
    'dice_one': 60911,  #  U+EDEF
    'dice_six': 60912,  #  U+EDF0
    'dice_three': 60913,  #  U+EDF1
    'dice_two': 60914,  #  U+EDF2
    'divide': 60915,  #  U+EDF3
    'door_closed': 60916,  #  U+EDF4
    'door_open': 60917,  #  U+EDF5
    'equals': 60918,  #  U+EDF6
    'feather': 60919,  #  U+EDF7
    'frog': 60920,  #  U+EDF8
    'gas_pump': 60921,  #  U+EDF9
    'glasses': 60922,  #  U+EDFA
    'greater_than': 60923,  #  U+EDFB
    'greater_than_equal': 60924,  #  U+EDFC
    'helicopter': 60925,  #  U+EDFD
    'infinity#1': 60926,  #  U+EDFE
    'kiwi_bird': 60927,  #  U+EDFF
    'receipt': 60940,  #  U+EE0C
    'robot': 60941,  #  U+EE0D
    'ruler#1': 60942,  #  U+EE0E
    'ruler_combined': 60943,  #  U+EE0F
    'ruler_horizontal': 60944,  #  U+EE10
    'ruler_vertical': 60945,  #  U+EE11
    'school': 60946,  #  U+EE12
    'screwdriver': 60947,  #  U+EE13
    'shoe_prints': 60948,  #  U+EE14
    'skull': 60949,  #  U+EE15
    'ban_smoking': 60950,  #  U+EE16
    'store': 60951,  #  U+EE17
    'shop': 60952,  #  U+EE18
    'bars_staggered': 60953,  #  U+EE19
    'stroopwafel': 60954,  #  U+EE1A
    'toolbox': 60955,  #  U+EE1B
    'shirt#1': 60956,  #  U+EE1C
    'person_walking': 60957,  #  U+EE1D
    'wallet#1': 60958,  #  U+EE1E
    'face_angry': 60959,  #  U+EE1F
    'archway': 60960,  #  U+EE20
    'book_atlas': 60961,  #  U+EE21
    'award': 60962,  #  U+EE22
    'delete_left': 60963,  #  U+EE23
    'bezier_curve': 60964,  #  U+EE24
    'bong': 60965,  #  U+EE25
    'brush': 60966,  #  U+EE26
    'bus_simple': 60967,  #  U+EE27
    'cannabis': 60968,  #  U+EE28
    'check_double': 60969,  #  U+EE29
    'martini_glass_citrus': 60970,  #  U+EE2A
    'bell_concierge': 60971,  #  U+EE2B
    'cookie': 60972,  #  U+EE2C
    'cookie_bite': 60973,  #  U+EE2D
    'crop_simple': 60974,  #  U+EE2E
    'tachograph_digital': 60975,  #  U+EE2F
    'face_dizzy': 60976,  #  U+EE30
    'compass_drafting': 60977,  #  U+EE31
    'drum': 60978,  #  U+EE32
    'drum_steelpan': 60979,  #  U+EE33
    'feather_pointed': 60980,  #  U+EE34
    'file_contract': 60981,  #  U+EE35
    'file_arrow_down': 60982,  #  U+EE36
    'file_export': 60983,  #  U+EE37
    'file_import': 60984,  #  U+EE38
    'file_invoice': 60985,  #  U+EE39
    'file_invoice_dollar': 60986,  #  U+EE3A
    'file_prescription': 60987,  #  U+EE3B
    'file_signature': 60988,  #  U+EE3C
    'file_arrow_up': 60989,  #  U+EE3D
    'fill': 60990,  #  U+EE3E
    'fill_drip': 60991,  #  U+EE3F
    'fingerprint#1': 60992,  #  U+EE40
    'fish': 60993,  #  U+EE41
    'face_flushed': 60994,  #  U+EE42
    'face_frown_open': 60995,  #  U+EE43
    'martini_glass': 60996,  #  U+EE44
    'earth_africa': 60997,  #  U+EE45
    'earth_americas': 60998,  #  U+EE46
    'earth_asia': 60999,  #  U+EE47
    'face_grimace': 61000,  #  U+EE48
    'face_grin': 61001,  #  U+EE49
    'face_grin_wide': 61002,  #  U+EE4A
    'face_grin_beam': 61003,  #  U+EE4B
    'face_grin_beam_sweat': 61004,  #  U+EE4C
    'face_grin_hearts': 61005,  #  U+EE4D
    'face_grin_squint': 61006,  #  U+EE4E
    'face_grin_squint_tears': 61007,  #  U+EE4F
    'face_grin_stars': 61008,  #  U+EE50
    'face_grin_tears': 61009,  #  U+EE51
    'face_grin_tongue': 61010,  #  U+EE52
    'face_grin_tongue_squint': 61011,  #  U+EE53
    'face_grin_tongue_wink': 61012,  #  U+EE54
    'face_grin_wink': 61013,  #  U+EE55
    'grip': 61014,  #  U+EE56
    'grip_vertical': 61015,  #  U+EE57
    'headphones_simple': 61016,  #  U+EE58
    'headset': 61017,  #  U+EE59
    'highlighter': 61018,  #  U+EE5A
    'hornbill': 61019,  #  U+EE5B
    'hot_tub_person': 61020,  #  U+EE5C
    'hotel_building': 61021,  #  U+EE5D
    'joint': 61022,  #  U+EE5E
    'face_kiss': 61023,  #  U+EE5F
    'face_kiss_beam': 61024,  #  U+EE60
    'face_kiss_wink_heart': 61025,  #  U+EE61
    'face_laugh': 61026,  #  U+EE62
    'face_laugh_beam': 61027,  #  U+EE63
    'face_laugh_squint': 61028,  #  U+EE64
    'face_laugh_wink': 61029,  #  U+EE65
    'cart_flatbed_suitcase': 61030,  #  U+EE66
    'mailchimp': 61031,  #  U+EE67
    'map_location': 61032,  #  U+EE68
    'map_location_dot': 61033,  #  U+EE69
    'marker': 61034,  #  U+EE6A
    'medal': 61035,  #  U+EE6B
    'megaport': 61036,  #  U+EE6C
    'face_meh_blank': 61037,  #  U+EE6D
    'face_rolling_eyes': 61038,  #  U+EE6E
    'monument': 61039,  #  U+EE6F
    'mortar_pestle': 61040,  #  U+EE70
    'nimblr': 61041,  #  U+EE71
    'paint_roller': 61042,  #  U+EE72
    'passport': 61043,  #  U+EE73
    'pen_fancy': 61044,  #  U+EE74
    'pen_nib': 61045,  #  U+EE75
    'pen_ruler': 61046,  #  U+EE76
    'plane_arrival': 61047,  #  U+EE77
    'plane_departure': 61048,  #  U+EE78
    'prescription': 61049,  #  U+EE79
    'rev': 61050,  #  U+EE7A
    'face_sad_cry': 61051,  #  U+EE7B
    'face_sad_tear': 61052,  #  U+EE7C
    'shopware': 61053,  #  U+EE7D
    'van_shuttle': 61054,  #  U+EE7E
    'signature': 61055,  #  U+EE7F
    'face_smile_beam': 61056,  #  U+EE80
    'solar_panel': 61057,  #  U+EE81
    'spa': 61058,  #  U+EE82
    'splotch': 61059,  #  U+EE83
    'spray_can': 61060,  #  U+EE84
    'squarespace': 61061,  #  U+EE85
    'stamp': 61062,  #  U+EE86
    'star_half_stroke': 61063,  #  U+EE87
    'suitcase_rolling': 61064,  #  U+EE88
    'face_surprise': 61065,  #  U+EE89
    'swatchbook': 61066,  #  U+EE8A
    'person_swimming': 61067,  #  U+EE8B
    'water_ladder': 61068,  #  U+EE8C
    'themeco': 61069,  #  U+EE8D
    'droplet_slash': 61070,  #  U+EE8E
    'face_tired': 61071,  #  U+EE8F
    'tooth#1': 61072,  #  U+EE90
    'umbrella_beach': 61073,  #  U+EE91
    'vector_square': 61074,  #  U+EE92
    'weebly': 61075,  #  U+EE93
    'weight_hanging': 61076,  #  U+EE94
    'wine_glass_empty': 61077,  #  U+EE95
    'wix': 61078,  #  U+EE96
    'spray_can_sparkles': 61079,  #  U+EE97
    'apple_whole': 61080,  #  U+EE98
    'atom#1': 61081,  #  U+EE99
    'bone': 61082,  #  U+EE9A
    'book_open_reader': 61083,  #  U+EE9B
    'brain#1': 61084,  #  U+EE9C
    'car_rear': 61085,  #  U+EE9D
    'car_battery': 61086,  #  U+EE9E
    'car_burst': 61087,  #  U+EE9F
    'car_side': 61088,  #  U+EEA0
    'charging_station': 61089,  #  U+EEA1
    'diamond_turn_right': 61090,  #  U+EEA2
    'draw_polygon': 61091,  #  U+EEA3
    'ello#1': 61092,  #  U+EEA4
    'hackerrank': 61093,  #  U+EEA5
    'kaggle': 61094,  #  U+EEA6
    'laptop_code': 61095,  #  U+EEA7
    'layer_group': 61096,  #  U+EEA8
    'location_crosshairs': 61097,  #  U+EEA9
    'lungs': 61098,  #  U+EEAA
    'markdown#1': 61099,  #  U+EEAB
    'microscope#1': 61100,  #  U+EEAC
    'neos': 61101,  #  U+EEAD
    'oil_can': 61102,  #  U+EEAE
    'poop': 61103,  #  U+EEAF
    'shapes': 61104,  #  U+EEB0
    'star_of_life': 61105,  #  U+EEB1
    'gauge': 61106,  #  U+EEB2
    'gauge_simple': 61107,  #  U+EEB3
    'teeth': 61108,  #  U+EEB4
    'teeth_open': 61109,  #  U+EEB5
    'masks_theater': 61110,  #  U+EEB6
    'traffic_light': 61111,  #  U+EEB7
    'truck_monster': 61112,  #  U+EEB8
    'truck_pickup': 61113,  #  U+EEB9
    'zhihu': 61114,  #  U+EEBA
    'rectangle_ad': 61115,  #  U+EEBB
    'alipay': 61116,  #  U+EEBC
    'ankh': 61117,  #  U+EEBD
    'book_bible': 61118,  #  U+EEBE
    'business_time': 61119,  #  U+EEBF
    'city': 61120,  #  U+EEC0
    'comment_dollar': 61121,  #  U+EEC1
    'comments_dollar': 61122,  #  U+EEC2
    'cross': 61123,  #  U+EEC3
    'dharmachakra': 61124,  #  U+EEC4
    'envelope_open_text': 61125,  #  U+EEC5
    'folder_minus': 61126,  #  U+EEC6
    'folder_plus': 61127,  #  U+EEC7
    'filter_circle_dollar': 61128,  #  U+EEC8
    'gopuram': 61129,  #  U+EEC9
    'hamsa': 61130,  #  U+EECA
    'bahai': 61131,  #  U+EECB
    'jedi': 61132,  #  U+EECC
    'book_journal_whills': 61133,  #  U+EECD
    'kaaba': 61134,  #  U+EECE
    'khanda': 61135,  #  U+EECF
    'landmark': 61136,  #  U+EED0
    'envelopes_bulk': 61137,  #  U+EED1
    'menorah': 61138,  #  U+EED2
    'mosque': 61139,  #  U+EED3
    'om': 61140,  #  U+EED4
    'spaghetti_monster_flying': 61141,  #  U+EED5
    'peace': 61142,  #  U+EED6
    'place_of_worship': 61143,  #  U+EED7
    'square_poll_vertical': 61144,  #  U+EED8
    'square_poll_horizontal': 61145,  #  U+EED9
    'person_praying': 61146,  #  U+EEDA
    'hands_praying': 61147,  #  U+EEDB
    'book_quran': 61148,  #  U+EEDC
    'magnifying_glass_dollar': 61149,  #  U+EEDD
    'magnifying_glass_location': 61150,  #  U+EEDE
    'socks': 61151,  #  U+EEDF
    'square_root_variable': 61152,  #  U+EEE0
    'star_and_crescent': 61153,  #  U+EEE1
    'star_of_david': 61154,  #  U+EEE2
    'synagogue': 61155,  #  U+EEE3
    'the_red_yeti': 61156,  #  U+EEE4
    'scroll_torah': 61157,  #  U+EEE5
    'torii_gate': 61158,  #  U+EEE6
    'vihara': 61159,  #  U+EEE7
    'volume_xmark': 61160,  #  U+EEE8
    'yin_yang': 61161,  #  U+EEE9
    'blender_phone': 61162,  #  U+EEEA
    'book_skull': 61163,  #  U+EEEB
    'campground': 61164,  #  U+EEEC
    'cat': 61165,  #  U+EEED
    'chair': 61166,  #  U+EEEE
    'cloud_moon': 61167,  #  U+EEEF
    'cloud_sun': 61168,  #  U+EEF0
    'cow': 61169,  #  U+EEF1
    'critical_role': 61170,  #  U+EEF2
    'd_and_d_beyond': 61171,  #  U+EEF3
    'dev': 61172,  #  U+EEF4
    'dice_d20': 61173,  #  U+EEF5
    'dice_d6': 61174,  #  U+EEF6
    'dog': 61175,  #  U+EEF7
    'dragon': 61176,  #  U+EEF8
    'drumstick_bite': 61177,  #  U+EEF9
    'dungeon': 61178,  #  U+EEFA
    'fantasy_flight_games': 61179,  #  U+EEFB
    'file_csv': 61180,  #  U+EEFC
    'hand_fist': 61181,  #  U+EEFD
    'ghost': 61182,  #  U+EEFE
    'hammer': 61183,  #  U+EEFF
    'hanukiah': 61184,  #  U+EF00
    'hat_wizard': 61185,  #  U+EF01
    'person_hiking': 61186,  #  U+EF02
    'hippo': 61187,  #  U+EF03
    'horse': 61188,  #  U+EF04
    'house_chimney_crack': 61189,  #  U+EF05
    'hryvnia_sign': 61190,  #  U+EF06
    'mask': 61191,  #  U+EF07
    'mountain': 61192,  #  U+EF08
    'network_wired': 61193,  #  U+EF09
    'otter': 61194,  #  U+EF0A
    'ring#1': 61195,  #  U+EF0B
    'person_running': 61196,  #  U+EF0C
    'scroll': 61197,  #  U+EF0D
    'skull_crossbones': 61198,  #  U+EF0E
    'slash#1': 61199,  #  U+EF0F
    'spider': 61200,  #  U+EF10
    'toilet_paper': 61201,  #  U+EF11
    'tractor': 61202,  #  U+EF12
    'user_injured': 61203,  #  U+EF13
    'vr_cardboard': 61204,  #  U+EF14
    'wand_sparkles': 61205,  #  U+EF15
    'wind#1': 61206,  #  U+EF16
    'wine_bottle': 61207,  #  U+EF17
    'wizards_of_the_coast': 61208,  #  U+EF18
    'think_peaks': 61209,  #  U+EF19
    'cloud_meatball': 61210,  #  U+EF1A
    'cloud_moon_rain': 61211,  #  U+EF1B
    'cloud_rain': 61212,  #  U+EF1C
    'cloud_showers_heavy': 61213,  #  U+EF1D
    'cloud_sun_rain': 61214,  #  U+EF1E
    'democrat': 61215,  #  U+EF1F
    'flag_usa': 61216,  #  U+EF20
    'hurricane': 61217,  #  U+EF21
    'landmark_dome': 61218,  #  U+EF22
    'meteor': 61219,  #  U+EF23
    'person_booth': 61220,  #  U+EF24
    'poo_storm': 61221,  #  U+EF25
    'rainbow': 61222,  #  U+EF26
    'reacteurope': 61223,  #  U+EF27
    'republican': 61224,  #  U+EF28
    'smog': 61225,  #  U+EF29
    'temperature_high': 61226,  #  U+EF2A
    'temperature_low': 61227,  #  U+EF2B
    'cloud_bolt': 61228,  #  U+EF2C
    'tornado': 61229,  #  U+EF2D
    'volcano': 61230,  #  U+EF2E
    'check_to_slot': 61231,  #  U+EF2F
    'water': 61232,  #  U+EF30
    'artstation': 61233,  #  U+EF31
    'atlassian': 61234,  #  U+EF32
    'baby': 61235,  #  U+EF33
    'baby_carriage': 61236,  #  U+EF34
    'biohazard#1': 61237,  #  U+EF35
    'blog': 61238,  #  U+EF36
    'calendar_day': 61239,  #  U+EF37
    'calendar_week': 61240,  #  U+EF38
    'canadian_maple_leaf': 61241,  #  U+EF39
    'candy_cane': 61242,  #  U+EF3A
    'carrot': 61243,  #  U+EF3B
    'cash_register': 61244,  #  U+EF3C
    'centos': 61245,  #  U+EF3D
    'minimize#1': 61246,  #  U+EF3E
    'confluence': 61247,  #  U+EF3F
    'dhl': 61248,  #  U+EF40
    'diaspora': 61249,  #  U+EF41
    'dumpster': 61250,  #  U+EF42
    'dumpster_fire': 61251,  #  U+EF43
    'ethernet': 61252,  #  U+EF44
    'fedex': 61253,  #  U+EF45
    'fedora': 61254,  #  U+EF46
    'figma': 61255,  #  U+EF47
    'gifts': 61256,  #  U+EF48
    'champagne_glasses': 61257,  #  U+EF49
    'whiskey_glass': 61258,  #  U+EF4A
    'earth_europe': 61259,  #  U+EF4B
    'grip_lines': 61260,  #  U+EF4C
    'grip_lines_vertical': 61261,  #  U+EF4D
    'guitar#1': 61262,  #  U+EF4E
    'heart_crack': 61263,  #  U+EF4F
    'holly_berry': 61264,  #  U+EF50
    'horse_head': 61265,  #  U+EF51
    'icicles': 61266,  #  U+EF52
    'igloo': 61267,  #  U+EF53
    'intercom': 61268,  #  U+EF54
    'invision': 61269,  #  U+EF55
    'jira': 61270,  #  U+EF56
    'mendeley': 61271,  #  U+EF57
    'mitten': 61272,  #  U+EF58
    'mug_hot': 61273,  #  U+EF59
    'radiation': 61274,  #  U+EF5A
    'circle_radiation': 61275,  #  U+EF5B
    'raspberry_pi': 61276,  #  U+EF5C
    'redhat': 61277,  #  U+EF5D
    'restroom': 61278,  #  U+EF5E
    'satellite': 61279,  #  U+EF5F
    'satellite_dish': 61280,  #  U+EF60
    'sd_card': 61281,  #  U+EF61
    'sim_card': 61282,  #  U+EF62
    'person_skating': 61283,  #  U+EF63
    'sketch': 61284,  #  U+EF64
    'person_skiing': 61285,  #  U+EF65
    'person_skiing_nordic': 61286,  #  U+EF66
    'sleigh': 61287,  #  U+EF67
    'comment_sms': 61288,  #  U+EF68
    'person_snowboarding': 61289,  #  U+EF69
    'snowman': 61290,  #  U+EF6A
    'snowplow': 61291,  #  U+EF6B
    'sourcetree': 61292,  #  U+EF6C
    'suse': 61293,  #  U+EF6D
    'tenge_sign': 61294,  #  U+EF6E
    'toilet#1': 61295,  #  U+EF6F
    'screwdriver_wrench': 61296,  #  U+EF70
    'cable_car': 61297,  #  U+EF71
    'ubuntu': 61298,  #  U+EF72
    'ups': 61299,  #  U+EF73
    'usps': 61300,  #  U+EF74
    'yarn': 61301,  #  U+EF75
    'fire_flame_curved': 61302,  #  U+EF76
    'bacon': 61303,  #  U+EF77
    'book_medical': 61304,  #  U+EF78
    'bread_slice': 61305,  #  U+EF79
    'cheese': 61306,  #  U+EF7A
    'house_chimney_medical': 61307,  #  U+EF7B
    'clipboard_user': 61308,  #  U+EF7C
    'comment_medical': 61309,  #  U+EF7D
    'crutch': 61310,  #  U+EF7E
    'disease': 61311,  #  U+EF7F
    'egg': 61312,  #  U+EF80
    'folder_tree': 61313,  #  U+EF81
    'burger': 61314,  #  U+EF82
    'hand_middle_finger': 61315,  #  U+EF83
    'helmet_safety': 61316,  #  U+EF84
    'house_chimney': 61317,  #  U+EF85
    'hospital_user': 61318,  #  U+EF86
    'hotdog#1': 61319,  #  U+EF87
    'ice_cream': 61320,  #  U+EF88
    'laptop_medical': 61321,  #  U+EF89
    'pager': 61322,  #  U+EF8A
    'pepper_hot': 61323,  #  U+EF8B
    'pizza_slice': 61324,  #  U+EF8C
    'sack_dollar': 61325,  #  U+EF8D
    'book_tanakh': 61326,  #  U+EF8E
    'bars_progress': 61327,  #  U+EF8F
    'trash_arrow_up': 61328,  #  U+EF90
    'trash_can_arrow_up': 61329,  #  U+EF91
    'user_nurse': 61330,  #  U+EF92
    'airbnb': 61331,  #  U+EF93
    'battle_net': 61332,  #  U+EF94
    'bootstrap': 61333,  #  U+EF95
    'buffer': 61334,  #  U+EF96
    'chromecast': 61335,  #  U+EF97
    'evernote': 61336,  #  U+EF98
    'itch_io': 61337,  #  U+EF99
    'salesforce': 61338,  #  U+EF9A
    'speaker_deck': 61339,  #  U+EF9B
    'symfony': 61340,  #  U+EF9C
    'wave_square': 61341,  #  U+EF9D
    'waze': 61342,  #  U+EF9E
    'yammer': 61343,  #  U+EF9F
    'git_alt': 61344,  #  U+EFA0
    'stackpath': 61345,  #  U+EFA1
    'person_biking': 61346,  #  U+EFA2
    'border_all': 61347,  #  U+EFA3
    'border_none': 61348,  #  U+EFA4
    'border_top_left': 61349,  #  U+EFA5
    'person_digging': 61350,  #  U+EFA6
    'fan': 61351,  #  U+EFA7
    'icons': 61352,  #  U+EFA8
    'phone_flip': 61353,  #  U+EFA9
    'square_phone_flip': 61354,  #  U+EFAA
    'photo_film': 61355,  #  U+EFAB
    'text_slash': 61356,  #  U+EFAC
    'arrow_down_z_a': 61357,  #  U+EFAD
    'arrow_up_z_a': 61358,  #  U+EFAE
    'arrow_down_short_wide': 61359,  #  U+EFAF
    'arrow_up_short_wide': 61360,  #  U+EFB0
    'arrow_down_9_1': 61361,  #  U+EFB1
    'arrow_up_9_1': 61362,  #  U+EFB2
    'spell_check': 61363,  #  U+EFB3
    'voicemail': 61364,  #  U+EFB4
    'cotton_bureau': 61365,  #  U+EFB5
    'buy_n_large': 61366,  #  U+EFB6
    'hat_cowboy': 61367,  #  U+EFB7
    'hat_cowboy_side': 61368,  #  U+EFB8
    'mdb': 61369,  #  U+EFB9
    'computer_mouse': 61370,  #  U+EFBA
    'orcid': 61371,  #  U+EFBB
    'radio': 61372,  #  U+EFBC
    'record_vinyl': 61373,  #  U+EFBD
    'swift': 61374,  #  U+EFBE
    'umbraco': 61375,  #  U+EFBF
    'walkie_talkie': 61376,  #  U+EFC0
    'caravan': 61377,  #  U+EFC1
    'avianex': 61378,  #  U+EFC2
    'less_than': 61379,  #  U+EFC3
    'less_than_equal': 61380,  #  U+EFC4
    'memory': 61381,  #  U+EFC5
    'microphone_lines_slash': 61382,  #  U+EFC6
    'money_bill_wave': 61383,  #  U+EFC7
    'money_bill_1_wave': 61384,  #  U+EFC8
    'money_check': 61385,  #  U+EFC9
    'money_check_dollar': 61386,  #  U+EFCA
    'not_equal': 61387,  #  U+EFCB
    'palette': 61388,  #  U+EFCC
    'square_parking': 61389,  #  U+EFCD
    'diagram_project': 61390,  #  U+EFCE
    'martini_glass_empty': 61440,  #  U+F000
    'music#1': 61441,  #  U+F001
    'magnifying_glass': 61442,  #  U+F002
    'envelope_o': 61443,  #  U+F003
    'heart#2': 61444,  #  U+F004
    'star': 61445,  #  U+F005
    'star_o': 61446,  #  U+F006
    'user': 61447,  #  U+F007
    'film': 61448,  #  U+F008
    'table_cells_large': 61449,  #  U+F009
    'table_cells': 61450,  #  U+F00A
    'table_list': 61451,  #  U+F00B
    'check#1': 61452,  #  U+F00C
    'xmark': 61453,  #  U+F00D
    'magnifying_glass_plus': 61454,  #  U+F00E
    'images': 61455,  #  U+F00F
    'magnifying_glass_minus': 61456,  #  U+F010
    'power_off': 61457,  #  U+F011
    'signal': 61458,  #  U+F012
    'gear#1': 61459,  #  U+F013
    'trash_can': 61460,  #  U+F014
    'house': 61461,  #  U+F015
    'file_o': 61462,  #  U+F016
    'clock': 61463,  #  U+F017
    'road': 61464,  #  U+F018
    'download': 61465,  #  U+F019
    'circle_down': 61466,  #  U+F01A
    'circle_up': 61467,  #  U+F01B
    'inbox#1': 61468,  #  U+F01C
    'play_circle_o': 61469,  #  U+F01D
    'arrow_rotate_right': 61470,  #  U+F01E
    'pen': 61471,  #  U+F01F
    'pen_clip': 61472,  #  U+F020
    'arrows_rotate': 61473,  #  U+F021
    'rectangle_list': 61474,  #  U+F022
    'lock#1': 61475,  #  U+F023
    'flag': 61476,  #  U+F024
    'headphones': 61477,  #  U+F025
    'volume_off': 61478,  #  U+F026
    'volume_low': 61479,  #  U+F027
    'volume_high': 61480,  #  U+F028
    'qrcode': 61481,  #  U+F029
    'barcode': 61482,  #  U+F02A
    'tag#1': 61483,  #  U+F02B
    'tags': 61484,  #  U+F02C
    'book#1': 61485,  #  U+F02D
    'bookmark#1': 61486,  #  U+F02E
    'print': 61487,  #  U+F02F
    'camera': 61488,  #  U+F030
    'font': 61489,  #  U+F031
    'bold#1': 61490,  #  U+F032
    'italic#1': 61491,  #  U+F033
    'text_height': 61492,  #  U+F034
    'text_width': 61493,  #  U+F035
    'align_left': 61494,  #  U+F036
    'align_center': 61495,  #  U+F037
    'align_right': 61496,  #  U+F038
    'align_justify': 61497,  #  U+F039
    'list': 61498,  #  U+F03A
    'outdent': 61499,  #  U+F03B
    'indent#1': 61500,  #  U+F03C
    'video': 61501,  #  U+F03D
    'image': 61502,  #  U+F03E
    'down_long': 61503,  #  U+F03F
    'pencil': 61504,  #  U+F040
    'location_pin': 61505,  #  U+F041
    'circle_half_stroke': 61506,  #  U+F042
    'droplet': 61507,  #  U+F043
    'pen_to_square': 61508,  #  U+F044
    'share_square_o': 61509,  #  U+F045
    'check_square_o': 61510,  #  U+F046
    'arrows_up_down_left_right': 61511,  #  U+F047
    'backward_step': 61512,  #  U+F048
    'backward_fast': 61513,  #  U+F049
    'backward': 61514,  #  U+F04A
    'play#1': 61515,  #  U+F04B
    'pause': 61516,  #  U+F04C
    'stop': 61517,  #  U+F04D
    'forward': 61518,  #  U+F04E
    'left_long': 61519,  #  U+F04F
    'forward_fast': 61520,  #  U+F050
    'forward_step': 61521,  #  U+F051
    'eject': 61522,  #  U+F052
    'chevron_left': 61523,  #  U+F053
    'chevron_right': 61524,  #  U+F054
    'circle_plus': 61525,  #  U+F055
    'circle_minus': 61526,  #  U+F056
    'remove_sign': 61527,  #  U+F057
    'ok_sign': 61528,  #  U+F058
    'circle_question': 61529,  #  U+F059
    'circle_info': 61530,  #  U+F05A
    'crosshairs': 61531,  #  U+F05B
    'circle_xmark': 61532,  #  U+F05C
    'circle_check': 61533,  #  U+F05D
    'ban': 61534,  #  U+F05E
    'file_pen': 61535,  #  U+F05F
    'arrow_left': 61536,  #  U+F060
    'arrow_right': 61537,  #  U+F061
    'arrow_up': 61538,  #  U+F062
    'arrow_down': 61539,  #  U+F063
    'share': 61540,  #  U+F064
    'expand': 61541,  #  U+F065
    'compress': 61542,  #  U+F066
    'plus': 61543,  #  U+F067
    'minus': 61544,  #  U+F068
    'asterisk': 61545,  #  U+F069
    'circle_exclamation': 61546,  #  U+F06A
    'gift#1': 61547,  #  U+F06B
    'leaf': 61548,  #  U+F06C
    'fire': 61549,  #  U+F06D
    'eye#1': 61550,  #  U+F06E
    'maximize#1': 61551,  #  U+F06F
    'eye_slash': 61552,  #  U+F070
    'triangle_exclamation': 61553,  #  U+F071
    'plane': 61554,  #  U+F072
    'calendar_days': 61555,  #  U+F073
    'shuffle': 61556,  #  U+F074
    'comment#1': 61557,  #  U+F075
    'magnet#1': 61558,  #  U+F076
    'chevron_up': 61559,  #  U+F077
    'chevron_down': 61560,  #  U+F078
    'retweet': 61561,  #  U+F079
    'cart_shopping': 61562,  #  U+F07A
    'folder#1': 61563,  #  U+F07B
    'folder_open': 61564,  #  U+F07C
    'arrows_up_down': 61565,  #  U+F07D
    'arrows_left_right': 61566,  #  U+F07E
    'clipboard_alt': 61567,  #  U+F07F
    'chart_bar': 61568,  #  U+F080
    'square_twitter': 61569,  #  U+F081
    'square_facebook': 61570,  #  U+F082
    'camera_retro': 61571,  #  U+F083
    'key#1': 61572,  #  U+F084
    'gears': 61573,  #  U+F085
    'comments': 61574,  #  U+F086
    'thumbs_o_up': 61575,  #  U+F087
    'thumbs_o_down': 61576,  #  U+F088
    'star_half': 61577,  #  U+F089
    'heard_o': 61578,  #  U+F08A
    'arrow_right_from_bracket': 61579,  #  U+F08B
    'linkedin_square': 61580,  #  U+F08C
    'thumbtack': 61581,  #  U+F08D
    'arrow_up_right_from_square': 61582,  #  U+F08E
    'left_right': 61583,  #  U+F08F
    'arrow_right_to_bracket': 61584,  #  U+F090
    'trophy': 61585,  #  U+F091
    'square_github': 61586,  #  U+F092
    'upload': 61587,  #  U+F093
    'lemon': 61588,  #  U+F094
    'phone': 61589,  #  U+F095
    'square_o': 61590,  #  U+F096
    'bookmark_o': 61591,  #  U+F097
    'square_phone': 61592,  #  U+F098
    'twitter#1': 61593,  #  U+F099
    'facebook': 61594,  #  U+F09A
    'github#1': 61595,  #  U+F09B
    'unlock#1': 61596,  #  U+F09C
    'credit_card': 61597,  #  U+F09D
    'rss#1': 61598,  #  U+F09E
    'up_down': 61599,  #  U+F09F
    'hard_drive': 61600,  #  U+F0A0
    'bullhorn': 61601,  #  U+F0A1
    'bell_o': 61602,  #  U+F0A2
    'certificate': 61603,  #  U+F0A3
    'hand_point_right': 61604,  #  U+F0A4
    'hand_point_left': 61605,  #  U+F0A5
    'hand_point_up': 61606,  #  U+F0A6
    'hand_point_down': 61607,  #  U+F0A7
    'circle_arrow_left': 61608,  #  U+F0A8
    'circle_arrow_right': 61609,  #  U+F0A9
    'circle_arrow_up': 61610,  #  U+F0AA
    'circle_arrow_down': 61611,  #  U+F0AB
    'globe#1': 61612,  #  U+F0AC
    'wrench': 61613,  #  U+F0AD
    'list_check': 61614,  #  U+F0AE
    'square_font_awesome_stroke': 61615,  #  U+F0AF
    'filter#1': 61616,  #  U+F0B0
    'briefcase#1': 61617,  #  U+F0B1
    'up_down_left_right': 61618,  #  U+F0B2
    'up_right_from_square': 61619,  #  U+F0B3
    'square_up_right': 61620,  #  U+F0B4
    'right_left': 61621,  #  U+F0B5
    'repeat_alt': 61622,  #  U+F0B6
    'accusoft': 61623,  #  U+F0B7
    'adversal': 61624,  #  U+F0B8
    'affiliatetheme': 61625,  #  U+F0B9
    'algolia': 61626,  #  U+F0BA
    'amilia': 61627,  #  U+F0BB
    'angrycreative': 61628,  #  U+F0BC
    'app_store': 61629,  #  U+F0BD
    'app_store_ios': 61630,  #  U+F0BE
    'apper': 61631,  #  U+F0BF
    'users': 61632,  #  U+F0C0
    'link#1': 61633,  #  U+F0C1
    'cloud#2': 61634,  #  U+F0C2
    'flask': 61635,  #  U+F0C3
    'scissors': 61636,  #  U+F0C4
    'copy#1': 61637,  #  U+F0C5
    'paperclip': 61638,  #  U+F0C6
    'floppy_disk': 61639,  #  U+F0C7
    'square': 61640,  #  U+F0C8
    'bars': 61641,  #  U+F0C9
    'list_ul': 61642,  #  U+F0CA
    'list_ol': 61643,  #  U+F0CB
    'strikethrough': 61644,  #  U+F0CC
    'underline': 61645,  #  U+F0CD
    'table#1': 61646,  #  U+F0CE
    'asymmetrik': 61647,  #  U+F0CF
    'wand_magic': 61648,  #  U+F0D0
    'truck': 61649,  #  U+F0D1
    'pinterest': 61650,  #  U+F0D2
    'square_pinterest': 61651,  #  U+F0D3
    'square_google_plus': 61652,  #  U+F0D4
    'google_plus': 61653,  #  U+F0D5
    'money_bill': 61654,  #  U+F0D6
    'caret_down': 61655,  #  U+F0D7
    'caret_up': 61656,  #  U+F0D8
    'caret_left': 61657,  #  U+F0D9
    'caret_right': 61658,  #  U+F0DA
    'table_columns': 61659,  #  U+F0DB
    'sort': 61660,  #  U+F0DC
    'sort_down': 61661,  #  U+F0DD
    'sort_up': 61662,  #  U+F0DE
    'audible': 61663,  #  U+F0DF
    'envelope': 61664,  #  U+F0E0
    'linkedin_in': 61665,  #  U+F0E1
    'arrow_rotate_left': 61666,  #  U+F0E2
    'gavel': 61667,  #  U+F0E3
    'gauge_simple_high': 61668,  #  U+F0E4
    'comment_o': 61669,  #  U+F0E5
    'comments_o': 61670,  #  U+F0E6
    'bolt': 61671,  #  U+F0E7
    'sitemap': 61672,  #  U+F0E8
    'umbrella#1': 61673,  #  U+F0E9
    'paste': 61674,  #  U+F0EA
    'lightbulb#1': 61675,  #  U+F0EB
    'arrow_right_arrow_left': 61676,  #  U+F0EC
    'cloud_arrow_down': 61677,  #  U+F0ED
    'cloud_arrow_up': 61678,  #  U+F0EE
    'aws': 61679,  #  U+F0EF
    'user_doctor': 61680,  #  U+F0F0
    'stethoscope': 61681,  #  U+F0F1
    'suitcase': 61682,  #  U+F0F2
    'bell#1': 61683,  #  U+F0F3
    'mug_saucer': 61684,  #  U+F0F4
    'utensils': 61685,  #  U+F0F5
    'file_text_o': 61686,  #  U+F0F6
    'building_o': 61687,  #  U+F0F7
    'hospital': 61688,  #  U+F0F8
    'truck_medical': 61689,  #  U+F0F9
    'suitcase_medical': 61690,  #  U+F0FA
    'jet_fighter': 61691,  #  U+F0FB
    'beer_mug_empty': 61692,  #  U+F0FC
    'square_h': 61693,  #  U+F0FD
    'square_plus': 61694,  #  U+F0FE
    'bimobject': 61695,  #  U+F0FF
    'angles_left': 61696,  #  U+F100
    'angles_right': 61697,  #  U+F101
    'angles_up': 61698,  #  U+F102
    'angles_down': 61699,  #  U+F103
    'angle_left': 61700,  #  U+F104
    'angle_right': 61701,  #  U+F105
    'angle_up': 61702,  #  U+F106
    'angle_down': 61703,  #  U+F107
    'desktop': 61704,  #  U+F108
    'laptop': 61705,  #  U+F109
    'tablet_screen_button': 61706,  #  U+F10A
    'mobile_screen_button': 61707,  #  U+F10B
    'circle_o': 61708,  #  U+F10C
    'quote_left': 61709,  #  U+F10D
    'quote_right': 61710,  #  U+F10E
    'bitcoin': 61711,  #  U+F10F
    'spinner': 61712,  #  U+F110
    'circle#1': 61713,  #  U+F111
    'reply#1': 61714,  #  U+F112
    'github_alt': 61715,  #  U+F113
    'folder_o': 61716,  #  U+F114
    'folder_open_o': 61717,  #  U+F115
    'bity': 61718,  #  U+F116
    'blackberry': 61719,  #  U+F117
    'face_smile': 61720,  #  U+F118
    'face_frown': 61721,  #  U+F119
    'face_meh': 61722,  #  U+F11A
    'gamepad': 61723,  #  U+F11B
    'keyboard': 61724,  #  U+F11C
    'flag_o': 61725,  #  U+F11D
    'flag_checkered': 61726,  #  U+F11E
    'blogger': 61727,  #  U+F11F
    'terminal#1': 61728,  #  U+F120
    'code#1': 61729,  #  U+F121
    'reply_all': 61730,  #  U+F122
    'star_half_o': 61731,  #  U+F123
    'location_arrow': 61732,  #  U+F124
    'crop': 61733,  #  U+F125
    'code_branch': 61734,  #  U+F126
    'link_slash': 61735,  #  U+F127
    'question#1': 61736,  #  U+F128
    'info#1': 61737,  #  U+F129
    'exclamation': 61738,  #  U+F12A
    'superscript': 61739,  #  U+F12B
    'subscript': 61740,  #  U+F12C
    'eraser': 61741,  #  U+F12D
    'puzzle_piece': 61742,  #  U+F12E
    'blogger_b': 61743,  #  U+F12F
    'microphone': 61744,  #  U+F130
    'microphone_slash': 61745,  #  U+F131
    'shield#1': 61746,  #  U+F132
    'calendar_o': 61747,  #  U+F133
    'fire_extinguisher': 61748,  #  U+F134
    'rocket#1': 61749,  #  U+F135
    'maxcdn': 61750,  #  U+F136
    'circle_chevron_left': 61751,  #  U+F137
    'circle_chevron_right': 61752,  #  U+F138
    'circle_chevron_up': 61753,  #  U+F139
    'circle_chevron_down': 61754,  #  U+F13A
    'html5': 61755,  #  U+F13B
    'css3': 61756,  #  U+F13C
    'anchor': 61757,  #  U+F13D
    'unlock_keyhole': 61758,  #  U+F13E
    'buromobelexperte': 61759,  #  U+F13F
    'bullseye': 61760,  #  U+F140
    'ellipsis#1': 61761,  #  U+F141
    'ellipsis_vertical': 61762,  #  U+F142
    'square_rss': 61763,  #  U+F143
    'circle_play': 61764,  #  U+F144
    'ticket': 61765,  #  U+F145
    'square_minus': 61766,  #  U+F146
    'minus_square_o': 61767,  #  U+F147
    'arrow_turn_up': 61768,  #  U+F148
    'arrow_turn_down': 61769,  #  U+F149
    'square_check': 61770,  #  U+F14A
    'square_pen': 61771,  #  U+F14B
    'square_arrow_up_right': 61772,  #  U+F14C
    'share_from_square': 61773,  #  U+F14D
    'compass#1': 61774,  #  U+F14E
    'centercode': 61775,  #  U+F14F
    'square_caret_down': 61776,  #  U+F150
    'square_caret_up': 61777,  #  U+F151
    'square_caret_right': 61778,  #  U+F152
    'euro_sign': 61779,  #  U+F153
    'sterling_sign': 61780,  #  U+F154
    'dollar_sign': 61781,  #  U+F155
    'rupee_sign': 61782,  #  U+F156
    'yen_sign': 61783,  #  U+F157
    'ruble_sign': 61784,  #  U+F158
    'won_sign': 61785,  #  U+F159
    'btc': 61786,  #  U+F15A
    'file#1': 61787,  #  U+F15B
    'file_lines': 61788,  #  U+F15C
    'arrow_down_a_z': 61789,  #  U+F15D
    'arrow_up_a_z': 61790,  #  U+F15E
    'cloudscale': 61791,  #  U+F15F
    'arrow_down_wide_short': 61792,  #  U+F160
    'arrow_up_wide_short': 61793,  #  U+F161
    'arrow_down_1_9': 61794,  #  U+F162
    'arrow_up_1_9': 61795,  #  U+F163
    'thumbs_up': 61796,  #  U+F164
    'thumbs_down': 61797,  #  U+F165
    'square_youtube': 61798,  #  U+F166
    'cloudsmith': 61799,  #  U+F167
    'xing': 61800,  #  U+F168
    'square_xing': 61801,  #  U+F169
    'youtube': 61802,  #  U+F16A
    'dropbox': 61803,  #  U+F16B
    'stack_overflow': 61804,  #  U+F16C
    'instagram': 61805,  #  U+F16D
    'flickr': 61806,  #  U+F16E
    'cloudversify': 61807,  #  U+F16F
    'adn': 61808,  #  U+F170
    'bitbucket': 61809,  #  U+F171
    'code_commit': 61810,  #  U+F172
    'tumblr': 61811,  #  U+F173
    'square_tumblr': 61812,  #  U+F174
    'arrow_down_long': 61813,  #  U+F175
    'arrow_up_long': 61814,  #  U+F176
    'arrow_left_long': 61815,  #  U+F177
    'arrow_right_long': 61816,  #  U+F178
    'apple': 61817,  #  U+F179
    'windows': 61818,  #  U+F17A
    'android': 61819,  #  U+F17B
    'linux': 61820,  #  U+F17C
    'dribbble': 61821,  #  U+F17D
    'skype': 61822,  #  U+F17E
    'code_merge': 61823,  #  U+F17F
    'foursquare': 61824,  #  U+F180
    'trello': 61825,  #  U+F181
    'person_dress': 61826,  #  U+F182
    'person#1': 61827,  #  U+F183
    'gratipay': 61828,  #  U+F184
    'sun': 61829,  #  U+F185
    'moon': 61830,  #  U+F186
    'box_archive': 61831,  #  U+F187
    'bug#1': 61832,  #  U+F188
    'vk': 61833,  #  U+F189
    'weibo': 61834,  #  U+F18A
    'renren': 61835,  #  U+F18B
    'pagelines': 61836,  #  U+F18C
    'stack_exchange': 61837,  #  U+F18D
    'circle_right': 61838,  #  U+F18E
    'cpanel': 61839,  #  U+F18F
    'circle_left': 61840,  #  U+F190
    'square_caret_left': 61841,  #  U+F191
    'circle_dot': 61842,  #  U+F192
    'wheelchair': 61843,  #  U+F193
    'square_vimeo': 61844,  #  U+F194
    'lira_sign': 61845,  #  U+F195
    'plus_square_o': 61846,  #  U+F196
    'shuttle_space': 61847,  #  U+F197
    'slack': 61848,  #  U+F198
    'square_envelope': 61849,  #  U+F199
    'wordpress': 61850,  #  U+F19A
    'openid': 61851,  #  U+F19B
    'building_columns': 61852,  #  U+F19C
    'graduation_cap': 61853,  #  U+F19D
    'yahoo': 61854,  #  U+F19E
    'css3_alt': 61855,  #  U+F19F
    'google': 61856,  #  U+F1A0
    'reddit': 61857,  #  U+F1A1
    'square_reddit': 61858,  #  U+F1A2
    'stumbleupon_circle': 61859,  #  U+F1A3
    'stumbleupon': 61860,  #  U+F1A4
    'delicious': 61861,  #  U+F1A5
    'digg': 61862,  #  U+F1A6
    'pied_piper_pp': 61863,  #  U+F1A7
    'pied_piper_alt': 61864,  #  U+F1A8
    'drupal': 61865,  #  U+F1A9
    'joomla': 61866,  #  U+F1AA
    'language': 61867,  #  U+F1AB
    'fax': 61868,  #  U+F1AC
    'building': 61869,  #  U+F1AD
    'child': 61870,  #  U+F1AE
    'cuttlefish': 61871,  #  U+F1AF
    'paw': 61872,  #  U+F1B0
    'spoon': 61873,  #  U+F1B1
    'cube': 61874,  #  U+F1B2
    'cubes': 61875,  #  U+F1B3
    'behance': 61876,  #  U+F1B4
    'square_behance': 61877,  #  U+F1B5
    'steam': 61878,  #  U+F1B6
    'square_steam': 61879,  #  U+F1B7
    'recycle': 61880,  #  U+F1B8
    'car': 61881,  #  U+F1B9
    'taxi': 61882,  #  U+F1BA
    'tree#1': 61883,  #  U+F1BB
    'spotify': 61884,  #  U+F1BC
    'deviantart': 61885,  #  U+F1BD
    'soundcloud': 61886,  #  U+F1BE
    'd_and_d': 61887,  #  U+F1BF
    'database#1': 61888,  #  U+F1C0
    'file_pdf': 61889,  #  U+F1C1
    'file_word': 61890,  #  U+F1C2
    'file_excel': 61891,  #  U+F1C3
    'file_powerpoint': 61892,  #  U+F1C4
    'file_image': 61893,  #  U+F1C5
    'file_zipper': 61894,  #  U+F1C6
    'file_audio': 61895,  #  U+F1C7
    'file_video': 61896,  #  U+F1C8
    'file_code': 61897,  #  U+F1C9
    'vine': 61898,  #  U+F1CA
    'codepen': 61899,  #  U+F1CB
    'jsfiddle': 61900,  #  U+F1CC
    'life_ring': 61901,  #  U+F1CD
    'circle_notch': 61902,  #  U+F1CE
    'deploydog': 61903,  #  U+F1CF
    'rebel': 61904,  #  U+F1D0
    'empire': 61905,  #  U+F1D1
    'square_git': 61906,  #  U+F1D2
    'git': 61907,  #  U+F1D3
    'hacker_news': 61908,  #  U+F1D4
    'tencent_weibo': 61909,  #  U+F1D5
    'qq': 61910,  #  U+F1D6
    'weixin': 61911,  #  U+F1D7
    'paper_plane': 61912,  #  U+F1D8
    'paper_plane_o': 61913,  #  U+F1D9
    'clock_rotate_left': 61914,  #  U+F1DA
    'circle_thin': 61915,  #  U+F1DB
    'heading': 61916,  #  U+F1DC
    'paragraph': 61917,  #  U+F1DD
    'sliders': 61918,  #  U+F1DE
    'deskpro': 61919,  #  U+F1DF
    'share_nodes': 61920,  #  U+F1E0
    'square_share_nodes': 61921,  #  U+F1E1
    'bomb': 61922,  #  U+F1E2
    'futbol': 61923,  #  U+F1E3
    'tty': 61924,  #  U+F1E4
    'binoculars': 61925,  #  U+F1E5
    'plug#1': 61926,  #  U+F1E6
    'slideshare': 61927,  #  U+F1E7
    'twitch': 61928,  #  U+F1E8
    'yelp': 61929,  #  U+F1E9
    'newspaper': 61930,  #  U+F1EA
    'wifi': 61931,  #  U+F1EB
    'calculator': 61932,  #  U+F1EC
    'paypal': 61933,  #  U+F1ED
    'google_wallet': 61934,  #  U+F1EE
    'digital_ocean': 61935,  #  U+F1EF
    'cc_visa': 61936,  #  U+F1F0
    'cc_mastercard': 61937,  #  U+F1F1
    'cc_discover': 61938,  #  U+F1F2
    'cc_amex': 61939,  #  U+F1F3
    'cc_paypal': 61940,  #  U+F1F4
    'cc_stripe': 61941,  #  U+F1F5
    'bell_slash': 61942,  #  U+F1F6
    'bell_slash_o': 61943,  #  U+F1F7
    'trash#1': 61944,  #  U+F1F8
    'copyright': 61945,  #  U+F1F9
    'at': 61946,  #  U+F1FA
    'eye_dropper': 61947,  #  U+F1FB
    'paintbrush': 61948,  #  U+F1FC
    'cake_candles': 61949,  #  U+F1FD
    'chart_area': 61950,  #  U+F1FE
    'discord': 61951,  #  U+F1FF
    'chart_pie': 61952,  #  U+F200
    'chart_line': 61953,  #  U+F201
    'lastfm': 61954,  #  U+F202
    'square_lastfm': 61955,  #  U+F203
    'toggle_off': 61956,  #  U+F204
    'toggle_on': 61957,  #  U+F205
    'bicycle': 61958,  #  U+F206
    'bus': 61959,  #  U+F207
    'ioxhost': 61960,  #  U+F208
    'angellist': 61961,  #  U+F209
    'closed_captioning': 61962,  #  U+F20A
    'shekel_sign': 61963,  #  U+F20B
    'discourse': 61964,  #  U+F20C
    'buysellads': 61965,  #  U+F20D
    'connectdevelop': 61966,  #  U+F20E
    'dochub': 61967,  #  U+F20F
    'dashcube': 61968,  #  U+F210
    'forumbee': 61969,  #  U+F211
    'leanpub': 61970,  #  U+F212
    'sellsy': 61971,  #  U+F213
    'shirtsinbulk': 61972,  #  U+F214
    'simplybuilt': 61973,  #  U+F215
    'skyatlas': 61974,  #  U+F216
    'cart_plus': 61975,  #  U+F217
    'cart_arrow_down': 61976,  #  U+F218
    'gem': 61977,  #  U+F219
    'ship': 61978,  #  U+F21A
    'user_secret': 61979,  #  U+F21B
    'motorcycle': 61980,  #  U+F21C
    'street_view': 61981,  #  U+F21D
    'heart_pulse': 61982,  #  U+F21E
    'docker': 61983,  #  U+F21F
    'draft2digital': 61984,  #  U+F220
    'venus': 61985,  #  U+F221
    'mars': 61986,  #  U+F222
    'mercury': 61987,  #  U+F223
    'transgender': 61988,  #  U+F224
    'transgender_alt': 61989,  #  U+F225
    'venus_double': 61990,  #  U+F226
    'mars_double': 61991,  #  U+F227
    'venus_mars': 61992,  #  U+F228
    'mars_stroke': 61993,  #  U+F229
    'mars_stroke_up': 61994,  #  U+F22A
    'mars_stroke_right': 61995,  #  U+F22B
    'neuter': 61996,  #  U+F22C
    'genderless': 61997,  #  U+F22D
    'square_dribbble': 61998,  #  U+F22E
    'dyalog': 61999,  #  U+F22F
    'earlybirds': 62000,  #  U+F230
    'pinterest_p': 62001,  #  U+F231
    'whatsapp': 62002,  #  U+F232
    'server#1': 62003,  #  U+F233
    'user_plus': 62004,  #  U+F234
    'user_xmark': 62005,  #  U+F235
    'bed#1': 62006,  #  U+F236
    'viacoin': 62007,  #  U+F237
    'train': 62008,  #  U+F238
    'train_subway': 62009,  #  U+F239
    'medium': 62010,  #  U+F23A
    'y_combinator': 62011,  #  U+F23B
    'optin_monster': 62012,  #  U+F23C
    'opencart': 62013,  #  U+F23D
    'expeditedssl': 62014,  #  U+F23E
    'erlang': 62015,  #  U+F23F
    'battery_full': 62016,  #  U+F240
    'battery_three_quarters': 62017,  #  U+F241
    'battery_half': 62018,  #  U+F242
    'battery_quarter': 62019,  #  U+F243
    'battery_empty': 62020,  #  U+F244
    'arrow_pointer': 62021,  #  U+F245
    'i_cursor': 62022,  #  U+F246
    'object_group': 62023,  #  U+F247
    'object_ungroup': 62024,  #  U+F248
    'note_sticky': 62025,  #  U+F249
    'sticky_note_o': 62026,  #  U+F24A
    'cc_jcb': 62027,  #  U+F24B
    'cc_diners_club': 62028,  #  U+F24C
    'clone': 62029,  #  U+F24D
    'scale_balanced': 62030,  #  U+F24E
    'facebook_f': 62031,  #  U+F24F
    'hourglass_o': 62032,  #  U+F250
    'hourglass_start': 62033,  #  U+F251
    'hourglass_half': 62034,  #  U+F252
    'hourglass_end': 62035,  #  U+F253
    'hourglass': 62036,  #  U+F254
    'hand_back_fist': 62037,  #  U+F255
    'hand': 62038,  #  U+F256
    'hand_scissors': 62039,  #  U+F257
    'hand_lizard': 62040,  #  U+F258
    'hand_spock': 62041,  #  U+F259
    'hand_pointer': 62042,  #  U+F25A
    'hand_peace': 62043,  #  U+F25B
    'trademark': 62044,  #  U+F25C
    'registered': 62045,  #  U+F25D
    'creative_commons': 62046,  #  U+F25E
    'facebook_messenger': 62047,  #  U+F25F
    'gg': 62048,  #  U+F260
    'gg_circle': 62049,  #  U+F261
    'firstdraft': 62050,  #  U+F262
    'odnoklassniki': 62051,  #  U+F263
    'square_odnoklassniki': 62052,  #  U+F264
    'get_pocket': 62053,  #  U+F265
    'wikipedia_w': 62054,  #  U+F266
    'safari': 62055,  #  U+F267
    'chrome': 62056,  #  U+F268
    'firefox': 62057,  #  U+F269
    'opera': 62058,  #  U+F26A
    'internet_explorer': 62059,  #  U+F26B
    'tv': 62060,  #  U+F26C
    'contao': 62061,  #  U+F26D
    '500px': 62062,  #  U+F26E
    'fonticons_fi': 62063,  #  U+F26F
    'amazon': 62064,  #  U+F270
    'calendar_plus': 62065,  #  U+F271
    'calendar_minus': 62066,  #  U+F272
    'calendar_xmark': 62067,  #  U+F273
    'calendar_check': 62068,  #  U+F274
    'industry': 62069,  #  U+F275
    'map_pin': 62070,  #  U+F276
    'signs_post': 62071,  #  U+F277
    'map_o': 62072,  #  U+F278
    'map#1': 62073,  #  U+F279
    'message': 62074,  #  U+F27A
    'comment_dots': 62075,  #  U+F27B
    'houzz': 62076,  #  U+F27C
    'vimeo_v': 62077,  #  U+F27D
    'black_tie': 62078,  #  U+F27E
    'fort_awesome_alt': 62079,  #  U+F27F
    'fonticons': 62080,  #  U+F280
    'reddit_alien': 62081,  #  U+F281
    'edge': 62082,  #  U+F282
    'credit_card_alt': 62083,  #  U+F283
    'codiepie': 62084,  #  U+F284
    'modx': 62085,  #  U+F285
    'fort_awesome': 62086,  #  U+F286
    'usb': 62087,  #  U+F287
    'product_hunt': 62088,  #  U+F288
    'mixcloud': 62089,  #  U+F289
    'scribd': 62090,  #  U+F28A
    'circle_pause': 62091,  #  U+F28B
    'pause_circle_o': 62092,  #  U+F28C
    'circle_stop': 62093,  #  U+F28D
    'stop_circle_o': 62094,  #  U+F28E
    'freebsd': 62095,  #  U+F28F
    'bag_shopping': 62096,  #  U+F290
    'basket_shopping': 62097,  #  U+F291
    'hashtag': 62098,  #  U+F292
    'bluetooth': 62099,  #  U+F293
    'bluetooth_b': 62100,  #  U+F294
    'percent': 62101,  #  U+F295
    'gitlab': 62102,  #  U+F296
    'wpbeginner': 62103,  #  U+F297
    'wpforms': 62104,  #  U+F298
    'envira': 62105,  #  U+F299
    'universal_access': 62106,  #  U+F29A
    'accessible_icon': 62107,  #  U+F29B
    'question_circle_o': 62108,  #  U+F29C
    'person_walking_with_cane': 62109,  #  U+F29D
    'audio_description': 62110,  #  U+F29E
    'diamond': 62111,  #  U+F29F
    'phone_volume': 62112,  #  U+F2A0
    'braille': 62113,  #  U+F2A1
    'ear_listen': 62114,  #  U+F2A2
    'hands_asl_interpreting': 62115,  #  U+F2A3
    'ear_deaf': 62116,  #  U+F2A4
    'glide': 62117,  #  U+F2A5
    'glide_g': 62118,  #  U+F2A6
    'hands': 62119,  #  U+F2A7
    'eye_low_vision': 62120,  #  U+F2A8
    'viadeo': 62121,  #  U+F2A9
    'square_viadeo': 62122,  #  U+F2AA
    'snapchat': 62123,  #  U+F2AB
    'gitkraken': 62124,  #  U+F2AC
    'square_snapchat': 62125,  #  U+F2AD
    'pied_piper': 62126,  #  U+F2AE
    'gofore': 62127,  #  U+F2AF
    'first_order': 62128,  #  U+F2B0
    'yoast': 62129,  #  U+F2B1
    'themeisle': 62130,  #  U+F2B2
    'google_plus_circle': 62131,  #  U+F2B3
    'font_awesome': 62132,  #  U+F2B4
    'handshake': 62133,  #  U+F2B5
    'envelope_open': 62134,  #  U+F2B6
    'envelope_open_o': 62135,  #  U+F2B7
    'linode': 62136,  #  U+F2B8
    'address_book': 62137,  #  U+F2B9
    'address_book_o': 62138,  #  U+F2BA
    'address_card': 62139,  #  U+F2BB
    'address_card_o': 62140,  #  U+F2BC
    'circle_user': 62141,  #  U+F2BD
    'user_circle_o': 62142,  #  U+F2BE
    'goodreads': 62143,  #  U+F2BF
    'user_o': 62144,  #  U+F2C0
    'id_badge': 62145,  #  U+F2C1
    'id_card': 62146,  #  U+F2C2
    'id_card_o': 62147,  #  U+F2C3
    'quora': 62148,  #  U+F2C4
    'free_code_camp': 62149,  #  U+F2C5
    'telegram#1': 62150,  #  U+F2C6
    'temperature_full': 62151,  #  U+F2C7
    'temperature_three_quarters': 62152,  #  U+F2C8
    'temperature_half': 62153,  #  U+F2C9
    'temperature_quarter': 62154,  #  U+F2CA
    'temperature_empty': 62155,  #  U+F2CB
    'shower': 62156,  #  U+F2CC
    'bath#1': 62157,  #  U+F2CD
    'podcast': 62158,  #  U+F2CE
    'goodreads_g': 62159,  #  U+F2CF
    'window_maximize': 62160,  #  U+F2D0
    'window_minimize': 62161,  #  U+F2D1
    'window_restore': 62162,  #  U+F2D2
    'square_xmark': 62163,  #  U+F2D3
    'rectangle_xmark': 62164,  #  U+F2D4
    'bandcamp': 62165,  #  U+F2D5
    'grav#1': 62166,  #  U+F2D6
    'etsy': 62167,  #  U+F2D7
    'imdb#1': 62168,  #  U+F2D8
    'ravelry': 62169,  #  U+F2D9
    'sellcast': 62170,  #  U+F2DA
    'microchip': 62171,  #  U+F2DB
    'snowflake': 62172,  #  U+F2DC
    'superpowers': 62173,  #  U+F2DD
    'wpexplorer': 62174,  #  U+F2DE
    'google_drive': 62175,  #  U+F2DF
    'meetup': 62176,  #  U+F2E0
    'google_play': 62177,  #  U+F2E1
    'gripfire': 62178,  #  U+F2E2
    'grunt': 62179,  #  U+F2E3
    'gulp': 62180,  #  U+F2E4
    'square_hacker_news': 62181,  #  U+F2E5
    'hire_a_helper': 62182,  #  U+F2E6
    'hotjar': 62183,  #  U+F2E7
    'hubspot': 62184,  #  U+F2E8
    'itunes': 62185,  #  U+F2E9
    'rotate_left': 62186,  #  U+F2EA
    'itunes_note': 62187,  #  U+F2EB
    'jenkins': 62188,  #  U+F2EC
    'joget': 62189,  #  U+F2ED
    'js': 62190,  #  U+F2EE
    'square_js': 62191,  #  U+F2EF
    'keycdn': 62192,  #  U+F2F0
    'rotate': 62193,  #  U+F2F1
    'stopwatch': 62194,  #  U+F2F2
    'kickstarter': 62195,  #  U+F2F3
    'kickstarter_k': 62196,  #  U+F2F4
    'right_from_bracket': 62197,  #  U+F2F5
    'right_to_bracket': 62198,  #  U+F2F6
    'laravel': 62199,  #  U+F2F7
    'turn_down': 62200,  #  U+F2F8
    'rotate_right': 62201,  #  U+F2F9
    'turn_up': 62202,  #  U+F2FA
    'line': 62203,  #  U+F2FB
    'lock_open': 62204,  #  U+F2FC
    'lyft': 62205,  #  U+F2FD
    'poo': 62206,  #  U+F2FE
    'magento': 62207,  #  U+F2FF
    'Alpine': 62208,  #  U+F300
    'AOSC OS': 62209,  #  U+F301
    'Apple': 62210,  #  U+F302
    'Arch Linux': 62211,  #  U+F303
    'CentOS': 62212,  #  U+F304
    'CoreOS': 62213,  #  U+F305
    'Debian': 62214,  #  U+F306
    'Devuan': 62215,  #  U+F307
    'Docker': 62216,  #  U+F308
    'elementary OS': 62217,  #  U+F309
    'Fedora': 62218,  #  U+F30A
    'Fedora (inverse)': 62219,  #  U+F30B
    'FreeBSD': 62220,  #  U+F30C
    'Gentoo': 62221,  #  U+F30D
    'Linux Mint': 62222,  #  U+F30E
    'Linux Mint (inverse)': 62223,  #  U+F30F
    'Mageia': 62224,  #  U+F310
    'Mandriva': 62225,  #  U+F311
    'Manjaro': 62226,  #  U+F312
    'NixOS': 62227,  #  U+F313
    'OpenSUSE': 62228,  #  U+F314
    'Raspberry pi': 62229,  #  U+F315
    'Red Hat': 62230,  #  U+F316
    'Sabayon': 62231,  #  U+F317
    'Slackware': 62232,  #  U+F318
    'Slackware (inverse)': 62233,  #  U+F319
    'Tux': 62234,  #  U+F31A
    'Ubuntu': 62235,  #  U+F31B
    'Ubuntu (inverse)': 62236,  #  U+F31C
    'Alma Linux': 62237,  #  U+F31D
    'ArchLabs': 62238,  #  U+F31E
    'Artix Linux': 62239,  #  U+F31F
    'Budgie': 62240,  #  U+F320
    'Deepin': 62241,  #  U+F321
    'Endeavour OS': 62242,  #  U+F322
    'Ferris': 62243,  #  U+F323
    'Flathub': 62244,  #  U+F324
    'GNU Guix': 62245,  #  U+F325
    'illumos': 62246,  #  U+F326
    'Kali Linux': 62247,  #  U+F327
    'OpenBSD': 62248,  #  U+F328
    'Parrot OS': 62249,  #  U+F329
    'Pop!_OS': 62250,  #  U+F32A
    'Rocky Linux': 62251,  #  U+F32B
    'Snappy': 62252,  #  U+F32C
    'Solus': 62253,  #  U+F32D
    'Void': 62254,  #  U+F32E
    'Zorin OS': 62255,  #  U+F32F
    'Codeberg': 62256,  #  U+F330
    'KDE Neon': 62257,  #  U+F331
    'KDE Plasma': 62258,  #  U+F332
    'Kubuntu': 62259,  #  U+F333
    'Kubuntu (inverse)': 62260,  #  U+F334
    'Forgejo': 62261,  #  U+F335
    'FreeCAD': 62262,  #  U+F336
    'Garuda Linux': 62263,  #  U+F337
    'GIMP': 62264,  #  U+F338
    'Gitea': 62265,  #  U+F339
    'Hyperbola GNU/Linux-libre': 62266,  #  U+F33A
    'Inkscape': 62267,  #  U+F33B
    'Kdenlive': 62268,  #  U+F33C
    'Krita': 62269,  #  U+F33D
    'LXLE Linux': 62270,  #  U+F33E
    'MX Linux': 62271,  #  U+F33F
    'Parabola GNU/Linux-libre': 62272,  #  U+F340
    'Puppy Linux': 62273,  #  U+F341
    'QubesOS': 62274,  #  U+F342
    'Tails': 62275,  #  U+F343
    'Trisquel GNU/Linux': 62276,  #  U+F344
    'Archcraft': 62277,  #  U+F345
    'ArcoLinux': 62278,  #  U+F346
    'BigLinux': 62279,  #  U+F347
    'Crystal Linux': 62280,  #  U+F348
    'Loc-OS': 62281,  #  U+F349
    'XeroLinux': 62282,  #  U+F34A
    'Arduino': 62283,  #  U+F34B
    'KiCad': 62284,  #  U+F34C
    'Octoprint': 62285,  #  U+F34D
    'OpenSCAD': 62286,  #  U+F34E
    'OSH': 62287,  #  U+F34F
    'OSHWA': 62288,  #  U+F350
    'Prusa Slicer': 62289,  #  U+F351
    'RepRap': 62290,  #  U+F352
    'RISC-V': 62291,  #  U+F353
    'Awesome WM': 62292,  #  U+F354
    'bspwm': 62293,  #  U+F355
    'dwm': 62294,  #  U+F356
    'Enlightenment': 62295,  #  U+F357
    'Fluxbox': 62296,  #  U+F358
    'Hyprland': 62297,  #  U+F359
    'i3': 62298,  #  U+F35A
    'JWM': 62299,  #  U+F35B
    'Qtile': 62300,  #  U+F35C
    'Sway': 62301,  #  U+F35D
    'Xmonad': 62302,  #  U+F35E
    'Cinnamon': 62303,  #  U+F35F
    'freedesktop.org': 62304,  #  U+F360
    'GNOME': 62305,  #  U+F361
    'GTK': 62306,  #  U+F362
    'LXDE': 62307,  #  U+F363
    'LXQt': 62308,  #  U+F364
    'MATE': 62309,  #  U+F365
    'Vanilla OS': 62310,  #  U+F366
    'Wayland': 62311,  #  U+F367
    'XFCE': 62312,  #  U+F368
    'Xorg': 62313,  #  U+F369
    'F-droid': 62314,  #  U+F36A
    'FOSDEM': 62315,  #  U+F36B
    'OSI': 62316,  #  U+F36C
    'Wikimedia': 62317,  #  U+F36D
    'mpv': 62318,  #  U+F36E
    'Neovim': 62319,  #  U+F36F
    'Thunderbird': 62320,  #  U+F370
    'Tor Browser': 62321,  #  U+F371
    'VS Codium': 62322,  #  U+F372
    'KDE': 62323,  #  U+F373
    'PostmarketOS': 62324,  #  U+F374
    'Qt': 62325,  #  U+F375
    'light-bulb': 62464,  #  U+F400
    'repo#1': 62465,  #  U+F401
    'repo-forked#1': 62466,  #  U+F402
    'repo-push#1': 62467,  #  U+F403
    'repo-pull#1': 62468,  #  U+F404
    'book#2': 62469,  #  U+F405
    'accessibility': 62470,  #  U+F406
    'git-pull-request#1': 62471,  #  U+F407
    'mark-github': 62472,  #  U+F408
    'download#1': 62473,  #  U+F409
    'upload#1': 62474,  #  U+F40A
    'accessibility-inset': 62475,  #  U+F40B
    'alert-fill': 62476,  #  U+F40C
    'file-code#2': 62477,  #  U+F40D
    'apps': 62478,  #  U+F40E
    'file-media#1': 62479,  #  U+F40F
    'file-zip#1': 62480,  #  U+F410
    'archive#1': 62481,  #  U+F411
    'tag#2': 62482,  #  U+F412
    'file-directory': 62483,  #  U+F413
    'file-submodule#1': 62484,  #  U+F414
    'person#2': 62485,  #  U+F415
    'arrow-both#1': 62486,  #  U+F416
    'git-commit#1': 62487,  #  U+F417
    'git-branch': 62488,  #  U+F418
    'git-merge#1': 62489,  #  U+F419
    'mirror#1': 62490,  #  U+F41A
    'issue-opened': 62491,  #  U+F41B
    'issue-reopened#1': 62492,  #  U+F41C
    'issue-closed': 62493,  #  U+F41D
    'star#1': 62494,  #  U+F41E
    'comment#2': 62495,  #  U+F41F
    'question#2': 62496,  #  U+F420
    'alert': 62497,  #  U+F421
    'search#1': 62498,  #  U+F422
    'gear#2': 62499,  #  U+F423
    'arrow-down-left': 62500,  #  U+F424
    'tools#2': 62501,  #  U+F425
    'sign-out#1': 62502,  #  U+F426
    'rocket#2': 62503,  #  U+F427
    'rss#2': 62504,  #  U+F428
    'paste#1': 62505,  #  U+F429
    'sign-in#1': 62506,  #  U+F42A
    'organization#1': 62507,  #  U+F42B
    'device-mobile#1': 62508,  #  U+F42C
    'unfold#1': 62509,  #  U+F42D
    'check#2': 62510,  #  U+F42E
    'mail#1': 62511,  #  U+F42F
    'read': 62512,  #  U+F430
    'arrow-up#1': 62513,  #  U+F431
    'arrow-right#1': 62514,  #  U+F432
    'arrow-down#1': 62515,  #  U+F433
    'arrow-left#1': 62516,  #  U+F434
    'pin#1': 62517,  #  U+F435
    'gift#2': 62518,  #  U+F436
    'graph#1': 62519,  #  U+F437
    'triangle-left#1': 62520,  #  U+F438
    'credit-card#1': 62521,  #  U+F439
    'clock#1': 62522,  #  U+F43A
    'ruby#2': 62523,  #  U+F43B
    'broadcast#1': 62524,  #  U+F43C
    'key#2': 62525,  #  U+F43D
    'arrow-down-right': 62526,  #  U+F43E
    'repo-clone#1': 62527,  #  U+F43F
    'diff#1': 62528,  #  U+F440
    'eye#2': 62529,  #  U+F441
    'comment-discussion#1': 62530,  #  U+F442
    'arrow-switch': 62531,  #  U+F443
    'dot-fill': 62532,  #  U+F444
    'square-fill': 62533,  #  U+F445
    'device-camera#1': 62534,  #  U+F446
    'device-camera-video#1': 62535,  #  U+F447
    'pencil#1': 62536,  #  U+F448
    'info#2': 62537,  #  U+F449
    'triangle-right#1': 62538,  #  U+F44A
    'triangle-down#1': 62539,  #  U+F44B
    'link#2': 62540,  #  U+F44C
    'plus#1': 62541,  #  U+F44D
    'three-bars#1': 62542,  #  U+F44E
    'code#2': 62543,  #  U+F44F
    'location#1': 62544,  #  U+F450
    'list-unordered#1': 62545,  #  U+F451
    'list-ordered#1': 62546,  #  U+F452
    'quote#1': 62547,  #  U+F453
    'versions#1': 62548,  #  U+F454
    'calendar#1': 62549,  #  U+F455
    'lock#2': 62550,  #  U+F456
    'diff-added#1': 62551,  #  U+F457
    'diff-removed#1': 62552,  #  U+F458
    'diff-modified#1': 62553,  #  U+F459
    'diff-renamed#1': 62554,  #  U+F45A
    'horizontal-rule#1': 62555,  #  U+F45B
    'arrow-up-left': 62556,  #  U+F45C
    'milestone#1': 62557,  #  U+F45D
    'checklist#1': 62558,  #  U+F45E
    'megaphone#1': 62559,  #  U+F45F
    'chevron-right#1': 62560,  #  U+F460
    'bookmark#2': 62561,  #  U+F461
    'sliders#1': 62562,  #  U+F462
    'meter': 62563,  #  U+F463
    'history#1': 62564,  #  U+F464
    'link-external#1': 62565,  #  U+F465
    'mute#1': 62566,  #  U+F466
    'x': 62567,  #  U+F467
    'circle-slash#1': 62568,  #  U+F468
    'pulse#2': 62569,  #  U+F469
    'sync#1': 62570,  #  U+F46A
    'telescope#2': 62571,  #  U+F46B
    'arrow-up-right': 62572,  #  U+F46C
    'home#1': 62573,  #  U+F46D
    'stop#1': 62574,  #  U+F46E
    'bug#2': 62575,  #  U+F46F
    'logo-github': 62576,  #  U+F470
    'file-binary#1': 62577,  #  U+F471
    'database#2': 62578,  #  U+F472
    'server#2': 62579,  #  U+F473
    'diff-ignored#1': 62580,  #  U+F474
    'ellipsis#2': 62581,  #  U+F475
    'bell-fill': 62582,  #  U+F476
    'hubot#1': 62583,  #  U+F477
    'bell-slash#1': 62584,  #  U+F478
    'blocked': 62585,  #  U+F479
    'bookmark-fill': 62586,  #  U+F47A
    'chevron-up#1': 62587,  #  U+F47B
    'chevron-down#1': 62588,  #  U+F47C
    'chevron-left#1': 62589,  #  U+F47D
    'triangle-up#1': 62590,  #  U+F47E
    'git-compare#1': 62591,  #  U+F47F
    'logo-gist': 62592,  #  U+F480
    'file-symlink-file#1': 62593,  #  U+F481
    'file-symlink-directory#1': 62594,  #  U+F482
    'squirrel#1': 62595,  #  U+F483
    'globe#2': 62596,  #  U+F484
    'unmute#1': 62597,  #  U+F485
    'mention#1': 62598,  #  U+F486
    'package#1': 62599,  #  U+F487
    'browser#1': 62600,  #  U+F488
    'terminal#2': 62601,  #  U+F489
    'markdown#2': 62602,  #  U+F48A
    'dash#1': 62603,  #  U+F48B
    'fold#1': 62604,  #  U+F48C
    'inbox#2': 62605,  #  U+F48D
    'trash#2': 62606,  #  U+F48E
    'paintbrush#1': 62607,  #  U+F48F
    'flame#1': 62608,  #  U+F490
    'briefcase#2': 62609,  #  U+F491
    'plug#2': 62610,  #  U+F492
    'bookmark-slash-fill': 62611,  #  U+F493
    'mortar-board#1': 62612,  #  U+F494
    'law#1': 62613,  #  U+F495
    'thumbsup#1': 62614,  #  U+F496
    'thumbsdown#1': 62615,  #  U+F497
    'desktop-download#1': 62616,  #  U+F498
    'beaker#1': 62617,  #  U+F499
    'bell#2': 62618,  #  U+F49A
    'cache': 62619,  #  U+F49B
    'shield#2': 62620,  #  U+F49C
    'bold#2': 62621,  #  U+F49D
    'check-circle': 62622,  #  U+F49E
    'italic#2': 62623,  #  U+F49F
    'tasklist#1': 62624,  #  U+F4A0
    'verified#1': 62625,  #  U+F4A1
    'smiley#1': 62626,  #  U+F4A2
    'unverified#1': 62627,  #  U+F4A3
    'check-circle-fill': 62628,  #  U+F4A4
    'file#2': 62629,  #  U+F4A5
    'grabber#1': 62630,  #  U+F4A6
    'checkbox': 62631,  #  U+F4A7
    'reply#2': 62632,  #  U+F4A8
    'device-desktop': 62633,  #  U+F4A9
    'circle#2': 62634,  #  U+F4AA
    'clock-fill': 62635,  #  U+F4AB
    'cloud#3': 62636,  #  U+F4AC
    'cloud-offline': 62637,  #  U+F4AD
    'code-of-conduct': 62638,  #  U+F4AE
    'code-review': 62639,  #  U+F4AF
    'code-square': 62640,  #  U+F4B0
    'codescan': 62641,  #  U+F4B1
    'codescan-checkmark': 62642,  #  U+F4B2
    'codespaces': 62643,  #  U+F4B3
    'columns': 62644,  #  U+F4B4
    'command-palette': 62645,  #  U+F4B5
    'commit': 62646,  #  U+F4B6
    'container': 62647,  #  U+F4B7
    'copilot#1': 62648,  #  U+F4B8
    'copilot-error': 62649,  #  U+F4B9
    'copilot-warning': 62650,  #  U+F4BA
    'copy#2': 62651,  #  U+F4BB
    'cpu': 62652,  #  U+F4BC
    'cross-reference': 62653,  #  U+F4BD
    'dependabot': 62654,  #  U+F4BE
    'diamond#1': 62655,  #  U+F4BF
    'discussion-closed': 62656,  #  U+F4C0
    'discussion-duplicate': 62657,  #  U+F4C1
    'discussion-outdated': 62658,  #  U+F4C2
    'dot': 62659,  #  U+F4C3
    'duplicate': 62660,  #  U+F4C4
    'eye-closed#1': 62661,  #  U+F4C5
    'feed-discussion': 62662,  #  U+F4C6
    'feed-forked': 62663,  #  U+F4C7
    'feed-heart': 62664,  #  U+F4C8
    'feed-merged': 62665,  #  U+F4C9
    'feed-person': 62666,  #  U+F4CA
    'feed-repo': 62667,  #  U+F4CB
    'feed-rocket': 62668,  #  U+F4CC
    'feed-star': 62669,  #  U+F4CD
    'feed-tag': 62670,  #  U+F4CE
    'feed-trophy': 62671,  #  U+F4CF
    'file-added': 62672,  #  U+F4D0
    'file-badge': 62673,  #  U+F4D1
    'file-diff': 62674,  #  U+F4D2
    'file-directory-fill': 62675,  #  U+F4D3
    'file-directory-open-fill': 62676,  #  U+F4D4
    'file-moved': 62677,  #  U+F4D5
    'file-removed': 62678,  #  U+F4D6
    'filter#2': 62679,  #  U+F4D7
    'fiscal-host': 62680,  #  U+F4D8
    'fold-down#1': 62681,  #  U+F4D9
    'fold-up#1': 62682,  #  U+F4DA
    'git-merge-queue': 62683,  #  U+F4DB
    'git-pull-request-closed#1': 62684,  #  U+F4DC
    'git-pull-request-draft#1': 62685,  #  U+F4DD
    'goal': 62686,  #  U+F4DE
    'hash': 62687,  #  U+F4DF
    'heading#1': 62688,  #  U+F4E0
    'heart-fill': 62689,  #  U+F4E1
    'home-fill': 62690,  #  U+F4E2
    'hourglass#1': 62691,  #  U+F4E3
    'id-badge': 62692,  #  U+F4E4
    'image#1': 62693,  #  U+F4E5
    'infinity#2': 62694,  #  U+F4E6
    'issue-draft#1': 62695,  #  U+F4E7
    'issue-tracked-by': 62696,  #  U+F4E8
    'issue-tracks': 62697,  #  U+F4E9
    'iterations': 62698,  #  U+F4EA
    'kebab-horizontal': 62699,  #  U+F4EB
    'key-asterisk': 62700,  #  U+F4EC
    'log': 62701,  #  U+F4ED
    'moon#1': 62702,  #  U+F4EE
    'move-to-bottom': 62703,  #  U+F4EF
    'move-to-end': 62704,  #  U+F4F0
    'move-to-start': 62705,  #  U+F4F1
    'move-to-top': 62706,  #  U+F4F2
    'multi-select': 62707,  #  U+F4F3
    'no-entry': 62708,  #  U+F4F4
    'north-star': 62709,  #  U+F4F5
    'note#1': 62710,  #  U+F4F6
    'number': 62711,  #  U+F4F7
    'package-dependencies': 62712,  #  U+F4F8
    'package-dependents': 62713,  #  U+F4F9
    'paper-airplane': 62714,  #  U+F4FA
    'paperclip#1': 62715,  #  U+F4FB
    'passkey-fill': 62716,  #  U+F4FC
    'people': 62717,  #  U+F4FD
    'person-add#1': 62718,  #  U+F4FE
    'person-fill': 62719,  #  U+F4FF
    'play#2': 62720,  #  U+F500
    'plus-circle': 62721,  #  U+F501
    'project#1': 62722,  #  U+F502
    'project-roadmap': 62723,  #  U+F503
    'project-symlink': 62724,  #  U+F504
    'project-template': 62725,  #  U+F505
    'rel-file-path': 62726,  #  U+F506
    'repo-deleted': 62727,  #  U+F507
    'repo-locked': 62728,  #  U+F508
    'repo-template': 62729,  #  U+F509
    'report#1': 62730,  #  U+F50A
    'rows': 62731,  #  U+F50B
    'screen-full#1': 62732,  #  U+F50C
    'screen-normal#1': 62733,  #  U+F50D
    'share#1': 62734,  #  U+F50E
    'share-android': 62735,  #  U+F50F
    'shield-check': 62736,  #  U+F510
    'shield-lock': 62737,  #  U+F511
    'shield-slash': 62738,  #  U+F512
    'shield-x': 62739,  #  U+F513
    'sidebar-collapse': 62740,  #  U+F514
    'sidebar-expand': 62741,  #  U+F515
    'single-select': 62742,  #  U+F516
    'skip': 62743,  #  U+F517
    'skip-fill': 62744,  #  U+F518
    'sort-asc': 62745,  #  U+F519
    'sort-desc': 62746,  #  U+F51A
    'sparkle-fill': 62747,  #  U+F51B
    'sponsor-tiers': 62748,  #  U+F51C
    'square#1': 62749,  #  U+F51D
    'stack': 62750,  #  U+F51E
    'star-fill': 62751,  #  U+F51F
    'stopwatch#1': 62752,  #  U+F520
    'strikethrough#1': 62753,  #  U+F521
    'sun#1': 62754,  #  U+F522
    'tab': 62755,  #  U+F523
    'tab-external': 62756,  #  U+F524
    'table#2': 62757,  #  U+F525
    'telescope-fill': 62758,  #  U+F526
    'trophy#1': 62759,  #  U+F527
    'typography': 62760,  #  U+F528
    'unlink': 62761,  #  U+F529
    'unlock#2': 62762,  #  U+F52A
    'unread': 62763,  #  U+F52B
    'video#1': 62764,  #  U+F52C
    'webhook': 62765,  #  U+F52D
    'workflow': 62766,  #  U+F52E
    'x-circle': 62767,  #  U+F52F
    'x-circle-fill': 62768,  #  U+F530
    'zoom-in#1': 62769,  #  U+F531
    'zoom-out#1': 62770,  #  U+F532
    'bookmark-slash': 62771,  #  U+F533
    'vector-square': 983041,  # 󰀁 U+F0001
    'access-point-network': 983042,  # 󰀂 U+F0002
    'access-point': 983043,  # 󰀃 U+F0003
    'account#1': 983044,  # 󰀄 U+F0004
    'account-alert': 983045,  # 󰀅 U+F0005
    'account-box': 983046,  # 󰀆 U+F0006
    'account-box-outline': 983047,  # 󰀇 U+F0007
    'account-check': 983048,  # 󰀈 U+F0008
    'account-circle': 983049,  # 󰀉 U+F0009
    'account-convert': 983050,  # 󰀊 U+F000A
    'account-key': 983051,  # 󰀋 U+F000B
    'tooltip-account': 983052,  # 󰀌 U+F000C
    'account-minus': 983053,  # 󰀍 U+F000D
    'account-multiple': 983054,  # 󰀎 U+F000E
    'account-multiple-outline': 983055,  # 󰀏 U+F000F
    'account-multiple-plus': 983056,  # 󰀐 U+F0010
    'account-network': 983057,  # 󰀑 U+F0011
    'account-off': 983058,  # 󰀒 U+F0012
    'account-outline': 983059,  # 󰀓 U+F0013
    'account-plus': 983060,  # 󰀔 U+F0014
    'account-remove': 983061,  # 󰀕 U+F0015
    'account-search': 983062,  # 󰀖 U+F0016
    'account-star': 983063,  # 󰀗 U+F0017
    'orbit': 983064,  # 󰀘 U+F0018
    'account-switch': 983065,  # 󰀙 U+F0019
    'adjust': 983066,  # 󰀚 U+F001A
    'air-conditioner': 983067,  # 󰀛 U+F001B
    'airballoon': 983068,  # 󰀜 U+F001C
    'airplane': 983069,  # 󰀝 U+F001D
    'airplane-off': 983070,  # 󰀞 U+F001E
    'cast-variant': 983071,  # 󰀟 U+F001F
    'alarm': 983072,  # 󰀠 U+F0020
    'alarm-check': 983073,  # 󰀡 U+F0021
    'alarm-multiple': 983074,  # 󰀢 U+F0022
    'alarm-off': 983075,  # 󰀣 U+F0023
    'alarm-plus': 983076,  # 󰀤 U+F0024
    'album': 983077,  # 󰀥 U+F0025
    'alert#1': 983078,  # 󰀦 U+F0026
    'alert-box': 983079,  # 󰀧 U+F0027
    'alert-circle': 983080,  # 󰀨 U+F0028
    'alert-octagon': 983081,  # 󰀩 U+F0029
    'alert-outline': 983082,  # 󰀪 U+F002A
    'alpha': 983083,  # 󰀫 U+F002B
    'alphabetical': 983084,  # 󰀬 U+F002C
    'greenhouse': 983085,  # 󰀭 U+F002D
    'rollerblade-off': 983086,  # 󰀮 U+F002E
    'ambulance': 983087,  # 󰀯 U+F002F
    'amplifier': 983088,  # 󰀰 U+F0030
    'anchor#1': 983089,  # 󰀱 U+F0031
    'android#1': 983090,  # 󰀲 U+F0032
    'web-plus': 983091,  # 󰀳 U+F0033
    'android-studio': 983092,  # 󰀴 U+F0034
    'apple#1': 983093,  # 󰀵 U+F0035
    'apple-finder': 983094,  # 󰀶 U+F0036
    'apple-ios': 983095,  # 󰀷 U+F0037
    'apple-icloud': 983096,  # 󰀸 U+F0038
    'apple-safari': 983097,  # 󰀹 U+F0039
    'font-awesome': 983098,  # 󰀺 U+F003A
    'apps#1': 983099,  # 󰀻 U+F003B
    'archive#2': 983100,  # 󰀼 U+F003C
    'arrange-bring-forward': 983101,  # 󰀽 U+F003D
    'arrange-bring-to-front': 983102,  # 󰀾 U+F003E
    'arrange-send-backward': 983103,  # 󰀿 U+F003F
    'arrange-send-to-back': 983104,  # 󰁀 U+F0040
    'arrow-all': 983105,  # 󰁁 U+F0041
    'arrow-bottom-left': 983106,  # 󰁂 U+F0042
    'arrow-bottom-right': 983107,  # 󰁃 U+F0043
    'arrow-collapse-all': 983108,  # 󰁄 U+F0044
    'arrow-down#2': 983109,  # 󰁅 U+F0045
    'arrow-down-thick': 983110,  # 󰁆 U+F0046
    'arrow-down-bold-circle': 983111,  # 󰁇 U+F0047
    'arrow-down-bold-circle-outline': 983112,  # 󰁈 U+F0048
    'arrow-down-bold-hexagon-outline': 983113,  # 󰁉 U+F0049
    'arrow-down-drop-circle': 983114,  # 󰁊 U+F004A
    'arrow-down-drop-circle-outline': 983115,  # 󰁋 U+F004B
    'arrow-expand-all': 983116,  # 󰁌 U+F004C
    'arrow-left#2': 983117,  # 󰁍 U+F004D
    'arrow-left-thick': 983118,  # 󰁎 U+F004E
    'arrow-left-bold-circle': 983119,  # 󰁏 U+F004F
    'arrow-left-bold-circle-outline': 983120,  # 󰁐 U+F0050
    'arrow-left-bold-hexagon-outline': 983121,  # 󰁑 U+F0051
    'arrow-left-drop-circle': 983122,  # 󰁒 U+F0052
    'arrow-left-drop-circle-outline': 983123,  # 󰁓 U+F0053
    'arrow-right#2': 983124,  # 󰁔 U+F0054
    'arrow-right-thick': 983125,  # 󰁕 U+F0055
    'arrow-right-bold-circle': 983126,  # 󰁖 U+F0056
    'arrow-right-bold-circle-outline': 983127,  # 󰁗 U+F0057
    'arrow-right-bold-hexagon-outline': 983128,  # 󰁘 U+F0058
    'arrow-right-drop-circle': 983129,  # 󰁙 U+F0059
    'arrow-right-drop-circle-outline': 983130,  # 󰁚 U+F005A
    'arrow-top-left': 983131,  # 󰁛 U+F005B
    'arrow-top-right': 983132,  # 󰁜 U+F005C
    'arrow-up#2': 983133,  # 󰁝 U+F005D
    'arrow-up-thick': 983134,  # 󰁞 U+F005E
    'arrow-up-bold-circle': 983135,  # 󰁟 U+F005F
    'arrow-up-bold-circle-outline': 983136,  # 󰁠 U+F0060
    'arrow-up-bold-hexagon-outline': 983137,  # 󰁡 U+F0061
    'arrow-up-drop-circle': 983138,  # 󰁢 U+F0062
    'arrow-up-drop-circle-outline': 983139,  # 󰁣 U+F0063
    'assistant': 983140,  # 󰁤 U+F0064
    'at#1': 983141,  # 󰁥 U+F0065
    'attachment': 983142,  # 󰁦 U+F0066
    'book-music': 983143,  # 󰁧 U+F0067
    'auto-fix': 983144,  # 󰁨 U+F0068
    'auto-upload': 983145,  # 󰁩 U+F0069
    'autorenew': 983146,  # 󰁪 U+F006A
    'av-timer': 983147,  # 󰁫 U+F006B
    'baby#1': 983148,  # 󰁬 U+F006C
    'backburger': 983149,  # 󰁭 U+F006D
    'backspace': 983150,  # 󰁮 U+F006E
    'backup-restore': 983151,  # 󰁯 U+F006F
    'bank': 983152,  # 󰁰 U+F0070
    'barcode#1': 983153,  # 󰁱 U+F0071
    'barcode-scan': 983154,  # 󰁲 U+F0072
    'barley': 983155,  # 󰁳 U+F0073
    'barrel': 983156,  # 󰁴 U+F0074
    'incognito-off': 983157,  # 󰁵 U+F0075
    'basket': 983158,  # 󰁶 U+F0076
    'basket-fill': 983159,  # 󰁷 U+F0077
    'basket-unfill': 983160,  # 󰁸 U+F0078
    'battery': 983161,  # 󰁹 U+F0079
    'battery-10': 983162,  # 󰁺 U+F007A
    'battery-20': 983163,  # 󰁻 U+F007B
    'battery-30': 983164,  # 󰁼 U+F007C
    'battery-40': 983165,  # 󰁽 U+F007D
    'battery-50': 983166,  # 󰁾 U+F007E
    'battery-60': 983167,  # 󰁿 U+F007F
    'battery-70': 983168,  # 󰂀 U+F0080
    'battery-80': 983169,  # 󰂁 U+F0081
    'battery-90': 983170,  # 󰂂 U+F0082
    'battery-alert': 983171,  # 󰂃 U+F0083
    'battery-charging': 983172,  # 󰂄 U+F0084
    'battery-charging-100': 983173,  # 󰂅 U+F0085
    'battery-charging-20': 983174,  # 󰂆 U+F0086
    'battery-charging-30': 983175,  # 󰂇 U+F0087
    'battery-charging-40': 983176,  # 󰂈 U+F0088
    'battery-charging-60': 983177,  # 󰂉 U+F0089
    'battery-charging-80': 983178,  # 󰂊 U+F008A
    'battery-charging-90': 983179,  # 󰂋 U+F008B
    'battery-minus-variant': 983180,  # 󰂌 U+F008C
    'battery-negative': 983181,  # 󰂍 U+F008D
    'battery-outline': 983182,  # 󰂎 U+F008E
    'battery-plus-variant': 983183,  # 󰂏 U+F008F
    'battery-positive': 983184,  # 󰂐 U+F0090
    'battery-unknown': 983185,  # 󰂑 U+F0091
    'beach': 983186,  # 󰂒 U+F0092
    'flask#1': 983187,  # 󰂓 U+F0093
    'flask-empty': 983188,  # 󰂔 U+F0094
    'flask-empty-outline': 983189,  # 󰂕 U+F0095
    'flask-outline': 983190,  # 󰂖 U+F0096
    'bunk-bed-outline': 983191,  # 󰂗 U+F0097
    'beer': 983192,  # 󰂘 U+F0098
    'bed-outline': 983193,  # 󰂙 U+F0099
    'bell#3': 983194,  # 󰂚 U+F009A
    'bell-off': 983195,  # 󰂛 U+F009B
    'bell-outline': 983196,  # 󰂜 U+F009C
    'bell-plus': 983197,  # 󰂝 U+F009D
    'bell-ring': 983198,  # 󰂞 U+F009E
    'bell-ring-outline': 983199,  # 󰂟 U+F009F
    'bell-sleep': 983200,  # 󰂠 U+F00A0
    'beta': 983201,  # 󰂡 U+F00A1
    'book-cross': 983202,  # 󰂢 U+F00A2
    'bike': 983203,  # 󰂣 U+F00A3
    'microsoft-bing': 983204,  # 󰂤 U+F00A4
    'binoculars#1': 983205,  # 󰂥 U+F00A5
    'bio': 983206,  # 󰂦 U+F00A6
    'biohazard#2': 983207,  # 󰂧 U+F00A7
    'bitbucket#1': 983208,  # 󰂨 U+F00A8
    'black-mesa': 983209,  # 󰂩 U+F00A9
    'shield-refresh': 983210,  # 󰂪 U+F00AA
    'blender-software': 983211,  # 󰂫 U+F00AB
    'blinds': 983212,  # 󰂬 U+F00AC
    'block-helper': 983213,  # 󰂭 U+F00AD
    'application-edit': 983214,  # 󰂮 U+F00AE
    'bluetooth#1': 983215,  # 󰂯 U+F00AF
    'bluetooth-audio': 983216,  # 󰂰 U+F00B0
    'bluetooth-connect': 983217,  # 󰂱 U+F00B1
    'bluetooth-off': 983218,  # 󰂲 U+F00B2
    'bluetooth-settings': 983219,  # 󰂳 U+F00B3
    'bluetooth-transfer': 983220,  # 󰂴 U+F00B4
    'blur': 983221,  # 󰂵 U+F00B5
    'blur-linear': 983222,  # 󰂶 U+F00B6
    'blur-off': 983223,  # 󰂷 U+F00B7
    'blur-radial': 983224,  # 󰂸 U+F00B8
    'bone#1': 983225,  # 󰂹 U+F00B9
    'book#3': 983226,  # 󰂺 U+F00BA
    'book-multiple': 983227,  # 󰂻 U+F00BB
    'book-variant-multiple': 983228,  # 󰂼 U+F00BC
    'book-open#1': 983229,  # 󰂽 U+F00BD
    'book-open-blank-variant': 983230,  # 󰂾 U+F00BE
    'book-variant': 983231,  # 󰂿 U+F00BF
    'bookmark#3': 983232,  # 󰃀 U+F00C0
    'bookmark-check': 983233,  # 󰃁 U+F00C1
    'bookmark-music': 983234,  # 󰃂 U+F00C2
    'bookmark-outline': 983235,  # 󰃃 U+F00C3
    'bookmark-plus-outline': 983236,  # 󰃄 U+F00C4
    'bookmark-plus': 983237,  # 󰃅 U+F00C5
    'bookmark-remove': 983238,  # 󰃆 U+F00C6
    'border-all': 983239,  # 󰃇 U+F00C7
    'border-bottom': 983240,  # 󰃈 U+F00C8
    'border-color': 983241,  # 󰃉 U+F00C9
    'border-horizontal': 983242,  # 󰃊 U+F00CA
    'border-inside': 983243,  # 󰃋 U+F00CB
    'border-left': 983244,  # 󰃌 U+F00CC
    'border-none': 983245,  # 󰃍 U+F00CD
    'border-outside': 983246,  # 󰃎 U+F00CE
    'border-right': 983247,  # 󰃏 U+F00CF
    'border-style': 983248,  # 󰃐 U+F00D0
    'border-top': 983249,  # 󰃑 U+F00D1
    'border-vertical': 983250,  # 󰃒 U+F00D2
    'bowling': 983251,  # 󰃓 U+F00D3
    'box#1': 983252,  # 󰃔 U+F00D4
    'box-cutter': 983253,  # 󰃕 U+F00D5
    'briefcase#3': 983254,  # 󰃖 U+F00D6
    'briefcase-check': 983255,  # 󰃗 U+F00D7
    'briefcase-download': 983256,  # 󰃘 U+F00D8
    'briefcase-upload': 983257,  # 󰃙 U+F00D9
    'brightness-1': 983258,  # 󰃚 U+F00DA
    'brightness-2': 983259,  # 󰃛 U+F00DB
    'brightness-3': 983260,  # 󰃜 U+F00DC
    'brightness-4': 983261,  # 󰃝 U+F00DD
    'brightness-5': 983262,  # 󰃞 U+F00DE
    'brightness-6': 983263,  # 󰃟 U+F00DF
    'brightness-7': 983264,  # 󰃠 U+F00E0
    'brightness-auto': 983265,  # 󰃡 U+F00E1
    'broom#1': 983266,  # 󰃢 U+F00E2
    'brush#1': 983267,  # 󰃣 U+F00E3
    'bug#3': 983268,  # 󰃤 U+F00E4
    'bulletin-board': 983269,  # 󰃥 U+F00E5
    'bullhorn#1': 983270,  # 󰃦 U+F00E6
    'bus#1': 983271,  # 󰃧 U+F00E7
    'cached': 983272,  # 󰃨 U+F00E8
    'cake': 983273,  # 󰃩 U+F00E9
    'cake-layered': 983274,  # 󰃪 U+F00EA
    'cake-variant': 983275,  # 󰃫 U+F00EB
    'calculator#1': 983276,  # 󰃬 U+F00EC
    'calendar#2': 983277,  # 󰃭 U+F00ED
    'calendar-blank': 983278,  # 󰃮 U+F00EE
    'calendar-check': 983279,  # 󰃯 U+F00EF
    'calendar-clock': 983280,  # 󰃰 U+F00F0
    'calendar-multiple': 983281,  # 󰃱 U+F00F1
    'calendar-multiple-check': 983282,  # 󰃲 U+F00F2
    'calendar-plus': 983283,  # 󰃳 U+F00F3
    'calendar-remove': 983284,  # 󰃴 U+F00F4
    'calendar-text': 983285,  # 󰃵 U+F00F5
    'calendar-today': 983286,  # 󰃶 U+F00F6
    'call-made': 983287,  # 󰃷 U+F00F7
    'call-merge': 983288,  # 󰃸 U+F00F8
    'call-missed': 983289,  # 󰃹 U+F00F9
    'call-received': 983290,  # 󰃺 U+F00FA
    'call-split': 983291,  # 󰃻 U+F00FB
    'camcorder': 983292,  # 󰃼 U+F00FC
    'video-box': 983293,  # 󰃽 U+F00FD
    'video-box-off': 983294,  # 󰃾 U+F00FE
    'camcorder-off': 983295,  # 󰃿 U+F00FF
    'camera#1': 983296,  # 󰄀 U+F0100
    'camera-enhance': 983297,  # 󰄁 U+F0101
    'camera-front': 983298,  # 󰄂 U+F0102
    'camera-front-variant': 983299,  # 󰄃 U+F0103
    'camera-iris': 983300,  # 󰄄 U+F0104
    'camera-party-mode': 983301,  # 󰄅 U+F0105
    'camera-rear': 983302,  # 󰄆 U+F0106
    'camera-rear-variant': 983303,  # 󰄇 U+F0107
    'camera-switch': 983304,  # 󰄈 U+F0108
    'camera-timer': 983305,  # 󰄉 U+F0109
    'candycane': 983306,  # 󰄊 U+F010A
    'car#1': 983307,  # 󰄋 U+F010B
    'car-battery': 983308,  # 󰄌 U+F010C
    'car-connected': 983309,  # 󰄍 U+F010D
    'car-wash': 983310,  # 󰄎 U+F010E
    'carrot#1': 983311,  # 󰄏 U+F010F
    'cart': 983312,  # 󰄐 U+F0110
    'cart-outline': 983313,  # 󰄑 U+F0111
    'cart-plus': 983314,  # 󰄒 U+F0112
    'case-sensitive-alt': 983315,  # 󰄓 U+F0113
    'cash': 983316,  # 󰄔 U+F0114
    'cash-100': 983317,  # 󰄕 U+F0115
    'cash-multiple': 983318,  # 󰄖 U+F0116
    'checkbox-blank-badge-outline': 983319,  # 󰄗 U+F0117
    'cast': 983320,  # 󰄘 U+F0118
    'cast-connected': 983321,  # 󰄙 U+F0119
    'castle': 983322,  # 󰄚 U+F011A
    'cat#1': 983323,  # 󰄛 U+F011B
    'cellphone': 983324,  # 󰄜 U+F011C
    'tray-arrow-up': 983325,  # 󰄝 U+F011D
    'cellphone-basic': 983326,  # 󰄞 U+F011E
    'cellphone-dock': 983327,  # 󰄟 U+F011F
    'tray-arrow-down': 983328,  # 󰄠 U+F0120
    'cellphone-link': 983329,  # 󰄡 U+F0121
    'cellphone-link-off': 983330,  # 󰄢 U+F0122
    'cellphone-settings': 983331,  # 󰄣 U+F0123
    'certificate#1': 983332,  # 󰄤 U+F0124
    'chair-school': 983333,  # 󰄥 U+F0125
    'chart-arc': 983334,  # 󰄦 U+F0126
    'chart-areaspline': 983335,  # 󰄧 U+F0127
    'chart-bar': 983336,  # 󰄨 U+F0128
    'chart-histogram': 983337,  # 󰄩 U+F0129
    'chart-line': 983338,  # 󰄪 U+F012A
    'chart-pie': 983339,  # 󰄫 U+F012B
    'check#3': 983340,  # 󰄬 U+F012C
    'check-all#1': 983341,  # 󰄭 U+F012D
    'checkbox-blank': 983342,  # 󰄮 U+F012E
    'checkbox-blank-circle': 983343,  # 󰄯 U+F012F
    'checkbox-blank-circle-outline': 983344,  # 󰄰 U+F0130
    'checkbox-blank-outline': 983345,  # 󰄱 U+F0131
    'checkbox-marked': 983346,  # 󰄲 U+F0132
    'checkbox-marked-circle': 983347,  # 󰄳 U+F0133
    'checkbox-marked-circle-outline': 983348,  # 󰄴 U+F0134
    'checkbox-marked-outline': 983349,  # 󰄵 U+F0135
    'checkbox-multiple-blank': 983350,  # 󰄶 U+F0136
    'checkbox-multiple-blank-outline': 983351,  # 󰄷 U+F0137
    'checkbox-multiple-marked': 983352,  # 󰄸 U+F0138
    'checkbox-multiple-marked-outline': 983353,  # 󰄹 U+F0139
    'checkerboard': 983354,  # 󰄺 U+F013A
    'chemical-weapon': 983355,  # 󰄻 U+F013B
    'chevron-double-down': 983356,  # 󰄼 U+F013C
    'chevron-double-left': 983357,  # 󰄽 U+F013D
    'chevron-double-right': 983358,  # 󰄾 U+F013E
    'chevron-double-up': 983359,  # 󰄿 U+F013F
    'chevron-down#2': 983360,  # 󰅀 U+F0140
    'chevron-left#2': 983361,  # 󰅁 U+F0141
    'chevron-right#2': 983362,  # 󰅂 U+F0142
    'chevron-up#2': 983363,  # 󰅃 U+F0143
    'church#1': 983364,  # 󰅄 U+F0144
    'roller-skate-off': 983365,  # 󰅅 U+F0145
    'city#1': 983366,  # 󰅆 U+F0146
    'clipboard': 983367,  # 󰅇 U+F0147
    'clipboard-account': 983368,  # 󰅈 U+F0148
    'clipboard-alert': 983369,  # 󰅉 U+F0149
    'clipboard-arrow-down': 983370,  # 󰅊 U+F014A
    'clipboard-arrow-left': 983371,  # 󰅋 U+F014B
    'clipboard-outline': 983372,  # 󰅌 U+F014C
    'clipboard-text': 983373,  # 󰅍 U+F014D
    'clipboard-check': 983374,  # 󰅎 U+F014E
    'clippy#1': 983375,  # 󰅏 U+F014F
    'clock-outline': 983376,  # 󰅐 U+F0150
    'clock-end': 983377,  # 󰅑 U+F0151
    'clock-fast': 983378,  # 󰅒 U+F0152
    'clock-in': 983379,  # 󰅓 U+F0153
    'clock-out': 983380,  # 󰅔 U+F0154
    'clock-start': 983381,  # 󰅕 U+F0155
    'close#1': 983382,  # 󰅖 U+F0156
    'close-box': 983383,  # 󰅗 U+F0157
    'close-box-outline': 983384,  # 󰅘 U+F0158
    'close-circle': 983385,  # 󰅙 U+F0159
    'close-circle-outline': 983386,  # 󰅚 U+F015A
    'close-network': 983387,  # 󰅛 U+F015B
    'close-octagon': 983388,  # 󰅜 U+F015C
    'close-octagon-outline': 983389,  # 󰅝 U+F015D
    'closed-caption': 983390,  # 󰅞 U+F015E
    'cloud#4': 983391,  # 󰅟 U+F015F
    'cloud-check': 983392,  # 󰅠 U+F0160
    'cloud-circle': 983393,  # 󰅡 U+F0161
    'cloud-download#1': 983394,  # 󰅢 U+F0162
    'cloud-outline': 983395,  # 󰅣 U+F0163
    'cloud-off-outline': 983396,  # 󰅤 U+F0164
    'cloud-print': 983397,  # 󰅥 U+F0165
    'cloud-print-outline': 983398,  # 󰅦 U+F0166
    'cloud-upload#1': 983399,  # 󰅧 U+F0167
    'code-array': 983400,  # 󰅨 U+F0168
    'code-braces': 983401,  # 󰅩 U+F0169
    'code-brackets': 983402,  # 󰅪 U+F016A
    'code-equal': 983403,  # 󰅫 U+F016B
    'code-greater-than': 983404,  # 󰅬 U+F016C
    'code-greater-than-or-equal': 983405,  # 󰅭 U+F016D
    'code-less-than': 983406,  # 󰅮 U+F016E
    'code-less-than-or-equal': 983407,  # 󰅯 U+F016F
    'code-not-equal': 983408,  # 󰅰 U+F0170
    'code-not-equal-variant': 983409,  # 󰅱 U+F0171
    'code-parentheses': 983410,  # 󰅲 U+F0172
    'code-string': 983411,  # 󰅳 U+F0173
    'code-tags': 983412,  # 󰅴 U+F0174
    'codepen#1': 983413,  # 󰅵 U+F0175
    'coffee#1': 983414,  # 󰅶 U+F0176
    'coffee-to-go': 983415,  # 󰅷 U+F0177
    'bell-badge-outline': 983416,  # 󰅸 U+F0178
    'color-helper': 983417,  # 󰅹 U+F0179
    'comment#3': 983418,  # 󰅺 U+F017A
    'comment-account': 983419,  # 󰅻 U+F017B
    'comment-account-outline': 983420,  # 󰅼 U+F017C
    'comment-alert': 983421,  # 󰅽 U+F017D
    'comment-alert-outline': 983422,  # 󰅾 U+F017E
    'comment-check': 983423,  # 󰅿 U+F017F
    'comment-check-outline': 983424,  # 󰆀 U+F0180
    'comment-multiple-outline': 983425,  # 󰆁 U+F0181
    'comment-outline': 983426,  # 󰆂 U+F0182
    'comment-plus-outline': 983427,  # 󰆃 U+F0183
    'comment-processing': 983428,  # 󰆄 U+F0184
    'comment-processing-outline': 983429,  # 󰆅 U+F0185
    'comment-question-outline': 983430,  # 󰆆 U+F0186
    'comment-remove-outline': 983431,  # 󰆇 U+F0187
    'comment-text': 983432,  # 󰆈 U+F0188
    'comment-text-outline': 983433,  # 󰆉 U+F0189
    'compare': 983434,  # 󰆊 U+F018A
    'compass#2': 983435,  # 󰆋 U+F018B
    'compass-outline': 983436,  # 󰆌 U+F018C
    'console': 983437,  # 󰆍 U+F018D
    'card-account-mail': 983438,  # 󰆎 U+F018E
    'content-copy': 983439,  # 󰆏 U+F018F
    'content-cut': 983440,  # 󰆐 U+F0190
    'content-duplicate': 983441,  # 󰆑 U+F0191
    'content-paste': 983442,  # 󰆒 U+F0192
    'content-save': 983443,  # 󰆓 U+F0193
    'content-save-all': 983444,  # 󰆔 U+F0194
    'contrast': 983445,  # 󰆕 U+F0195
    'contrast-box': 983446,  # 󰆖 U+F0196
    'contrast-circle': 983447,  # 󰆗 U+F0197
    'cookie#1': 983448,  # 󰆘 U+F0198
    'counter': 983449,  # 󰆙 U+F0199
    'cow#1': 983450,  # 󰆚 U+F019A
    'credit-card-outline': 983451,  # 󰆛 U+F019B
    'credit-card-multiple-outline': 983452,  # 󰆜 U+F019C
    'credit-card-scan-outline': 983453,  # 󰆝 U+F019D
    'crop#1': 983454,  # 󰆞 U+F019E
    'crop-free': 983455,  # 󰆟 U+F019F
    'crop-landscape': 983456,  # 󰆠 U+F01A0
    'crop-portrait': 983457,  # 󰆡 U+F01A1
    'crop-square': 983458,  # 󰆢 U+F01A2
    'crosshairs#1': 983459,  # 󰆣 U+F01A3
    'crosshairs-gps': 983460,  # 󰆤 U+F01A4
    'crown#2': 983461,  # 󰆥 U+F01A5
    'cube#1': 983462,  # 󰆦 U+F01A6
    'cube-outline': 983463,  # 󰆧 U+F01A7
    'cube-send': 983464,  # 󰆨 U+F01A8
    'cube-unfolded': 983465,  # 󰆩 U+F01A9
    'cup': 983466,  # 󰆪 U+F01AA
    'cup-water': 983467,  # 󰆫 U+F01AB
    'currency-btc': 983468,  # 󰆬 U+F01AC
    'currency-eur': 983469,  # 󰆭 U+F01AD
    'currency-gbp': 983470,  # 󰆮 U+F01AE
    'currency-inr': 983471,  # 󰆯 U+F01AF
    'currency-ngn': 983472,  # 󰆰 U+F01B0
    'currency-rub': 983473,  # 󰆱 U+F01B1
    'currency-try': 983474,  # 󰆲 U+F01B2
    'delete-variant': 983475,  # 󰆳 U+F01B3
    'delete': 983476,  # 󰆴 U+F01B4
    'decimal-increase': 983477,  # 󰆵 U+F01B5
    'decimal-decrease': 983478,  # 󰆶 U+F01B6
    'debug-step-over#1': 983479,  # 󰆷 U+F01B7
    'debug-step-out#1': 983480,  # 󰆸 U+F01B8
    'debug-step-into#1': 983481,  # 󰆹 U+F01B9
    'database-plus': 983482,  # 󰆺 U+F01BA
    'database-minus': 983483,  # 󰆻 U+F01BB
    'database#3': 983484,  # 󰆼 U+F01BC
    'cursor-pointer': 983485,  # 󰆽 U+F01BD
    'cursor-move': 983486,  # 󰆾 U+F01BE
    'cursor-default-outline': 983487,  # 󰆿 U+F01BF
    'cursor-default': 983488,  # 󰇀 U+F01C0
    'currency-usd': 983489,  # 󰇁 U+F01C1
    'delta': 983490,  # 󰇂 U+F01C2
    'deskphone': 983491,  # 󰇃 U+F01C3
    'desktop-mac': 983492,  # 󰇄 U+F01C4
    'desktop-tower': 983493,  # 󰇅 U+F01C5
    'details': 983494,  # 󰇆 U+F01C6
    'deviantart#1': 983495,  # 󰇇 U+F01C7
    'diamond-stone': 983496,  # 󰇈 U+F01C8
    'ab-testing': 983497,  # 󰇉 U+F01C9
    'dice-1': 983498,  # 󰇊 U+F01CA
    'dice-2': 983499,  # 󰇋 U+F01CB
    'dice-3': 983500,  # 󰇌 U+F01CC
    'dice-4': 983501,  # 󰇍 U+F01CD
    'dice-5': 983502,  # 󰇎 U+F01CE
    'dice-6': 983503,  # 󰇏 U+F01CF
    'directions': 983504,  # 󰇐 U+F01D0
    'disc-alert': 983505,  # 󰇑 U+F01D1
    'disqus': 983506,  # 󰇒 U+F01D2
    'video-plus-outline': 983507,  # 󰇓 U+F01D3
    'division': 983508,  # 󰇔 U+F01D4
    'division-box': 983509,  # 󰇕 U+F01D5
    'dns': 983510,  # 󰇖 U+F01D6
    'domain': 983511,  # 󰇗 U+F01D7
    'dots-horizontal': 983512,  # 󰇘 U+F01D8
    'dots-vertical': 983513,  # 󰇙 U+F01D9
    'download#2': 983514,  # 󰇚 U+F01DA
    'drag': 983515,  # 󰇛 U+F01DB
    'drag-horizontal': 983516,  # 󰇜 U+F01DC
    'drag-vertical': 983517,  # 󰇝 U+F01DD
    'drawing': 983518,  # 󰇞 U+F01DE
    'drawing-box': 983519,  # 󰇟 U+F01DF
    'shield-refresh-outline': 983520,  # 󰇠 U+F01E0
    'calendar-refresh': 983521,  # 󰇡 U+F01E1
    'drone': 983522,  # 󰇢 U+F01E2
    'dropbox#1': 983523,  # 󰇣 U+F01E3
    'drupal#1': 983524,  # 󰇤 U+F01E4
    'duck': 983525,  # 󰇥 U+F01E5
    'dumbbell#1': 983526,  # 󰇦 U+F01E6
    'earth': 983527,  # 󰇧 U+F01E7
    'earth-off': 983528,  # 󰇨 U+F01E8
    'microsoft-edge': 983529,  # 󰇩 U+F01E9
    'eject#1': 983530,  # 󰇪 U+F01EA
    'elevation-decline': 983531,  # 󰇫 U+F01EB
    'elevation-rise': 983532,  # 󰇬 U+F01EC
    'elevator': 983533,  # 󰇭 U+F01ED
    'email': 983534,  # 󰇮 U+F01EE
    'email-open': 983535,  # 󰇯 U+F01EF
    'email-outline': 983536,  # 󰇰 U+F01F0
    'email-lock': 983537,  # 󰇱 U+F01F1
    'emoticon-outline': 983538,  # 󰇲 U+F01F2
    'emoticon-cool-outline': 983539,  # 󰇳 U+F01F3
    'emoticon-devil-outline': 983540,  # 󰇴 U+F01F4
    'emoticon-happy-outline': 983541,  # 󰇵 U+F01F5
    'emoticon-neutral-outline': 983542,  # 󰇶 U+F01F6
    'emoticon-poop': 983543,  # 󰇷 U+F01F7
    'emoticon-sad-outline': 983544,  # 󰇸 U+F01F8
    'emoticon-tongue': 983545,  # 󰇹 U+F01F9
    'engine': 983546,  # 󰇺 U+F01FA
    'engine-outline': 983547,  # 󰇻 U+F01FB
    'equal#1': 983548,  # 󰇼 U+F01FC
    'equal-box': 983549,  # 󰇽 U+F01FD
    'eraser#1': 983550,  # 󰇾 U+F01FE
    'escalator': 983551,  # 󰇿 U+F01FF
    'ethernet#1': 983552,  # 󰈀 U+F0200
    'ethernet-cable': 983553,  # 󰈁 U+F0201
    'ethernet-cable-off': 983554,  # 󰈂 U+F0202
    'calendar-refresh-outline': 983555,  # 󰈃 U+F0203
    'evernote#1': 983556,  # 󰈄 U+F0204
    'exclamation#1': 983557,  # 󰈅 U+F0205
    'exit-to-app': 983558,  # 󰈆 U+F0206
    'export#1': 983559,  # 󰈇 U+F0207
    'eye#3': 983560,  # 󰈈 U+F0208
    'eye-off': 983561,  # 󰈉 U+F0209
    'eyedropper': 983562,  # 󰈊 U+F020A
    'eyedropper-variant': 983563,  # 󰈋 U+F020B
    'facebook#1': 983564,  # 󰈌 U+F020C
    'order-alphabetical-ascending': 983565,  # 󰈍 U+F020D
    'facebook-messenger': 983566,  # 󰈎 U+F020E
    'factory': 983567,  # 󰈏 U+F020F
    'fan#1': 983568,  # 󰈐 U+F0210
    'fast-forward': 983569,  # 󰈑 U+F0211
    'fax#1': 983570,  # 󰈒 U+F0212
    'ferry': 983571,  # 󰈓 U+F0213
    'file#3': 983572,  # 󰈔 U+F0214
    'file-chart': 983573,  # 󰈕 U+F0215
    'file-check': 983574,  # 󰈖 U+F0216
    'file-cloud': 983575,  # 󰈗 U+F0217
    'file-delimited': 983576,  # 󰈘 U+F0218
    'file-document': 983577,  # 󰈙 U+F0219
    'text-box': 983578,  # 󰈚 U+F021A
    'file-excel': 983579,  # 󰈛 U+F021B
    'file-excel-box': 983580,  # 󰈜 U+F021C
    'file-export#1': 983581,  # 󰈝 U+F021D
    'file-find': 983582,  # 󰈞 U+F021E
    'file-image': 983583,  # 󰈟 U+F021F
    'file-import#1': 983584,  # 󰈠 U+F0220
    'file-lock': 983585,  # 󰈡 U+F0221
    'file-multiple': 983586,  # 󰈢 U+F0222
    'file-music': 983587,  # 󰈣 U+F0223
    'file-outline': 983588,  # 󰈤 U+F0224
    'file-jpg-box': 983589,  # 󰈥 U+F0225
    'file-pdf-box': 983590,  # 󰈦 U+F0226
    'file-powerpoint': 983591,  # 󰈧 U+F0227
    'file-powerpoint-box': 983592,  # 󰈨 U+F0228
    'file-presentation-box': 983593,  # 󰈩 U+F0229
    'file-send': 983594,  # 󰈪 U+F022A
    'file-video': 983595,  # 󰈫 U+F022B
    'file-word': 983596,  # 󰈬 U+F022C
    'file-word-box': 983597,  # 󰈭 U+F022D
    'file-code#3': 983598,  # 󰈮 U+F022E
    'film#1': 983599,  # 󰈯 U+F022F
    'filmstrip': 983600,  # 󰈰 U+F0230
    'filmstrip-off': 983601,  # 󰈱 U+F0231
    'filter#3': 983602,  # 󰈲 U+F0232
    'filter-outline': 983603,  # 󰈳 U+F0233
    'filter-remove': 983604,  # 󰈴 U+F0234
    'filter-remove-outline': 983605,  # 󰈵 U+F0235
    'filter-variant': 983606,  # 󰈶 U+F0236
    'fingerprint#2': 983607,  # 󰈷 U+F0237
    'fire#1': 983608,  # 󰈸 U+F0238
    'firefox#1': 983609,  # 󰈹 U+F0239
    'fish#1': 983610,  # 󰈺 U+F023A
    'flag#1': 983611,  # 󰈻 U+F023B
    'flag-checkered': 983612,  # 󰈼 U+F023C
    'flag-outline': 983613,  # 󰈽 U+F023D
    'flag-variant-outline': 983614,  # 󰈾 U+F023E
    'flag-triangle': 983615,  # 󰈿 U+F023F
    'flag-variant': 983616,  # 󰉀 U+F0240
    'flash': 983617,  # 󰉁 U+F0241
    'flash-auto': 983618,  # 󰉂 U+F0242
    'flash-off': 983619,  # 󰉃 U+F0243
    'flashlight': 983620,  # 󰉄 U+F0244
    'flashlight-off': 983621,  # 󰉅 U+F0245
    'star-half#1': 983622,  # 󰉆 U+F0246
    'flip-to-back': 983623,  # 󰉇 U+F0247
    'flip-to-front': 983624,  # 󰉈 U+F0248
    'floppy#1': 983625,  # 󰉉 U+F0249
    'flower': 983626,  # 󰉊 U+F024A
    'folder#2': 983627,  # 󰉋 U+F024B
    'folder-account': 983628,  # 󰉌 U+F024C
    'folder-download': 983629,  # 󰉍 U+F024D
    'folder-google-drive': 983630,  # 󰉎 U+F024E
    'folder-image': 983631,  # 󰉏 U+F024F
    'folder-lock': 983632,  # 󰉐 U+F0250
    'folder-lock-open': 983633,  # 󰉑 U+F0251
    'folder-move': 983634,  # 󰉒 U+F0252
    'folder-multiple': 983635,  # 󰉓 U+F0253
    'folder-multiple-image': 983636,  # 󰉔 U+F0254
    'folder-multiple-outline': 983637,  # 󰉕 U+F0255
    'folder-outline': 983638,  # 󰉖 U+F0256
    'folder-plus': 983639,  # 󰉗 U+F0257
    'folder-remove': 983640,  # 󰉘 U+F0258
    'folder-upload': 983641,  # 󰉙 U+F0259
    'food': 983642,  # 󰉚 U+F025A
    'food-apple': 983643,  # 󰉛 U+F025B
    'food-variant': 983644,  # 󰉜 U+F025C
    'football#1': 983645,  # 󰉝 U+F025D
    'football-australian': 983646,  # 󰉞 U+F025E
    'football-helmet': 983647,  # 󰉟 U+F025F
    'format-align-center': 983648,  # 󰉠 U+F0260
    'format-align-justify': 983649,  # 󰉡 U+F0261
    'format-align-left': 983650,  # 󰉢 U+F0262
    'format-align-right': 983651,  # 󰉣 U+F0263
    'format-bold': 983652,  # 󰉤 U+F0264
    'format-clear': 983653,  # 󰉥 U+F0265
    'format-color-fill': 983654,  # 󰉦 U+F0266
    'format-float-center': 983655,  # 󰉧 U+F0267
    'format-float-left': 983656,  # 󰉨 U+F0268
    'format-float-none': 983657,  # 󰉩 U+F0269
    'format-float-right': 983658,  # 󰉪 U+F026A
    'format-header-1': 983659,  # 󰉫 U+F026B
    'format-header-2': 983660,  # 󰉬 U+F026C
    'format-header-3': 983661,  # 󰉭 U+F026D
    'format-header-4': 983662,  # 󰉮 U+F026E
    'format-header-5': 983663,  # 󰉯 U+F026F
    'format-header-6': 983664,  # 󰉰 U+F0270
    'format-header-decrease': 983665,  # 󰉱 U+F0271
    'format-header-equal': 983666,  # 󰉲 U+F0272
    'format-header-increase': 983667,  # 󰉳 U+F0273
    'format-header-pound': 983668,  # 󰉴 U+F0274
    'format-indent-decrease': 983669,  # 󰉵 U+F0275
    'format-indent-increase': 983670,  # 󰉶 U+F0276
    'format-italic': 983671,  # 󰉷 U+F0277
    'format-line-spacing': 983672,  # 󰉸 U+F0278
    'format-list-bulleted': 983673,  # 󰉹 U+F0279
    'format-list-bulleted-type': 983674,  # 󰉺 U+F027A
    'format-list-numbered': 983675,  # 󰉻 U+F027B
    'format-paint': 983676,  # 󰉼 U+F027C
    'format-paragraph': 983677,  # 󰉽 U+F027D
    'format-quote-close': 983678,  # 󰉾 U+F027E
    'format-size': 983679,  # 󰉿 U+F027F
    'format-strikethrough': 983680,  # 󰊀 U+F0280
    'format-strikethrough-variant': 983681,  # 󰊁 U+F0281
    'format-subscript': 983682,  # 󰊂 U+F0282
    'format-superscript': 983683,  # 󰊃 U+F0283
    'format-text': 983684,  # 󰊄 U+F0284
    'format-textdirection-l-to-r': 983685,  # 󰊅 U+F0285
    'format-textdirection-r-to-l': 983686,  # 󰊆 U+F0286
    'format-underline': 983687,  # 󰊇 U+F0287
    'format-wrap-inline': 983688,  # 󰊈 U+F0288
    'format-wrap-square': 983689,  # 󰊉 U+F0289
    'format-wrap-tight': 983690,  # 󰊊 U+F028A
    'format-wrap-top-bottom': 983691,  # 󰊋 U+F028B
    'forum': 983692,  # 󰊌 U+F028C
    'forward#1': 983693,  # 󰊍 U+F028D
    'bowl': 983694,  # 󰊎 U+F028E
    'fridge-outline': 983695,  # 󰊏 U+F028F
    'fridge': 983696,  # 󰊐 U+F0290
    'fridge-top': 983697,  # 󰊑 U+F0291
    'fridge-bottom': 983698,  # 󰊒 U+F0292
    'fullscreen': 983699,  # 󰊓 U+F0293
    'fullscreen-exit': 983700,  # 󰊔 U+F0294
    'function': 983701,  # 󰊕 U+F0295
    'gamepad#1': 983702,  # 󰊖 U+F0296
    'gamepad-variant': 983703,  # 󰊗 U+F0297
    'gas-station': 983704,  # 󰊘 U+F0298
    'gate': 983705,  # 󰊙 U+F0299
    'gauge#1': 983706,  # 󰊚 U+F029A
    'gavel#1': 983707,  # 󰊛 U+F029B
    'gender-female': 983708,  # 󰊜 U+F029C
    'gender-male': 983709,  # 󰊝 U+F029D
    'gender-male-female': 983710,  # 󰊞 U+F029E
    'gender-transgender': 983711,  # 󰊟 U+F029F
    'ghost#1': 983712,  # 󰊠 U+F02A0
    'gift-outline': 983713,  # 󰊡 U+F02A1
    'git#1': 983714,  # 󰊢 U+F02A2
    'card-account-details-star': 983715,  # 󰊣 U+F02A3
    'github#2': 983716,  # 󰊤 U+F02A4
    'glass-flute': 983717,  # 󰊥 U+F02A5
    'glass-mug': 983718,  # 󰊦 U+F02A6
    'glass-stange': 983719,  # 󰊧 U+F02A7
    'glass-tulip': 983720,  # 󰊨 U+F02A8
    'bowl-outline': 983721,  # 󰊩 U+F02A9
    'glasses#1': 983722,  # 󰊪 U+F02AA
    'gmail': 983723,  # 󰊫 U+F02AB
    'gnome': 983724,  # 󰊬 U+F02AC
    'google#1': 983725,  # 󰊭 U+F02AD
    'google-cardboard': 983726,  # 󰊮 U+F02AE
    'google-chrome': 983727,  # 󰊯 U+F02AF
    'google-circles': 983728,  # 󰊰 U+F02B0
    'google-circles-communities': 983729,  # 󰊱 U+F02B1
    'google-circles-extended': 983730,  # 󰊲 U+F02B2
    'google-circles-group': 983731,  # 󰊳 U+F02B3
    'google-controller': 983732,  # 󰊴 U+F02B4
    'google-controller-off': 983733,  # 󰊵 U+F02B5
    'google-drive#1': 983734,  # 󰊶 U+F02B6
    'google-earth': 983735,  # 󰊷 U+F02B7
    'google-glass': 983736,  # 󰊸 U+F02B8
    'google-nearby': 983737,  # 󰊹 U+F02B9
    'video-minus-outline': 983738,  # 󰊺 U+F02BA
    'microsoft-teams': 983739,  # 󰊻 U+F02BB
    'google-play#1': 983740,  # 󰊼 U+F02BC
    'google-plus': 983741,  # 󰊽 U+F02BD
    'order-bool-ascending': 983742,  # 󰊾 U+F02BE
    'google-translate': 983743,  # 󰊿 U+F02BF
    'google-classroom': 983744,  # 󰋀 U+F02C0
    'grid': 983745,  # 󰋁 U+F02C1
    'grid-off': 983746,  # 󰋂 U+F02C2
    'group': 983747,  # 󰋃 U+F02C3
    'guitar-electric': 983748,  # 󰋄 U+F02C4
    'guitar-pick': 983749,  # 󰋅 U+F02C5
    'guitar-pick-outline': 983750,  # 󰋆 U+F02C6
    'hand-pointing-right': 983751,  # 󰋇 U+F02C7
    'hanger': 983752,  # 󰋈 U+F02C8
    'google-hangouts': 983753,  # 󰋉 U+F02C9
    'harddisk': 983754,  # 󰋊 U+F02CA
    'headphones#1': 983755,  # 󰋋 U+F02CB
    'headphones-box': 983756,  # 󰋌 U+F02CC
    'headphones-settings': 983757,  # 󰋍 U+F02CD
    'headset#1': 983758,  # 󰋎 U+F02CE
    'headset-dock': 983759,  # 󰋏 U+F02CF
    'headset-off': 983760,  # 󰋐 U+F02D0
    'heart#3': 983761,  # 󰋑 U+F02D1
    'heart-box': 983762,  # 󰋒 U+F02D2
    'heart-box-outline': 983763,  # 󰋓 U+F02D3
    'heart-broken': 983764,  # 󰋔 U+F02D4
    'heart-outline': 983765,  # 󰋕 U+F02D5
    'help': 983766,  # 󰋖 U+F02D6
    'help-circle': 983767,  # 󰋗 U+F02D7
    'hexagon#1': 983768,  # 󰋘 U+F02D8
    'hexagon-outline': 983769,  # 󰋙 U+F02D9
    'history#2': 983770,  # 󰋚 U+F02DA
    'hololens': 983771,  # 󰋛 U+F02DB
    'home#2': 983772,  # 󰋜 U+F02DC
    'home-modern': 983773,  # 󰋝 U+F02DD
    'home-variant': 983774,  # 󰋞 U+F02DE
    'hops': 983775,  # 󰋟 U+F02DF
    'hospital-box': 983776,  # 󰋠 U+F02E0
    'hospital-building': 983777,  # 󰋡 U+F02E1
    'hospital-marker': 983778,  # 󰋢 U+F02E2
    'bed#2': 983779,  # 󰋣 U+F02E3
    'bowl-mix-outline': 983780,  # 󰋤 U+F02E4
    'pot': 983781,  # 󰋥 U+F02E5
    'human': 983782,  # 󰋦 U+F02E6
    'human-child': 983783,  # 󰋧 U+F02E7
    'human-male-female': 983784,  # 󰋨 U+F02E8
    'image#2': 983785,  # 󰋩 U+F02E9
    'image-album': 983786,  # 󰋪 U+F02EA
    'image-area': 983787,  # 󰋫 U+F02EB
    'image-area-close': 983788,  # 󰋬 U+F02EC
    'image-broken': 983789,  # 󰋭 U+F02ED
    'image-broken-variant': 983790,  # 󰋮 U+F02EE
    'image-multiple-outline': 983791,  # 󰋯 U+F02EF
    'image-filter-black-white': 983792,  # 󰋰 U+F02F0
    'image-filter-center-focus': 983793,  # 󰋱 U+F02F1
    'image-filter-center-focus-weak': 983794,  # 󰋲 U+F02F2
    'image-filter-drama': 983795,  # 󰋳 U+F02F3
    'image-filter-frames': 983796,  # 󰋴 U+F02F4
    'image-filter-hdr': 983797,  # 󰋵 U+F02F5
    'image-filter-none': 983798,  # 󰋶 U+F02F6
    'image-filter-tilt-shift': 983799,  # 󰋷 U+F02F7
    'image-filter-vintage': 983800,  # 󰋸 U+F02F8
    'image-multiple': 983801,  # 󰋹 U+F02F9
    'import': 983802,  # 󰋺 U+F02FA
    'inbox-arrow-down': 983803,  # 󰋻 U+F02FB
    'information': 983804,  # 󰋼 U+F02FC
    'information-outline': 983805,  # 󰋽 U+F02FD
    'instagram#1': 983806,  # 󰋾 U+F02FE
    'pot-outline': 983807,  # 󰋿 U+F02FF
    'microsoft-internet-explorer': 983808,  # 󰌀 U+F0300
    'invert-colors': 983809,  # 󰌁 U+F0301
    'jeepney': 983810,  # 󰌂 U+F0302
    'jira#1': 983811,  # 󰌃 U+F0303
    'jsfiddle#1': 983812,  # 󰌄 U+F0304
    'keg': 983813,  # 󰌅 U+F0305
    'key#3': 983814,  # 󰌆 U+F0306
    'key-change': 983815,  # 󰌇 U+F0307
    'key-minus': 983816,  # 󰌈 U+F0308
    'key-plus': 983817,  # 󰌉 U+F0309
    'key-remove': 983818,  # 󰌊 U+F030A
    'key-variant': 983819,  # 󰌋 U+F030B
    'keyboard#1': 983820,  # 󰌌 U+F030C
    'keyboard-backspace': 983821,  # 󰌍 U+F030D
    'keyboard-caps': 983822,  # 󰌎 U+F030E
    'keyboard-close': 983823,  # 󰌏 U+F030F
    'keyboard-off': 983824,  # 󰌐 U+F0310
    'keyboard-return': 983825,  # 󰌑 U+F0311
    'keyboard-tab': 983826,  # 󰌒 U+F0312
    'keyboard-variant': 983827,  # 󰌓 U+F0313
    'kodi': 983828,  # 󰌔 U+F0314
    'label': 983829,  # 󰌕 U+F0315
    'label-outline': 983830,  # 󰌖 U+F0316
    'lan': 983831,  # 󰌗 U+F0317
    'lan-connect': 983832,  # 󰌘 U+F0318
    'lan-disconnect': 983833,  # 󰌙 U+F0319
    'lan-pending': 983834,  # 󰌚 U+F031A
    'language-csharp': 983835,  # 󰌛 U+F031B
    'language-css3': 983836,  # 󰌜 U+F031C
    'language-html5': 983837,  # 󰌝 U+F031D
    'language-javascript': 983838,  # 󰌞 U+F031E
    'language-php': 983839,  # 󰌟 U+F031F
    'language-python': 983840,  # 󰌠 U+F0320
    'contactless-payment-circle': 983841,  # 󰌡 U+F0321
    'laptop#1': 983842,  # 󰌢 U+F0322
    'magazine-rifle': 983843,  # 󰌣 U+F0323
    'magazine-pistol': 983844,  # 󰌤 U+F0324
    'keyboard-tab-reverse': 983845,  # 󰌥 U+F0325
    'pot-steam-outline': 983846,  # 󰌦 U+F0326
    'launch': 983847,  # 󰌧 U+F0327
    'layers#2': 983848,  # 󰌨 U+F0328
    'layers-off': 983849,  # 󰌩 U+F0329
    'leaf#1': 983850,  # 󰌪 U+F032A
    'led-off': 983851,  # 󰌫 U+F032B
    'led-on': 983852,  # 󰌬 U+F032C
    'led-outline': 983853,  # 󰌭 U+F032D
    'led-variant-off': 983854,  # 󰌮 U+F032E
    'led-variant-on': 983855,  # 󰌯 U+F032F
    'led-variant-outline': 983856,  # 󰌰 U+F0330
    'library#1': 983857,  # 󰌱 U+F0331
    'filmstrip-box': 983858,  # 󰌲 U+F0332
    'music-box-multiple': 983859,  # 󰌳 U+F0333
    'plus-box-multiple': 983860,  # 󰌴 U+F0334
    'lightbulb#2': 983861,  # 󰌵 U+F0335
    'lightbulb-outline': 983862,  # 󰌶 U+F0336
    'link#3': 983863,  # 󰌷 U+F0337
    'link-off': 983864,  # 󰌸 U+F0338
    'link-variant': 983865,  # 󰌹 U+F0339
    'link-variant-off': 983866,  # 󰌺 U+F033A
    'linkedin': 983867,  # 󰌻 U+F033B
    'sort-reverse-variant': 983868,  # 󰌼 U+F033C
    'linux#1': 983869,  # 󰌽 U+F033D
    'lock#3': 983870,  # 󰌾 U+F033E
    'lock-open': 983871,  # 󰌿 U+F033F
    'lock-open-outline': 983872,  # 󰍀 U+F0340
    'lock-outline': 983873,  # 󰍁 U+F0341
    'login': 983874,  # 󰍂 U+F0342
    'logout': 983875,  # 󰍃 U+F0343
    'looks': 983876,  # 󰍄 U+F0344
    'loupe': 983877,  # 󰍅 U+F0345
    'lumx': 983878,  # 󰍆 U+F0346
    'magnet#2': 983879,  # 󰍇 U+F0347
    'magnet-on': 983880,  # 󰍈 U+F0348
    'magnify': 983881,  # 󰍉 U+F0349
    'magnify-minus': 983882,  # 󰍊 U+F034A
    'magnify-plus': 983883,  # 󰍋 U+F034B
    'plus-circle-multiple': 983884,  # 󰍌 U+F034C
    'map#2': 983885,  # 󰍍 U+F034D
    'map-marker': 983886,  # 󰍎 U+F034E
    'map-marker-circle': 983887,  # 󰍏 U+F034F
    'map-marker-multiple': 983888,  # 󰍐 U+F0350
    'map-marker-off': 983889,  # 󰍑 U+F0351
    'map-marker-radius': 983890,  # 󰍒 U+F0352
    'margin': 983891,  # 󰍓 U+F0353
    'language-markdown': 983892,  # 󰍔 U+F0354
    'marker-check': 983893,  # 󰍕 U+F0355
    'glass-cocktail': 983894,  # 󰍖 U+F0356
    'material-ui': 983895,  # 󰍗 U+F0357
    'math-compass': 983896,  # 󰍘 U+F0358
    'stackpath#1': 983897,  # 󰍙 U+F0359
    'minus-circle-multiple': 983898,  # 󰍚 U+F035A
    'memory#1': 983899,  # 󰍛 U+F035B
    'menu#1': 983900,  # 󰍜 U+F035C
    'menu-down': 983901,  # 󰍝 U+F035D
    'menu-left': 983902,  # 󰍞 U+F035E
    'menu-right': 983903,  # 󰍟 U+F035F
    'menu-up': 983904,  # 󰍠 U+F0360
    'message#1': 983905,  # 󰍡 U+F0361
    'message-alert': 983906,  # 󰍢 U+F0362
    'message-draw': 983907,  # 󰍣 U+F0363
    'message-image': 983908,  # 󰍤 U+F0364
    'message-outline': 983909,  # 󰍥 U+F0365
    'message-processing': 983910,  # 󰍦 U+F0366
    'message-reply': 983911,  # 󰍧 U+F0367
    'message-reply-text': 983912,  # 󰍨 U+F0368
    'message-text': 983913,  # 󰍩 U+F0369
    'message-text-outline': 983914,  # 󰍪 U+F036A
    'message-video': 983915,  # 󰍫 U+F036B
    'microphone#1': 983916,  # 󰍬 U+F036C
    'microphone-off': 983917,  # 󰍭 U+F036D
    'microphone-outline': 983918,  # 󰍮 U+F036E
    'microphone-settings': 983919,  # 󰍯 U+F036F
    'microphone-variant': 983920,  # 󰍰 U+F0370
    'microphone-variant-off': 983921,  # 󰍱 U+F0371
    'microsoft#1': 983922,  # 󰍲 U+F0372
    'minecraft': 983923,  # 󰍳 U+F0373
    'minus#1': 983924,  # 󰍴 U+F0374
    'minus-box': 983925,  # 󰍵 U+F0375
    'minus-circle': 983926,  # 󰍶 U+F0376
    'minus-circle-outline': 983927,  # 󰍷 U+F0377
    'minus-network': 983928,  # 󰍸 U+F0378
    'monitor': 983929,  # 󰍹 U+F0379
    'monitor-multiple': 983930,  # 󰍺 U+F037A
    'more': 983931,  # 󰍻 U+F037B
    'motorbike': 983932,  # 󰍼 U+F037C
    'mouse': 983933,  # 󰍽 U+F037D
    'mouse-off': 983934,  # 󰍾 U+F037E
    'mouse-variant': 983935,  # 󰍿 U+F037F
    'mouse-variant-off': 983936,  # 󰎀 U+F0380
    'movie': 983937,  # 󰎁 U+F0381
    'multiplication': 983938,  # 󰎂 U+F0382
    'multiplication-box': 983939,  # 󰎃 U+F0383
    'music-box': 983940,  # 󰎄 U+F0384
    'music-box-outline': 983941,  # 󰎅 U+F0385
    'music-circle': 983942,  # 󰎆 U+F0386
    'music-note': 983943,  # 󰎇 U+F0387
    'music-note#1': 983944,  # 󰎈 U+F0388
    'music-note-half': 983945,  # 󰎉 U+F0389
    'music-note-off': 983946,  # 󰎊 U+F038A
    'music-note-quarter': 983947,  # 󰎋 U+F038B
    'music-note-sixteenth': 983948,  # 󰎌 U+F038C
    'music-note-whole': 983949,  # 󰎍 U+F038D
    'nature': 983950,  # 󰎎 U+F038E
    'nature-people': 983951,  # 󰎏 U+F038F
    'navigation': 983952,  # 󰎐 U+F0390
    'needle': 983953,  # 󰎑 U+F0391
    'smoke-detector': 983954,  # 󰎒 U+F0392
    'thermostat': 983955,  # 󰎓 U+F0393
    'new-box': 983956,  # 󰎔 U+F0394
    'newspaper#1': 983957,  # 󰎕 U+F0395
    'nfc': 983958,  # 󰎖 U+F0396
    'nfc-tap': 983959,  # 󰎗 U+F0397
    'nfc-variant': 983960,  # 󰎘 U+F0398
    'nodejs': 983961,  # 󰎙 U+F0399
    'note#2': 983962,  # 󰎚 U+F039A
    'note-outline': 983963,  # 󰎛 U+F039B
    'note-plus': 983964,  # 󰎜 U+F039C
    'note-plus-outline': 983965,  # 󰎝 U+F039D
    'note-text': 983966,  # 󰎞 U+F039E
    'notification-clear-all': 983967,  # 󰎟 U+F039F
    'numeric': 983968,  # 󰎠 U+F03A0
    'numeric-0-box': 983969,  # 󰎡 U+F03A1
    'numeric-0-box-multiple-outline': 983970,  # 󰎢 U+F03A2
    'numeric-0-box-outline': 983971,  # 󰎣 U+F03A3
    'numeric-1-box': 983972,  # 󰎤 U+F03A4
    'numeric-1-box-multiple-outline': 983973,  # 󰎥 U+F03A5
    'numeric-1-box-outline': 983974,  # 󰎦 U+F03A6
    'numeric-2-box': 983975,  # 󰎧 U+F03A7
    'numeric-2-box-multiple-outline': 983976,  # 󰎨 U+F03A8
    'numeric-2-box-outline': 983977,  # 󰎩 U+F03A9
    'numeric-3-box': 983978,  # 󰎪 U+F03AA
    'numeric-3-box-multiple-outline': 983979,  # 󰎫 U+F03AB
    'numeric-3-box-outline': 983980,  # 󰎬 U+F03AC
    'numeric-4-box': 983981,  # 󰎭 U+F03AD
    'numeric-4-box-outline': 983982,  # 󰎮 U+F03AE
    'numeric-5-box-multiple-outline': 983983,  # 󰎯 U+F03AF
    'numeric-5-box-outline': 983984,  # 󰎰 U+F03B0
    'numeric-5-box': 983985,  # 󰎱 U+F03B1
    'numeric-4-box-multiple-outline': 983986,  # 󰎲 U+F03B2
    'numeric-6-box': 983987,  # 󰎳 U+F03B3
    'numeric-6-box-multiple-outline': 983988,  # 󰎴 U+F03B4
    'numeric-6-box-outline': 983989,  # 󰎵 U+F03B5
    'numeric-7-box': 983990,  # 󰎶 U+F03B6
    'numeric-7-box-multiple-outline': 983991,  # 󰎷 U+F03B7
    'numeric-7-box-outline': 983992,  # 󰎸 U+F03B8
    'numeric-8-box': 983993,  # 󰎹 U+F03B9
    'numeric-8-box-multiple-outline': 983994,  # 󰎺 U+F03BA
    'numeric-8-box-outline': 983995,  # 󰎻 U+F03BB
    'numeric-9-box': 983996,  # 󰎼 U+F03BC
    'numeric-9-box-multiple-outline': 983997,  # 󰎽 U+F03BD
    'numeric-9-box-outline': 983998,  # 󰎾 U+F03BE
    'numeric-9-plus-box': 983999,  # 󰎿 U+F03BF
    'numeric-9-plus-box-multiple-outline': 984000,  # 󰏀 U+F03C0
    'numeric-9-plus-box-outline': 984001,  # 󰏁 U+F03C1
    'nutrition': 984002,  # 󰏂 U+F03C2
    'octagon': 984003,  # 󰏃 U+F03C3
    'octagon-outline': 984004,  # 󰏄 U+F03C4
    'odnoklassniki#1': 984005,  # 󰏅 U+F03C5
    'microsoft-office': 984006,  # 󰏆 U+F03C6
    'oil': 984007,  # 󰏇 U+F03C7
    'coolant-temperature': 984008,  # 󰏈 U+F03C8
    'omega': 984009,  # 󰏉 U+F03C9
    'microsoft-onedrive': 984010,  # 󰏊 U+F03CA
    'open-in-app': 984011,  # 󰏋 U+F03CB
    'open-in-new': 984012,  # 󰏌 U+F03CC
    'openid#1': 984013,  # 󰏍 U+F03CD
    'opera#1': 984014,  # 󰏎 U+F03CE
    'ornament': 984015,  # 󰏏 U+F03CF
    'ornament-variant': 984016,  # 󰏐 U+F03D0
    'inbox-arrow-up': 984017,  # 󰏑 U+F03D1
    'owl': 984018,  # 󰏒 U+F03D2
    'package#2': 984019,  # 󰏓 U+F03D3
    'package-down': 984020,  # 󰏔 U+F03D4
    'package-up': 984021,  # 󰏕 U+F03D5
    'package-variant': 984022,  # 󰏖 U+F03D6
    'package-variant-closed': 984023,  # 󰏗 U+F03D7
    'palette#1': 984024,  # 󰏘 U+F03D8
    'palette-advanced': 984025,  # 󰏙 U+F03D9
    'panda': 984026,  # 󰏚 U+F03DA
    'pandora': 984027,  # 󰏛 U+F03DB
    'panorama': 984028,  # 󰏜 U+F03DC
    'panorama-fisheye': 984029,  # 󰏝 U+F03DD
    'panorama-horizontal-outline': 984030,  # 󰏞 U+F03DE
    'panorama-vertical-outline': 984031,  # 󰏟 U+F03DF
    'panorama-wide-angle-outline': 984032,  # 󰏠 U+F03E0
    'paper-cut-vertical': 984033,  # 󰏡 U+F03E1
    'paperclip#2': 984034,  # 󰏢 U+F03E2
    'parking': 984035,  # 󰏣 U+F03E3
    'pause#1': 984036,  # 󰏤 U+F03E4
    'pause-circle': 984037,  # 󰏥 U+F03E5
    'pause-circle-outline': 984038,  # 󰏦 U+F03E6
    'pause-octagon': 984039,  # 󰏧 U+F03E7
    'pause-octagon-outline': 984040,  # 󰏨 U+F03E8
    'paw#1': 984041,  # 󰏩 U+F03E9
    'pen#1': 984042,  # 󰏪 U+F03EA
    'pencil#2': 984043,  # 󰏫 U+F03EB
    'pencil-box': 984044,  # 󰏬 U+F03EC
    'pencil-box-outline': 984045,  # 󰏭 U+F03ED
    'pencil-lock': 984046,  # 󰏮 U+F03EE
    'pencil-off': 984047,  # 󰏯 U+F03EF
    'percent#1': 984048,  # 󰏰 U+F03F0
    'mortar-pestle-plus': 984049,  # 󰏱 U+F03F1
    'phone#1': 984050,  # 󰏲 U+F03F2
    'phone-bluetooth': 984051,  # 󰏳 U+F03F3
    'phone-forward': 984052,  # 󰏴 U+F03F4
    'phone-hangup': 984053,  # 󰏵 U+F03F5
    'phone-in-talk': 984054,  # 󰏶 U+F03F6
    'phone-incoming': 984055,  # 󰏷 U+F03F7
    'phone-lock': 984056,  # 󰏸 U+F03F8
    'phone-log': 984057,  # 󰏹 U+F03F9
    'phone-missed': 984058,  # 󰏺 U+F03FA
    'phone-outgoing': 984059,  # 󰏻 U+F03FB
    'phone-paused': 984060,  # 󰏼 U+F03FC
    'phone-settings': 984061,  # 󰏽 U+F03FD
    'phone-voip': 984062,  # 󰏾 U+F03FE
    'pi#1': 984063,  # 󰏿 U+F03FF
    'pi-box': 984064,  # 󰐀 U+F0400
    'pig': 984065,  # 󰐁 U+F0401
    'pill': 984066,  # 󰐂 U+F0402
    'pin#2': 984067,  # 󰐃 U+F0403
    'pin-off': 984068,  # 󰐄 U+F0404
    'pine-tree': 984069,  # 󰐅 U+F0405
    'pine-tree-box': 984070,  # 󰐆 U+F0406
    'pinterest#1': 984071,  # 󰐇 U+F0407
    'contactless-payment-circle-outline': 984072,  # 󰐈 U+F0408
    'pizza#1': 984073,  # 󰐉 U+F0409
    'play#3': 984074,  # 󰐊 U+F040A
    'play-box-outline': 984075,  # 󰐋 U+F040B
    'play-circle#1': 984076,  # 󰐌 U+F040C
    'play-circle-outline': 984077,  # 󰐍 U+F040D
    'play-pause': 984078,  # 󰐎 U+F040E
    'play-protected-content': 984079,  # 󰐏 U+F040F
    'playlist-minus': 984080,  # 󰐐 U+F0410
    'playlist-play': 984081,  # 󰐑 U+F0411
    'playlist-plus': 984082,  # 󰐒 U+F0412
    'playlist-remove': 984083,  # 󰐓 U+F0413
    'sony-playstation': 984084,  # 󰐔 U+F0414
    'plus#2': 984085,  # 󰐕 U+F0415
    'plus-box': 984086,  # 󰐖 U+F0416
    'plus-circle#1': 984087,  # 󰐗 U+F0417
    'plus-circle-multiple-outline': 984088,  # 󰐘 U+F0418
    'plus-circle-outline': 984089,  # 󰐙 U+F0419
    'plus-network': 984090,  # 󰐚 U+F041A
    'sledding': 984091,  # 󰐛 U+F041B
    'wall-sconce-flat-variant': 984092,  # 󰐜 U+F041C
    'pokeball': 984093,  # 󰐝 U+F041D
    'polaroid': 984094,  # 󰐞 U+F041E
    'poll': 984095,  # 󰐟 U+F041F
    'account-eye': 984096,  # 󰐠 U+F0420
    'polymer': 984097,  # 󰐡 U+F0421
    'popcorn#1': 984098,  # 󰐢 U+F0422
    'pound': 984099,  # 󰐣 U+F0423
    'pound-box': 984100,  # 󰐤 U+F0424
    'power': 984101,  # 󰐥 U+F0425
    'power-settings': 984102,  # 󰐦 U+F0426
    'power-socket': 984103,  # 󰐧 U+F0427
    'presentation': 984104,  # 󰐨 U+F0428
    'presentation-play': 984105,  # 󰐩 U+F0429
    'printer': 984106,  # 󰐪 U+F042A
    'printer-3d': 984107,  # 󰐫 U+F042B
    'printer-alert': 984108,  # 󰐬 U+F042C
    'professional-hexagon': 984109,  # 󰐭 U+F042D
    'projector': 984110,  # 󰐮 U+F042E
    'projector-screen': 984111,  # 󰐯 U+F042F
    'pulse#3': 984112,  # 󰐰 U+F0430
    'puzzle': 984113,  # 󰐱 U+F0431
    'qrcode#1': 984114,  # 󰐲 U+F0432
    'qrcode-scan': 984115,  # 󰐳 U+F0433
    'quadcopter': 984116,  # 󰐴 U+F0434
    'quality-high': 984117,  # 󰐵 U+F0435
    'book-multiple-outline': 984118,  # 󰐶 U+F0436
    'radar': 984119,  # 󰐷 U+F0437
    'radiator': 984120,  # 󰐸 U+F0438
    'radio#1': 984121,  # 󰐹 U+F0439
    'radio-handheld': 984122,  # 󰐺 U+F043A
    'radio-tower#1': 984123,  # 󰐻 U+F043B
    'radioactive#1': 984124,  # 󰐼 U+F043C
    'checkbox-blank-circle-outline#1': 984125,  # 󰐽 U+F043D
    'radiobox-marked': 984126,  # 󰐾 U+F043E
    'raspberry-pi': 984127,  # 󰐿 U+F043F
    'ray-end': 984128,  # 󰑀 U+F0440
    'ray-end-arrow': 984129,  # 󰑁 U+F0441
    'ray-start': 984130,  # 󰑂 U+F0442
    'ray-start-arrow': 984131,  # 󰑃 U+F0443
    'ray-start-end': 984132,  # 󰑄 U+F0444
    'ray-vertex': 984133,  # 󰑅 U+F0445
    'lastpass': 984134,  # 󰑆 U+F0446
    'read#1': 984135,  # 󰑇 U+F0447
    'youtube-tv': 984136,  # 󰑈 U+F0448
    'receipt#1': 984137,  # 󰑉 U+F0449
    'record#1': 984138,  # 󰑊 U+F044A
    'record-rec': 984139,  # 󰑋 U+F044B
    'recycle#1': 984140,  # 󰑌 U+F044C
    'reddit#1': 984141,  # 󰑍 U+F044D
    'redo#1': 984142,  # 󰑎 U+F044E
    'redo-variant': 984143,  # 󰑏 U+F044F
    'refresh#1': 984144,  # 󰑐 U+F0450
    'regex#1': 984145,  # 󰑑 U+F0451
    'relative-scale': 984146,  # 󰑒 U+F0452
    'reload': 984147,  # 󰑓 U+F0453
    'remote#1': 984148,  # 󰑔 U+F0454
    'rename-box': 984149,  # 󰑕 U+F0455
    'repeat': 984150,  # 󰑖 U+F0456
    'repeat-off': 984151,  # 󰑗 U+F0457
    'repeat-once': 984152,  # 󰑘 U+F0458
    'replay': 984153,  # 󰑙 U+F0459
    'reply#3': 984154,  # 󰑚 U+F045A
    'reply-all': 984155,  # 󰑛 U+F045B
    'reproduction': 984156,  # 󰑜 U+F045C
    'resize-bottom-right': 984157,  # 󰑝 U+F045D
    'responsive': 984158,  # 󰑞 U+F045E
    'rewind': 984159,  # 󰑟 U+F045F
    'ribbon#1': 984160,  # 󰑠 U+F0460
    'road#1': 984161,  # 󰑡 U+F0461
    'road-variant': 984162,  # 󰑢 U+F0462
    'rocket#3': 984163,  # 󰑣 U+F0463
    'rotate-3d-variant': 984164,  # 󰑤 U+F0464
    'rotate-left': 984165,  # 󰑥 U+F0465
    'rotate-left-variant': 984166,  # 󰑦 U+F0466
    'rotate-right': 984167,  # 󰑧 U+F0467
    'rotate-right-variant': 984168,  # 󰑨 U+F0468
    'router-wireless': 984169,  # 󰑩 U+F0469
    'routes': 984170,  # 󰑪 U+F046A
    'rss#3': 984171,  # 󰑫 U+F046B
    'rss-box': 984172,  # 󰑬 U+F046C
    'ruler#2': 984173,  # 󰑭 U+F046D
    'run-fast': 984174,  # 󰑮 U+F046E
    'sale': 984175,  # 󰑯 U+F046F
    'satellite#1': 984176,  # 󰑰 U+F0470
    'satellite-variant': 984177,  # 󰑱 U+F0471
    'scale': 984178,  # 󰑲 U+F0472
    'scale-bathroom': 984179,  # 󰑳 U+F0473
    'school#1': 984180,  # 󰑴 U+F0474
    'screen-rotation': 984181,  # 󰑵 U+F0475
    'screwdriver#1': 984182,  # 󰑶 U+F0476
    'script-outline': 984183,  # 󰑷 U+F0477
    'screen-rotation-lock': 984184,  # 󰑸 U+F0478
    'sd': 984185,  # 󰑹 U+F0479
    'seal': 984186,  # 󰑺 U+F047A
    'seat-flat': 984187,  # 󰑻 U+F047B
    'seat-flat-angled': 984188,  # 󰑼 U+F047C
    'seat-individual-suite': 984189,  # 󰑽 U+F047D
    'seat-legroom-extra': 984190,  # 󰑾 U+F047E
    'seat-legroom-normal': 984191,  # 󰑿 U+F047F
    'seat-legroom-reduced': 984192,  # 󰒀 U+F0480
    'seat-recline-extra': 984193,  # 󰒁 U+F0481
    'seat-recline-normal': 984194,  # 󰒂 U+F0482
    'security': 984195,  # 󰒃 U+F0483
    'security-network': 984196,  # 󰒄 U+F0484
    'select': 984197,  # 󰒅 U+F0485
    'select-all': 984198,  # 󰒆 U+F0486
    'select-inverse': 984199,  # 󰒇 U+F0487
    'select-off': 984200,  # 󰒈 U+F0488
    'selection': 984201,  # 󰒉 U+F0489
    'send#1': 984202,  # 󰒊 U+F048A
    'server#3': 984203,  # 󰒋 U+F048B
    'server-minus': 984204,  # 󰒌 U+F048C
    'server-network': 984205,  # 󰒍 U+F048D
    'server-network-off': 984206,  # 󰒎 U+F048E
    'server-off': 984207,  # 󰒏 U+F048F
    'server-plus': 984208,  # 󰒐 U+F0490
    'server-remove': 984209,  # 󰒑 U+F0491
    'server-security': 984210,  # 󰒒 U+F0492
    'cog': 984211,  # 󰒓 U+F0493
    'cog-box': 984212,  # 󰒔 U+F0494
    'shape-plus': 984213,  # 󰒕 U+F0495
    'share#2': 984214,  # 󰒖 U+F0496
    'share-variant': 984215,  # 󰒗 U+F0497
    'shield#3': 984216,  # 󰒘 U+F0498
    'shield-outline': 984217,  # 󰒙 U+F0499
    'shopping': 984218,  # 󰒚 U+F049A
    'shopping-music': 984219,  # 󰒛 U+F049B
    'shredder': 984220,  # 󰒜 U+F049C
    'shuffle#1': 984221,  # 󰒝 U+F049D
    'shuffle-disabled': 984222,  # 󰒞 U+F049E
    'shuffle-variant': 984223,  # 󰒟 U+F049F
    'sigma': 984224,  # 󰒠 U+F04A0
    'sign-caution': 984225,  # 󰒡 U+F04A1
    'signal#1': 984226,  # 󰒢 U+F04A2
    'silverware': 984227,  # 󰒣 U+F04A3
    'silverware-fork': 984228,  # 󰒤 U+F04A4
    'silverware-spoon': 984229,  # 󰒥 U+F04A5
    'silverware-variant': 984230,  # 󰒦 U+F04A6
    'sim': 984231,  # 󰒧 U+F04A7
    'sim-alert': 984232,  # 󰒨 U+F04A8
    'sim-off': 984233,  # 󰒩 U+F04A9
    'sitemap#1': 984234,  # 󰒪 U+F04AA
    'skip-backward': 984235,  # 󰒫 U+F04AB
    'skip-forward': 984236,  # 󰒬 U+F04AC
    'skip-next': 984237,  # 󰒭 U+F04AD
    'skip-previous': 984238,  # 󰒮 U+F04AE
    'skype#1': 984239,  # 󰒯 U+F04AF
    'skype-business': 984240,  # 󰒰 U+F04B0
    'slack#1': 984241,  # 󰒱 U+F04B1
    'sleep': 984242,  # 󰒲 U+F04B2
    'sleep-off': 984243,  # 󰒳 U+F04B3
    'smoking#1': 984244,  # 󰒴 U+F04B4
    'smoking-off': 984245,  # 󰒵 U+F04B5
    'snapchat#1': 984246,  # 󰒶 U+F04B6
    'snowman#1': 984247,  # 󰒷 U+F04B7
    'soccer': 984248,  # 󰒸 U+F04B8
    'sofa#1': 984249,  # 󰒹 U+F04B9
    'sort#1': 984250,  # 󰒺 U+F04BA
    'sort-alphabetical-variant': 984251,  # 󰒻 U+F04BB
    'sort-ascending': 984252,  # 󰒼 U+F04BC
    'sort-descending': 984253,  # 󰒽 U+F04BD
    'sort-numeric-variant': 984254,  # 󰒾 U+F04BE
    'sort-variant': 984255,  # 󰒿 U+F04BF
    'soundcloud#1': 984256,  # 󰓀 U+F04C0
    'source-fork': 984257,  # 󰓁 U+F04C1
    'source-pull': 984258,  # 󰓂 U+F04C2
    'speaker': 984259,  # 󰓃 U+F04C3
    'speaker-off': 984260,  # 󰓄 U+F04C4
    'speedometer': 984261,  # 󰓅 U+F04C5
    'spellcheck': 984262,  # 󰓆 U+F04C6
    'spotify#1': 984263,  # 󰓇 U+F04C7
    'spotlight': 984264,  # 󰓈 U+F04C8
    'spotlight-beam': 984265,  # 󰓉 U+F04C9
    'book-remove-multiple-outline': 984266,  # 󰓊 U+F04CA
    'account-switch-outline': 984267,  # 󰓋 U+F04CB
    'stack-overflow': 984268,  # 󰓌 U+F04CC
    'stairs': 984269,  # 󰓍 U+F04CD
    'star#2': 984270,  # 󰓎 U+F04CE
    'star-circle': 984271,  # 󰓏 U+F04CF
    'star-half-full': 984272,  # 󰓐 U+F04D0
    'star-off': 984273,  # 󰓑 U+F04D1
    'star-outline': 984274,  # 󰓒 U+F04D2
    'steam#1': 984275,  # 󰓓 U+F04D3
    'steering': 984276,  # 󰓔 U+F04D4
    'step-backward': 984277,  # 󰓕 U+F04D5
    'step-backward-2': 984278,  # 󰓖 U+F04D6
    'step-forward': 984279,  # 󰓗 U+F04D7
    'step-forward-2': 984280,  # 󰓘 U+F04D8
    'stethoscope#1': 984281,  # 󰓙 U+F04D9
    'stocking': 984282,  # 󰓚 U+F04DA
    'stop#2': 984283,  # 󰓛 U+F04DB
    'store#1': 984284,  # 󰓜 U+F04DC
    'store-24-hour': 984285,  # 󰓝 U+F04DD
    'stove': 984286,  # 󰓞 U+F04DE
    'subway-variant': 984287,  # 󰓟 U+F04DF
    'sunglasses': 984288,  # 󰓠 U+F04E0
    'swap-horizontal': 984289,  # 󰓡 U+F04E1
    'swap-vertical': 984290,  # 󰓢 U+F04E2
    'swim': 984291,  # 󰓣 U+F04E3
    'switch': 984292,  # 󰓤 U+F04E4
    'sword': 984293,  # 󰓥 U+F04E5
    'sync#2': 984294,  # 󰓦 U+F04E6
    'sync-alert': 984295,  # 󰓧 U+F04E7
    'sync-off': 984296,  # 󰓨 U+F04E8
    'tab#1': 984297,  # 󰓩 U+F04E9
    'tab-unselected': 984298,  # 󰓪 U+F04EA
    'table#3': 984299,  # 󰓫 U+F04EB
    'table-column-plus-after': 984300,  # 󰓬 U+F04EC
    'table-column-plus-before': 984301,  # 󰓭 U+F04ED
    'table-column-remove': 984302,  # 󰓮 U+F04EE
    'table-column-width': 984303,  # 󰓯 U+F04EF
    'table-edit': 984304,  # 󰓰 U+F04F0
    'table-large': 984305,  # 󰓱 U+F04F1
    'table-row-height': 984306,  # 󰓲 U+F04F2
    'table-row-plus-after': 984307,  # 󰓳 U+F04F3
    'table-row-plus-before': 984308,  # 󰓴 U+F04F4
    'table-row-remove': 984309,  # 󰓵 U+F04F5
    'tablet#1': 984310,  # 󰓶 U+F04F6
    'tablet-android': 984311,  # 󰓷 U+F04F7
    'tangram': 984312,  # 󰓸 U+F04F8
    'tag#3': 984313,  # 󰓹 U+F04F9
    'tag-faces': 984314,  # 󰓺 U+F04FA
    'tag-multiple': 984315,  # 󰓻 U+F04FB
    'tag-outline': 984316,  # 󰓼 U+F04FC
    'tag-text-outline': 984317,  # 󰓽 U+F04FD
    'target#1': 984318,  # 󰓾 U+F04FE
    'taxi#1': 984319,  # 󰓿 U+F04FF
    'teamviewer': 984320,  # 󰔀 U+F0500
    'skateboarding': 984321,  # 󰔁 U+F0501
    'television': 984322,  # 󰔂 U+F0502
    'television-guide': 984323,  # 󰔃 U+F0503
    'temperature-celsius': 984324,  # 󰔄 U+F0504
    'temperature-fahrenheit': 984325,  # 󰔅 U+F0505
    'temperature-kelvin': 984326,  # 󰔆 U+F0506
    'tennis-ball': 984327,  # 󰔇 U+F0507
    'tent': 984328,  # 󰔈 U+F0508
    'image-filter-hdr#1': 984329,  # 󰔉 U+F0509
    'text-to-speech': 984330,  # 󰔊 U+F050A
    'text-to-speech-off': 984331,  # 󰔋 U+F050B
    'texture': 984332,  # 󰔌 U+F050C
    'theater': 984333,  # 󰔍 U+F050D
    'theme-light-dark': 984334,  # 󰔎 U+F050E
    'thermometer#1': 984335,  # 󰔏 U+F050F
    'thermometer-lines': 984336,  # 󰔐 U+F0510
    'thumb-down': 984337,  # 󰔑 U+F0511
    'thumb-down-outline': 984338,  # 󰔒 U+F0512
    'thumb-up': 984339,  # 󰔓 U+F0513
    'thumb-up-outline': 984340,  # 󰔔 U+F0514
    'thumbs-up-down': 984341,  # 󰔕 U+F0515
    'ticket#1': 984342,  # 󰔖 U+F0516
    'ticket-account': 984343,  # 󰔗 U+F0517
    'ticket-confirmation': 984344,  # 󰔘 U+F0518
    'tie': 984345,  # 󰔙 U+F0519
    'timelapse': 984346,  # 󰔚 U+F051A
    'timer-outline': 984347,  # 󰔛 U+F051B
    'timer-10': 984348,  # 󰔜 U+F051C
    'timer-3': 984349,  # 󰔝 U+F051D
    'timer-off-outline': 984350,  # 󰔞 U+F051E
    'timer-sand': 984351,  # 󰔟 U+F051F
    'timetable': 984352,  # 󰔠 U+F0520
    'toggle-switch': 984353,  # 󰔡 U+F0521
    'toggle-switch-off': 984354,  # 󰔢 U+F0522
    'tooltip': 984355,  # 󰔣 U+F0523
    'tooltip-edit': 984356,  # 󰔤 U+F0524
    'tooltip-image': 984357,  # 󰔥 U+F0525
    'tooltip-outline': 984358,  # 󰔦 U+F0526
    'tooltip-plus-outline': 984359,  # 󰔧 U+F0527
    'tooltip-text': 984360,  # 󰔨 U+F0528
    'tooth-outline': 984361,  # 󰔩 U+F0529
    'cloud-refresh': 984362,  # 󰔪 U+F052A
    'traffic-light': 984363,  # 󰔫 U+F052B
    'train#1': 984364,  # 󰔬 U+F052C
    'tram': 984365,  # 󰔭 U+F052D
    'transcribe': 984366,  # 󰔮 U+F052E
    'transcribe-close': 984367,  # 󰔯 U+F052F
    'transfer-right': 984368,  # 󰔰 U+F0530
    'tree#2': 984369,  # 󰔱 U+F0531
    'trello#1': 984370,  # 󰔲 U+F0532
    'trending-down': 984371,  # 󰔳 U+F0533
    'trending-neutral': 984372,  # 󰔴 U+F0534
    'trending-up': 984373,  # 󰔵 U+F0535
    'triangle': 984374,  # 󰔶 U+F0536
    'triangle-outline': 984375,  # 󰔷 U+F0537
    'trophy#2': 984376,  # 󰔸 U+F0538
    'trophy-award': 984377,  # 󰔹 U+F0539
    'trophy-outline': 984378,  # 󰔺 U+F053A
    'trophy-variant': 984379,  # 󰔻 U+F053B
    'trophy-variant-outline': 984380,  # 󰔼 U+F053C
    'truck#1': 984381,  # 󰔽 U+F053D
    'truck-delivery': 984382,  # 󰔾 U+F053E
    'tshirt-crew-outline': 984383,  # 󰔿 U+F053F
    'tshirt-v-outline': 984384,  # 󰕀 U+F0540
    'file-refresh-outline': 984385,  # 󰕁 U+F0541
    'folder-refresh-outline': 984386,  # 󰕂 U+F0542
    'twitch#1': 984387,  # 󰕃 U+F0543
    'twitter#2': 984388,  # 󰕄 U+F0544
    'order-numeric-ascending': 984389,  # 󰕅 U+F0545
    'order-numeric-descending': 984390,  # 󰕆 U+F0546
    'repeat-variant': 984391,  # 󰕇 U+F0547
    'ubuntu#1': 984392,  # 󰕈 U+F0548
    'umbraco#1': 984393,  # 󰕉 U+F0549
    'umbrella#2': 984394,  # 󰕊 U+F054A
    'umbrella-outline': 984395,  # 󰕋 U+F054B
    'undo': 984396,  # 󰕌 U+F054C
    'undo-variant': 984397,  # 󰕍 U+F054D
    'unfold-less-horizontal': 984398,  # 󰕎 U+F054E
    'unfold-more-horizontal': 984399,  # 󰕏 U+F054F
    'ungroup': 984400,  # 󰕐 U+F0550
    'web-remove': 984401,  # 󰕑 U+F0551
    'upload#2': 984402,  # 󰕒 U+F0552
    'usb#1': 984403,  # 󰕓 U+F0553
    'vector-arrange-above': 984404,  # 󰕔 U+F0554
    'vector-arrange-below': 984405,  # 󰕕 U+F0555
    'vector-circle': 984406,  # 󰕖 U+F0556
    'vector-circle-variant': 984407,  # 󰕗 U+F0557
    'vector-combine': 984408,  # 󰕘 U+F0558
    'vector-curve': 984409,  # 󰕙 U+F0559
    'vector-difference': 984410,  # 󰕚 U+F055A
    'vector-difference-ab': 984411,  # 󰕛 U+F055B
    'vector-difference-ba': 984412,  # 󰕜 U+F055C
    'vector-intersection': 984413,  # 󰕝 U+F055D
    'vector-line': 984414,  # 󰕞 U+F055E
    'vector-point': 984415,  # 󰕟 U+F055F
    'vector-polygon': 984416,  # 󰕠 U+F0560
    'vector-polyline': 984417,  # 󰕡 U+F0561
    'vector-selection': 984418,  # 󰕢 U+F0562
    'vector-triangle': 984419,  # 󰕣 U+F0563
    'vector-union': 984420,  # 󰕤 U+F0564
    'shield-check#1': 984421,  # 󰕥 U+F0565
    'vibrate': 984422,  # 󰕦 U+F0566
    'video#2': 984423,  # 󰕧 U+F0567
    'video-off': 984424,  # 󰕨 U+F0568
    'video-switch': 984425,  # 󰕩 U+F0569
    'view-agenda': 984426,  # 󰕪 U+F056A
    'view-array': 984427,  # 󰕫 U+F056B
    'view-carousel': 984428,  # 󰕬 U+F056C
    'view-column': 984429,  # 󰕭 U+F056D
    'view-dashboard': 984430,  # 󰕮 U+F056E
    'view-day': 984431,  # 󰕯 U+F056F
    'view-grid': 984432,  # 󰕰 U+F0570
    'view-headline': 984433,  # 󰕱 U+F0571
    'view-list': 984434,  # 󰕲 U+F0572
    'view-module': 984435,  # 󰕳 U+F0573
    'view-quilt': 984436,  # 󰕴 U+F0574
    'view-stream': 984437,  # 󰕵 U+F0575
    'view-week': 984438,  # 󰕶 U+F0576
    'vimeo#1': 984439,  # 󰕷 U+F0577
    'buffet': 984440,  # 󰕸 U+F0578
    'hands-pray': 984441,  # 󰕹 U+F0579
    'credit-card-wireless-off': 984442,  # 󰕺 U+F057A
    'credit-card-wireless-off-outline': 984443,  # 󰕻 U+F057B
    'vlc': 984444,  # 󰕼 U+F057C
    'voicemail#1': 984445,  # 󰕽 U+F057D
    'volume-high': 984446,  # 󰕾 U+F057E
    'volume-low': 984447,  # 󰕿 U+F057F
    'volume-medium': 984448,  # 󰖀 U+F0580
    'volume-off': 984449,  # 󰖁 U+F0581
    'vpn': 984450,  # 󰖂 U+F0582
    'walk': 984451,  # 󰖃 U+F0583
    'wallet#2': 984452,  # 󰖄 U+F0584
    'wallet-giftcard': 984453,  # 󰖅 U+F0585
    'wallet-membership': 984454,  # 󰖆 U+F0586
    'wallet-travel': 984455,  # 󰖇 U+F0587
    'wan': 984456,  # 󰖈 U+F0588
    'watch#1': 984457,  # 󰖉 U+F0589
    'watch-export': 984458,  # 󰖊 U+F058A
    'watch-import': 984459,  # 󰖋 U+F058B
    'water#1': 984460,  # 󰖌 U+F058C
    'water-off': 984461,  # 󰖍 U+F058D
    'water-percent': 984462,  # 󰖎 U+F058E
    'water-pump': 984463,  # 󰖏 U+F058F
    'weather-cloudy': 984464,  # 󰖐 U+F0590
    'weather-fog': 984465,  # 󰖑 U+F0591
    'weather-hail': 984466,  # 󰖒 U+F0592
    'weather-lightning': 984467,  # 󰖓 U+F0593
    'weather-night': 984468,  # 󰖔 U+F0594
    'weather-partly-cloudy': 984469,  # 󰖕 U+F0595
    'weather-pouring': 984470,  # 󰖖 U+F0596
    'weather-rainy': 984471,  # 󰖗 U+F0597
    'weather-snowy': 984472,  # 󰖘 U+F0598
    'weather-sunny': 984473,  # 󰖙 U+F0599
    'weather-sunset': 984474,  # 󰖚 U+F059A
    'weather-sunset-down': 984475,  # 󰖛 U+F059B
    'weather-sunset-up': 984476,  # 󰖜 U+F059C
    'weather-windy': 984477,  # 󰖝 U+F059D
    'weather-windy-variant': 984478,  # 󰖞 U+F059E
    'web': 984479,  # 󰖟 U+F059F
    'webcam': 984480,  # 󰖠 U+F05A0
    'weight': 984481,  # 󰖡 U+F05A1
    'weight-kilogram': 984482,  # 󰖢 U+F05A2
    'whatsapp#1': 984483,  # 󰖣 U+F05A3
    'wheelchair-accessibility': 984484,  # 󰖤 U+F05A4
    'white-balance-auto': 984485,  # 󰖥 U+F05A5
    'white-balance-incandescent': 984486,  # 󰖦 U+F05A6
    'white-balance-iridescent': 984487,  # 󰖧 U+F05A7
    'white-balance-sunny': 984488,  # 󰖨 U+F05A8
    'wifi#1': 984489,  # 󰖩 U+F05A9
    'wifi-off': 984490,  # 󰖪 U+F05AA
    'nintendo-wii': 984491,  # 󰖫 U+F05AB
    'wikipedia': 984492,  # 󰖬 U+F05AC
    'window-close': 984493,  # 󰖭 U+F05AD
    'window-closed': 984494,  # 󰖮 U+F05AE
    'window-maximize': 984495,  # 󰖯 U+F05AF
    'window-minimize': 984496,  # 󰖰 U+F05B0
    'window-open': 984497,  # 󰖱 U+F05B1
    'window-restore': 984498,  # 󰖲 U+F05B2
    'microsoft-windows': 984499,  # 󰖳 U+F05B3
    'wordpress#1': 984500,  # 󰖴 U+F05B4
    'account-hard-hat': 984501,  # 󰖵 U+F05B5
    'wrap': 984502,  # 󰖶 U+F05B6
    'wrench#1': 984503,  # 󰖷 U+F05B7
    'contacts-outline': 984504,  # 󰖸 U+F05B8
    'microsoft-xbox': 984505,  # 󰖹 U+F05B9
    'microsoft-xbox-controller': 984506,  # 󰖺 U+F05BA
    'microsoft-xbox-controller-off': 984507,  # 󰖻 U+F05BB
    'table-furniture': 984508,  # 󰖼 U+F05BC
    'sort-alphabetical-ascending': 984509,  # 󰖽 U+F05BD
    'firewire': 984510,  # 󰖾 U+F05BE
    'sort-alphabetical-descending': 984511,  # 󰖿 U+F05BF
    'xml': 984512,  # 󰗀 U+F05C0
    'yeast': 984513,  # 󰗁 U+F05C1
    'database-refresh': 984514,  # 󰗂 U+F05C2
    'youtube#1': 984515,  # 󰗃 U+F05C3
    'zip-box': 984516,  # 󰗄 U+F05C4
    'surround-sound': 984517,  # 󰗅 U+F05C5
    'vector-rectangle': 984518,  # 󰗆 U+F05C6
    'playlist-check': 984519,  # 󰗇 U+F05C7
    'format-line-style': 984520,  # 󰗈 U+F05C8
    'format-line-weight': 984521,  # 󰗉 U+F05C9
    'translate': 984522,  # 󰗊 U+F05CA
    'account-voice': 984523,  # 󰗋 U+F05CB
    'opacity': 984524,  # 󰗌 U+F05CC
    'near-me': 984525,  # 󰗍 U+F05CD
    'clock-alert-outline': 984526,  # 󰗎 U+F05CE
    'human-pregnant': 984527,  # 󰗏 U+F05CF
    'sticker-circle-outline': 984528,  # 󰗐 U+F05D0
    'scale-balance': 984529,  # 󰗑 U+F05D1
    'card-account-details': 984530,  # 󰗒 U+F05D2
    'account-multiple-minus': 984531,  # 󰗓 U+F05D3
    'airplane-landing': 984532,  # 󰗔 U+F05D4
    'airplane-takeoff': 984533,  # 󰗕 U+F05D5
    'alert-circle-outline': 984534,  # 󰗖 U+F05D6
    'altimeter': 984535,  # 󰗗 U+F05D7
    'animation': 984536,  # 󰗘 U+F05D8
    'book-minus': 984537,  # 󰗙 U+F05D9
    'book-open-page-variant': 984538,  # 󰗚 U+F05DA
    'book-plus': 984539,  # 󰗛 U+F05DB
    'boombox': 984540,  # 󰗜 U+F05DC
    'bullseye#1': 984541,  # 󰗝 U+F05DD
    'comment-remove': 984542,  # 󰗞 U+F05DE
    'camera-off': 984543,  # 󰗟 U+F05DF
    'check-circle#1': 984544,  # 󰗠 U+F05E0
    'check-circle-outline': 984545,  # 󰗡 U+F05E1
    'candle': 984546,  # 󰗢 U+F05E2
    'chart-bubble': 984547,  # 󰗣 U+F05E3
    'credit-card-off-outline': 984548,  # 󰗤 U+F05E4
    'cup-off': 984549,  # 󰗥 U+F05E5
    'copyright#1': 984550,  # 󰗦 U+F05E6
    'cursor-text': 984551,  # 󰗧 U+F05E7
    'delete-forever': 984552,  # 󰗨 U+F05E8
    'delete-sweep': 984553,  # 󰗩 U+F05E9
    'dice-d20-outline': 984554,  # 󰗪 U+F05EA
    'dice-d4-outline': 984555,  # 󰗫 U+F05EB
    'dice-d8-outline': 984556,  # 󰗬 U+F05EC
    'dice-d6-outline': 984557,  # 󰗭 U+F05ED
    'disc': 984558,  # 󰗮 U+F05EE
    'email-open-outline': 984559,  # 󰗯 U+F05EF
    'email-variant': 984560,  # 󰗰 U+F05F0
    'ev-station': 984561,  # 󰗱 U+F05F1
    'food-fork-drink': 984562,  # 󰗲 U+F05F2
    'food-off': 984563,  # 󰗳 U+F05F3
    'format-title': 984564,  # 󰗴 U+F05F4
    'google-maps': 984565,  # 󰗵 U+F05F5
    'heart-pulse': 984566,  # 󰗶 U+F05F6
    'highway': 984567,  # 󰗷 U+F05F7
    'home-map-marker': 984568,  # 󰗸 U+F05F8
    'incognito': 984569,  # 󰗹 U+F05F9
    'kettle': 984570,  # 󰗺 U+F05FA
    'lock-plus': 984571,  # 󰗻 U+F05FB
    'exit-to-app#1': 984572,  # 󰗼 U+F05FC
    'logout-variant': 984573,  # 󰗽 U+F05FD
    'music-note-bluetooth': 984574,  # 󰗾 U+F05FE
    'music-note-bluetooth-off': 984575,  # 󰗿 U+F05FF
    'page-first': 984576,  # 󰘀 U+F0600
    'page-last': 984577,  # 󰘁 U+F0601
    'phone-classic': 984578,  # 󰘂 U+F0602
    'priority-high': 984579,  # 󰘃 U+F0603
    'priority-low': 984580,  # 󰘄 U+F0604
    'qqchat': 984581,  # 󰘅 U+F0605
    'pool': 984582,  # 󰘆 U+F0606
    'rounded-corner': 984583,  # 󰘇 U+F0607
    'rowing': 984584,  # 󰘈 U+F0608
    'saxophone': 984585,  # 󰘉 U+F0609
    'signal-variant': 984586,  # 󰘊 U+F060A
    'stack-exchange': 984587,  # 󰘋 U+F060B
    'subdirectory-arrow-left': 984588,  # 󰘌 U+F060C
    'subdirectory-arrow-right': 984589,  # 󰘍 U+F060D
    'form-textbox': 984590,  # 󰘎 U+F060E
    'violin': 984591,  # 󰘏 U+F060F
    'microsoft-visual-studio': 984592,  # 󰘐 U+F0610
    'wechat': 984593,  # 󰘑 U+F0611
    'watermark': 984594,  # 󰘒 U+F0612
    'file-hidden': 984595,  # 󰘓 U+F0613
    'application-outline': 984596,  # 󰘔 U+F0614
    'arrow-collapse': 984597,  # 󰘕 U+F0615
    'arrow-expand': 984598,  # 󰘖 U+F0616
    'bowl-mix': 984599,  # 󰘗 U+F0617
    'bridge': 984600,  # 󰘘 U+F0618
    'application-edit-outline': 984601,  # 󰘙 U+F0619
    'chip#2': 984602,  # 󰘚 U+F061A
    'content-save-settings': 984603,  # 󰘛 U+F061B
    'dialpad': 984604,  # 󰘜 U+F061C
    'book-alphabet': 984605,  # 󰘝 U+F061D
    'format-horizontal-align-center': 984606,  # 󰘞 U+F061E
    'format-horizontal-align-left': 984607,  # 󰘟 U+F061F
    'format-horizontal-align-right': 984608,  # 󰘠 U+F0620
    'format-vertical-align-bottom': 984609,  # 󰘡 U+F0621
    'format-vertical-align-center': 984610,  # 󰘢 U+F0622
    'format-vertical-align-top': 984611,  # 󰘣 U+F0623
    'line-scan': 984612,  # 󰘤 U+F0624
    'help-circle-outline': 984613,  # 󰘥 U+F0625
    'code-json': 984614,  # 󰘦 U+F0626
    'lambda': 984615,  # 󰘧 U+F0627
    'matrix': 984616,  # 󰘨 U+F0628
    'meteor#1': 984617,  # 󰘩 U+F0629
    'close-circle-multiple': 984618,  # 󰘪 U+F062A
    'sigma-lower': 984619,  # 󰘫 U+F062B
    'source-branch': 984620,  # 󰘬 U+F062C
    'source-merge': 984621,  # 󰘭 U+F062D
    'tune': 984622,  # 󰘮 U+F062E
    'webhook#1': 984623,  # 󰘯 U+F062F
    'account-settings': 984624,  # 󰘰 U+F0630
    'account-details': 984625,  # 󰘱 U+F0631
    'apple-keyboard-caps': 984626,  # 󰘲 U+F0632
    'apple-keyboard-command': 984627,  # 󰘳 U+F0633
    'apple-keyboard-control': 984628,  # 󰘴 U+F0634
    'apple-keyboard-option': 984629,  # 󰘵 U+F0635
    'apple-keyboard-shift': 984630,  # 󰘶 U+F0636
    'box-shadow': 984631,  # 󰘷 U+F0637
    'cards': 984632,  # 󰘸 U+F0638
    'cards-outline': 984633,  # 󰘹 U+F0639
    'cards-playing-outline': 984634,  # 󰘺 U+F063A
    'checkbox-multiple-blank-circle': 984635,  # 󰘻 U+F063B
    'checkbox-multiple-blank-circle-outline': 984636,  # 󰘼 U+F063C
    'checkbox-multiple-marked-circle': 984637,  # 󰘽 U+F063D
    'checkbox-multiple-marked-circle-outline': 984638,  # 󰘾 U+F063E
    'cloud-sync': 984639,  # 󰘿 U+F063F
    'collage': 984640,  # 󰙀 U+F0640
    'directions-fork': 984641,  # 󰙁 U+F0641
    'eraser-variant': 984642,  # 󰙂 U+F0642
    'face-man': 984643,  # 󰙃 U+F0643
    'face-man-profile': 984644,  # 󰙄 U+F0644
    'file-tree': 984645,  # 󰙅 U+F0645
    'format-annotation-plus': 984646,  # 󰙆 U+F0646
    'gas-cylinder': 984647,  # 󰙇 U+F0647
    'grease-pencil': 984648,  # 󰙈 U+F0648
    'human-female': 984649,  # 󰙉 U+F0649
    'human-greeting-variant': 984650,  # 󰙊 U+F064A
    'human-handsdown': 984651,  # 󰙋 U+F064B
    'human-handsup': 984652,  # 󰙌 U+F064C
    'human-male': 984653,  # 󰙍 U+F064D
    'information-variant': 984654,  # 󰙎 U+F064E
    'lead-pencil': 984655,  # 󰙏 U+F064F
    'map-marker-minus': 984656,  # 󰙐 U+F0650
    'map-marker-plus': 984657,  # 󰙑 U+F0651
    'marker#1': 984658,  # 󰙒 U+F0652
    'message-plus': 984659,  # 󰙓 U+F0653
    'microscope#2': 984660,  # 󰙔 U+F0654
    'move-resize': 984661,  # 󰙕 U+F0655
    'move-resize-variant': 984662,  # 󰙖 U+F0656
    'paw-off': 984663,  # 󰙗 U+F0657
    'phone-minus': 984664,  # 󰙘 U+F0658
    'phone-plus': 984665,  # 󰙙 U+F0659
    'pot-steam': 984666,  # 󰙚 U+F065A
    'pot-mix': 984667,  # 󰙛 U+F065B
    'serial-port': 984668,  # 󰙜 U+F065C
    'shape-circle-plus': 984669,  # 󰙝 U+F065D
    'shape-polygon-plus': 984670,  # 󰙞 U+F065E
    'shape-rectangle-plus': 984671,  # 󰙟 U+F065F
    'shape-square-plus': 984672,  # 󰙠 U+F0660
    'skip-next-circle': 984673,  # 󰙡 U+F0661
    'skip-next-circle-outline': 984674,  # 󰙢 U+F0662
    'skip-previous-circle': 984675,  # 󰙣 U+F0663
    'skip-previous-circle-outline': 984676,  # 󰙤 U+F0664
    'spray': 984677,  # 󰙥 U+F0665
    'stop-circle#1': 984678,  # 󰙦 U+F0666
    'stop-circle-outline': 984679,  # 󰙧 U+F0667
    'test-tube': 984680,  # 󰙨 U+F0668
    'text-shadow': 984681,  # 󰙩 U+F0669
    'tune-vertical': 984682,  # 󰙪 U+F066A
    'cart-off': 984683,  # 󰙫 U+F066B
    'chart-gantt': 984684,  # 󰙬 U+F066C
    'chart-scatter-plot-hexbin': 984685,  # 󰙭 U+F066D
    'chart-timeline': 984686,  # 󰙮 U+F066E
    'discord#1': 984687,  # 󰙯 U+F066F
    'file-restore': 984688,  # 󰙰 U+F0670
    'language-c': 984689,  # 󰙱 U+F0671
    'language-cpp': 984690,  # 󰙲 U+F0672
    'language-xaml': 984691,  # 󰙳 U+F0673
    'creation': 984692,  # 󰙴 U+F0674
    'application-cog': 984693,  # 󰙵 U+F0675
    'credit-card-plus-outline': 984694,  # 󰙶 U+F0676
    'pot-mix-outline': 984695,  # 󰙷 U+F0677
    'bow-tie': 984696,  # 󰙸 U+F0678
    'calendar-range': 984697,  # 󰙹 U+F0679
    'currency-usd-off': 984698,  # 󰙺 U+F067A
    'flash-red-eye': 984699,  # 󰙻 U+F067B
    'oar': 984700,  # 󰙼 U+F067C
    'piano#1': 984701,  # 󰙽 U+F067D
    'weather-lightning-rainy': 984702,  # 󰙾 U+F067E
    'weather-snowy-rainy': 984703,  # 󰙿 U+F067F
    'yin-yang': 984704,  # 󰚀 U+F0680
    'tower-beach': 984705,  # 󰚁 U+F0681
    'tower-fire': 984706,  # 󰚂 U+F0682
    'delete-circle': 984707,  # 󰚃 U+F0683
    'dna#2': 984708,  # 󰚄 U+F0684
    'hamburger#1': 984709,  # 󰚅 U+F0685
    'gondola': 984710,  # 󰚆 U+F0686
    'inbox#3': 984711,  # 󰚇 U+F0687
    'reorder-horizontal': 984712,  # 󰚈 U+F0688
    'reorder-vertical': 984713,  # 󰚉 U+F0689
    'shield-home': 984714,  # 󰚊 U+F068A
    'tag-heart': 984715,  # 󰚋 U+F068B
    'skull#1': 984716,  # 󰚌 U+F068C
    'solid': 984717,  # 󰚍 U+F068D
    'alarm-snooze': 984718,  # 󰚎 U+F068E
    'baby-carriage': 984719,  # 󰚏 U+F068F
    'beaker-outline': 984720,  # 󰚐 U+F0690
    'bomb#1': 984721,  # 󰚑 U+F0691
    'calendar-question': 984722,  # 󰚒 U+F0692
    'camera-burst': 984723,  # 󰚓 U+F0693
    'code-tags-check': 984724,  # 󰚔 U+F0694
    'circle-multiple-outline': 984725,  # 󰚕 U+F0695
    'crop-rotate': 984726,  # 󰚖 U+F0696
    'developer-board': 984727,  # 󰚗 U+F0697
    'piano-off': 984728,  # 󰚘 U+F0698
    'skate-off': 984729,  # 󰚙 U+F0699
    'message-star': 984730,  # 󰚚 U+F069A
    'emoticon-dead-outline': 984731,  # 󰚛 U+F069B
    'emoticon-excited-outline': 984732,  # 󰚜 U+F069C
    'folder-star': 984733,  # 󰚝 U+F069D
    'format-color-text': 984734,  # 󰚞 U+F069E
    'format-section': 984735,  # 󰚟 U+F069F
    'gradient-vertical': 984736,  # 󰚠 U+F06A0
    'home-outline': 984737,  # 󰚡 U+F06A1
    'message-bulleted': 984738,  # 󰚢 U+F06A2
    'message-bulleted-off': 984739,  # 󰚣 U+F06A3
    'nuke': 984740,  # 󰚤 U+F06A4
    'power-plug': 984741,  # 󰚥 U+F06A5
    'power-plug-off': 984742,  # 󰚦 U+F06A6
    'publish': 984743,  # 󰚧 U+F06A7
    'credit-card-marker': 984744,  # 󰚨 U+F06A8
    'robot#1': 984745,  # 󰚩 U+F06A9
    'format-rotate-90': 984746,  # 󰚪 U+F06AA
    'scanner': 984747,  # 󰚫 U+F06AB
    'subway': 984748,  # 󰚬 U+F06AC
    'timer-sand-empty': 984749,  # 󰚭 U+F06AD
    'transit-transfer': 984750,  # 󰚮 U+F06AE
    'unity': 984751,  # 󰚯 U+F06AF
    'update': 984752,  # 󰚰 U+F06B0
    'watch-vibrate': 984753,  # 󰚱 U+F06B1
    'angular#1': 984754,  # 󰚲 U+F06B2
    'dolby': 984755,  # 󰚳 U+F06B3
    'emby': 984756,  # 󰚴 U+F06B4
    'lamp': 984757,  # 󰚵 U+F06B5
    'menu-down-outline': 984758,  # 󰚶 U+F06B6
    'menu-up-outline': 984759,  # 󰚷 U+F06B7
    'note-multiple': 984760,  # 󰚸 U+F06B8
    'note-multiple-outline': 984761,  # 󰚹 U+F06B9
    'plex': 984762,  # 󰚺 U+F06BA
    'shield-airplane': 984763,  # 󰚻 U+F06BB
    'account-edit': 984764,  # 󰚼 U+F06BC
    'alert-decagram': 984765,  # 󰚽 U+F06BD
    'all-inclusive': 984766,  # 󰚾 U+F06BE
    'angularjs': 984767,  # 󰚿 U+F06BF
    'arrow-down-box': 984768,  # 󰛀 U+F06C0
    'arrow-left-box': 984769,  # 󰛁 U+F06C1
    'arrow-right-box': 984770,  # 󰛂 U+F06C2
    'arrow-up-box': 984771,  # 󰛃 U+F06C3
    'asterisk#1': 984772,  # 󰛄 U+F06C4
    'bomb-off': 984773,  # 󰛅 U+F06C5
    'bootstrap#1': 984774,  # 󰛆 U+F06C6
    'cards-variant': 984775,  # 󰛇 U+F06C7
    'clipboard-flow': 984776,  # 󰛈 U+F06C8
    'close-outline': 984777,  # 󰛉 U+F06C9
    'coffee-outline': 984778,  # 󰛊 U+F06CA
    'contacts': 984779,  # 󰛋 U+F06CB
    'delete-empty': 984780,  # 󰛌 U+F06CC
    'earth-box': 984781,  # 󰛍 U+F06CD
    'earth-box-off': 984782,  # 󰛎 U+F06CE
    'email-alert': 984783,  # 󰛏 U+F06CF
    'eye-outline': 984784,  # 󰛐 U+F06D0
    'eye-off-outline': 984785,  # 󰛑 U+F06D1
    'fast-forward-outline': 984786,  # 󰛒 U+F06D2
    'feather#1': 984787,  # 󰛓 U+F06D3
    'find-replace': 984788,  # 󰛔 U+F06D4
    'flash-outline': 984789,  # 󰛕 U+F06D5
    'format-font': 984790,  # 󰛖 U+F06D6
    'format-page-break': 984791,  # 󰛗 U+F06D7
    'format-pilcrow': 984792,  # 󰛘 U+F06D8
    'garage': 984793,  # 󰛙 U+F06D9
    'garage-open': 984794,  # 󰛚 U+F06DA
    'card-account-details-star-outline': 984795,  # 󰛛 U+F06DB
    'google-keep': 984796,  # 󰛜 U+F06DC
    'snowmobile': 984797,  # 󰛝 U+F06DD
    'heart-half-full': 984798,  # 󰛞 U+F06DE
    'heart-half': 984799,  # 󰛟 U+F06DF
    'heart-half-outline': 984800,  # 󰛠 U+F06E0
    'hexagon-multiple': 984801,  # 󰛡 U+F06E1
    'hook': 984802,  # 󰛢 U+F06E2
    'hook-off': 984803,  # 󰛣 U+F06E3
    'infinity#3': 984804,  # 󰛤 U+F06E4
    'language-swift': 984805,  # 󰛥 U+F06E5
    'language-typescript': 984806,  # 󰛦 U+F06E6
    'laptop-off': 984807,  # 󰛧 U+F06E7
    'lightbulb-on': 984808,  # 󰛨 U+F06E8
    'lightbulb-on-outline': 984809,  # 󰛩 U+F06E9
    'lock-pattern': 984810,  # 󰛪 U+F06EA
    'folder-zip': 984811,  # 󰛫 U+F06EB
    'magnify-minus-outline': 984812,  # 󰛬 U+F06EC
    'magnify-plus-outline': 984813,  # 󰛭 U+F06ED
    'mailbox': 984814,  # 󰛮 U+F06EE
    'medical-bag': 984815,  # 󰛯 U+F06EF
    'message-settings': 984816,  # 󰛰 U+F06F0
    'message-cog': 984817,  # 󰛱 U+F06F1
    'minus-box-outline': 984818,  # 󰛲 U+F06F2
    'network': 984819,  # 󰛳 U+F06F3
    'download-network': 984820,  # 󰛴 U+F06F4
    'help-network': 984821,  # 󰛵 U+F06F5
    'upload-network': 984822,  # 󰛶 U+F06F6
    'npm#1': 984823,  # 󰛷 U+F06F7
    'nut': 984824,  # 󰛸 U+F06F8
    'octagram': 984825,  # 󰛹 U+F06F9
    'page-layout-body': 984826,  # 󰛺 U+F06FA
    'page-layout-footer': 984827,  # 󰛻 U+F06FB
    'page-layout-header': 984828,  # 󰛼 U+F06FC
    'page-layout-sidebar-left': 984829,  # 󰛽 U+F06FD
    'page-layout-sidebar-right': 984830,  # 󰛾 U+F06FE
    'pencil-circle': 984831,  # 󰛿 U+F06FF
    'pentagon-outline': 984832,  # 󰜀 U+F0700
    'pentagon': 984833,  # 󰜁 U+F0701
    'pillar': 984834,  # 󰜂 U+F0702
    'pistol': 984835,  # 󰜃 U+F0703
    'plus-box-outline': 984836,  # 󰜄 U+F0704
    'plus-outline': 984837,  # 󰜅 U+F0705
    'prescription#1': 984838,  # 󰜆 U+F0706
    'printer-settings': 984839,  # 󰜇 U+F0707
    'react#1': 984840,  # 󰜈 U+F0708
    'restart': 984841,  # 󰜉 U+F0709
    'rewind-outline': 984842,  # 󰜊 U+F070A
    'rhombus': 984843,  # 󰜋 U+F070B
    'rhombus-outline': 984844,  # 󰜌 U+F070C
    'robot-vacuum': 984845,  # 󰜍 U+F070D
    'run': 984846,  # 󰜎 U+F070E
    'search-web': 984847,  # 󰜏 U+F070F
    'shovel': 984848,  # 󰜐 U+F0710
    'shovel-off': 984849,  # 󰜑 U+F0711
    'signal-2g': 984850,  # 󰜒 U+F0712
    'signal-3g': 984851,  # 󰜓 U+F0713
    'signal-4g': 984852,  # 󰜔 U+F0714
    'signal-hspa': 984853,  # 󰜕 U+F0715
    'signal-hspa-plus': 984854,  # 󰜖 U+F0716
    'snowflake#1': 984855,  # 󰜗 U+F0717
    'source-commit': 984856,  # 󰜘 U+F0718
    'source-commit-end': 984857,  # 󰜙 U+F0719
    'source-commit-end-local': 984858,  # 󰜚 U+F071A
    'source-commit-local': 984859,  # 󰜛 U+F071B
    'source-commit-next-local': 984860,  # 󰜜 U+F071C
    'source-commit-start': 984861,  # 󰜝 U+F071D
    'source-commit-start-next-local': 984862,  # 󰜞 U+F071E
    'speaker-wireless': 984863,  # 󰜟 U+F071F
    'stadium-variant': 984864,  # 󰜠 U+F0720
    'svg': 984865,  # 󰜡 U+F0721
    'tag-plus': 984866,  # 󰜢 U+F0722
    'tag-remove': 984867,  # 󰜣 U+F0723
    'ticket-percent': 984868,  # 󰜤 U+F0724
    'tilde': 984869,  # 󰜥 U+F0725
    'treasure-chest': 984870,  # 󰜦 U+F0726
    'truck-trailer': 984871,  # 󰜧 U+F0727
    'view-parallel': 984872,  # 󰜨 U+F0728
    'view-sequential': 984873,  # 󰜩 U+F0729
    'washing-machine': 984874,  # 󰜪 U+F072A
    'webpack': 984875,  # 󰜫 U+F072B
    'widgets': 984876,  # 󰜬 U+F072C
    'nintendo-wiiu': 984877,  # 󰜭 U+F072D
    'arrow-down-bold': 984878,  # 󰜮 U+F072E
    'arrow-down-bold-box': 984879,  # 󰜯 U+F072F
    'arrow-down-bold-box-outline': 984880,  # 󰜰 U+F0730
    'arrow-left-bold': 984881,  # 󰜱 U+F0731
    'arrow-left-bold-box': 984882,  # 󰜲 U+F0732
    'arrow-left-bold-box-outline': 984883,  # 󰜳 U+F0733
    'arrow-right-bold': 984884,  # 󰜴 U+F0734
    'arrow-right-bold-box': 984885,  # 󰜵 U+F0735
    'arrow-right-bold-box-outline': 984886,  # 󰜶 U+F0736
    'arrow-up-bold': 984887,  # 󰜷 U+F0737
    'arrow-up-bold-box': 984888,  # 󰜸 U+F0738
    'arrow-up-bold-box-outline': 984889,  # 󰜹 U+F0739
    'cancel': 984890,  # 󰜺 U+F073A
    'file-account': 984891,  # 󰜻 U+F073B
    'gesture-double-tap': 984892,  # 󰜼 U+F073C
    'gesture-swipe-down': 984893,  # 󰜽 U+F073D
    'gesture-swipe-left': 984894,  # 󰜾 U+F073E
    'gesture-swipe-right': 984895,  # 󰜿 U+F073F
    'gesture-swipe-up': 984896,  # 󰝀 U+F0740
    'gesture-tap': 984897,  # 󰝁 U+F0741
    'gesture-two-double-tap': 984898,  # 󰝂 U+F0742
    'gesture-two-tap': 984899,  # 󰝃 U+F0743
    'humble-bundle': 984900,  # 󰝄 U+F0744
    'kickstarter#1': 984901,  # 󰝅 U+F0745
    'netflix': 984902,  # 󰝆 U+F0746
    'microsoft-onenote': 984903,  # 󰝇 U+F0747
    'wall-sconce-round': 984904,  # 󰝈 U+F0748
    'folder-refresh': 984905,  # 󰝉 U+F0749
    'vector-radius': 984906,  # 󰝊 U+F074A
    'microsoft-xbox-controller-battery-alert': 984907,  # 󰝋 U+F074B
    'microsoft-xbox-controller-battery-empty': 984908,  # 󰝌 U+F074C
    'microsoft-xbox-controller-battery-full': 984909,  # 󰝍 U+F074D
    'microsoft-xbox-controller-battery-low': 984910,  # 󰝎 U+F074E
    'microsoft-xbox-controller-battery-medium': 984911,  # 󰝏 U+F074F
    'microsoft-xbox-controller-battery-medium': 984911,  # 󰝏 U+F074F
    'microsoft-xbox-controller-battery-unknown': 984912,  # 󰝐 U+F0750
    'clipboard-plus': 984913,  # 󰝑 U+F0751
    'file-plus': 984914,  # 󰝒 U+F0752
    'format-align-bottom': 984915,  # 󰝓 U+F0753
    'format-align-middle': 984916,  # 󰝔 U+F0754
    'format-align-top': 984917,  # 󰝕 U+F0755
    'format-list-checks': 984918,  # 󰝖 U+F0756
    'format-quote-open': 984919,  # 󰝗 U+F0757
    'grid-large': 984920,  # 󰝘 U+F0758
    'heart-off': 984921,  # 󰝙 U+F0759
    'music#2': 984922,  # 󰝚 U+F075A
    'music-off': 984923,  # 󰝛 U+F075B
    'tab-plus': 984924,  # 󰝜 U+F075C
    'volume-plus': 984925,  # 󰝝 U+F075D
    'volume-minus': 984926,  # 󰝞 U+F075E
    'volume-mute': 984927,  # 󰝟 U+F075F
    'unfold-less-vertical': 984928,  # 󰝠 U+F0760
    'unfold-more-vertical': 984929,  # 󰝡 U+F0761
    'taco': 984930,  # 󰝢 U+F0762
    'square-outline': 984931,  # 󰝣 U+F0763
    'square#2': 984932,  # 󰝤 U+F0764
    'checkbox-blank-circle#1': 984933,  # 󰝥 U+F0765
    'checkbox-blank-circle-outline#2': 984934,  # 󰝦 U+F0766
    'alert-octagram': 984935,  # 󰝧 U+F0767
    'atom#2': 984936,  # 󰝨 U+F0768
    'ceiling-light': 984937,  # 󰝩 U+F0769
    'chart-bar-stacked': 984938,  # 󰝪 U+F076A
    'chart-line-stacked': 984939,  # 󰝫 U+F076B
    'decagram': 984940,  # 󰝬 U+F076C
    'decagram-outline': 984941,  # 󰝭 U+F076D
    'dice-multiple': 984942,  # 󰝮 U+F076E
    'dice-d10-outline': 984943,  # 󰝯 U+F076F
    'folder-open': 984944,  # 󰝰 U+F0770
    'guitar-acoustic': 984945,  # 󰝱 U+F0771
    'loading#1': 984946,  # 󰝲 U+F0772
    'lock-reset': 984947,  # 󰝳 U+F0773
    'ninja': 984948,  # 󰝴 U+F0774
    'octagram-outline': 984949,  # 󰝵 U+F0775
    'pencil-circle-outline': 984950,  # 󰝶 U+F0776
    'selection-off': 984951,  # 󰝷 U+F0777
    'set-all': 984952,  # 󰝸 U+F0778
    'set-center': 984953,  # 󰝹 U+F0779
    'set-center-right': 984954,  # 󰝺 U+F077A
    'set-left': 984955,  # 󰝻 U+F077B
    'set-left-center': 984956,  # 󰝼 U+F077C
    'set-left-right': 984957,  # 󰝽 U+F077D
    'set-none': 984958,  # 󰝾 U+F077E
    'set-right': 984959,  # 󰝿 U+F077F
    'shield-half-full': 984960,  # 󰞀 U+F0780
    'sign-direction': 984961,  # 󰞁 U+F0781
    'sign-text': 984962,  # 󰞂 U+F0782
    'signal-off': 984963,  # 󰞃 U+F0783
    'square-root': 984964,  # 󰞄 U+F0784
    'sticker-emoji': 984965,  # 󰞅 U+F0785
    'summit': 984966,  # 󰞆 U+F0786
    'sword-cross': 984967,  # 󰞇 U+F0787
    'truck-fast': 984968,  # 󰞈 U+F0788
    'web-check': 984969,  # 󰞉 U+F0789
    'cast-off': 984970,  # 󰞊 U+F078A
    'help-box': 984971,  # 󰞋 U+F078B
    'timer-sand-full': 984972,  # 󰞌 U+F078C
    'waves': 984973,  # 󰞍 U+F078D
    'alarm-bell': 984974,  # 󰞎 U+F078E
    'alarm-light': 984975,  # 󰞏 U+F078F
    'video-switch-outline': 984976,  # 󰞐 U+F0790
    'check-decagram': 984977,  # 󰞑 U+F0791
    'arrow-collapse-down': 984978,  # 󰞒 U+F0792
    'arrow-collapse-left': 984979,  # 󰞓 U+F0793
    'arrow-collapse-right': 984980,  # 󰞔 U+F0794
    'arrow-collapse-up': 984981,  # 󰞕 U+F0795
    'arrow-expand-down': 984982,  # 󰞖 U+F0796
    'arrow-expand-left': 984983,  # 󰞗 U+F0797
    'arrow-expand-right': 984984,  # 󰞘 U+F0798
    'arrow-expand-up': 984985,  # 󰞙 U+F0799
    'book-lock': 984986,  # 󰞚 U+F079A
    'book-lock-open': 984987,  # 󰞛 U+F079B
    'bus-articulated-end': 984988,  # 󰞜 U+F079C
    'bus-articulated-front': 984989,  # 󰞝 U+F079D
    'bus-double-decker': 984990,  # 󰞞 U+F079E
    'bus-school': 984991,  # 󰞟 U+F079F
    'bus-side': 984992,  # 󰞠 U+F07A0
    'camera-gopro': 984993,  # 󰞡 U+F07A1
    'camera-metering-center': 984994,  # 󰞢 U+F07A2
    'camera-metering-matrix': 984995,  # 󰞣 U+F07A3
    'camera-metering-partial': 984996,  # 󰞤 U+F07A4
    'camera-metering-spot': 984997,  # 󰞥 U+F07A5
    'cannabis#1': 984998,  # 󰞦 U+F07A6
    'car-convertible': 984999,  # 󰞧 U+F07A7
    'car-estate': 985000,  # 󰞨 U+F07A8
    'car-hatchback': 985001,  # 󰞩 U+F07A9
    'car-pickup': 985002,  # 󰞪 U+F07AA
    'car-side': 985003,  # 󰞫 U+F07AB
    'car-sports': 985004,  # 󰞬 U+F07AC
    'caravan#1': 985005,  # 󰞭 U+F07AD
    'cctv': 985006,  # 󰞮 U+F07AE
    'chart-donut': 985007,  # 󰞯 U+F07AF
    'chart-donut-variant': 985008,  # 󰞰 U+F07B0
    'chart-line-variant': 985009,  # 󰞱 U+F07B1
    'chili-hot': 985010,  # 󰞲 U+F07B2
    'chili-medium': 985011,  # 󰞳 U+F07B3
    'chili-mild': 985012,  # 󰞴 U+F07B4
    'cloud-braces': 985013,  # 󰞵 U+F07B5
    'cloud-tags': 985014,  # 󰞶 U+F07B6
    'console-line': 985015,  # 󰞷 U+F07B7
    'corn': 985016,  # 󰞸 U+F07B8
    'folder-zip-outline': 985017,  # 󰞹 U+F07B9
    'currency-cny': 985018,  # 󰞺 U+F07BA
    'currency-eth': 985019,  # 󰞻 U+F07BB
    'currency-jpy': 985020,  # 󰞼 U+F07BC
    'currency-krw': 985021,  # 󰞽 U+F07BD
    'currency-sign': 985022,  # 󰞾 U+F07BE
    'currency-twd': 985023,  # 󰞿 U+F07BF
    'desktop-classic': 985024,  # 󰟀 U+F07C0
    'dip-switch': 985025,  # 󰟁 U+F07C1
    'donkey': 985026,  # 󰟂 U+F07C2
    'dots-horizontal-circle': 985027,  # 󰟃 U+F07C3
    'dots-vertical-circle': 985028,  # 󰟄 U+F07C4
    'ear-hearing': 985029,  # 󰟅 U+F07C5
    'elephant': 985030,  # 󰟆 U+F07C6
    'storefront': 985031,  # 󰟇 U+F07C7
    'food-croissant': 985032,  # 󰟈 U+F07C8
    'forklift': 985033,  # 󰟉 U+F07C9
    'fuel': 985034,  # 󰟊 U+F07CA
    'gesture': 985035,  # 󰟋 U+F07CB
    'google-analytics': 985036,  # 󰟌 U+F07CC
    'google-assistant': 985037,  # 󰟍 U+F07CD
    'headphones-off': 985038,  # 󰟎 U+F07CE
    'high-definition': 985039,  # 󰟏 U+F07CF
    'home-assistant': 985040,  # 󰟐 U+F07D0
    'home-automation': 985041,  # 󰟑 U+F07D1
    'home-circle': 985042,  # 󰟒 U+F07D2
    'language-go': 985043,  # 󰟓 U+F07D3
    'language-r': 985044,  # 󰟔 U+F07D4
    'lava-lamp': 985045,  # 󰟕 U+F07D5
    'led-strip': 985046,  # 󰟖 U+F07D6
    'locker': 985047,  # 󰟗 U+F07D7
    'locker-multiple': 985048,  # 󰟘 U+F07D8
    'map-marker-outline': 985049,  # 󰟙 U+F07D9
    'metronome': 985050,  # 󰟚 U+F07DA
    'metronome-tick': 985051,  # 󰟛 U+F07DB
    'micro-sd': 985052,  # 󰟜 U+F07DC
    'facebook-gaming': 985053,  # 󰟝 U+F07DD
    'movie-roll': 985054,  # 󰟞 U+F07DE
    'mushroom#1': 985055,  # 󰟟 U+F07DF
    'mushroom-outline': 985056,  # 󰟠 U+F07E0
    'nintendo-switch': 985057,  # 󰟡 U+F07E1
    'null': 985058,  # 󰟢 U+F07E2
    'passport#1': 985059,  # 󰟣 U+F07E3
    'molecule-co2': 985060,  # 󰟤 U+F07E4
    'pipe': 985061,  # 󰟥 U+F07E5
    'pipe-disconnected': 985062,  # 󰟦 U+F07E6
    'power-socket-eu': 985063,  # 󰟧 U+F07E7
    'power-socket-uk': 985064,  # 󰟨 U+F07E8
    'power-socket-us': 985065,  # 󰟩 U+F07E9
    'rice': 985066,  # 󰟪 U+F07EA
    'ring#2': 985067,  # 󰟫 U+F07EB
    'sass#1': 985068,  # 󰟬 U+F07EC
    'send-lock': 985069,  # 󰟭 U+F07ED
    'soy-sauce': 985070,  # 󰟮 U+F07EE
    'standard-definition': 985071,  # 󰟯 U+F07EF
    'surround-sound-2-0': 985072,  # 󰟰 U+F07F0
    'surround-sound-3-1': 985073,  # 󰟱 U+F07F1
    'surround-sound-5-1': 985074,  # 󰟲 U+F07F2
    'surround-sound-7-1': 985075,  # 󰟳 U+F07F3
    'television-classic': 985076,  # 󰟴 U+F07F4
    'form-textbox-password': 985077,  # 󰟵 U+F07F5
    'thought-bubble': 985078,  # 󰟶 U+F07F6
    'thought-bubble-outline': 985079,  # 󰟷 U+F07F7
    'trackpad': 985080,  # 󰟸 U+F07F8
    'ultra-high-definition': 985081,  # 󰟹 U+F07F9
    'van-passenger': 985082,  # 󰟺 U+F07FA
    'van-utility': 985083,  # 󰟻 U+F07FB
    'vanish': 985084,  # 󰟼 U+F07FC
    'video-3d': 985085,  # 󰟽 U+F07FD
    'wall': 985086,  # 󰟾 U+F07FE
    'xmpp': 985087,  # 󰟿 U+F07FF
    'account-multiple-plus-outline': 985088,  # 󰠀 U+F0800
    'account-plus-outline': 985089,  # 󰠁 U+F0801
    'credit-card-wireless': 985090,  # 󰠂 U+F0802
    'account-music': 985091,  # 󰠃 U+F0803
    'atlassian#1': 985092,  # 󰠄 U+F0804
    'microsoft-azure': 985093,  # 󰠅 U+F0805
    'basketball#1': 985094,  # 󰠆 U+F0806
    'battery-charging-wireless': 985095,  # 󰠇 U+F0807
    'battery-charging-wireless-10': 985096,  # 󰠈 U+F0808
    'battery-charging-wireless-20': 985097,  # 󰠉 U+F0809
    'battery-charging-wireless-30': 985098,  # 󰠊 U+F080A
    'battery-charging-wireless-40': 985099,  # 󰠋 U+F080B
    'battery-charging-wireless-50': 985100,  # 󰠌 U+F080C
    'battery-charging-wireless-60': 985101,  # 󰠍 U+F080D
    'battery-charging-wireless-70': 985102,  # 󰠎 U+F080E
    'battery-charging-wireless-80': 985103,  # 󰠏 U+F080F
    'battery-charging-wireless-90': 985104,  # 󰠐 U+F0810
    'battery-charging-wireless-alert': 985105,  # 󰠑 U+F0811
    'battery-charging-wireless-outline': 985106,  # 󰠒 U+F0812
    'bitcoin#1': 985107,  # 󰠓 U+F0813
    'briefcase-outline': 985108,  # 󰠔 U+F0814
    'cellphone-wireless': 985109,  # 󰠕 U+F0815
    'clover': 985110,  # 󰠖 U+F0816
    'comment-question': 985111,  # 󰠗 U+F0817
    'content-save-outline': 985112,  # 󰠘 U+F0818
    'delete-restore': 985113,  # 󰠙 U+F0819
    'door': 985114,  # 󰠚 U+F081A
    'door-closed': 985115,  # 󰠛 U+F081B
    'door-open': 985116,  # 󰠜 U+F081C
    'fan-off': 985117,  # 󰠝 U+F081D
    'file-percent': 985118,  # 󰠞 U+F081E
    'finance': 985119,  # 󰠟 U+F081F
    'lightning-bolt-circle': 985120,  # 󰠠 U+F0820
    'floor-plan': 985121,  # 󰠡 U+F0821
    'forum-outline': 985122,  # 󰠢 U+F0822
    'golf': 985123,  # 󰠣 U+F0823
    'google-home': 985124,  # 󰠤 U+F0824
    'guy-fawkes-mask': 985125,  # 󰠥 U+F0825
    'home-account': 985126,  # 󰠦 U+F0826
    'home-heart': 985127,  # 󰠧 U+F0827
    'hot-tub': 985128,  # 󰠨 U+F0828
    'hulu': 985129,  # 󰠩 U+F0829
    'ice-cream#1': 985130,  # 󰠪 U+F082A
    'image-off': 985131,  # 󰠫 U+F082B
    'karate': 985132,  # 󰠬 U+F082C
    'ladybug': 985133,  # 󰠭 U+F082D
    'notebook#1': 985134,  # 󰠮 U+F082E
    'phone-return': 985135,  # 󰠯 U+F082F
    'poker-chip': 985136,  # 󰠰 U+F0830
    'shape': 985137,  # 󰠱 U+F0831
    'shape-outline': 985138,  # 󰠲 U+F0832
    'ship-wheel': 985139,  # 󰠳 U+F0833
    'soccer-field': 985140,  # 󰠴 U+F0834
    'table-column': 985141,  # 󰠵 U+F0835
    'table-of-contents': 985142,  # 󰠶 U+F0836
    'table-row': 985143,  # 󰠷 U+F0837
    'table-settings': 985144,  # 󰠸 U+F0838
    'television-box': 985145,  # 󰠹 U+F0839
    'television-classic-off': 985146,  # 󰠺 U+F083A
    'television-off': 985147,  # 󰠻 U+F083B
    'tow-truck': 985148,  # 󰠼 U+F083C
    'upload-multiple': 985149,  # 󰠽 U+F083D
    'video-4k-box': 985150,  # 󰠾 U+F083E
    'video-input-antenna': 985151,  # 󰠿 U+F083F
    'video-input-component': 985152,  # 󰡀 U+F0840
    'video-input-hdmi': 985153,  # 󰡁 U+F0841
    'video-input-svideo': 985154,  # 󰡂 U+F0842
    'view-dashboard-variant': 985155,  # 󰡃 U+F0843
    'vuejs#1': 985156,  # 󰡄 U+F0844
    'xamarin': 985157,  # 󰡅 U+F0845
    'human-male-board-poll': 985158,  # 󰡆 U+F0846
    'youtube-studio': 985159,  # 󰡇 U+F0847
    'youtube-gaming': 985160,  # 󰡈 U+F0848
    'account-group': 985161,  # 󰡉 U+F0849
    'camera-switch-outline': 985162,  # 󰡊 U+F084A
    'airport': 985163,  # 󰡋 U+F084B
    'arrow-collapse-horizontal': 985164,  # 󰡌 U+F084C
    'arrow-collapse-vertical': 985165,  # 󰡍 U+F084D
    'arrow-expand-horizontal': 985166,  # 󰡎 U+F084E
    'arrow-expand-vertical': 985167,  # 󰡏 U+F084F
    'augmented-reality': 985168,  # 󰡐 U+F0850
    'badminton': 985169,  # 󰡑 U+F0851
    'baseball#1': 985170,  # 󰡒 U+F0852
    'baseball-bat': 985171,  # 󰡓 U+F0853
    'bottle-wine': 985172,  # 󰡔 U+F0854
    'check-outline': 985173,  # 󰡕 U+F0855
    'checkbox-intermediate': 985174,  # 󰡖 U+F0856
    'chess-king#1': 985175,  # 󰡗 U+F0857
    'chess-knight': 985176,  # 󰡘 U+F0858
    'chess-pawn#1': 985177,  # 󰡙 U+F0859
    'chess-queen#1': 985178,  # 󰡚 U+F085A
    'chess-rook': 985179,  # 󰡛 U+F085B
    'chess-bishop#1': 985180,  # 󰡜 U+F085C
    'clipboard-pulse': 985181,  # 󰡝 U+F085D
    'clipboard-pulse-outline': 985182,  # 󰡞 U+F085E
    'comment-multiple': 985183,  # 󰡟 U+F085F
    'comment-text-multiple': 985184,  # 󰡠 U+F0860
    'comment-text-multiple-outline': 985185,  # 󰡡 U+F0861
    'crane': 985186,  # 󰡢 U+F0862
    'curling': 985187,  # 󰡣 U+F0863
    'currency-bdt': 985188,  # 󰡤 U+F0864
    'currency-kzt': 985189,  # 󰡥 U+F0865
    'database-search': 985190,  # 󰡦 U+F0866
    'dice-d12-outline': 985191,  # 󰡧 U+F0867
    'docker#1': 985192,  # 󰡨 U+F0868
    'doorbell-video': 985193,  # 󰡩 U+F0869
    'ethereum#1': 985194,  # 󰡪 U+F086A
    'eye-plus': 985195,  # 󰡫 U+F086B
    'eye-plus-outline': 985196,  # 󰡬 U+F086C
    'eye-settings': 985197,  # 󰡭 U+F086D
    'eye-settings-outline': 985198,  # 󰡮 U+F086E
    'file-question': 985199,  # 󰡯 U+F086F
    'folder-network': 985200,  # 󰡰 U+F0870
    'function-variant': 985201,  # 󰡱 U+F0871
    'garage-alert': 985202,  # 󰡲 U+F0872
    'gauge-empty': 985203,  # 󰡳 U+F0873
    'gauge-full': 985204,  # 󰡴 U+F0874
    'gauge-low': 985205,  # 󰡵 U+F0875
    'glass-wine': 985206,  # 󰡶 U+F0876
    'graphql': 985207,  # 󰡷 U+F0877
    'high-definition-box': 985208,  # 󰡸 U+F0878
    'hockey-puck': 985209,  # 󰡹 U+F0879
    'hockey-sticks': 985210,  # 󰡺 U+F087A
    'home-alert': 985211,  # 󰡻 U+F087B
    'image-plus': 985212,  # 󰡼 U+F087C
    'jquery': 985213,  # 󰡽 U+F087D
    'lifebuoy': 985214,  # 󰡾 U+F087E
    'mixed-reality': 985215,  # 󰡿 U+F087F
    'nativescript': 985216,  # 󰢀 U+F0880
    'onepassword': 985217,  # 󰢁 U+F0881
    'patreon#1': 985218,  # 󰢂 U+F0882
    'close-circle-multiple-outline': 985219,  # 󰢃 U+F0883
    'peace#1': 985220,  # 󰢄 U+F0884
    'phone-rotate-landscape': 985221,  # 󰢅 U+F0885
    'phone-rotate-portrait': 985222,  # 󰢆 U+F0886
    'pier': 985223,  # 󰢇 U+F0887
    'pier-crane': 985224,  # 󰢈 U+F0888
    'pipe-leak': 985225,  # 󰢉 U+F0889
    'piston': 985226,  # 󰢊 U+F088A
    'play-network': 985227,  # 󰢋 U+F088B
    'reminder': 985228,  # 󰢌 U+F088C
    'room-service': 985229,  # 󰢍 U+F088D
    'salesforce#1': 985230,  # 󰢎 U+F088E
    'shield-account': 985231,  # 󰢏 U+F088F
    'human-male-board': 985232,  # 󰢐 U+F0890
    'thermostat-box': 985233,  # 󰢑 U+F0891
    'tractor#1': 985234,  # 󰢒 U+F0892
    'vector-ellipse': 985235,  # 󰢓 U+F0893
    'virtual-reality': 985236,  # 󰢔 U+F0894
    'watch-export-variant': 985237,  # 󰢕 U+F0895
    'watch-import-variant': 985238,  # 󰢖 U+F0896
    'watch-variant': 985239,  # 󰢗 U+F0897
    'weather-hurricane': 985240,  # 󰢘 U+F0898
    'account-heart': 985241,  # 󰢙 U+F0899
    'alien': 985242,  # 󰢚 U+F089A
    'anvil': 985243,  # 󰢛 U+F089B
    'battery-charging-10': 985244,  # 󰢜 U+F089C
    'battery-charging-50': 985245,  # 󰢝 U+F089D
    'battery-charging-70': 985246,  # 󰢞 U+F089E
    'battery-charging-outline': 985247,  # 󰢟 U+F089F
    'bed-empty': 985248,  # 󰢠 U+F08A0
    'border-all-variant': 985249,  # 󰢡 U+F08A1
    'border-bottom-variant': 985250,  # 󰢢 U+F08A2
    'border-left-variant': 985251,  # 󰢣 U+F08A3
    'border-none-variant': 985252,  # 󰢤 U+F08A4
    'border-right-variant': 985253,  # 󰢥 U+F08A5
    'border-top-variant': 985254,  # 󰢦 U+F08A6
    'calendar-edit': 985255,  # 󰢧 U+F08A7
    'clipboard-check-outline': 985256,  # 󰢨 U+F08A8
    'console-network': 985257,  # 󰢩 U+F08A9
    'file-compare': 985258,  # 󰢪 U+F08AA
    'fire-truck': 985259,  # 󰢫 U+F08AB
    'folder-key': 985260,  # 󰢬 U+F08AC
    'folder-key-network': 985261,  # 󰢭 U+F08AD
    'expansion-card': 985262,  # 󰢮 U+F08AE
    'kayaking': 985263,  # 󰢯 U+F08AF
    'inbox-multiple': 985264,  # 󰢰 U+F08B0
    'language-lua': 985265,  # 󰢱 U+F08B1
    'lock-smart': 985266,  # 󰢲 U+F08B2
    'microphone-minus': 985267,  # 󰢳 U+F08B3
    'microphone-plus': 985268,  # 󰢴 U+F08B4
    'palette-swatch': 985269,  # 󰢵 U+F08B5
    'periodic-table': 985270,  # 󰢶 U+F08B6
    'pickaxe': 985271,  # 󰢷 U+F08B7
    'qrcode-edit': 985272,  # 󰢸 U+F08B8
    'remote-desktop': 985273,  # 󰢹 U+F08B9
    'sausage': 985274,  # 󰢺 U+F08BA
    'cog-outline': 985275,  # 󰢻 U+F08BB
    'signal-cellular-1': 985276,  # 󰢼 U+F08BC
    'signal-cellular-2': 985277,  # 󰢽 U+F08BD
    'signal-cellular-3': 985278,  # 󰢾 U+F08BE
    'signal-cellular-outline': 985279,  # 󰢿 U+F08BF
    'ssh': 985280,  # 󰣀 U+F08C0
    'swap-horizontal-variant': 985281,  # 󰣁 U+F08C1
    'swap-vertical-variant': 985282,  # 󰣂 U+F08C2
    'tooth#2': 985283,  # 󰣃 U+F08C3
    'train-variant': 985284,  # 󰣄 U+F08C4
    'account-multiple-check': 985285,  # 󰣅 U+F08C5
    'application': 985286,  # 󰣆 U+F08C6
    'arch': 985287,  # 󰣇 U+F08C7
    'axe': 985288,  # 󰣈 U+F08C8
    'bullseye-arrow': 985289,  # 󰣉 U+F08C9
    'bus-clock': 985290,  # 󰣊 U+F08CA
    'camera-account': 985291,  # 󰣋 U+F08CB
    'camera-image': 985292,  # 󰣌 U+F08CC
    'car-limousine': 985293,  # 󰣍 U+F08CD
    'cards-club': 985294,  # 󰣎 U+F08CE
    'cards-diamond': 985295,  # 󰣏 U+F08CF
    'heart#4': 985296,  # 󰣐 U+F08D0
    'cards-spade': 985297,  # 󰣑 U+F08D1
    'cellphone-text': 985298,  # 󰣒 U+F08D2
    'cellphone-message': 985299,  # 󰣓 U+F08D3
    'chart-multiline': 985300,  # 󰣔 U+F08D4
    'circle-edit-outline': 985301,  # 󰣕 U+F08D5
    'cogs': 985302,  # 󰣖 U+F08D6
    'credit-card-settings-outline': 985303,  # 󰣗 U+F08D7
    'death-star': 985304,  # 󰣘 U+F08D8
    'death-star-variant': 985305,  # 󰣙 U+F08D9
    'debian': 985306,  # 󰣚 U+F08DA
    'fedora#1': 985307,  # 󰣛 U+F08DB
    'file-undo': 985308,  # 󰣜 U+F08DC
    'floor-lamp': 985309,  # 󰣝 U+F08DD
    'folder-edit': 985310,  # 󰣞 U+F08DE
    'format-columns': 985311,  # 󰣟 U+F08DF
    'freebsd#1': 985312,  # 󰣠 U+F08E0
    'gate-and': 985313,  # 󰣡 U+F08E1
    'gate-nand': 985314,  # 󰣢 U+F08E2
    'gate-nor': 985315,  # 󰣣 U+F08E3
    'gate-not': 985316,  # 󰣤 U+F08E4
    'gate-or': 985317,  # 󰣥 U+F08E5
    'gate-xnor': 985318,  # 󰣦 U+F08E6
    'gate-xor': 985319,  # 󰣧 U+F08E7
    'gentoo': 985320,  # 󰣨 U+F08E8
    'globe-model': 985321,  # 󰣩 U+F08E9
    'hammer#1': 985322,  # 󰣪 U+F08EA
    'home-lock': 985323,  # 󰣫 U+F08EB
    'home-lock-open': 985324,  # 󰣬 U+F08EC
    'linux-mint': 985325,  # 󰣭 U+F08ED
    'lock-alert': 985326,  # 󰣮 U+F08EE
    'lock-question': 985327,  # 󰣯 U+F08EF
    'map-marker-distance': 985328,  # 󰣰 U+F08F0
    'midi': 985329,  # 󰣱 U+F08F1
    'midi-port': 985330,  # 󰣲 U+F08F2
    'nas': 985331,  # 󰣳 U+F08F3
    'network-strength-1': 985332,  # 󰣴 U+F08F4
    'network-strength-1-alert': 985333,  # 󰣵 U+F08F5
    'network-strength-2': 985334,  # 󰣶 U+F08F6
    'network-strength-2-alert': 985335,  # 󰣷 U+F08F7
    'network-strength-3': 985336,  # 󰣸 U+F08F8
    'network-strength-3-alert': 985337,  # 󰣹 U+F08F9
    'network-strength-4': 985338,  # 󰣺 U+F08FA
    'network-strength-4-alert': 985339,  # 󰣻 U+F08FB
    'network-strength-off': 985340,  # 󰣼 U+F08FC
    'network-strength-off-outline': 985341,  # 󰣽 U+F08FD
    'network-strength-outline': 985342,  # 󰣾 U+F08FE
    'play-speed': 985343,  # 󰣿 U+F08FF
    'playlist-edit': 985344,  # 󰤀 U+F0900
    'power-cycle': 985345,  # 󰤁 U+F0901
    'power-off': 985346,  # 󰤂 U+F0902
    'power-on': 985347,  # 󰤃 U+F0903
    'power-sleep': 985348,  # 󰤄 U+F0904
    'power-socket-au': 985349,  # 󰤅 U+F0905
    'power-standby': 985350,  # 󰤆 U+F0906
    'rabbit': 985351,  # 󰤇 U+F0907
    'robot-vacuum-variant': 985352,  # 󰤈 U+F0908
    'satellite-uplink': 985353,  # 󰤉 U+F0909
    'scanner-off': 985354,  # 󰤊 U+F090A
    'book-minus-multiple-outline': 985355,  # 󰤋 U+F090B
    'square-edit-outline': 985356,  # 󰤌 U+F090C
    'sort-numeric-ascending-variant': 985357,  # 󰤍 U+F090D
    'steering-off': 985358,  # 󰤎 U+F090E
    'table-search': 985359,  # 󰤏 U+F090F
    'tag-minus': 985360,  # 󰤐 U+F0910
    'test-tube-empty': 985361,  # 󰤑 U+F0911
    'test-tube-off': 985362,  # 󰤒 U+F0912
    'ticket-outline': 985363,  # 󰤓 U+F0913
    'track-light': 985364,  # 󰤔 U+F0914
    'transition': 985365,  # 󰤕 U+F0915
    'transition-masked': 985366,  # 󰤖 U+F0916
    'tumble-dryer': 985367,  # 󰤗 U+F0917
    'file-refresh': 985368,  # 󰤘 U+F0918
    'video-account': 985369,  # 󰤙 U+F0919
    'video-image': 985370,  # 󰤚 U+F091A
    'video-stabilization': 985371,  # 󰤛 U+F091B
    'wall-sconce': 985372,  # 󰤜 U+F091C
    'wall-sconce-flat': 985373,  # 󰤝 U+F091D
    'wall-sconce-round-variant': 985374,  # 󰤞 U+F091E
    'wifi-strength-1': 985375,  # 󰤟 U+F091F
    'wifi-strength-1-alert': 985376,  # 󰤠 U+F0920
    'wifi-strength-1-lock': 985377,  # 󰤡 U+F0921
    'wifi-strength-2': 985378,  # 󰤢 U+F0922
    'wifi-strength-2-alert': 985379,  # 󰤣 U+F0923
    'wifi-strength-2-lock': 985380,  # 󰤤 U+F0924
    'wifi-strength-3': 985381,  # 󰤥 U+F0925
    'wifi-strength-3-alert': 985382,  # 󰤦 U+F0926
    'wifi-strength-3-lock': 985383,  # 󰤧 U+F0927
    'wifi-strength-4': 985384,  # 󰤨 U+F0928
    'wifi-strength-4-alert': 985385,  # 󰤩 U+F0929
    'wifi-strength-4-lock': 985386,  # 󰤪 U+F092A
    'wifi-strength-alert-outline': 985387,  # 󰤫 U+F092B
    'wifi-strength-lock-outline': 985388,  # 󰤬 U+F092C
    'wifi-strength-off': 985389,  # 󰤭 U+F092D
    'wifi-strength-off-outline': 985390,  # 󰤮 U+F092E
    'wifi-strength-outline': 985391,  # 󰤯 U+F092F
    'pin-off-outline': 985392,  # 󰤰 U+F0930
    'pin-outline': 985393,  # 󰤱 U+F0931
    'share-outline': 985394,  # 󰤲 U+F0932
    'trackpad-lock': 985395,  # 󰤳 U+F0933
    'account-box-multiple': 985396,  # 󰤴 U+F0934
    'account-search-outline': 985397,  # 󰤵 U+F0935
    'account-filter': 985398,  # 󰤶 U+F0936
    'angle-acute': 985399,  # 󰤷 U+F0937
    'angle-obtuse': 985400,  # 󰤸 U+F0938
    'angle-right': 985401,  # 󰤹 U+F0939
    'animation-play': 985402,  # 󰤺 U+F093A
    'arrow-split-horizontal': 985403,  # 󰤻 U+F093B
    'arrow-split-vertical': 985404,  # 󰤼 U+F093C
    'audio-video': 985405,  # 󰤽 U+F093D
    'battery-10-bluetooth': 985406,  # 󰤾 U+F093E
    'battery-20-bluetooth': 985407,  # 󰤿 U+F093F
    'battery-30-bluetooth': 985408,  # 󰥀 U+F0940
    'battery-40-bluetooth': 985409,  # 󰥁 U+F0941
    'battery-50-bluetooth': 985410,  # 󰥂 U+F0942
    'battery-60-bluetooth': 985411,  # 󰥃 U+F0943
    'battery-70-bluetooth': 985412,  # 󰥄 U+F0944
    'battery-80-bluetooth': 985413,  # 󰥅 U+F0945
    'battery-90-bluetooth': 985414,  # 󰥆 U+F0946
    'battery-alert-bluetooth': 985415,  # 󰥇 U+F0947
    'battery-bluetooth': 985416,  # 󰥈 U+F0948
    'battery-bluetooth-variant': 985417,  # 󰥉 U+F0949
    'battery-unknown-bluetooth': 985418,  # 󰥊 U+F094A
    'dharmachakra#1': 985419,  # 󰥋 U+F094B
    'calendar-search': 985420,  # 󰥌 U+F094C
    'cellphone-remove': 985421,  # 󰥍 U+F094D
    'cellphone-key': 985422,  # 󰥎 U+F094E
    'cellphone-lock': 985423,  # 󰥏 U+F094F
    'cellphone-off': 985424,  # 󰥐 U+F0950
    'cellphone-cog': 985425,  # 󰥑 U+F0951
    'cellphone-sound': 985426,  # 󰥒 U+F0952
    'cross#1': 985427,  # 󰥓 U+F0953
    'clock#2': 985428,  # 󰥔 U+F0954
    'clock-alert': 985429,  # 󰥕 U+F0955
    'cloud-search': 985430,  # 󰥖 U+F0956
    'cloud-search-outline': 985431,  # 󰥗 U+F0957
    'cordova': 985432,  # 󰥘 U+F0958
    'cryengine': 985433,  # 󰥙 U+F0959
    'cupcake': 985434,  # 󰥚 U+F095A
    'sine-wave': 985435,  # 󰥛 U+F095B
    'current-dc': 985436,  # 󰥜 U+F095C
    'database-import': 985437,  # 󰥝 U+F095D
    'database-export': 985438,  # 󰥞 U+F095E
    'desk-lamp': 985439,  # 󰥟 U+F095F
    'disc-player': 985440,  # 󰥠 U+F0960
    'email-search': 985441,  # 󰥡 U+F0961
    'email-search-outline': 985442,  # 󰥢 U+F0962
    'exponent': 985443,  # 󰥣 U+F0963
    'exponent-box': 985444,  # 󰥤 U+F0964
    'file-download': 985445,  # 󰥥 U+F0965
    'file-download-outline': 985446,  # 󰥦 U+F0966
    'firebase': 985447,  # 󰥧 U+F0967
    'folder-search': 985448,  # 󰥨 U+F0968
    'folder-search-outline': 985449,  # 󰥩 U+F0969
    'format-list-checkbox': 985450,  # 󰥪 U+F096A
    'fountain': 985451,  # 󰥫 U+F096B
    'google-fit': 985452,  # 󰥬 U+F096C
    'greater-than': 985453,  # 󰥭 U+F096D
    'greater-than-or-equal': 985454,  # 󰥮 U+F096E
    'hard-hat': 985455,  # 󰥯 U+F096F
    'headphones-bluetooth': 985456,  # 󰥰 U+F0970
    'heart-circle': 985457,  # 󰥱 U+F0971
    'heart-circle-outline': 985458,  # 󰥲 U+F0972
    'om#1': 985459,  # 󰥳 U+F0973
    'home-minus': 985460,  # 󰥴 U+F0974
    'home-plus': 985461,  # 󰥵 U+F0975
    'image-outline': 985462,  # 󰥶 U+F0976
    'image-search': 985463,  # 󰥷 U+F0977
    'image-search-outline': 985464,  # 󰥸 U+F0978
    'star-crescent': 985465,  # 󰥹 U+F0979
    'star-david': 985466,  # 󰥺 U+F097A
    'keyboard-outline': 985467,  # 󰥻 U+F097B
    'less-than': 985468,  # 󰥼 U+F097C
    'less-than-or-equal': 985469,  # 󰥽 U+F097D
    'light-switch': 985470,  # 󰥾 U+F097E
    'lock-clock': 985471,  # 󰥿 U+F097F
    'magnify-close': 985472,  # 󰦀 U+F0980
    'map-minus': 985473,  # 󰦁 U+F0981
    'map-outline': 985474,  # 󰦂 U+F0982
    'map-plus': 985475,  # 󰦃 U+F0983
    'map-search': 985476,  # 󰦄 U+F0984
    'map-search-outline': 985477,  # 󰦅 U+F0985
    'material-design': 985478,  # 󰦆 U+F0986
    'medal#1': 985479,  # 󰦇 U+F0987
    'microsoft-dynamics-365': 985480,  # 󰦈 U+F0988
    'monitor-cellphone': 985481,  # 󰦉 U+F0989
    'monitor-cellphone-star': 985482,  # 󰦊 U+F098A
    'mouse-bluetooth': 985483,  # 󰦋 U+F098B
    'muffin': 985484,  # 󰦌 U+F098C
    'not-equal': 985485,  # 󰦍 U+F098D
    'not-equal-variant': 985486,  # 󰦎 U+F098E
    'order-bool-ascending-variant': 985487,  # 󰦏 U+F098F
    'order-bool-descending-variant': 985488,  # 󰦐 U+F0990
    'office-building': 985489,  # 󰦑 U+F0991
    'plus-minus': 985490,  # 󰦒 U+F0992
    'plus-minus-box': 985491,  # 󰦓 U+F0993
    'podcast#1': 985492,  # 󰦔 U+F0994
    'progress-check': 985493,  # 󰦕 U+F0995
    'progress-clock': 985494,  # 󰦖 U+F0996
    'progress-download': 985495,  # 󰦗 U+F0997
    'progress-upload': 985496,  # 󰦘 U+F0998
    'qi': 985497,  # 󰦙 U+F0999
    'record-player': 985498,  # 󰦚 U+F099A
    'restore#1': 985499,  # 󰦛 U+F099B
    'shield-off-outline': 985500,  # 󰦜 U+F099C
    'shield-lock#1': 985501,  # 󰦝 U+F099D
    'shield-off': 985502,  # 󰦞 U+F099E
    'set-top-box': 985503,  # 󰦟 U+F099F
    'shower#1': 985504,  # 󰦠 U+F09A0
    'shower-head': 985505,  # 󰦡 U+F09A1
    'speaker-bluetooth': 985506,  # 󰦢 U+F09A2
    'square-root-box': 985507,  # 󰦣 U+F09A3
    'star-circle-outline': 985508,  # 󰦤 U+F09A4
    'star-face': 985509,  # 󰦥 U+F09A5
    'table-merge-cells': 985510,  # 󰦦 U+F09A6
    'tablet-cellphone': 985511,  # 󰦧 U+F09A7
    'text': 985512,  # 󰦨 U+F09A8
    'text-short': 985513,  # 󰦩 U+F09A9
    'text-long': 985514,  # 󰦪 U+F09AA
    'toilet#2': 985515,  # 󰦫 U+F09AB
    'toolbox#1': 985516,  # 󰦬 U+F09AC
    'toolbox-outline': 985517,  # 󰦭 U+F09AD
    'tournament': 985518,  # 󰦮 U+F09AE
    'two-factor-authentication': 985519,  # 󰦯 U+F09AF
    'umbrella-closed': 985520,  # 󰦰 U+F09B0
    'unreal': 985521,  # 󰦱 U+F09B1
    'video-minus': 985522,  # 󰦲 U+F09B2
    'video-plus': 985523,  # 󰦳 U+F09B3
    'volleyball#1': 985524,  # 󰦴 U+F09B4
    'weight-pound': 985525,  # 󰦵 U+F09B5
    'whistle': 985526,  # 󰦶 U+F09B6
    'arrow-bottom-left-bold-outline': 985527,  # 󰦷 U+F09B7
    'arrow-bottom-left-thick': 985528,  # 󰦸 U+F09B8
    'arrow-bottom-right-bold-outline': 985529,  # 󰦹 U+F09B9
    'arrow-bottom-right-thick': 985530,  # 󰦺 U+F09BA
    'arrow-decision': 985531,  # 󰦻 U+F09BB
    'arrow-decision-auto': 985532,  # 󰦼 U+F09BC
    'arrow-decision-auto-outline': 985533,  # 󰦽 U+F09BD
    'arrow-decision-outline': 985534,  # 󰦾 U+F09BE
    'arrow-down-bold-outline': 985535,  # 󰦿 U+F09BF
    'arrow-left-bold-outline': 985536,  # 󰧀 U+F09C0
    'arrow-left-right-bold-outline': 985537,  # 󰧁 U+F09C1
    'arrow-right-bold-outline': 985538,  # 󰧂 U+F09C2
    'arrow-top-left-bold-outline': 985539,  # 󰧃 U+F09C3
    'arrow-top-left-thick': 985540,  # 󰧄 U+F09C4
    'arrow-top-right-bold-outline': 985541,  # 󰧅 U+F09C5
    'arrow-top-right-thick': 985542,  # 󰧆 U+F09C6
    'arrow-up-bold-outline': 985543,  # 󰧇 U+F09C7
    'arrow-up-down-bold-outline': 985544,  # 󰧈 U+F09C8
    'ballot': 985545,  # 󰧉 U+F09C9
    'ballot-outline': 985546,  # 󰧊 U+F09CA
    'betamax': 985547,  # 󰧋 U+F09CB
    'bookmark-minus': 985548,  # 󰧌 U+F09CC
    'bookmark-minus-outline': 985549,  # 󰧍 U+F09CD
    'bookmark-off': 985550,  # 󰧎 U+F09CE
    'bookmark-off-outline': 985551,  # 󰧏 U+F09CF
    'braille#1': 985552,  # 󰧐 U+F09D0
    'brain#2': 985553,  # 󰧑 U+F09D1
    'calendar-heart': 985554,  # 󰧒 U+F09D2
    'calendar-star': 985555,  # 󰧓 U+F09D3
    'cassette': 985556,  # 󰧔 U+F09D4
    'cellphone-arrow-down': 985557,  # 󰧕 U+F09D5
    'chevron-down-box': 985558,  # 󰧖 U+F09D6
    'chevron-down-box-outline': 985559,  # 󰧗 U+F09D7
    'chevron-left-box': 985560,  # 󰧘 U+F09D8
    'chevron-left-box-outline': 985561,  # 󰧙 U+F09D9
    'chevron-right-box': 985562,  # 󰧚 U+F09DA
    'chevron-right-box-outline': 985563,  # 󰧛 U+F09DB
    'chevron-up-box': 985564,  # 󰧜 U+F09DC
    'chevron-up-box-outline': 985565,  # 󰧝 U+F09DD
    'circle-medium': 985566,  # 󰧞 U+F09DE
    'circle-small#1': 985567,  # 󰧟 U+F09DF
    'cloud-alert': 985568,  # 󰧠 U+F09E0
    'comment-arrow-left': 985569,  # 󰧡 U+F09E1
    'comment-arrow-left-outline': 985570,  # 󰧢 U+F09E2
    'comment-arrow-right': 985571,  # 󰧣 U+F09E3
    'comment-arrow-right-outline': 985572,  # 󰧤 U+F09E4
    'comment-plus': 985573,  # 󰧥 U+F09E5
    'currency-php': 985574,  # 󰧦 U+F09E6
    'delete-outline': 985575,  # 󰧧 U+F09E7
    'desktop-mac-dashboard': 985576,  # 󰧨 U+F09E8
    'download-multiple': 985577,  # 󰧩 U+F09E9
    'eight-track': 985578,  # 󰧪 U+F09EA
    'email-plus': 985579,  # 󰧫 U+F09EB
    'email-plus-outline': 985580,  # 󰧬 U+F09EC
    'text-box-outline': 985581,  # 󰧭 U+F09ED
    'file-document-outline': 985582,  # 󰧮 U+F09EE
    'floppy-variant': 985583,  # 󰧯 U+F09EF
    'flower-outline': 985584,  # 󰧰 U+F09F0
    'flower-tulip': 985585,  # 󰧱 U+F09F1
    'flower-tulip-outline': 985586,  # 󰧲 U+F09F2
    'format-font-size-decrease': 985587,  # 󰧳 U+F09F3
    'format-font-size-increase': 985588,  # 󰧴 U+F09F4
    'ghost-off': 985589,  # 󰧵 U+F09F5
    'google-lens': 985590,  # 󰧶 U+F09F6
    'google-spreadsheet': 985591,  # 󰧷 U+F09F7
    'image-move': 985592,  # 󰧸 U+F09F8
    'keyboard-settings': 985593,  # 󰧹 U+F09F9
    'keyboard-settings-outline': 985594,  # 󰧺 U+F09FA
    'knife': 985595,  # 󰧻 U+F09FB
    'knife-military': 985596,  # 󰧼 U+F09FC
    'layers-off-outline': 985597,  # 󰧽 U+F09FD
    'layers-outline': 985598,  # 󰧾 U+F09FE
    'lighthouse': 985599,  # 󰧿 U+F09FF
    'lighthouse-on': 985600,  # 󰨀 U+F0A00
    'map-legend': 985601,  # 󰨁 U+F0A01
    'menu-left-outline': 985602,  # 󰨂 U+F0A02
    'menu-right-outline': 985603,  # 󰨃 U+F0A03
    'message-alert-outline': 985604,  # 󰨄 U+F0A04
    'mini-sd': 985605,  # 󰨅 U+F0A05
    'minidisc': 985606,  # 󰨆 U+F0A06
    'monitor-dashboard': 985607,  # 󰨇 U+F0A07
    'pirate': 985608,  # 󰨈 U+F0A08
    'pokemon-go': 985609,  # 󰨉 U+F0A09
    'powershell': 985610,  # 󰨊 U+F0A0A
    'printer-wireless': 985611,  # 󰨋 U+F0A0B
    'quality-low': 985612,  # 󰨌 U+F0A0C
    'quality-medium': 985613,  # 󰨍 U+F0A0D
    'reflect-horizontal': 985614,  # 󰨎 U+F0A0E
    'reflect-vertical': 985615,  # 󰨏 U+F0A0F
    'rhombus-medium': 985616,  # 󰨐 U+F0A10
    'rhombus-split': 985617,  # 󰨑 U+F0A11
    'shield-account-outline': 985618,  # 󰨒 U+F0A12
    'square-medium': 985619,  # 󰨓 U+F0A13
    'square-medium-outline': 985620,  # 󰨔 U+F0A14
    'square-small': 985621,  # 󰨕 U+F0A15
    'subtitles': 985622,  # 󰨖 U+F0A16
    'subtitles-outline': 985623,  # 󰨗 U+F0A17
    'table-border': 985624,  # 󰨘 U+F0A18
    'toggle-switch-off-outline': 985625,  # 󰨙 U+F0A19
    'toggle-switch-outline': 985626,  # 󰨚 U+F0A1A
    'vhs': 985627,  # 󰨛 U+F0A1B
    'video-vintage': 985628,  # 󰨜 U+F0A1C
    'view-dashboard-outline': 985629,  # 󰨝 U+F0A1D
    'microsoft-visual-studio-code': 985630,  # 󰨞 U+F0A1E
    'vote': 985631,  # 󰨟 U+F0A1F
    'vote-outline': 985632,  # 󰨠 U+F0A20
    'microsoft-windows-classic': 985633,  # 󰨡 U+F0A21
    'microsoft-xbox-controller-battery-charging': 985634,  # 󰨢 U+F0A22
    'zip-disk': 985635,  # 󰨣 U+F0A23
    'aspect-ratio': 985636,  # 󰨤 U+F0A24
    'babel': 985637,  # 󰨥 U+F0A25
    'balloon': 985638,  # 󰨦 U+F0A26
    'bank-transfer': 985639,  # 󰨧 U+F0A27
    'bank-transfer-in': 985640,  # 󰨨 U+F0A28
    'bank-transfer-out': 985641,  # 󰨩 U+F0A29
    'briefcase-minus': 985642,  # 󰨪 U+F0A2A
    'briefcase-plus': 985643,  # 󰨫 U+F0A2B
    'briefcase-remove': 985644,  # 󰨬 U+F0A2C
    'briefcase-search': 985645,  # 󰨭 U+F0A2D
    'bug-check': 985646,  # 󰨮 U+F0A2E
    'bug-check-outline': 985647,  # 󰨯 U+F0A2F
    'bug-outline': 985648,  # 󰨰 U+F0A30
    'calendar-alert': 985649,  # 󰨱 U+F0A31
    'calendar-multiselect': 985650,  # 󰨲 U+F0A32
    'calendar-week': 985651,  # 󰨳 U+F0A33
    'calendar-week-begin': 985652,  # 󰨴 U+F0A34
    'cellphone-screenshot': 985653,  # 󰨵 U+F0A35
    'city-variant': 985654,  # 󰨶 U+F0A36
    'city-variant-outline': 985655,  # 󰨷 U+F0A37
    'clipboard-text-outline': 985656,  # 󰨸 U+F0A38
    'cloud-question': 985657,  # 󰨹 U+F0A39
    'comment-eye': 985658,  # 󰨺 U+F0A3A
    'comment-eye-outline': 985659,  # 󰨻 U+F0A3B
    'comment-search': 985660,  # 󰨼 U+F0A3C
    'comment-search-outline': 985661,  # 󰨽 U+F0A3D
    'contain': 985662,  # 󰨾 U+F0A3E
    'contain-end': 985663,  # 󰨿 U+F0A3F
    'contain-start': 985664,  # 󰩀 U+F0A40
    'dlna': 985665,  # 󰩁 U+F0A41
    'doctor': 985666,  # 󰩂 U+F0A42
    'dog#1': 985667,  # 󰩃 U+F0A43
    'dog-side': 985668,  # 󰩄 U+F0A44
    'ear-hearing-off': 985669,  # 󰩅 U+F0A45
    'engine-off': 985670,  # 󰩆 U+F0A46
    'engine-off-outline': 985671,  # 󰩇 U+F0A47
    'exit-run': 985672,  # 󰩈 U+F0A48
    'feature-search': 985673,  # 󰩉 U+F0A49
    'feature-search-outline': 985674,  # 󰩊 U+F0A4A
    'file-alert': 985675,  # 󰩋 U+F0A4B
    'file-alert-outline': 985676,  # 󰩌 U+F0A4C
    'file-upload': 985677,  # 󰩍 U+F0A4D
    'file-upload-outline': 985678,  # 󰩎 U+F0A4E
    'hand-front-right': 985679,  # 󰩏 U+F0A4F
    'hand-okay': 985680,  # 󰩐 U+F0A50
    'hand-peace': 985681,  # 󰩑 U+F0A51
    'hand-peace-variant': 985682,  # 󰩒 U+F0A52
    'hand-pointing-down': 985683,  # 󰩓 U+F0A53
    'hand-pointing-left': 985684,  # 󰩔 U+F0A54
    'hand-pointing-up': 985685,  # 󰩕 U+F0A55
    'heart-multiple': 985686,  # 󰩖 U+F0A56
    'heart-multiple-outline': 985687,  # 󰩗 U+F0A57
    'horseshoe': 985688,  # 󰩘 U+F0A58
    'human-female-boy': 985689,  # 󰩙 U+F0A59
    'human-female-female': 985690,  # 󰩚 U+F0A5A
    'human-female-girl': 985691,  # 󰩛 U+F0A5B
    'human-male-boy': 985692,  # 󰩜 U+F0A5C
    'human-male-girl': 985693,  # 󰩝 U+F0A5D
    'human-male-male': 985694,  # 󰩞 U+F0A5E
    'ip': 985695,  # 󰩟 U+F0A5F
    'ip-network': 985696,  # 󰩠 U+F0A60
    'litecoin': 985697,  # 󰩡 U+F0A61
    'magnify-minus-cursor': 985698,  # 󰩢 U+F0A62
    'magnify-plus-cursor': 985699,  # 󰩣 U+F0A63
    'menu-swap': 985700,  # 󰩤 U+F0A64
    'menu-swap-outline': 985701,  # 󰩥 U+F0A65
    'puzzle-outline': 985702,  # 󰩦 U+F0A66
    'registered-trademark': 985703,  # 󰩧 U+F0A67
    'resize': 985704,  # 󰩨 U+F0A68
    'router-wireless-settings': 985705,  # 󰩩 U+F0A69
    'safe': 985706,  # 󰩪 U+F0A6A
    'scissors-cutting': 985707,  # 󰩫 U+F0A6B
    'select-drag': 985708,  # 󰩬 U+F0A6C
    'selection-drag': 985709,  # 󰩭 U+F0A6D
    'settings-helper': 985710,  # 󰩮 U+F0A6E
    'signal-5g': 985711,  # 󰩯 U+F0A6F
    'silverware-fork-knife': 985712,  # 󰩰 U+F0A70
    'smog#1': 985713,  # 󰩱 U+F0A71
    'solar-power': 985714,  # 󰩲 U+F0A72
    'star-box': 985715,  # 󰩳 U+F0A73
    'star-box-outline': 985716,  # 󰩴 U+F0A74
    'table-plus': 985717,  # 󰩵 U+F0A75
    'table-remove': 985718,  # 󰩶 U+F0A76
    'target-variant': 985719,  # 󰩷 U+F0A77
    'trademark#1': 985720,  # 󰩸 U+F0A78
    'trash-can': 985721,  # 󰩹 U+F0A79
    'trash-can-outline': 985722,  # 󰩺 U+F0A7A
    'tshirt-crew': 985723,  # 󰩻 U+F0A7B
    'tshirt-v': 985724,  # 󰩼 U+F0A7C
    'zodiac-aquarius': 985725,  # 󰩽 U+F0A7D
    'zodiac-aries': 985726,  # 󰩾 U+F0A7E
    'zodiac-cancer': 985727,  # 󰩿 U+F0A7F
    'zodiac-capricorn': 985728,  # 󰪀 U+F0A80
    'zodiac-gemini': 985729,  # 󰪁 U+F0A81
    'zodiac-leo': 985730,  # 󰪂 U+F0A82
    'zodiac-libra': 985731,  # 󰪃 U+F0A83
    'zodiac-pisces': 985732,  # 󰪄 U+F0A84
    'zodiac-sagittarius': 985733,  # 󰪅 U+F0A85
    'zodiac-scorpio': 985734,  # 󰪆 U+F0A86
    'zodiac-taurus': 985735,  # 󰪇 U+F0A87
    'zodiac-virgo': 985736,  # 󰪈 U+F0A88
    'account-child': 985737,  # 󰪉 U+F0A89
    'account-child-circle': 985738,  # 󰪊 U+F0A8A
    'account-supervisor': 985739,  # 󰪋 U+F0A8B
    'account-supervisor-circle': 985740,  # 󰪌 U+F0A8C
    'ampersand': 985741,  # 󰪍 U+F0A8D
    'web-off': 985742,  # 󰪎 U+F0A8E
    'animation-outline': 985743,  # 󰪏 U+F0A8F
    'animation-play-outline': 985744,  # 󰪐 U+F0A90
    'bell-off-outline': 985745,  # 󰪑 U+F0A91
    'bell-plus-outline': 985746,  # 󰪒 U+F0A92
    'bell-sleep-outline': 985747,  # 󰪓 U+F0A93
    'book-minus-multiple': 985748,  # 󰪔 U+F0A94
    'book-plus-multiple': 985749,  # 󰪕 U+F0A95
    'book-remove-multiple': 985750,  # 󰪖 U+F0A96
    'book-remove': 985751,  # 󰪗 U+F0A97
    'briefcase-edit': 985752,  # 󰪘 U+F0A98
    'bus-alert': 985753,  # 󰪙 U+F0A99
    'calculator-variant': 985754,  # 󰪚 U+F0A9A
    'caps-lock': 985755,  # 󰪛 U+F0A9B
    'cash-refund': 985756,  # 󰪜 U+F0A9C
    'checkbook': 985757,  # 󰪝 U+F0A9D
    'circle-slice-1': 985758,  # 󰪞 U+F0A9E
    'circle-slice-2': 985759,  # 󰪟 U+F0A9F
    'circle-slice-3': 985760,  # 󰪠 U+F0AA0
    'circle-slice-4': 985761,  # 󰪡 U+F0AA1
    'circle-slice-5': 985762,  # 󰪢 U+F0AA2
    'circle-slice-6': 985763,  # 󰪣 U+F0AA3
    'circle-slice-7': 985764,  # 󰪤 U+F0AA4
    'circle-slice-8': 985765,  # 󰪥 U+F0AA5
    'collapse-all#1': 985766,  # 󰪦 U+F0AA6
    'collapse-all-outline': 985767,  # 󰪧 U+F0AA7
    'credit-card-refund-outline': 985768,  # 󰪨 U+F0AA8
    'database-check': 985769,  # 󰪩 U+F0AA9
    'database-lock': 985770,  # 󰪪 U+F0AAA
    'desktop-tower-monitor': 985771,  # 󰪫 U+F0AAB
    'dishwasher': 985772,  # 󰪬 U+F0AAC
    'dog-service': 985773,  # 󰪭 U+F0AAD
    'dot-net': 985774,  # 󰪮 U+F0AAE
    'egg#1': 985775,  # 󰪯 U+F0AAF
    'egg-easter': 985776,  # 󰪰 U+F0AB0
    'email-check': 985777,  # 󰪱 U+F0AB1
    'email-check-outline': 985778,  # 󰪲 U+F0AB2
    'et': 985779,  # 󰪳 U+F0AB3
    'expand-all#1': 985780,  # 󰪴 U+F0AB4
    'expand-all-outline': 985781,  # 󰪵 U+F0AB5
    'file-cabinet': 985782,  # 󰪶 U+F0AB6
    'text-box-multiple': 985783,  # 󰪷 U+F0AB7
    'text-box-multiple-outline': 985784,  # 󰪸 U+F0AB8
    'file-move': 985785,  # 󰪹 U+F0AB9
    'folder-clock': 985786,  # 󰪺 U+F0ABA
    'folder-clock-outline': 985787,  # 󰪻 U+F0ABB
    'format-annotation-minus': 985788,  # 󰪼 U+F0ABC
    'gesture-pinch': 985789,  # 󰪽 U+F0ABD
    'gesture-spread': 985790,  # 󰪾 U+F0ABE
    'gesture-swipe-horizontal': 985791,  # 󰪿 U+F0ABF
    'gesture-swipe-vertical': 985792,  # 󰫀 U+F0AC0
    'hail': 985793,  # 󰫁 U+F0AC1
    'helicopter#1': 985794,  # 󰫂 U+F0AC2
    'hexagon-slice-1': 985795,  # 󰫃 U+F0AC3
    'hexagon-slice-2': 985796,  # 󰫄 U+F0AC4
    'hexagon-slice-3': 985797,  # 󰫅 U+F0AC5
    'hexagon-slice-4': 985798,  # 󰫆 U+F0AC6
    'hexagon-slice-5': 985799,  # 󰫇 U+F0AC7
    'hexagon-slice-6': 985800,  # 󰫈 U+F0AC8
    'hexagram': 985801,  # 󰫉 U+F0AC9
    'hexagram-outline': 985802,  # 󰫊 U+F0ACA
    'label-off': 985803,  # 󰫋 U+F0ACB
    'label-off-outline': 985804,  # 󰫌 U+F0ACC
    'label-variant': 985805,  # 󰫍 U+F0ACD
    'label-variant-outline': 985806,  # 󰫎 U+F0ACE
    'language-ruby-on-rails': 985807,  # 󰫏 U+F0ACF
    'laravel#1': 985808,  # 󰫐 U+F0AD0
    'mastodon#1': 985809,  # 󰫑 U+F0AD1
    'sort-numeric-descending-variant': 985810,  # 󰫒 U+F0AD2
    'minus-circle-multiple-outline': 985811,  # 󰫓 U+F0AD3
    'music-circle-outline': 985812,  # 󰫔 U+F0AD4
    'pinwheel': 985813,  # 󰫕 U+F0AD5
    'pinwheel-outline': 985814,  # 󰫖 U+F0AD6
    'radiator-disabled': 985815,  # 󰫗 U+F0AD7
    'radiator-off': 985816,  # 󰫘 U+F0AD8
    'select-compare': 985817,  # 󰫙 U+F0AD9
    'shield-plus': 985818,  # 󰫚 U+F0ADA
    'shield-plus-outline': 985819,  # 󰫛 U+F0ADB
    'shield-remove': 985820,  # 󰫜 U+F0ADC
    'shield-remove-outline': 985821,  # 󰫝 U+F0ADD
    'book-plus-multiple-outline': 985822,  # 󰫞 U+F0ADE
    'sina-weibo': 985823,  # 󰫟 U+F0ADF
    'spray-bottle': 985824,  # 󰫠 U+F0AE0
    'squeegee': 985825,  # 󰫡 U+F0AE1
    'star-four-points': 985826,  # 󰫢 U+F0AE2
    'star-four-points-outline': 985827,  # 󰫣 U+F0AE3
    'star-three-points': 985828,  # 󰫤 U+F0AE4
    'star-three-points-outline': 985829,  # 󰫥 U+F0AE5
    'symfony#1': 985830,  # 󰫦 U+F0AE6
    'variable': 985831,  # 󰫧 U+F0AE7
    'vector-bezier': 985832,  # 󰫨 U+F0AE8
    'wiper': 985833,  # 󰫩 U+F0AE9
    'z-wave': 985834,  # 󰫪 U+F0AEA
    'zend': 985835,  # 󰫫 U+F0AEB
    'account-minus-outline': 985836,  # 󰫬 U+F0AEC
    'account-remove-outline': 985837,  # 󰫭 U+F0AED
    'alpha-a': 985838,  # 󰫮 U+F0AEE
    'alpha-b': 985839,  # 󰫯 U+F0AEF
    'alpha-c': 985840,  # 󰫰 U+F0AF0
    'alpha-d': 985841,  # 󰫱 U+F0AF1
    'alpha-e': 985842,  # 󰫲 U+F0AF2
    'alpha-f': 985843,  # 󰫳 U+F0AF3
    'alpha-g': 985844,  # 󰫴 U+F0AF4
    'alpha-h': 985845,  # 󰫵 U+F0AF5
    'alpha-i': 985846,  # 󰫶 U+F0AF6
    'alpha-j': 985847,  # 󰫷 U+F0AF7
    'alpha-k': 985848,  # 󰫸 U+F0AF8
    'alpha-l': 985849,  # 󰫹 U+F0AF9
    'alpha-m': 985850,  # 󰫺 U+F0AFA
    'alpha-n': 985851,  # 󰫻 U+F0AFB
    'alpha-o': 985852,  # 󰫼 U+F0AFC
    'alpha-p': 985853,  # 󰫽 U+F0AFD
    'alpha-q': 985854,  # 󰫾 U+F0AFE
    'alpha-r': 985855,  # 󰫿 U+F0AFF
    'alpha-s': 985856,  # 󰬀 U+F0B00
    'alpha-t': 985857,  # 󰬁 U+F0B01
    'alpha-u': 985858,  # 󰬂 U+F0B02
    'alpha-v': 985859,  # 󰬃 U+F0B03
    'alpha-w': 985860,  # 󰬄 U+F0B04
    'alpha-x': 985861,  # 󰬅 U+F0B05
    'alpha-y': 985862,  # 󰬆 U+F0B06
    'alpha-z': 985863,  # 󰬇 U+F0B07
    'alpha-a-box': 985864,  # 󰬈 U+F0B08
    'alpha-b-box': 985865,  # 󰬉 U+F0B09
    'alpha-c-box': 985866,  # 󰬊 U+F0B0A
    'alpha-d-box': 985867,  # 󰬋 U+F0B0B
    'alpha-e-box': 985868,  # 󰬌 U+F0B0C
    'alpha-f-box': 985869,  # 󰬍 U+F0B0D
    'alpha-g-box': 985870,  # 󰬎 U+F0B0E
    'alpha-h-box': 985871,  # 󰬏 U+F0B0F
    'alpha-i-box': 985872,  # 󰬐 U+F0B10
    'alpha-j-box': 985873,  # 󰬑 U+F0B11
    'alpha-k-box': 985874,  # 󰬒 U+F0B12
    'alpha-l-box': 985875,  # 󰬓 U+F0B13
    'alpha-m-box': 985876,  # 󰬔 U+F0B14
    'alpha-n-box': 985877,  # 󰬕 U+F0B15
    'alpha-o-box': 985878,  # 󰬖 U+F0B16
    'alpha-p-box': 985879,  # 󰬗 U+F0B17
    'alpha-q-box': 985880,  # 󰬘 U+F0B18
    'alpha-r-box': 985881,  # 󰬙 U+F0B19
    'alpha-s-box': 985882,  # 󰬚 U+F0B1A
    'alpha-t-box': 985883,  # 󰬛 U+F0B1B
    'alpha-u-box': 985884,  # 󰬜 U+F0B1C
    'alpha-v-box': 985885,  # 󰬝 U+F0B1D
    'alpha-w-box': 985886,  # 󰬞 U+F0B1E
    'alpha-x-box': 985887,  # 󰬟 U+F0B1F
    'alpha-y-box': 985888,  # 󰬠 U+F0B20
    'alpha-z-box': 985889,  # 󰬡 U+F0B21
    'bulldozer': 985890,  # 󰬢 U+F0B22
    'bullhorn-outline': 985891,  # 󰬣 U+F0B23
    'calendar-export': 985892,  # 󰬤 U+F0B24
    'calendar-import': 985893,  # 󰬥 U+F0B25
    'chevron-down-circle': 985894,  # 󰬦 U+F0B26
    'chevron-down-circle-outline': 985895,  # 󰬧 U+F0B27
    'chevron-left-circle': 985896,  # 󰬨 U+F0B28
    'chevron-left-circle-outline': 985897,  # 󰬩 U+F0B29
    'chevron-right-circle': 985898,  # 󰬪 U+F0B2A
    'chevron-right-circle-outline': 985899,  # 󰬫 U+F0B2B
    'chevron-up-circle': 985900,  # 󰬬 U+F0B2C
    'chevron-up-circle-outline': 985901,  # 󰬭 U+F0B2D
    'content-save-settings-outline': 985902,  # 󰬮 U+F0B2E
    'crystal-ball': 985903,  # 󰬯 U+F0B2F
    'ember#1': 985904,  # 󰬰 U+F0B30
    'facebook-workplace': 985905,  # 󰬱 U+F0B31
    'file-replace': 985906,  # 󰬲 U+F0B32
    'file-replace-outline': 985907,  # 󰬳 U+F0B33
    'format-letter-case': 985908,  # 󰬴 U+F0B34
    'format-letter-case-lower': 985909,  # 󰬵 U+F0B35
    'format-letter-case-upper': 985910,  # 󰬶 U+F0B36
    'language-java': 985911,  # 󰬷 U+F0B37
    'circle-multiple': 985912,  # 󰬸 U+F0B38
    'alpha-o#1': 985913,  # 󰬹 U+F0B39
    'numeric-1': 985914,  # 󰬺 U+F0B3A
    'numeric-2': 985915,  # 󰬻 U+F0B3B
    'numeric-3': 985916,  # 󰬼 U+F0B3C
    'numeric-4': 985917,  # 󰬽 U+F0B3D
    'numeric-5': 985918,  # 󰬾 U+F0B3E
    'numeric-6': 985919,  # 󰬿 U+F0B3F
    'numeric-7': 985920,  # 󰭀 U+F0B40
    'numeric-8': 985921,  # 󰭁 U+F0B41
    'numeric-9': 985922,  # 󰭂 U+F0B42
    'origin': 985923,  # 󰭃 U+F0B43
    'resistor': 985924,  # 󰭄 U+F0B44
    'resistor-nodes': 985925,  # 󰭅 U+F0B45
    'robot-industrial': 985926,  # 󰭆 U+F0B46
    'shoe-formal': 985927,  # 󰭇 U+F0B47
    'shoe-heel': 985928,  # 󰭈 U+F0B48
    'silo': 985929,  # 󰭉 U+F0B49
    'box-cutter-off': 985930,  # 󰭊 U+F0B4A
    'tab-minus': 985931,  # 󰭋 U+F0B4B
    'tab-remove': 985932,  # 󰭌 U+F0B4C
    'tape-measure': 985933,  # 󰭍 U+F0B4D
    'telescope#3': 985934,  # 󰭎 U+F0B4E
    'yahoo#1': 985935,  # 󰭏 U+F0B4F
    'account-alert-outline': 985936,  # 󰭐 U+F0B50
    'account-arrow-left': 985937,  # 󰭑 U+F0B51
    'account-arrow-left-outline': 985938,  # 󰭒 U+F0B52
    'account-arrow-right': 985939,  # 󰭓 U+F0B53
    'account-arrow-right-outline': 985940,  # 󰭔 U+F0B54
    'account-circle-outline': 985941,  # 󰭕 U+F0B55
    'account-clock': 985942,  # 󰭖 U+F0B56
    'account-clock-outline': 985943,  # 󰭗 U+F0B57
    'account-group-outline': 985944,  # 󰭘 U+F0B58
    'account-question': 985945,  # 󰭙 U+F0B59
    'account-question-outline': 985946,  # 󰭚 U+F0B5A
    'artstation#1': 985947,  # 󰭛 U+F0B5B
    'backspace-outline': 985948,  # 󰭜 U+F0B5C
    'barley-off': 985949,  # 󰭝 U+F0B5D
    'barn': 985950,  # 󰭞 U+F0B5E
    'bat': 985951,  # 󰭟 U+F0B5F
    'application-settings': 985952,  # 󰭠 U+F0B60
    'billiards': 985953,  # 󰭡 U+F0B61
    'billiards-rack': 985954,  # 󰭢 U+F0B62
    'book-open-outline': 985955,  # 󰭣 U+F0B63
    'book-outline': 985956,  # 󰭤 U+F0B64
    'boxing-glove': 985957,  # 󰭥 U+F0B65
    'calendar-blank-outline': 985958,  # 󰭦 U+F0B66
    'calendar-outline': 985959,  # 󰭧 U+F0B67
    'calendar-range-outline': 985960,  # 󰭨 U+F0B68
    'camera-control': 985961,  # 󰭩 U+F0B69
    'camera-enhance-outline': 985962,  # 󰭪 U+F0B6A
    'car-door': 985963,  # 󰭫 U+F0B6B
    'car-electric': 985964,  # 󰭬 U+F0B6C
    'car-key': 985965,  # 󰭭 U+F0B6D
    'car-multiple': 985966,  # 󰭮 U+F0B6E
    'card': 985967,  # 󰭯 U+F0B6F
    'card-bulleted': 985968,  # 󰭰 U+F0B70
    'card-bulleted-off': 985969,  # 󰭱 U+F0B71
    'card-bulleted-off-outline': 985970,  # 󰭲 U+F0B72
    'card-bulleted-outline': 985971,  # 󰭳 U+F0B73
    'card-bulleted-settings': 985972,  # 󰭴 U+F0B74
    'card-bulleted-settings-outline': 985973,  # 󰭵 U+F0B75
    'card-outline': 985974,  # 󰭶 U+F0B76
    'card-text': 985975,  # 󰭷 U+F0B77
    'card-text-outline': 985976,  # 󰭸 U+F0B78
    'chat': 985977,  # 󰭹 U+F0B79
    'chat-alert': 985978,  # 󰭺 U+F0B7A
    'chat-processing': 985979,  # 󰭻 U+F0B7B
    'chef-hat': 985980,  # 󰭼 U+F0B7C
    'cloud-download-outline': 985981,  # 󰭽 U+F0B7D
    'cloud-upload-outline': 985982,  # 󰭾 U+F0B7E
    'coffin': 985983,  # 󰭿 U+F0B7F
    'compass-off': 985984,  # 󰮀 U+F0B80
    'compass-off-outline': 985985,  # 󰮁 U+F0B81
    'controller-classic': 985986,  # 󰮂 U+F0B82
    'controller-classic-outline': 985987,  # 󰮃 U+F0B83
    'cube-scan': 985988,  # 󰮄 U+F0B84
    'currency-brl': 985989,  # 󰮅 U+F0B85
    'database-edit': 985990,  # 󰮆 U+F0B86
    'deathly-hallows': 985991,  # 󰮇 U+F0B87
    'delete-circle-outline': 985992,  # 󰮈 U+F0B88
    'delete-forever-outline': 985993,  # 󰮉 U+F0B89
    'diamond#2': 985994,  # 󰮊 U+F0B8A
    'diamond-outline': 985995,  # 󰮋 U+F0B8B
    'dns-outline': 985996,  # 󰮌 U+F0B8C
    'dots-horizontal-circle-outline': 985997,  # 󰮍 U+F0B8D
    'dots-vertical-circle-outline': 985998,  # 󰮎 U+F0B8E
    'download-outline': 985999,  # 󰮏 U+F0B8F
    'drag-variant': 986000,  # 󰮐 U+F0B90
    'eject-outline': 986001,  # 󰮑 U+F0B91
    'email-mark-as-unread': 986002,  # 󰮒 U+F0B92
    'export-variant': 986003,  # 󰮓 U+F0B93
    'eye-circle': 986004,  # 󰮔 U+F0B94
    'eye-circle-outline': 986005,  # 󰮕 U+F0B95
    'face-man-outline': 986006,  # 󰮖 U+F0B96
    'file-find-outline': 986007,  # 󰮗 U+F0B97
    'file-remove': 986008,  # 󰮘 U+F0B98
    'flag-minus': 986009,  # 󰮙 U+F0B99
    'flag-plus': 986010,  # 󰮚 U+F0B9A
    'flag-remove': 986011,  # 󰮛 U+F0B9B
    'folder-account-outline': 986012,  # 󰮜 U+F0B9C
    'folder-plus-outline': 986013,  # 󰮝 U+F0B9D
    'folder-remove-outline': 986014,  # 󰮞 U+F0B9E
    'folder-star-outline': 986015,  # 󰮟 U+F0B9F
    'gitlab#1': 986016,  # 󰮠 U+F0BA0
    'gog': 986017,  # 󰮡 U+F0BA1
    'grave-stone': 986018,  # 󰮢 U+F0BA2
    'halloween': 986019,  # 󰮣 U+F0BA3
    'hat-fedora': 986020,  # 󰮤 U+F0BA4
    'help-rhombus': 986021,  # 󰮥 U+F0BA5
    'help-rhombus-outline': 986022,  # 󰮦 U+F0BA6
    'home-variant-outline': 986023,  # 󰮧 U+F0BA7
    'inbox-multiple-outline': 986024,  # 󰮨 U+F0BA8
    'library-shelves': 986025,  # 󰮩 U+F0BA9
    'mapbox': 986026,  # 󰮪 U+F0BAA
    'menu-open': 986027,  # 󰮫 U+F0BAB
    'molecule#1': 986028,  # 󰮬 U+F0BAC
    'one-up': 986029,  # 󰮭 U+F0BAD
    'open-source-initiative': 986030,  # 󰮮 U+F0BAE
    'pac-man': 986031,  # 󰮯 U+F0BAF
    'page-next': 986032,  # 󰮰 U+F0BB0
    'page-next-outline': 986033,  # 󰮱 U+F0BB1
    'page-previous': 986034,  # 󰮲 U+F0BB2
    'page-previous-outline': 986035,  # 󰮳 U+F0BB3
    'pan': 986036,  # 󰮴 U+F0BB4
    'pan-bottom-left': 986037,  # 󰮵 U+F0BB5
    'pan-bottom-right': 986038,  # 󰮶 U+F0BB6
    'pan-down': 986039,  # 󰮷 U+F0BB7
    'pan-horizontal': 986040,  # 󰮸 U+F0BB8
    'pan-left': 986041,  # 󰮹 U+F0BB9
    'pan-right': 986042,  # 󰮺 U+F0BBA
    'pan-top-left': 986043,  # 󰮻 U+F0BBB
    'pan-top-right': 986044,  # 󰮼 U+F0BBC
    'pan-up': 986045,  # 󰮽 U+F0BBD
    'pan-vertical': 986046,  # 󰮾 U+F0BBE
    'pumpkin': 986047,  # 󰮿 U+F0BBF
    'rollupjs': 986048,  # 󰯀 U+F0BC0
    'script': 986049,  # 󰯁 U+F0BC1
    'script-text': 986050,  # 󰯂 U+F0BC2
    'script-text-outline': 986051,  # 󰯃 U+F0BC3
    'shield-key': 986052,  # 󰯄 U+F0BC4
    'shield-key-outline': 986053,  # 󰯅 U+F0BC5
    'skull-crossbones': 986054,  # 󰯆 U+F0BC6
    'skull-crossbones-outline': 986055,  # 󰯇 U+F0BC7
    'skull-outline': 986056,  # 󰯈 U+F0BC8
    'space-invaders': 986057,  # 󰯉 U+F0BC9
    'spider-web': 986058,  # 󰯊 U+F0BCA
    'view-split-horizontal': 986059,  # 󰯋 U+F0BCB
    'view-split-vertical': 986060,  # 󰯌 U+F0BCC
    'swap-horizontal-bold': 986061,  # 󰯍 U+F0BCD
    'swap-vertical-bold': 986062,  # 󰯎 U+F0BCE
    'tag-heart-outline': 986063,  # 󰯏 U+F0BCF
    'target-account': 986064,  # 󰯐 U+F0BD0
    'timeline': 986065,  # 󰯑 U+F0BD1
    'timeline-outline': 986066,  # 󰯒 U+F0BD2
    'timeline-text': 986067,  # 󰯓 U+F0BD3
    'timeline-text-outline': 986068,  # 󰯔 U+F0BD4
    'tooltip-image-outline': 986069,  # 󰯕 U+F0BD5
    'tooltip-plus': 986070,  # 󰯖 U+F0BD6
    'tooltip-text-outline': 986071,  # 󰯗 U+F0BD7
    'train-car': 986072,  # 󰯘 U+F0BD8
    'triforce': 986073,  # 󰯙 U+F0BD9
    'ubisoft': 986074,  # 󰯚 U+F0BDA
    'video-off-outline': 986075,  # 󰯛 U+F0BDB
    'video-outline': 986076,  # 󰯜 U+F0BDC
    'wallet-outline': 986077,  # 󰯝 U+F0BDD
    'waze#1': 986078,  # 󰯞 U+F0BDE
    'wrap-disabled': 986079,  # 󰯟 U+F0BDF
    'wrench-outline': 986080,  # 󰯠 U+F0BE0
    'access-point-network-off': 986081,  # 󰯡 U+F0BE1
    'account-check-outline': 986082,  # 󰯢 U+F0BE2
    'account-heart-outline': 986083,  # 󰯣 U+F0BE3
    'account-key-outline': 986084,  # 󰯤 U+F0BE4
    'account-multiple-minus-outline': 986085,  # 󰯥 U+F0BE5
    'account-network-outline': 986086,  # 󰯦 U+F0BE6
    'account-off-outline': 986087,  # 󰯧 U+F0BE7
    'account-star-outline': 986088,  # 󰯨 U+F0BE8
    'airbag': 986089,  # 󰯩 U+F0BE9
    'alarm-light-outline': 986090,  # 󰯪 U+F0BEA
    'alpha-a-box-outline': 986091,  # 󰯫 U+F0BEB
    'alpha-a-circle': 986092,  # 󰯬 U+F0BEC
    'alpha-a-circle-outline': 986093,  # 󰯭 U+F0BED
    'alpha-b-box-outline': 986094,  # 󰯮 U+F0BEE
    'alpha-b-circle': 986095,  # 󰯯 U+F0BEF
    'alpha-b-circle-outline': 986096,  # 󰯰 U+F0BF0
    'alpha-c-box-outline': 986097,  # 󰯱 U+F0BF1
    'alpha-c-circle': 986098,  # 󰯲 U+F0BF2
    'alpha-c-circle-outline': 986099,  # 󰯳 U+F0BF3
    'alpha-d-box-outline': 986100,  # 󰯴 U+F0BF4
    'alpha-d-circle': 986101,  # 󰯵 U+F0BF5
    'alpha-d-circle-outline': 986102,  # 󰯶 U+F0BF6
    'alpha-e-box-outline': 986103,  # 󰯷 U+F0BF7
    'alpha-e-circle': 986104,  # 󰯸 U+F0BF8
    'alpha-e-circle-outline': 986105,  # 󰯹 U+F0BF9
    'alpha-f-box-outline': 986106,  # 󰯺 U+F0BFA
    'alpha-f-circle': 986107,  # 󰯻 U+F0BFB
    'alpha-f-circle-outline': 986108,  # 󰯼 U+F0BFC
    'alpha-g-box-outline': 986109,  # 󰯽 U+F0BFD
    'alpha-g-circle': 986110,  # 󰯾 U+F0BFE
    'alpha-g-circle-outline': 986111,  # 󰯿 U+F0BFF
    'alpha-h-box-outline': 986112,  # 󰰀 U+F0C00
    'alpha-h-circle': 986113,  # 󰰁 U+F0C01
    'alpha-h-circle-outline': 986114,  # 󰰂 U+F0C02
    'alpha-i-box-outline': 986115,  # 󰰃 U+F0C03
    'alpha-i-circle': 986116,  # 󰰄 U+F0C04
    'alpha-i-circle-outline': 986117,  # 󰰅 U+F0C05
    'alpha-j-box-outline': 986118,  # 󰰆 U+F0C06
    'alpha-j-circle': 986119,  # 󰰇 U+F0C07
    'alpha-j-circle-outline': 986120,  # 󰰈 U+F0C08
    'alpha-k-box-outline': 986121,  # 󰰉 U+F0C09
    'alpha-k-circle': 986122,  # 󰰊 U+F0C0A
    'alpha-k-circle-outline': 986123,  # 󰰋 U+F0C0B
    'alpha-l-box-outline': 986124,  # 󰰌 U+F0C0C
    'alpha-l-circle': 986125,  # 󰰍 U+F0C0D
    'alpha-l-circle-outline': 986126,  # 󰰎 U+F0C0E
    'alpha-m-box-outline': 986127,  # 󰰏 U+F0C0F
    'alpha-m-circle': 986128,  # 󰰐 U+F0C10
    'alpha-m-circle-outline': 986129,  # 󰰑 U+F0C11
    'alpha-n-box-outline': 986130,  # 󰰒 U+F0C12
    'alpha-n-circle': 986131,  # 󰰓 U+F0C13
    'alpha-n-circle-outline': 986132,  # 󰰔 U+F0C14
    'alpha-o-box-outline': 986133,  # 󰰕 U+F0C15
    'alpha-o-circle': 986134,  # 󰰖 U+F0C16
    'alpha-o-circle-outline': 986135,  # 󰰗 U+F0C17
    'alpha-p-box-outline': 986136,  # 󰰘 U+F0C18
    'alpha-p-circle': 986137,  # 󰰙 U+F0C19
    'alpha-p-circle-outline': 986138,  # 󰰚 U+F0C1A
    'alpha-q-box-outline': 986139,  # 󰰛 U+F0C1B
    'alpha-q-circle': 986140,  # 󰰜 U+F0C1C
    'alpha-q-circle-outline': 986141,  # 󰰝 U+F0C1D
    'alpha-r-box-outline': 986142,  # 󰰞 U+F0C1E
    'alpha-r-circle': 986143,  # 󰰟 U+F0C1F
    'alpha-r-circle-outline': 986144,  # 󰰠 U+F0C20
    'alpha-s-box-outline': 986145,  # 󰰡 U+F0C21
    'alpha-s-circle': 986146,  # 󰰢 U+F0C22
    'alpha-s-circle-outline': 986147,  # 󰰣 U+F0C23
    'alpha-t-box-outline': 986148,  # 󰰤 U+F0C24
    'alpha-t-circle': 986149,  # 󰰥 U+F0C25
    'alpha-t-circle-outline': 986150,  # 󰰦 U+F0C26
    'alpha-u-box-outline': 986151,  # 󰰧 U+F0C27
    'alpha-u-circle': 986152,  # 󰰨 U+F0C28
    'alpha-u-circle-outline': 986153,  # 󰰩 U+F0C29
    'alpha-v-box-outline': 986154,  # 󰰪 U+F0C2A
    'alpha-v-circle': 986155,  # 󰰫 U+F0C2B
    'alpha-v-circle-outline': 986156,  # 󰰬 U+F0C2C
    'alpha-w-box-outline': 986157,  # 󰰭 U+F0C2D
    'alpha-w-circle': 986158,  # 󰰮 U+F0C2E
    'alpha-w-circle-outline': 986159,  # 󰰯 U+F0C2F
    'alpha-x-box-outline': 986160,  # 󰰰 U+F0C30
    'alpha-x-circle': 986161,  # 󰰱 U+F0C31
    'alpha-x-circle-outline': 986162,  # 󰰲 U+F0C32
    'alpha-y-box-outline': 986163,  # 󰰳 U+F0C33
    'alpha-y-circle': 986164,  # 󰰴 U+F0C34
    'alpha-y-circle-outline': 986165,  # 󰰵 U+F0C35
    'alpha-z-box-outline': 986166,  # 󰰶 U+F0C36
    'alpha-z-circle': 986167,  # 󰰷 U+F0C37
    'alpha-z-circle-outline': 986168,  # 󰰸 U+F0C38
    'ballot-recount': 986169,  # 󰰹 U+F0C39
    'ballot-recount-outline': 986170,  # 󰰺 U+F0C3A
    'basketball-hoop': 986171,  # 󰰻 U+F0C3B
    'basketball-hoop-outline': 986172,  # 󰰼 U+F0C3C
    'briefcase-download-outline': 986173,  # 󰰽 U+F0C3D
    'briefcase-edit-outline': 986174,  # 󰰾 U+F0C3E
    'briefcase-minus-outline': 986175,  # 󰰿 U+F0C3F
    'briefcase-plus-outline': 986176,  # 󰱀 U+F0C40
    'briefcase-remove-outline': 986177,  # 󰱁 U+F0C41
    'briefcase-search-outline': 986178,  # 󰱂 U+F0C42
    'briefcase-upload-outline': 986179,  # 󰱃 U+F0C43
    'calendar-check-outline': 986180,  # 󰱄 U+F0C44
    'calendar-remove-outline': 986181,  # 󰱅 U+F0C45
    'calendar-text-outline': 986182,  # 󰱆 U+F0C46
    'car-brake-abs': 986183,  # 󰱇 U+F0C47
    'car-brake-alert': 986184,  # 󰱈 U+F0C48
    'car-esp': 986185,  # 󰱉 U+F0C49
    'car-light-dimmed': 986186,  # 󰱊 U+F0C4A
    'car-light-fog': 986187,  # 󰱋 U+F0C4B
    'car-light-high': 986188,  # 󰱌 U+F0C4C
    'car-tire-alert': 986189,  # 󰱍 U+F0C4D
    'cart-arrow-right': 986190,  # 󰱎 U+F0C4E
    'charity': 986191,  # 󰱏 U+F0C4F
    'chart-bell-curve': 986192,  # 󰱐 U+F0C50
    'checkbox-multiple-outline': 986193,  # 󰱑 U+F0C51
    'checkbox-outline': 986194,  # 󰱒 U+F0C52
    'check-network': 986195,  # 󰱓 U+F0C53
    'check-network-outline': 986196,  # 󰱔 U+F0C54
    'clipboard-account-outline': 986197,  # 󰱕 U+F0C55
    'clipboard-arrow-down-outline': 986198,  # 󰱖 U+F0C56
    'clipboard-arrow-up': 986199,  # 󰱗 U+F0C57
    'clipboard-arrow-up-outline': 986200,  # 󰱘 U+F0C58
    'clipboard-play': 986201,  # 󰱙 U+F0C59
    'clipboard-play-outline': 986202,  # 󰱚 U+F0C5A
    'clipboard-text-play': 986203,  # 󰱛 U+F0C5B
    'clipboard-text-play-outline': 986204,  # 󰱜 U+F0C5C
    'close-box-multiple': 986205,  # 󰱝 U+F0C5D
    'close-box-multiple-outline': 986206,  # 󰱞 U+F0C5E
    'close-network-outline': 986207,  # 󰱟 U+F0C5F
    'console-network-outline': 986208,  # 󰱠 U+F0C60
    'currency-ils': 986209,  # 󰱡 U+F0C61
    'delete-sweep-outline': 986210,  # 󰱢 U+F0C62
    'diameter': 986211,  # 󰱣 U+F0C63
    'diameter-outline': 986212,  # 󰱤 U+F0C64
    'diameter-variant': 986213,  # 󰱥 U+F0C65
    'download-network-outline': 986214,  # 󰱦 U+F0C66
    'dump-truck': 986215,  # 󰱧 U+F0C67
    'emoticon': 986216,  # 󰱨 U+F0C68
    'emoticon-angry': 986217,  # 󰱩 U+F0C69
    'emoticon-angry-outline': 986218,  # 󰱪 U+F0C6A
    'emoticon-cool': 986219,  # 󰱫 U+F0C6B
    'emoticon-cry': 986220,  # 󰱬 U+F0C6C
    'emoticon-cry-outline': 986221,  # 󰱭 U+F0C6D
    'emoticon-dead': 986222,  # 󰱮 U+F0C6E
    'emoticon-devil': 986223,  # 󰱯 U+F0C6F
    'emoticon-excited': 986224,  # 󰱰 U+F0C70
    'emoticon-happy': 986225,  # 󰱱 U+F0C71
    'emoticon-kiss': 986226,  # 󰱲 U+F0C72
    'emoticon-kiss-outline': 986227,  # 󰱳 U+F0C73
    'emoticon-neutral': 986228,  # 󰱴 U+F0C74
    'emoticon-poop-outline': 986229,  # 󰱵 U+F0C75
    'emoticon-sad': 986230,  # 󰱶 U+F0C76
    'emoticon-tongue-outline': 986231,  # 󰱷 U+F0C77
    'emoticon-wink': 986232,  # 󰱸 U+F0C78
    'emoticon-wink-outline': 986233,  # 󰱹 U+F0C79
    'eslint': 986234,  # 󰱺 U+F0C7A
    'face-recognition': 986235,  # 󰱻 U+F0C7B
    'file-search': 986236,  # 󰱼 U+F0C7C
    'file-search-outline': 986237,  # 󰱽 U+F0C7D
    'file-table': 986238,  # 󰱾 U+F0C7E
    'file-table-outline': 986239,  # 󰱿 U+F0C7F
    'folder-key-network-outline': 986240,  # 󰲀 U+F0C80
    'folder-network-outline': 986241,  # 󰲁 U+F0C81
    'folder-text': 986242,  # 󰲂 U+F0C82
    'folder-text-outline': 986243,  # 󰲃 U+F0C83
    'food-apple-outline': 986244,  # 󰲄 U+F0C84
    'fuse': 986245,  # 󰲅 U+F0C85
    'fuse-blade': 986246,  # 󰲆 U+F0C86
    'google-ads': 986247,  # 󰲇 U+F0C87
    'google-street-view': 986248,  # 󰲈 U+F0C88
    'hazard-lights': 986249,  # 󰲉 U+F0C89
    'help-network-outline': 986250,  # 󰲊 U+F0C8A
    'application-brackets': 986251,  # 󰲋 U+F0C8B
    'application-brackets-outline': 986252,  # 󰲌 U+F0C8C
    'image-size-select-actual': 986253,  # 󰲍 U+F0C8D
    'image-size-select-large': 986254,  # 󰲎 U+F0C8E
    'image-size-select-small': 986255,  # 󰲏 U+F0C8F
    'ip-network-outline': 986256,  # 󰲐 U+F0C90
    'ipod': 986257,  # 󰲑 U+F0C91
    'language-haskell': 986258,  # 󰲒 U+F0C92
    'leaf-maple': 986259,  # 󰲓 U+F0C93
    'link-plus': 986260,  # 󰲔 U+F0C94
    'map-marker-check': 986261,  # 󰲕 U+F0C95
    'math-cos': 986262,  # 󰲖 U+F0C96
    'math-sin': 986263,  # 󰲗 U+F0C97
    'math-tan': 986264,  # 󰲘 U+F0C98
    'microwave': 986265,  # 󰲙 U+F0C99
    'minus-network-outline': 986266,  # 󰲚 U+F0C9A
    'network-off': 986267,  # 󰲛 U+F0C9B
    'network-off-outline': 986268,  # 󰲜 U+F0C9C
    'network-outline': 986269,  # 󰲝 U+F0C9D
    'alpha-o-circle#1': 986270,  # 󰲞 U+F0C9E
    'alpha-o-circle-outline#1': 986271,  # 󰲟 U+F0C9F
    'numeric-1-circle': 986272,  # 󰲠 U+F0CA0
    'numeric-1-circle-outline': 986273,  # 󰲡 U+F0CA1
    'numeric-2-circle': 986274,  # 󰲢 U+F0CA2
    'numeric-2-circle-outline': 986275,  # 󰲣 U+F0CA3
    'numeric-3-circle': 986276,  # 󰲤 U+F0CA4
    'numeric-3-circle-outline': 986277,  # 󰲥 U+F0CA5
    'numeric-4-circle': 986278,  # 󰲦 U+F0CA6
    'numeric-4-circle-outline': 986279,  # 󰲧 U+F0CA7
    'numeric-5-circle': 986280,  # 󰲨 U+F0CA8
    'numeric-5-circle-outline': 986281,  # 󰲩 U+F0CA9
    'numeric-6-circle': 986282,  # 󰲪 U+F0CAA
    'numeric-6-circle-outline': 986283,  # 󰲫 U+F0CAB
    'numeric-7-circle': 986284,  # 󰲬 U+F0CAC
    'numeric-7-circle-outline': 986285,  # 󰲭 U+F0CAD
    'numeric-8-circle': 986286,  # 󰲮 U+F0CAE
    'numeric-8-circle-outline': 986287,  # 󰲯 U+F0CAF
    'numeric-9-circle': 986288,  # 󰲰 U+F0CB0
    'numeric-9-circle-outline': 986289,  # 󰲱 U+F0CB1
    'numeric-9-plus-circle': 986290,  # 󰲲 U+F0CB2
    'numeric-9-plus-circle-outline': 986291,  # 󰲳 U+F0CB3
    'parachute': 986292,  # 󰲴 U+F0CB4
    'parachute-outline': 986293,  # 󰲵 U+F0CB5
    'pencil-outline': 986294,  # 󰲶 U+F0CB6
    'play-network-outline': 986295,  # 󰲷 U+F0CB7
    'playlist-music': 986296,  # 󰲸 U+F0CB8
    'playlist-music-outline': 986297,  # 󰲹 U+F0CB9
    'plus-network-outline': 986298,  # 󰲺 U+F0CBA
    'postage-stamp': 986299,  # 󰲻 U+F0CBB
    'progress-alert': 986300,  # 󰲼 U+F0CBC
    'progress-wrench': 986301,  # 󰲽 U+F0CBD
    'radio-am': 986302,  # 󰲾 U+F0CBE
    'radio-fm': 986303,  # 󰲿 U+F0CBF
    'radius': 986304,  # 󰳀 U+F0CC0
    'radius-outline': 986305,  # 󰳁 U+F0CC1
    'ruler-square': 986306,  # 󰳂 U+F0CC2
    'seat': 986307,  # 󰳃 U+F0CC3
    'seat-outline': 986308,  # 󰳄 U+F0CC4
    'seatbelt': 986309,  # 󰳅 U+F0CC5
    'sheep': 986310,  # 󰳆 U+F0CC6
    'shield-airplane-outline': 986311,  # 󰳇 U+F0CC7
    'shield-check-outline': 986312,  # 󰳈 U+F0CC8
    'shield-cross': 986313,  # 󰳉 U+F0CC9
    'shield-cross-outline': 986314,  # 󰳊 U+F0CCA
    'shield-home-outline': 986315,  # 󰳋 U+F0CCB
    'shield-lock-outline': 986316,  # 󰳌 U+F0CCC
    'sort-variant-lock': 986317,  # 󰳍 U+F0CCD
    'sort-variant-lock-open': 986318,  # 󰳎 U+F0CCE
    'source-repository': 986319,  # 󰳏 U+F0CCF
    'source-repository-multiple': 986320,  # 󰳐 U+F0CD0
    'spa#1': 986321,  # 󰳑 U+F0CD1
    'spa-outline': 986322,  # 󰳒 U+F0CD2
    'toaster-oven': 986323,  # 󰳓 U+F0CD3
    'truck-check': 986324,  # 󰳔 U+F0CD4
    'turnstile': 986325,  # 󰳕 U+F0CD5
    'turnstile-outline': 986326,  # 󰳖 U+F0CD6
    'turtle': 986327,  # 󰳗 U+F0CD7
    'upload-network-outline': 986328,  # 󰳘 U+F0CD8
    'vibrate-off': 986329,  # 󰳙 U+F0CD9
    'watch-vibrate-off': 986330,  # 󰳚 U+F0CDA
    'arrow-down-circle': 986331,  # 󰳛 U+F0CDB
    'arrow-down-circle-outline': 986332,  # 󰳜 U+F0CDC
    'arrow-left-circle': 986333,  # 󰳝 U+F0CDD
    'arrow-left-circle-outline': 986334,  # 󰳞 U+F0CDE
    'arrow-right-circle': 986335,  # 󰳟 U+F0CDF
    'arrow-right-circle-outline': 986336,  # 󰳠 U+F0CE0
    'arrow-up-circle': 986337,  # 󰳡 U+F0CE1
    'arrow-up-circle-outline': 986338,  # 󰳢 U+F0CE2
    'account-tie': 986339,  # 󰳣 U+F0CE3
    'alert-box-outline': 986340,  # 󰳤 U+F0CE4
    'alert-decagram-outline': 986341,  # 󰳥 U+F0CE5
    'alert-octagon-outline': 986342,  # 󰳦 U+F0CE6
    'alert-octagram-outline': 986343,  # 󰳧 U+F0CE7
    'ammunition': 986344,  # 󰳨 U+F0CE8
    'account-music-outline': 986345,  # 󰳩 U+F0CE9
    'beaker#2': 986346,  # 󰳪 U+F0CEA
    'blender#1': 986347,  # 󰳫 U+F0CEB
    'blood-bag': 986348,  # 󰳬 U+F0CEC
    'cross-bolnisi': 986349,  # 󰳭 U+F0CED
    'bread-slice': 986350,  # 󰳮 U+F0CEE
    'bread-slice-outline': 986351,  # 󰳯 U+F0CEF
    'briefcase-account': 986352,  # 󰳰 U+F0CF0
    'briefcase-account-outline': 986353,  # 󰳱 U+F0CF1
    'brightness-percent': 986354,  # 󰳲 U+F0CF2
    'bullet': 986355,  # 󰳳 U+F0CF3
    'cash-register': 986356,  # 󰳴 U+F0CF4
    'cross-celtic': 986357,  # 󰳵 U+F0CF5
    'cross-outline': 986358,  # 󰳶 U+F0CF6
    'clipboard-alert-outline': 986359,  # 󰳷 U+F0CF7
    'clipboard-arrow-left-outline': 986360,  # 󰳸 U+F0CF8
    'clipboard-arrow-right': 986361,  # 󰳹 U+F0CF9
    'clipboard-arrow-right-outline': 986362,  # 󰳺 U+F0CFA
    'content-save-edit': 986363,  # 󰳻 U+F0CFB
    'content-save-edit-outline': 986364,  # 󰳼 U+F0CFC
    'cursor-default-click': 986365,  # 󰳽 U+F0CFD
    'cursor-default-click-outline': 986366,  # 󰳾 U+F0CFE
    'database-sync': 986367,  # 󰳿 U+F0CFF
    'database-remove': 986368,  # 󰴀 U+F0D00
    'database-settings': 986369,  # 󰴁 U+F0D01
    'drama-masks': 986370,  # 󰴂 U+F0D02
    'email-box': 986371,  # 󰴃 U+F0D03
    'eye-check': 986372,  # 󰴄 U+F0D04
    'eye-check-outline': 986373,  # 󰴅 U+F0D05
    'fast-forward-30': 986374,  # 󰴆 U+F0D06
    'order-alphabetical-descending': 986375,  # 󰴇 U+F0D07
    'flower-poppy': 986376,  # 󰴈 U+F0D08
    'folder-pound': 986377,  # 󰴉 U+F0D09
    'folder-pound-outline': 986378,  # 󰴊 U+F0D0A
    'folder-sync': 986379,  # 󰴋 U+F0D0B
    'folder-sync-outline': 986380,  # 󰴌 U+F0D0C
    'format-list-numbered-rtl': 986381,  # 󰴍 U+F0D0D
    'format-text-wrapping-clip': 986382,  # 󰴎 U+F0D0E
    'format-text-wrapping-overflow': 986383,  # 󰴏 U+F0D0F
    'format-text-wrapping-wrap': 986384,  # 󰴐 U+F0D10
    'format-textbox': 986385,  # 󰴑 U+F0D11
    'fountain-pen': 986386,  # 󰴒 U+F0D12
    'fountain-pen-tip': 986387,  # 󰴓 U+F0D13
    'heart-broken-outline': 986388,  # 󰴔 U+F0D14
    'home-city': 986389,  # 󰴕 U+F0D15
    'home-city-outline': 986390,  # 󰴖 U+F0D16
    'hubspot#1': 986391,  # 󰴗 U+F0D17
    'filmstrip-box-multiple': 986392,  # 󰴘 U+F0D18
    'play-box-multiple': 986393,  # 󰴙 U+F0D19
    'link-box': 986394,  # 󰴚 U+F0D1A
    'link-box-outline': 986395,  # 󰴛 U+F0D1B
    'link-box-variant': 986396,  # 󰴜 U+F0D1C
    'link-box-variant-outline': 986397,  # 󰴝 U+F0D1D
    'map-clock': 986398,  # 󰴞 U+F0D1E
    'map-clock-outline': 986399,  # 󰴟 U+F0D1F
    'map-marker-path': 986400,  # 󰴠 U+F0D20
    'mother-nurse': 986401,  # 󰴡 U+F0D21
    'microsoft-outlook': 986402,  # 󰴢 U+F0D22
    'perspective-less': 986403,  # 󰴣 U+F0D23
    'perspective-more': 986404,  # 󰴤 U+F0D24
    'podium': 986405,  # 󰴥 U+F0D25
    'podium-bronze': 986406,  # 󰴦 U+F0D26
    'podium-gold': 986407,  # 󰴧 U+F0D27
    'podium-silver': 986408,  # 󰴨 U+F0D28
    'quora#1': 986409,  # 󰴩 U+F0D29
    'rewind-10': 986410,  # 󰴪 U+F0D2A
    'roller-skate': 986411,  # 󰴫 U+F0D2B
    'rollerblade': 986412,  # 󰴬 U+F0D2C
    'language-ruby': 986413,  # 󰴭 U+F0D2D
    'sack': 986414,  # 󰴮 U+F0D2E
    'sack-percent': 986415,  # 󰴯 U+F0D2F
    'safety-goggles': 986416,  # 󰴰 U+F0D30
    'select-color': 986417,  # 󰴱 U+F0D31
    'selection-ellipse': 986418,  # 󰴲 U+F0D32
    'shield-link-variant': 986419,  # 󰴳 U+F0D33
    'shield-link-variant-outline': 986420,  # 󰴴 U+F0D34
    'skate': 986421,  # 󰴵 U+F0D35
    'skew-less': 986422,  # 󰴶 U+F0D36
    'skew-more': 986423,  # 󰴷 U+F0D37
    'speaker-multiple': 986424,  # 󰴸 U+F0D38
    'stamper': 986425,  # 󰴹 U+F0D39
    'tank': 986426,  # 󰴺 U+F0D3A
    'tortoise': 986427,  # 󰴻 U+F0D3B
    'transit-connection': 986428,  # 󰴼 U+F0D3C
    'transit-connection-variant': 986429,  # 󰴽 U+F0D3D
    'transmission-tower': 986430,  # 󰴾 U+F0D3E
    'weight-gram': 986431,  # 󰴿 U+F0D3F
    'youtube-subscription': 986432,  # 󰵀 U+F0D40
    'zigbee': 986433,  # 󰵁 U+F0D41
    'email-alert-outline': 986434,  # 󰵂 U+F0D42
    'air-filter': 986435,  # 󰵃 U+F0D43
    'air-purifier': 986436,  # 󰵄 U+F0D44
    'android-messages': 986437,  # 󰵅 U+F0D45
    'apps-box': 986438,  # 󰵆 U+F0D46
    'atm': 986439,  # 󰵇 U+F0D47
    'axis': 986440,  # 󰵈 U+F0D48
    'axis-arrow': 986441,  # 󰵉 U+F0D49
    'axis-arrow-lock': 986442,  # 󰵊 U+F0D4A
    'axis-lock': 986443,  # 󰵋 U+F0D4B
    'axis-x-arrow': 986444,  # 󰵌 U+F0D4C
    'axis-x-arrow-lock': 986445,  # 󰵍 U+F0D4D
    'axis-x-rotate-clockwise': 986446,  # 󰵎 U+F0D4E
    'axis-x-rotate-counterclockwise': 986447,  # 󰵏 U+F0D4F
    'axis-x-y-arrow-lock': 986448,  # 󰵐 U+F0D50
    'axis-y-arrow': 986449,  # 󰵑 U+F0D51
    'axis-y-arrow-lock': 986450,  # 󰵒 U+F0D52
    'axis-y-rotate-clockwise': 986451,  # 󰵓 U+F0D53
    'axis-y-rotate-counterclockwise': 986452,  # 󰵔 U+F0D54
    'axis-z-arrow': 986453,  # 󰵕 U+F0D55
    'axis-z-arrow-lock': 986454,  # 󰵖 U+F0D56
    'axis-z-rotate-clockwise': 986455,  # 󰵗 U+F0D57
    'axis-z-rotate-counterclockwise': 986456,  # 󰵘 U+F0D58
    'bell-alert': 986457,  # 󰵙 U+F0D59
    'bell-circle': 986458,  # 󰵚 U+F0D5A
    'bell-circle-outline': 986459,  # 󰵛 U+F0D5B
    'calendar-minus': 986460,  # 󰵜 U+F0D5C
    'camera-outline': 986461,  # 󰵝 U+F0D5D
    'car-brake-hold': 986462,  # 󰵞 U+F0D5E
    'car-brake-parking': 986463,  # 󰵟 U+F0D5F
    'car-cruise-control': 986464,  # 󰵠 U+F0D60
    'car-defrost-front': 986465,  # 󰵡 U+F0D61
    'car-defrost-rear': 986466,  # 󰵢 U+F0D62
    'car-parking-lights': 986467,  # 󰵣 U+F0D63
    'car-traction-control': 986468,  # 󰵤 U+F0D64
    'bag-carry-on-check': 986469,  # 󰵥 U+F0D65
    'cart-arrow-down': 986470,  # 󰵦 U+F0D66
    'cart-arrow-up': 986471,  # 󰵧 U+F0D67
    'cart-minus': 986472,  # 󰵨 U+F0D68
    'cart-remove': 986473,  # 󰵩 U+F0D69
    'contactless-payment': 986474,  # 󰵪 U+F0D6A
    'creative-commons': 986475,  # 󰵫 U+F0D6B
    'credit-card-wireless-outline': 986476,  # 󰵬 U+F0D6C
    'cricket': 986477,  # 󰵭 U+F0D6D
    'dev-to': 986478,  # 󰵮 U+F0D6E
    'domain-off': 986479,  # 󰵯 U+F0D6F
    'face-agent': 986480,  # 󰵰 U+F0D70
    'fast-forward-10': 986481,  # 󰵱 U+F0D71
    'flare': 986482,  # 󰵲 U+F0D72
    'format-text-rotation-down': 986483,  # 󰵳 U+F0D73
    'format-text-rotation-none': 986484,  # 󰵴 U+F0D74
    'forwardburger': 986485,  # 󰵵 U+F0D75
    'gesture-swipe': 986486,  # 󰵶 U+F0D76
    'gesture-tap-hold': 986487,  # 󰵷 U+F0D77
    'file-gif-box': 986488,  # 󰵸 U+F0D78
    'go-kart': 986489,  # 󰵹 U+F0D79
    'go-kart-track': 986490,  # 󰵺 U+F0D7A
    'goodreads#1': 986491,  # 󰵻 U+F0D7B
    'grain': 986492,  # 󰵼 U+F0D7C
    'hdr': 986493,  # 󰵽 U+F0D7D
    'hdr-off': 986494,  # 󰵾 U+F0D7E
    'hiking': 986495,  # 󰵿 U+F0D7F
    'home-floor-1': 986496,  # 󰶀 U+F0D80
    'home-floor-2': 986497,  # 󰶁 U+F0D81
    'home-floor-3': 986498,  # 󰶂 U+F0D82
    'home-floor-a': 986499,  # 󰶃 U+F0D83
    'home-floor-b': 986500,  # 󰶄 U+F0D84
    'home-floor-g': 986501,  # 󰶅 U+F0D85
    'home-floor-l': 986502,  # 󰶆 U+F0D86
    'kabaddi': 986503,  # 󰶇 U+F0D87
    'mailbox-open': 986504,  # 󰶈 U+F0D88
    'mailbox-open-outline': 986505,  # 󰶉 U+F0D89
    'mailbox-open-up': 986506,  # 󰶊 U+F0D8A
    'mailbox-open-up-outline': 986507,  # 󰶋 U+F0D8B
    'mailbox-outline': 986508,  # 󰶌 U+F0D8C
    'mailbox-up': 986509,  # 󰶍 U+F0D8D
    'mailbox-up-outline': 986510,  # 󰶎 U+F0D8E
    'mixed-martial-arts': 986511,  # 󰶏 U+F0D8F
    'monitor-off': 986512,  # 󰶐 U+F0D90
    'motion-sensor': 986513,  # 󰶑 U+F0D91
    'point-of-sale': 986514,  # 󰶒 U+F0D92
    'racing-helmet': 986515,  # 󰶓 U+F0D93
    'racquetball': 986516,  # 󰶔 U+F0D94
    'restart-off': 986517,  # 󰶕 U+F0D95
    'rewind-30': 986518,  # 󰶖 U+F0D96
    'room-service-outline': 986519,  # 󰶗 U+F0D97
    'rotate-orbit': 986520,  # 󰶘 U+F0D98
    'rugby': 986521,  # 󰶙 U+F0D99
    'shield-search': 986522,  # 󰶚 U+F0D9A
    'solar-panel': 986523,  # 󰶛 U+F0D9B
    'solar-panel-large': 986524,  # 󰶜 U+F0D9C
    'subway-alert-variant': 986525,  # 󰶝 U+F0D9D
    'tea': 986526,  # 󰶞 U+F0D9E
    'tea-outline': 986527,  # 󰶟 U+F0D9F
    'tennis': 986528,  # 󰶠 U+F0DA0
    'transfer-down': 986529,  # 󰶡 U+F0DA1
    'transfer-left': 986530,  # 󰶢 U+F0DA2
    'transfer-up': 986531,  # 󰶣 U+F0DA3
    'trophy-broken': 986532,  # 󰶤 U+F0DA4
    'wind-turbine': 986533,  # 󰶥 U+F0DA5
    'wiper-wash': 986534,  # 󰶦 U+F0DA6
    'badge-account': 986535,  # 󰶧 U+F0DA7
    'badge-account-alert': 986536,  # 󰶨 U+F0DA8
    'badge-account-alert-outline': 986537,  # 󰶩 U+F0DA9
    'badge-account-outline': 986538,  # 󰶪 U+F0DAA
    'card-account-details-outline': 986539,  # 󰶫 U+F0DAB
    'air-horn': 986540,  # 󰶬 U+F0DAC
    'application-export': 986541,  # 󰶭 U+F0DAD
    'application-import': 986542,  # 󰶮 U+F0DAE
    'bandage#1': 986543,  # 󰶯 U+F0DAF
    'bank-minus': 986544,  # 󰶰 U+F0DB0
    'bank-plus': 986545,  # 󰶱 U+F0DB1
    'bank-remove': 986546,  # 󰶲 U+F0DB2
    'bolt#1': 986547,  # 󰶳 U+F0DB3
    'bugle': 986548,  # 󰶴 U+F0DB4
    'cactus': 986549,  # 󰶵 U+F0DB5
    'camera-wireless': 986550,  # 󰶶 U+F0DB6
    'camera-wireless-outline': 986551,  # 󰶷 U+F0DB7
    'cash-marker': 986552,  # 󰶸 U+F0DB8
    'chevron-triple-down': 986553,  # 󰶹 U+F0DB9
    'chevron-triple-left': 986554,  # 󰶺 U+F0DBA
    'chevron-triple-right': 986555,  # 󰶻 U+F0DBB
    'chevron-triple-up': 986556,  # 󰶼 U+F0DBC
    'closed-caption-outline': 986557,  # 󰶽 U+F0DBD
    'credit-card-marker-outline': 986558,  # 󰶾 U+F0DBE
    'diving-flippers': 986559,  # 󰶿 U+F0DBF
    'diving-helmet': 986560,  # 󰷀 U+F0DC0
    'diving-scuba': 986561,  # 󰷁 U+F0DC1
    'diving-scuba-flag': 986562,  # 󰷂 U+F0DC2
    'diving-scuba-tank': 986563,  # 󰷃 U+F0DC3
    'diving-scuba-tank-multiple': 986564,  # 󰷄 U+F0DC4
    'diving-snorkel': 986565,  # 󰷅 U+F0DC5
    'file-cancel': 986566,  # 󰷆 U+F0DC6
    'file-cancel-outline': 986567,  # 󰷇 U+F0DC7
    'file-document-edit': 986568,  # 󰷈 U+F0DC8
    'file-document-edit-outline': 986569,  # 󰷉 U+F0DC9
    'file-eye': 986570,  # 󰷊 U+F0DCA
    'file-eye-outline': 986571,  # 󰷋 U+F0DCB
    'folder-alert': 986572,  # 󰷌 U+F0DCC
    'folder-alert-outline': 986573,  # 󰷍 U+F0DCD
    'folder-edit-outline': 986574,  # 󰷎 U+F0DCE
    'folder-open-outline': 986575,  # 󰷏 U+F0DCF
    'format-list-bulleted-square': 986576,  # 󰷐 U+F0DD0
    'gantry-crane': 986577,  # 󰷑 U+F0DD1
    'home-floor-0': 986578,  # 󰷒 U+F0DD2
    'home-floor-negative-1': 986579,  # 󰷓 U+F0DD3
    'home-group': 986580,  # 󰷔 U+F0DD4
    'jabber': 986581,  # 󰷕 U+F0DD5
    'key-outline': 986582,  # 󰷖 U+F0DD6
    'leak': 986583,  # 󰷗 U+F0DD7
    'leak-off': 986584,  # 󰷘 U+F0DD8
    'marker-cancel': 986585,  # 󰷙 U+F0DD9
    'mine': 986586,  # 󰷚 U+F0DDA
    'monitor-lock': 986587,  # 󰷛 U+F0DDB
    'monitor-star': 986588,  # 󰷜 U+F0DDC
    'movie-outline': 986589,  # 󰷝 U+F0DDD
    'music-note-plus': 986590,  # 󰷞 U+F0DDE
    'nail': 986591,  # 󰷟 U+F0DDF
    'ocarina': 986592,  # 󰷠 U+F0DE0
    'passport-biometric': 986593,  # 󰷡 U+F0DE1
    'pen-lock': 986594,  # 󰷢 U+F0DE2
    'pen-minus': 986595,  # 󰷣 U+F0DE3
    'pen-off': 986596,  # 󰷤 U+F0DE4
    'pen-plus': 986597,  # 󰷥 U+F0DE5
    'pen-remove': 986598,  # 󰷦 U+F0DE6
    'pencil-lock-outline': 986599,  # 󰷧 U+F0DE7
    'pencil-minus': 986600,  # 󰷨 U+F0DE8
    'pencil-minus-outline': 986601,  # 󰷩 U+F0DE9
    'pencil-off-outline': 986602,  # 󰷪 U+F0DEA
    'pencil-plus': 986603,  # 󰷫 U+F0DEB
    'pencil-plus-outline': 986604,  # 󰷬 U+F0DEC
    'pencil-remove': 986605,  # 󰷭 U+F0DED
    'pencil-remove-outline': 986606,  # 󰷮 U+F0DEE
    'phone-off': 986607,  # 󰷯 U+F0DEF
    'phone-outline': 986608,  # 󰷰 U+F0DF0
    'pi-hole': 986609,  # 󰷱 U+F0DF1
    'playlist-star': 986610,  # 󰷲 U+F0DF2
    'screw-flat-top': 986611,  # 󰷳 U+F0DF3
    'screw-lag': 986612,  # 󰷴 U+F0DF4
    'screw-machine-flat-top': 986613,  # 󰷵 U+F0DF5
    'screw-machine-round-top': 986614,  # 󰷶 U+F0DF6
    'screw-round-top': 986615,  # 󰷷 U+F0DF7
    'send-circle': 986616,  # 󰷸 U+F0DF8
    'send-circle-outline': 986617,  # 󰷹 U+F0DF9
    'shoe-print': 986618,  # 󰷺 U+F0DFA
    'signature#1': 986619,  # 󰷻 U+F0DFB
    'signature-freehand': 986620,  # 󰷼 U+F0DFC
    'signature-image': 986621,  # 󰷽 U+F0DFD
    'signature-text': 986622,  # 󰷾 U+F0DFE
    'slope-downhill': 986623,  # 󰷿 U+F0DFF
    'slope-uphill': 986624,  # 󰸀 U+F0E00
    'thermometer-alert': 986625,  # 󰸁 U+F0E01
    'thermometer-chevron-down': 986626,  # 󰸂 U+F0E02
    'thermometer-chevron-up': 986627,  # 󰸃 U+F0E03
    'thermometer-minus': 986628,  # 󰸄 U+F0E04
    'thermometer-plus': 986629,  # 󰸅 U+F0E05
    'translate-off': 986630,  # 󰸆 U+F0E06
    'upload-outline': 986631,  # 󰸇 U+F0E07
    'volume-variant-off': 986632,  # 󰸈 U+F0E08
    'wallpaper': 986633,  # 󰸉 U+F0E09
    'water-outline': 986634,  # 󰸊 U+F0E0A
    'wifi-star': 986635,  # 󰸋 U+F0E0B
    'palette-outline': 986636,  # 󰸌 U+F0E0C
    'badge-account-horizontal': 986637,  # 󰸍 U+F0E0D
    'badge-account-horizontal-outline': 986638,  # 󰸎 U+F0E0E
    'aws#1': 986639,  # 󰸏 U+F0E0F
    'bag-personal': 986640,  # 󰸐 U+F0E10
    'bag-personal-off': 986641,  # 󰸑 U+F0E11
    'bag-personal-off-outline': 986642,  # 󰸒 U+F0E12
    'bag-personal-outline': 986643,  # 󰸓 U+F0E13
    'biathlon': 986644,  # 󰸔 U+F0E14
    'bookmark-multiple': 986645,  # 󰸕 U+F0E15
    'bookmark-multiple-outline': 986646,  # 󰸖 U+F0E16
    'calendar-month': 986647,  # 󰸗 U+F0E17
    'calendar-month-outline': 986648,  # 󰸘 U+F0E18
    'camera-retake': 986649,  # 󰸙 U+F0E19
    'camera-retake-outline': 986650,  # 󰸚 U+F0E1A
    'car-back': 986651,  # 󰸛 U+F0E1B
    'car-off': 986652,  # 󰸜 U+F0E1C
    'cast-education': 986653,  # 󰸝 U+F0E1D
    'check-bold': 986654,  # 󰸞 U+F0E1E
    'check-underline': 986655,  # 󰸟 U+F0E1F
    'check-underline-circle': 986656,  # 󰸠 U+F0E20
    'check-underline-circle-outline': 986657,  # 󰸡 U+F0E21
    'circular-saw': 986658,  # 󰸢 U+F0E22
    'comma': 986659,  # 󰸣 U+F0E23
    'comma-box-outline': 986660,  # 󰸤 U+F0E24
    'comma-circle': 986661,  # 󰸥 U+F0E25
    'comma-circle-outline': 986662,  # 󰸦 U+F0E26
    'content-save-move': 986663,  # 󰸧 U+F0E27
    'content-save-move-outline': 986664,  # 󰸨 U+F0E28
    'file-check-outline': 986665,  # 󰸩 U+F0E29
    'file-music-outline': 986666,  # 󰸪 U+F0E2A
    'comma-box': 986667,  # 󰸫 U+F0E2B
    'file-video-outline': 986668,  # 󰸬 U+F0E2C
    'file-png-box': 986669,  # 󰸭 U+F0E2D
    'fireplace': 986670,  # 󰸮 U+F0E2E
    'fireplace-off': 986671,  # 󰸯 U+F0E2F
    'firework': 986672,  # 󰸰 U+F0E30
    'format-color-highlight': 986673,  # 󰸱 U+F0E31
    'format-text-variant': 986674,  # 󰸲 U+F0E32
    'gamepad-circle': 986675,  # 󰸳 U+F0E33
    'gamepad-circle-down': 986676,  # 󰸴 U+F0E34
    'gamepad-circle-left': 986677,  # 󰸵 U+F0E35
    'gamepad-circle-outline': 986678,  # 󰸶 U+F0E36
    'gamepad-circle-right': 986679,  # 󰸷 U+F0E37
    'gamepad-circle-up': 986680,  # 󰸸 U+F0E38
    'gamepad-down': 986681,  # 󰸹 U+F0E39
    'gamepad-left': 986682,  # 󰸺 U+F0E3A
    'gamepad-right': 986683,  # 󰸻 U+F0E3B
    'gamepad-round': 986684,  # 󰸼 U+F0E3C
    'gamepad-round-down': 986685,  # 󰸽 U+F0E3D
    'gamepad-round-left': 986686,  # 󰸾 U+F0E3E
    'gamepad-round-outline': 986687,  # 󰸿 U+F0E3F
    'gamepad-round-right': 986688,  # 󰹀 U+F0E40
    'gamepad-round-up': 986689,  # 󰹁 U+F0E41
    'gamepad-up': 986690,  # 󰹂 U+F0E42
    'gatsby': 986691,  # 󰹃 U+F0E43
    'gift#3': 986692,  # 󰹄 U+F0E44
    'grill': 986693,  # 󰹅 U+F0E45
    'hand-back-left': 986694,  # 󰹆 U+F0E46
    'hand-back-right': 986695,  # 󰹇 U+F0E47
    'hand-saw': 986696,  # 󰹈 U+F0E48
    'image-frame': 986697,  # 󰹉 U+F0E49
    'invert-colors-off': 986698,  # 󰹊 U+F0E4A
    'keyboard-off-outline': 986699,  # 󰹋 U+F0E4B
    'layers-minus': 986700,  # 󰹌 U+F0E4C
    'layers-plus': 986701,  # 󰹍 U+F0E4D
    'layers-remove': 986702,  # 󰹎 U+F0E4E
    'lightbulb-off': 986703,  # 󰹏 U+F0E4F
    'lightbulb-off-outline': 986704,  # 󰹐 U+F0E50
    'monitor-screenshot': 986705,  # 󰹑 U+F0E51
    'ice-cream-off': 986706,  # 󰹒 U+F0E52
    'nfc-search-variant': 986707,  # 󰹓 U+F0E53
    'nfc-variant-off': 986708,  # 󰹔 U+F0E54
    'notebook-multiple': 986709,  # 󰹕 U+F0E55
    'hoop-house': 986710,  # 󰹖 U+F0E56
    'picture-in-picture-bottom-right': 986711,  # 󰹗 U+F0E57
    'picture-in-picture-bottom-right-outline': 986712,  # 󰹘 U+F0E58
    'picture-in-picture-top-right': 986713,  # 󰹙 U+F0E59
    'picture-in-picture-top-right-outline': 986714,  # 󰹚 U+F0E5A
    'printer-3d-nozzle': 986715,  # 󰹛 U+F0E5B
    'printer-3d-nozzle-outline': 986716,  # 󰹜 U+F0E5C
    'printer-off': 986717,  # 󰹝 U+F0E5D
    'rectangle': 986718,  # 󰹞 U+F0E5E
    'rectangle-outline': 986719,  # 󰹟 U+F0E5F
    'rivet': 986720,  # 󰹠 U+F0E60
    'saw-blade': 986721,  # 󰹡 U+F0E61
    'seed': 986722,  # 󰹢 U+F0E62
    'seed-outline': 986723,  # 󰹣 U+F0E63
    'signal-distance-variant': 986724,  # 󰹤 U+F0E64
    'spade': 986725,  # 󰹥 U+F0E65
    'sprout': 986726,  # 󰹦 U+F0E66
    'sprout-outline': 986727,  # 󰹧 U+F0E67
    'table-tennis': 986728,  # 󰹨 U+F0E68
    'tree-outline': 986729,  # 󰹩 U+F0E69
    'view-comfy': 986730,  # 󰹪 U+F0E6A
    'view-compact': 986731,  # 󰹫 U+F0E6B
    'view-compact-outline': 986732,  # 󰹬 U+F0E6C
    'vuetify': 986733,  # 󰹭 U+F0E6D
    'weather-cloudy-arrow-right': 986734,  # 󰹮 U+F0E6E
    'microsoft-xbox-controller-menu': 986735,  # 󰹯 U+F0E6F
    'microsoft-xbox-controller-view': 986736,  # 󰹰 U+F0E70
    'alarm-note': 986737,  # 󰹱 U+F0E71
    'alarm-note-off': 986738,  # 󰹲 U+F0E72
    'arrow-left-right': 986739,  # 󰹳 U+F0E73
    'arrow-left-right-bold': 986740,  # 󰹴 U+F0E74
    'arrow-top-left-bottom-right': 986741,  # 󰹵 U+F0E75
    'arrow-top-left-bottom-right-bold': 986742,  # 󰹶 U+F0E76
    'arrow-top-right-bottom-left': 986743,  # 󰹷 U+F0E77
    'arrow-top-right-bottom-left-bold': 986744,  # 󰹸 U+F0E78
    'arrow-up-down': 986745,  # 󰹹 U+F0E79
    'arrow-up-down-bold': 986746,  # 󰹺 U+F0E7A
    'atom-variant': 986747,  # 󰹻 U+F0E7B
    'baby-face': 986748,  # 󰹼 U+F0E7C
    'baby-face-outline': 986749,  # 󰹽 U+F0E7D
    'backspace-reverse': 986750,  # 󰹾 U+F0E7E
    'backspace-reverse-outline': 986751,  # 󰹿 U+F0E7F
    'bank-outline': 986752,  # 󰺀 U+F0E80
    'bell-alert-outline': 986753,  # 󰺁 U+F0E81
    'book-play': 986754,  # 󰺂 U+F0E82
    'book-play-outline': 986755,  # 󰺃 U+F0E83
    'book-search': 986756,  # 󰺄 U+F0E84
    'book-search-outline': 986757,  # 󰺅 U+F0E85
    'boom-gate': 986758,  # 󰺆 U+F0E86
    'boom-gate-alert': 986759,  # 󰺇 U+F0E87
    'boom-gate-alert-outline': 986760,  # 󰺈 U+F0E88
    'boom-gate-arrow-down': 986761,  # 󰺉 U+F0E89
    'boom-gate-arrow-down-outline': 986762,  # 󰺊 U+F0E8A
    'boom-gate-outline': 986763,  # 󰺋 U+F0E8B
    'boom-gate-arrow-up': 986764,  # 󰺌 U+F0E8C
    'boom-gate-arrow-up-outline': 986765,  # 󰺍 U+F0E8D
    'calendar-sync': 986766,  # 󰺎 U+F0E8E
    'calendar-sync-outline': 986767,  # 󰺏 U+F0E8F
    'cellphone-nfc': 986768,  # 󰺐 U+F0E90
    'chart-areaspline-variant': 986769,  # 󰺑 U+F0E91
    'chart-scatter-plot': 986770,  # 󰺒 U+F0E92
    'chart-timeline-variant': 986771,  # 󰺓 U+F0E93
    'chart-tree': 986772,  # 󰺔 U+F0E94
    'circle-double': 986773,  # 󰺕 U+F0E95
    'circle-expand': 986774,  # 󰺖 U+F0E96
    'clock-digital': 986775,  # 󰺗 U+F0E97
    'card-account-mail-outline': 986776,  # 󰺘 U+F0E98
    'card-account-phone': 986777,  # 󰺙 U+F0E99
    'card-account-phone-outline': 986778,  # 󰺚 U+F0E9A
    'account-cowboy-hat': 986779,  # 󰺛 U+F0E9B
    'currency-rial': 986780,  # 󰺜 U+F0E9C
    'delete-empty-outline': 986781,  # 󰺝 U+F0E9D
    'dolly#1': 986782,  # 󰺞 U+F0E9E
    'electric-switch': 986783,  # 󰺟 U+F0E9F
    'ellipse': 986784,  # 󰺠 U+F0EA0
    'ellipse-outline': 986785,  # 󰺡 U+F0EA1
    'equalizer': 986786,  # 󰺢 U+F0EA2
    'equalizer-outline': 986787,  # 󰺣 U+F0EA3
    'ferris-wheel': 986788,  # 󰺤 U+F0EA4
    'file-delimited-outline': 986789,  # 󰺥 U+F0EA5
    'text-box-check': 986790,  # 󰺦 U+F0EA6
    'text-box-check-outline': 986791,  # 󰺧 U+F0EA7
    'text-box-minus': 986792,  # 󰺨 U+F0EA8
    'text-box-minus-outline': 986793,  # 󰺩 U+F0EA9
    'text-box-plus': 986794,  # 󰺪 U+F0EAA
    'text-box-plus-outline': 986795,  # 󰺫 U+F0EAB
    'text-box-remove': 986796,  # 󰺬 U+F0EAC
    'text-box-remove-outline': 986797,  # 󰺭 U+F0EAD
    'text-box-search': 986798,  # 󰺮 U+F0EAE
    'text-box-search-outline': 986799,  # 󰺯 U+F0EAF
    'file-image-outline': 986800,  # 󰺰 U+F0EB0
    'fingerprint-off': 986801,  # 󰺱 U+F0EB1
    'format-list-bulleted-triangle': 986802,  # 󰺲 U+F0EB2
    'format-overline': 986803,  # 󰺳 U+F0EB3
    'frequently-asked-questions': 986804,  # 󰺴 U+F0EB4
    'gamepad-square': 986805,  # 󰺵 U+F0EB5
    'gamepad-square-outline': 986806,  # 󰺶 U+F0EB6
    'gamepad-variant-outline': 986807,  # 󰺷 U+F0EB7
    'gas-station-outline': 986808,  # 󰺸 U+F0EB8
    'google-podcast': 986809,  # 󰺹 U+F0EB9
    'home-analytics': 986810,  # 󰺺 U+F0EBA
    'mail#2': 986811,  # 󰺻 U+F0EBB
    'map-check': 986812,  # 󰺼 U+F0EBC
    'map-check-outline': 986813,  # 󰺽 U+F0EBD
    'ruler-square-compass': 986814,  # 󰺾 U+F0EBE
    'notebook-outline': 986815,  # 󰺿 U+F0EBF
    'penguin': 986816,  # 󰻀 U+F0EC0
    'radioactive-off': 986817,  # 󰻁 U+F0EC1
    'record-circle': 986818,  # 󰻂 U+F0EC2
    'record-circle-outline': 986819,  # 󰻃 U+F0EC3
    'remote-off': 986820,  # 󰻄 U+F0EC4
    'remote-tv': 986821,  # 󰻅 U+F0EC5
    'remote-tv-off': 986822,  # 󰻆 U+F0EC6
    'rotate-3d': 986823,  # 󰻇 U+F0EC7
    'sail-boat': 986824,  # 󰻈 U+F0EC8
    'scatter-plot': 986825,  # 󰻉 U+F0EC9
    'scatter-plot-outline': 986826,  # 󰻊 U+F0ECA
    'segment': 986827,  # 󰻋 U+F0ECB
    'shield-alert': 986828,  # 󰻌 U+F0ECC
    'shield-alert-outline': 986829,  # 󰻍 U+F0ECD
    'tablet-dashboard': 986830,  # 󰻎 U+F0ECE
    'television-play': 986831,  # 󰻏 U+F0ECF
    'unicode': 986832,  # 󰻐 U+F0ED0
    'video-3d-variant': 986833,  # 󰻑 U+F0ED1
    'video-wireless': 986834,  # 󰻒 U+F0ED2
    'video-wireless-outline': 986835,  # 󰻓 U+F0ED3
    'account-voice-off': 986836,  # 󰻔 U+F0ED4
    'bacteria#1': 986837,  # 󰻕 U+F0ED5
    'bacteria-outline': 986838,  # 󰻖 U+F0ED6
    'calendar-account': 986839,  # 󰻗 U+F0ED7
    'calendar-account-outline': 986840,  # 󰻘 U+F0ED8
    'calendar-weekend': 986841,  # 󰻙 U+F0ED9
    'calendar-weekend-outline': 986842,  # 󰻚 U+F0EDA
    'camera-plus': 986843,  # 󰻛 U+F0EDB
    'camera-plus-outline': 986844,  # 󰻜 U+F0EDC
    'campfire': 986845,  # 󰻝 U+F0EDD
    'chat-outline': 986846,  # 󰻞 U+F0EDE
    'cpu-32-bit': 986847,  # 󰻟 U+F0EDF
    'cpu-64-bit': 986848,  # 󰻠 U+F0EE0
    'credit-card-clock': 986849,  # 󰻡 U+F0EE1
    'credit-card-clock-outline': 986850,  # 󰻢 U+F0EE2
    'email-edit': 986851,  # 󰻣 U+F0EE3
    'email-edit-outline': 986852,  # 󰻤 U+F0EE4
    'email-minus': 986853,  # 󰻥 U+F0EE5
    'email-minus-outline': 986854,  # 󰻦 U+F0EE6
    'email-multiple': 986855,  # 󰻧 U+F0EE7
    'email-multiple-outline': 986856,  # 󰻨 U+F0EE8
    'email-open-multiple': 986857,  # 󰻩 U+F0EE9
    'email-open-multiple-outline': 986858,  # 󰻪 U+F0EEA
    'file-cad': 986859,  # 󰻫 U+F0EEB
    'file-cad-box': 986860,  # 󰻬 U+F0EEC
    'file-plus-outline': 986861,  # 󰻭 U+F0EED
    'filter-minus': 986862,  # 󰻮 U+F0EEE
    'filter-minus-outline': 986863,  # 󰻯 U+F0EEF
    'filter-plus': 986864,  # 󰻰 U+F0EF0
    'filter-plus-outline': 986865,  # 󰻱 U+F0EF1
    'fire-extinguisher': 986866,  # 󰻲 U+F0EF2
    'fishbowl': 986867,  # 󰻳 U+F0EF3
    'fishbowl-outline': 986868,  # 󰻴 U+F0EF4
    'fit-to-page': 986869,  # 󰻵 U+F0EF5
    'fit-to-page-outline': 986870,  # 󰻶 U+F0EF6
    'flash-alert': 986871,  # 󰻷 U+F0EF7
    'flash-alert-outline': 986872,  # 󰻸 U+F0EF8
    'heart-flash': 986873,  # 󰻹 U+F0EF9
    'home-flood': 986874,  # 󰻺 U+F0EFA
    'human-male-height': 986875,  # 󰻻 U+F0EFB
    'human-male-height-variant': 986876,  # 󰻼 U+F0EFC
    'ice-pop': 986877,  # 󰻽 U+F0EFD
    'identifier': 986878,  # 󰻾 U+F0EFE
    'image-filter-center-focus-strong': 986879,  # 󰻿 U+F0EFF
    'image-filter-center-focus-strong-outline': 986880,  # 󰼀 U+F0F00
    'jellyfish': 986881,  # 󰼁 U+F0F01
    'jellyfish-outline': 986882,  # 󰼂 U+F0F02
    'lasso': 986883,  # 󰼃 U+F0F03
    'music-box-multiple-outline': 986884,  # 󰼄 U+F0F04
    'map-marker-alert': 986885,  # 󰼅 U+F0F05
    'map-marker-alert-outline': 986886,  # 󰼆 U+F0F06
    'map-marker-question': 986887,  # 󰼇 U+F0F07
    'map-marker-question-outline': 986888,  # 󰼈 U+F0F08
    'map-marker-remove': 986889,  # 󰼉 U+F0F09
    'map-marker-remove-variant': 986890,  # 󰼊 U+F0F0A
    'necklace': 986891,  # 󰼋 U+F0F0B
    'newspaper-minus': 986892,  # 󰼌 U+F0F0C
    'newspaper-plus': 986893,  # 󰼍 U+F0F0D
    'numeric-0-box-multiple': 986894,  # 󰼎 U+F0F0E
    'numeric-1-box-multiple': 986895,  # 󰼏 U+F0F0F
    'numeric-2-box-multiple': 986896,  # 󰼐 U+F0F10
    'numeric-3-box-multiple': 986897,  # 󰼑 U+F0F11
    'numeric-4-box-multiple': 986898,  # 󰼒 U+F0F12
    'numeric-5-box-multiple': 986899,  # 󰼓 U+F0F13
    'numeric-6-box-multiple': 986900,  # 󰼔 U+F0F14
    'numeric-7-box-multiple': 986901,  # 󰼕 U+F0F15
    'numeric-8-box-multiple': 986902,  # 󰼖 U+F0F16
    'numeric-9-box-multiple': 986903,  # 󰼗 U+F0F17
    'numeric-9-plus-box-multiple': 986904,  # 󰼘 U+F0F18
    'oil-lamp': 986905,  # 󰼙 U+F0F19
    'phone-alert': 986906,  # 󰼚 U+F0F1A
    'play-outline': 986907,  # 󰼛 U+F0F1B
    'purse': 986908,  # 󰼜 U+F0F1C
    'purse-outline': 986909,  # 󰼝 U+F0F1D
    'railroad-light': 986910,  # 󰼞 U+F0F1E
    'reply-all-outline': 986911,  # 󰼟 U+F0F1F
    'reply-outline': 986912,  # 󰼠 U+F0F20
    'rss-off': 986913,  # 󰼡 U+F0F21
    'selection-ellipse-arrow-inside': 986914,  # 󰼢 U+F0F22
    'share-off': 986915,  # 󰼣 U+F0F23
    'share-off-outline': 986916,  # 󰼤 U+F0F24
    'skip-backward-outline': 986917,  # 󰼥 U+F0F25
    'skip-forward-outline': 986918,  # 󰼦 U+F0F26
    'skip-next-outline': 986919,  # 󰼧 U+F0F27
    'skip-previous-outline': 986920,  # 󰼨 U+F0F28
    'snowflake-alert': 986921,  # 󰼩 U+F0F29
    'snowflake-variant': 986922,  # 󰼪 U+F0F2A
    'stretch-to-page': 986923,  # 󰼫 U+F0F2B
    'stretch-to-page-outline': 986924,  # 󰼬 U+F0F2C
    'typewriter': 986925,  # 󰼭 U+F0F2D
    'wave': 986926,  # 󰼮 U+F0F2E
    'weather-cloudy-alert': 986927,  # 󰼯 U+F0F2F
    'weather-hazy': 986928,  # 󰼰 U+F0F30
    'weather-night-partly-cloudy': 986929,  # 󰼱 U+F0F31
    'weather-partly-lightning': 986930,  # 󰼲 U+F0F32
    'weather-partly-rainy': 986931,  # 󰼳 U+F0F33
    'weather-partly-snowy': 986932,  # 󰼴 U+F0F34
    'weather-partly-snowy-rainy': 986933,  # 󰼵 U+F0F35
    'weather-snowy-heavy': 986934,  # 󰼶 U+F0F36
    'weather-sunny-alert': 986935,  # 󰼷 U+F0F37
    'weather-tornado': 986936,  # 󰼸 U+F0F38
    'baby-bottle': 986937,  # 󰼹 U+F0F39
    'baby-bottle-outline': 986938,  # 󰼺 U+F0F3A
    'bag-carry-on': 986939,  # 󰼻 U+F0F3B
    'bag-carry-on-off': 986940,  # 󰼼 U+F0F3C
    'bag-checked': 986941,  # 󰼽 U+F0F3D
    'baguette': 986942,  # 󰼾 U+F0F3E
    'bus-multiple': 986943,  # 󰼿 U+F0F3F
    'car-shift-pattern': 986944,  # 󰽀 U+F0F40
    'cellphone-information': 986945,  # 󰽁 U+F0F41
    'content-save-alert': 986946,  # 󰽂 U+F0F42
    'content-save-alert-outline': 986947,  # 󰽃 U+F0F43
    'content-save-all-outline': 986948,  # 󰽄 U+F0F44
    'crosshairs-off': 986949,  # 󰽅 U+F0F45
    'cupboard': 986950,  # 󰽆 U+F0F46
    'cupboard-outline': 986951,  # 󰽇 U+F0F47
    'chair-rolling': 986952,  # 󰽈 U+F0F48
    'draw': 986953,  # 󰽉 U+F0F49
    'dresser': 986954,  # 󰽊 U+F0F4A
    'dresser-outline': 986955,  # 󰽋 U+F0F4B
    'emoticon-frown': 986956,  # 󰽌 U+F0F4C
    'emoticon-frown-outline': 986957,  # 󰽍 U+F0F4D
    'focus-auto': 986958,  # 󰽎 U+F0F4E
    'focus-field': 986959,  # 󰽏 U+F0F4F
    'focus-field-horizontal': 986960,  # 󰽐 U+F0F50
    'focus-field-vertical': 986961,  # 󰽑 U+F0F51
    'foot-print': 986962,  # 󰽒 U+F0F52
    'handball': 986963,  # 󰽓 U+F0F53
    'home-thermometer': 986964,  # 󰽔 U+F0F54
    'home-thermometer-outline': 986965,  # 󰽕 U+F0F55
    'kettle-outline': 986966,  # 󰽖 U+F0F56
    'latitude': 986967,  # 󰽗 U+F0F57
    'layers-triple': 986968,  # 󰽘 U+F0F58
    'layers-triple-outline': 986969,  # 󰽙 U+F0F59
    'longitude': 986970,  # 󰽚 U+F0F5A
    'language-markdown-outline': 986971,  # 󰽛 U+F0F5B
    'merge#1': 986972,  # 󰽜 U+F0F5C
    'middleware': 986973,  # 󰽝 U+F0F5D
    'middleware-outline': 986974,  # 󰽞 U+F0F5E
    'monitor-speaker': 986975,  # 󰽟 U+F0F5F
    'monitor-speaker-off': 986976,  # 󰽠 U+F0F60
    'moon-first-quarter': 986977,  # 󰽡 U+F0F61
    'moon-full': 986978,  # 󰽢 U+F0F62
    'moon-last-quarter': 986979,  # 󰽣 U+F0F63
    'moon-new': 986980,  # 󰽤 U+F0F64
    'moon-waning-crescent': 986981,  # 󰽥 U+F0F65
    'moon-waning-gibbous': 986982,  # 󰽦 U+F0F66
    'moon-waxing-crescent': 986983,  # 󰽧 U+F0F67
    'moon-waxing-gibbous': 986984,  # 󰽨 U+F0F68
    'music-accidental-double-flat': 986985,  # 󰽩 U+F0F69
    'music-accidental-double-sharp': 986986,  # 󰽪 U+F0F6A
    'music-accidental-flat': 986987,  # 󰽫 U+F0F6B
    'music-accidental-natural': 986988,  # 󰽬 U+F0F6C
    'music-accidental-sharp': 986989,  # 󰽭 U+F0F6D
    'music-clef-alto': 986990,  # 󰽮 U+F0F6E
    'music-clef-bass': 986991,  # 󰽯 U+F0F6F
    'music-clef-treble': 986992,  # 󰽰 U+F0F70
    'music-note-eighth-dotted': 986993,  # 󰽱 U+F0F71
    'music-note-half-dotted': 986994,  # 󰽲 U+F0F72
    'music-note-off-outline': 986995,  # 󰽳 U+F0F73
    'music-note-outline': 986996,  # 󰽴 U+F0F74
    'music-note-quarter-dotted': 986997,  # 󰽵 U+F0F75
    'music-note-sixteenth-dotted': 986998,  # 󰽶 U+F0F76
    'music-note-whole-dotted': 986999,  # 󰽷 U+F0F77
    'music-rest-eighth': 987000,  # 󰽸 U+F0F78
    'music-rest-half': 987001,  # 󰽹 U+F0F79
    'music-rest-quarter': 987002,  # 󰽺 U+F0F7A
    'music-rest-sixteenth': 987003,  # 󰽻 U+F0F7B
    'music-rest-whole': 987004,  # 󰽼 U+F0F7C
    'numeric-10-box': 987005,  # 󰽽 U+F0F7D
    'numeric-10-box-outline': 987006,  # 󰽾 U+F0F7E
    'page-layout-header-footer': 987007,  # 󰽿 U+F0F7F
    'patio-heater': 987008,  # 󰾀 U+F0F80
    'warehouse#1': 987009,  # 󰾁 U+F0F81
    'select-group': 987010,  # 󰾂 U+F0F82
    'shield-car': 987011,  # 󰾃 U+F0F83
    'shopping-search': 987012,  # 󰾄 U+F0F84
    'speedometer-medium': 987013,  # 󰾅 U+F0F85
    'speedometer-slow': 987014,  # 󰾆 U+F0F86
    'table-large-plus': 987015,  # 󰾇 U+F0F87
    'table-large-remove': 987016,  # 󰾈 U+F0F88
    'television-pause': 987017,  # 󰾉 U+F0F89
    'television-stop': 987018,  # 󰾊 U+F0F8A
    'transit-detour': 987019,  # 󰾋 U+F0F8B
    'video-input-scart': 987020,  # 󰾌 U+F0F8C
    'view-grid-plus': 987021,  # 󰾍 U+F0F8D
    'wallet-plus': 987022,  # 󰾎 U+F0F8E
    'wallet-plus-outline': 987023,  # 󰾏 U+F0F8F
    'wardrobe': 987024,  # 󰾐 U+F0F90
    'wardrobe-outline': 987025,  # 󰾑 U+F0F91
    'water-boiler': 987026,  # 󰾒 U+F0F92
    'water-pump-off': 987027,  # 󰾓 U+F0F93
    'web-box': 987028,  # 󰾔 U+F0F94
    'timeline-alert': 987029,  # 󰾕 U+F0F95
    'timeline-plus': 987030,  # 󰾖 U+F0F96
    'timeline-plus-outline': 987031,  # 󰾗 U+F0F97
    'timeline-alert-outline': 987032,  # 󰾘 U+F0F98
    'timeline-help': 987033,  # 󰾙 U+F0F99
    'timeline-help-outline': 987034,  # 󰾚 U+F0F9A
    'home-export-outline': 987035,  # 󰾛 U+F0F9B
    'home-import-outline': 987036,  # 󰾜 U+F0F9C
    'account-filter-outline': 987037,  # 󰾝 U+F0F9D
    'approximately-equal': 987038,  # 󰾞 U+F0F9E
    'approximately-equal-box': 987039,  # 󰾟 U+F0F9F
    'baby-carriage-off': 987040,  # 󰾠 U+F0FA0
    'bee': 987041,  # 󰾡 U+F0FA1
    'bee-flower': 987042,  # 󰾢 U+F0FA2
    'car-child-seat': 987043,  # 󰾣 U+F0FA3
    'car-seat': 987044,  # 󰾤 U+F0FA4
    'car-seat-cooler': 987045,  # 󰾥 U+F0FA5
    'car-seat-heater': 987046,  # 󰾦 U+F0FA6
    'chart-bell-curve-cumulative': 987047,  # 󰾧 U+F0FA7
    'clock-check': 987048,  # 󰾨 U+F0FA8
    'clock-check-outline': 987049,  # 󰾩 U+F0FA9
    'coffee-off': 987050,  # 󰾪 U+F0FAA
    'coffee-off-outline': 987051,  # 󰾫 U+F0FAB
    'credit-card-minus': 987052,  # 󰾬 U+F0FAC
    'credit-card-minus-outline': 987053,  # 󰾭 U+F0FAD
    'credit-card-remove': 987054,  # 󰾮 U+F0FAE
    'credit-card-remove-outline': 987055,  # 󰾯 U+F0FAF
    'devices': 987056,  # 󰾰 U+F0FB0
    'email-newsletter': 987057,  # 󰾱 U+F0FB1
    'expansion-card-variant': 987058,  # 󰾲 U+F0FB2
    'power-socket-ch': 987059,  # 󰾳 U+F0FB3
    'file-swap': 987060,  # 󰾴 U+F0FB4
    'file-swap-outline': 987061,  # 󰾵 U+F0FB5
    'folder-swap': 987062,  # 󰾶 U+F0FB6
    'folder-swap-outline': 987063,  # 󰾷 U+F0FB7
    'format-letter-ends-with': 987064,  # 󰾸 U+F0FB8
    'format-letter-matches': 987065,  # 󰾹 U+F0FB9
    'format-letter-starts-with': 987066,  # 󰾺 U+F0FBA
    'format-text-rotation-angle-down': 987067,  # 󰾻 U+F0FBB
    'format-text-rotation-angle-up': 987068,  # 󰾼 U+F0FBC
    'format-text-rotation-down-vertical': 987069,  # 󰾽 U+F0FBD
    'format-text-rotation-up': 987070,  # 󰾾 U+F0FBE
    'format-text-rotation-vertical': 987071,  # 󰾿 U+F0FBF
    'id-card#1': 987072,  # 󰿀 U+F0FC0
    'image-auto-adjust': 987073,  # 󰿁 U+F0FC1
    'key-wireless': 987074,  # 󰿂 U+F0FC2
    'license': 987075,  # 󰿃 U+F0FC3
    'location-enter': 987076,  # 󰿄 U+F0FC4
    'location-exit': 987077,  # 󰿅 U+F0FC5
    'lock-open-variant': 987078,  # 󰿆 U+F0FC6
    'lock-open-variant-outline': 987079,  # 󰿇 U+F0FC7
    'math-integral': 987080,  # 󰿈 U+F0FC8
    'math-integral-box': 987081,  # 󰿉 U+F0FC9
    'math-norm': 987082,  # 󰿊 U+F0FCA
    'math-norm-box': 987083,  # 󰿋 U+F0FCB
    'message-lock': 987084,  # 󰿌 U+F0FCC
    'message-text-lock': 987085,  # 󰿍 U+F0FCD
    'movie-open': 987086,  # 󰿎 U+F0FCE
    'movie-open-outline': 987087,  # 󰿏 U+F0FCF
    'bed-queen': 987088,  # 󰿐 U+F0FD0
    'bed-king-outline': 987089,  # 󰿑 U+F0FD1
    'bed-king': 987090,  # 󰿒 U+F0FD2
    'bed-double-outline': 987091,  # 󰿓 U+F0FD3
    'bed-double': 987092,  # 󰿔 U+F0FD4
    'microsoft-azure-devops': 987093,  # 󰿕 U+F0FD5
    'arm-flex-outline': 987094,  # 󰿖 U+F0FD6
    'arm-flex': 987095,  # 󰿗 U+F0FD7
    'protocol': 987096,  # 󰿘 U+F0FD8
    'seal-variant': 987097,  # 󰿙 U+F0FD9
    'select-place': 987098,  # 󰿚 U+F0FDA
    'bed-queen-outline': 987099,  # 󰿛 U+F0FDB
    'sign-direction-plus': 987100,  # 󰿜 U+F0FDC
    'sign-direction-remove': 987101,  # 󰿝 U+F0FDD
    'silverware-clean': 987102,  # 󰿞 U+F0FDE
    'slash-forward': 987103,  # 󰿟 U+F0FDF
    'slash-forward-box': 987104,  # 󰿠 U+F0FE0
    'swap-horizontal-circle': 987105,  # 󰿡 U+F0FE1
    'swap-horizontal-circle-outline': 987106,  # 󰿢 U+F0FE2
    'swap-vertical-circle': 987107,  # 󰿣 U+F0FE3
    'swap-vertical-circle-outline': 987108,  # 󰿤 U+F0FE4
    'tanker-truck': 987109,  # 󰿥 U+F0FE5
    'texture-box': 987110,  # 󰿦 U+F0FE6
    'tram-side': 987111,  # 󰿧 U+F0FE7
    'vector-link': 987112,  # 󰿨 U+F0FE8
    'numeric-10': 987113,  # 󰿩 U+F0FE9
    'numeric-10-box-multiple': 987114,  # 󰿪 U+F0FEA
    'numeric-10-box-multiple-outline': 987115,  # 󰿫 U+F0FEB
    'numeric-10-circle': 987116,  # 󰿬 U+F0FEC
    'numeric-10-circle-outline': 987117,  # 󰿭 U+F0FED
    'numeric-9-plus': 987118,  # 󰿮 U+F0FEE
    'credit-card#2': 987119,  # 󰿯 U+F0FEF
    'credit-card-multiple': 987120,  # 󰿰 U+F0FF0
    'credit-card-off': 987121,  # 󰿱 U+F0FF1
    'credit-card-plus': 987122,  # 󰿲 U+F0FF2
    'credit-card-refund': 987123,  # 󰿳 U+F0FF3
    'credit-card-scan': 987124,  # 󰿴 U+F0FF4
    'credit-card-settings': 987125,  # 󰿵 U+F0FF5
    'hospital#1': 987126,  # 󰿶 U+F0FF6
    'hospital-box-outline': 987127,  # 󰿷 U+F0FF7
    'oil-temperature': 987128,  # 󰿸 U+F0FF8
    'stadium': 987129,  # 󰿹 U+F0FF9
    'zip-box-outline': 987130,  # 󰿺 U+F0FFA
    'account-edit-outline': 987131,  # 󰿻 U+F0FFB
    'peanut': 987132,  # 󰿼 U+F0FFC
    'peanut-off': 987133,  # 󰿽 U+F0FFD
    'peanut-outline': 987134,  # 󰿾 U+F0FFE
    'peanut-off-outline': 987135,  # 󰿿 U+F0FFF
    'sign-direction-minus': 987136,  # 󱀀 U+F1000
    'newspaper-variant': 987137,  # 󱀁 U+F1001
    'newspaper-variant-multiple': 987138,  # 󱀂 U+F1002
    'newspaper-variant-multiple-outline': 987139,  # 󱀃 U+F1003
    'newspaper-variant-outline': 987140,  # 󱀄 U+F1004
    'overscan': 987141,  # 󱀅 U+F1005
    'pig-variant': 987142,  # 󱀆 U+F1006
    'piggy-bank': 987143,  # 󱀇 U+F1007
    'post': 987144,  # 󱀈 U+F1008
    'post-outline': 987145,  # 󱀉 U+F1009
    'account-box-multiple-outline': 987146,  # 󱀊 U+F100A
    'airballoon-outline': 987147,  # 󱀋 U+F100B
    'alphabetical-off': 987148,  # 󱀌 U+F100C
    'alphabetical-variant': 987149,  # 󱀍 U+F100D
    'alphabetical-variant-off': 987150,  # 󱀎 U+F100E
    'apache-kafka': 987151,  # 󱀏 U+F100F
    'billboard': 987152,  # 󱀐 U+F1010
    'blinds-open': 987153,  # 󱀑 U+F1011
    'bus-stop': 987154,  # 󱀒 U+F1012
    'bus-stop-covered': 987155,  # 󱀓 U+F1013
    'bus-stop-uncovered': 987156,  # 󱀔 U+F1014
    'car-2-plus': 987157,  # 󱀕 U+F1015
    'car-3-plus': 987158,  # 󱀖 U+F1016
    'car-brake-retarder': 987159,  # 󱀗 U+F1017
    'car-clutch': 987160,  # 󱀘 U+F1018
    'car-coolant-level': 987161,  # 󱀙 U+F1019
    'car-turbocharger': 987162,  # 󱀚 U+F101A
    'car-windshield': 987163,  # 󱀛 U+F101B
    'car-windshield-outline': 987164,  # 󱀜 U+F101C
    'cards-diamond-outline': 987165,  # 󱀝 U+F101D
    'cast-audio': 987166,  # 󱀞 U+F101E
    'cellphone-play': 987167,  # 󱀟 U+F101F
    'coach-lamp': 987168,  # 󱀠 U+F1020
    'comment-quote': 987169,  # 󱀡 U+F1021
    'comment-quote-outline': 987170,  # 󱀢 U+F1022
    'domino-mask': 987171,  # 󱀣 U+F1023
    'electron-framework': 987172,  # 󱀤 U+F1024
    'excavator': 987173,  # 󱀥 U+F1025
    'eye-minus': 987174,  # 󱀦 U+F1026
    'eye-minus-outline': 987175,  # 󱀧 U+F1027
    'file-account-outline': 987176,  # 󱀨 U+F1028
    'file-chart-outline': 987177,  # 󱀩 U+F1029
    'file-cloud-outline': 987178,  # 󱀪 U+F102A
    'file-code-outline': 987179,  # 󱀫 U+F102B
    'file-excel-box-outline': 987180,  # 󱀬 U+F102C
    'file-excel-outline': 987181,  # 󱀭 U+F102D
    'file-export-outline': 987182,  # 󱀮 U+F102E
    'file-import-outline': 987183,  # 󱀯 U+F102F
    'file-lock-outline': 987184,  # 󱀰 U+F1030
    'file-move-outline': 987185,  # 󱀱 U+F1031
    'file-multiple-outline': 987186,  # 󱀲 U+F1032
    'file-percent-outline': 987187,  # 󱀳 U+F1033
    'file-powerpoint-box-outline': 987188,  # 󱀴 U+F1034
    'file-powerpoint-outline': 987189,  # 󱀵 U+F1035
    'file-question-outline': 987190,  # 󱀶 U+F1036
    'file-remove-outline': 987191,  # 󱀷 U+F1037
    'file-restore-outline': 987192,  # 󱀸 U+F1038
    'file-send-outline': 987193,  # 󱀹 U+F1039
    'file-star': 987194,  # 󱀺 U+F103A
    'file-star-outline': 987195,  # 󱀻 U+F103B
    'file-undo-outline': 987196,  # 󱀼 U+F103C
    'file-word-box-outline': 987197,  # 󱀽 U+F103D
    'file-word-outline': 987198,  # 󱀾 U+F103E
    'filter-variant-remove': 987199,  # 󱀿 U+F103F
    'floor-lamp-dual': 987200,  # 󱁀 U+F1040
    'floor-lamp-torchiere-variant': 987201,  # 󱁁 U+F1041
    'fruit-cherries': 987202,  # 󱁂 U+F1042
    'fruit-citrus': 987203,  # 󱁃 U+F1043
    'fruit-grapes': 987204,  # 󱁄 U+F1044
    'fruit-grapes-outline': 987205,  # 󱁅 U+F1045
    'fruit-pineapple': 987206,  # 󱁆 U+F1046
    'fruit-watermelon': 987207,  # 󱁇 U+F1047
    'google-my-business': 987208,  # 󱁈 U+F1048
    'graph#2': 987209,  # 󱁉 U+F1049
    'graph-outline': 987210,  # 󱁊 U+F104A
    'harddisk-plus': 987211,  # 󱁋 U+F104B
    'harddisk-remove': 987212,  # 󱁌 U+F104C
    'home-circle-outline': 987213,  # 󱁍 U+F104D
    'instrument-triangle': 987214,  # 󱁎 U+F104E
    'island': 987215,  # 󱁏 U+F104F
    'keyboard-space': 987216,  # 󱁐 U+F1050
    'led-strip-variant': 987217,  # 󱁑 U+F1051
    'numeric-negative-1': 987218,  # 󱁒 U+F1052
    'oil-level': 987219,  # 󱁓 U+F1053
    'outdoor-lamp': 987220,  # 󱁔 U+F1054
    'palm-tree': 987221,  # 󱁕 U+F1055
    'party-popper': 987222,  # 󱁖 U+F1056
    'printer-pos': 987223,  # 󱁗 U+F1057
    'robber': 987224,  # 󱁘 U+F1058
    'routes-clock': 987225,  # 󱁙 U+F1059
    'scale-off': 987226,  # 󱁚 U+F105A
    'cog-transfer': 987227,  # 󱁛 U+F105B
    'cog-transfer-outline': 987228,  # 󱁜 U+F105C
    'shield-sun': 987229,  # 󱁝 U+F105D
    'shield-sun-outline': 987230,  # 󱁞 U+F105E
    'sprinkler': 987231,  # 󱁟 U+F105F
    'sprinkler-variant': 987232,  # 󱁠 U+F1060
    'table-chair': 987233,  # 󱁡 U+F1061
    'terraform': 987234,  # 󱁢 U+F1062
    'toaster': 987235,  # 󱁣 U+F1063
    'tools#3': 987236,  # 󱁤 U+F1064
    'transfer': 987237,  # 󱁥 U+F1065
    'valve': 987238,  # 󱁦 U+F1066
    'valve-closed': 987239,  # 󱁧 U+F1067
    'valve-open': 987240,  # 󱁨 U+F1068
    'video-check': 987241,  # 󱁩 U+F1069
    'video-check-outline': 987242,  # 󱁪 U+F106A
    'water-well': 987243,  # 󱁫 U+F106B
    'water-well-outline': 987244,  # 󱁬 U+F106C
    'bed-single': 987245,  # 󱁭 U+F106D
    'bed-single-outline': 987246,  # 󱁮 U+F106E
    'book-information-variant': 987247,  # 󱁯 U+F106F
    'bottle-soda': 987248,  # 󱁰 U+F1070
    'bottle-soda-classic': 987249,  # 󱁱 U+F1071
    'bottle-soda-outline': 987250,  # 󱁲 U+F1072
    'calendar-blank-multiple': 987251,  # 󱁳 U+F1073
    'card-search': 987252,  # 󱁴 U+F1074
    'card-search-outline': 987253,  # 󱁵 U+F1075
    'face-woman-profile': 987254,  # 󱁶 U+F1076
    'face-woman': 987255,  # 󱁷 U+F1077
    'face-woman-outline': 987256,  # 󱁸 U+F1078
    'file-settings': 987257,  # 󱁹 U+F1079
    'file-settings-outline': 987258,  # 󱁺 U+F107A
    'file-cog': 987259,  # 󱁻 U+F107B
    'file-cog-outline': 987260,  # 󱁼 U+F107C
    'folder-settings': 987261,  # 󱁽 U+F107D
    'folder-settings-outline': 987262,  # 󱁾 U+F107E
    'folder-cog': 987263,  # 󱁿 U+F107F
    'folder-cog-outline': 987264,  # 󱂀 U+F1080
    'furigana-horizontal': 987265,  # 󱂁 U+F1081
    'furigana-vertical': 987266,  # 󱂂 U+F1082
    'golf-tee': 987267,  # 󱂃 U+F1083
    'lungs#1': 987268,  # 󱂄 U+F1084
    'math-log': 987269,  # 󱂅 U+F1085
    'moped': 987270,  # 󱂆 U+F1086
    'router-network': 987271,  # 󱂇 U+F1087
    'alpha-i#1': 987272,  # 󱂈 U+F1088
    'roman-numeral-2': 987273,  # 󱂉 U+F1089
    'roman-numeral-3': 987274,  # 󱂊 U+F108A
    'roman-numeral-4': 987275,  # 󱂋 U+F108B
    'alpha-v#1': 987276,  # 󱂌 U+F108C
    'roman-numeral-6': 987277,  # 󱂍 U+F108D
    'roman-numeral-7': 987278,  # 󱂎 U+F108E
    'roman-numeral-8': 987279,  # 󱂏 U+F108F
    'roman-numeral-9': 987280,  # 󱂐 U+F1090
    'alpha-x#1': 987281,  # 󱂑 U+F1091
    'soldering-iron': 987282,  # 󱂒 U+F1092
    'stomach#1': 987283,  # 󱂓 U+F1093
    'table-eye': 987284,  # 󱂔 U+F1094
    'form-textarea': 987285,  # 󱂕 U+F1095
    'trumpet': 987286,  # 󱂖 U+F1096
    'account-cash': 987287,  # 󱂗 U+F1097
    'account-cash-outline': 987288,  # 󱂘 U+F1098
    'air-humidifier': 987289,  # 󱂙 U+F1099
    'ansible': 987290,  # 󱂚 U+F109A
    'api': 987291,  # 󱂛 U+F109B
    'bicycle#1': 987292,  # 󱂜 U+F109C
    'car-door-lock': 987293,  # 󱂝 U+F109D
    'coat-rack': 987294,  # 󱂞 U+F109E
    'coffee-maker': 987295,  # 󱂟 U+F109F
    'web-minus': 987296,  # 󱂠 U+F10A0
    'decimal': 987297,  # 󱂡 U+F10A1
    'decimal-comma': 987298,  # 󱂢 U+F10A2
    'decimal-comma-decrease': 987299,  # 󱂣 U+F10A3
    'decimal-comma-increase': 987300,  # 󱂤 U+F10A4
    'delete-alert': 987301,  # 󱂥 U+F10A5
    'delete-alert-outline': 987302,  # 󱂦 U+F10A6
    'delete-off': 987303,  # 󱂧 U+F10A7
    'delete-off-outline': 987304,  # 󱂨 U+F10A8
    'dock-bottom': 987305,  # 󱂩 U+F10A9
    'dock-left': 987306,  # 󱂪 U+F10AA
    'dock-right': 987307,  # 󱂫 U+F10AB
    'dock-window': 987308,  # 󱂬 U+F10AC
    'domain-plus': 987309,  # 󱂭 U+F10AD
    'domain-remove': 987310,  # 󱂮 U+F10AE
    'door-closed-lock': 987311,  # 󱂯 U+F10AF
    'download-off': 987312,  # 󱂰 U+F10B0
    'download-off-outline': 987313,  # 󱂱 U+F10B1
    'flag-minus-outline': 987314,  # 󱂲 U+F10B2
    'flag-plus-outline': 987315,  # 󱂳 U+F10B3
    'flag-remove-outline': 987316,  # 󱂴 U+F10B4
    'folder-home': 987317,  # 󱂵 U+F10B5
    'folder-home-outline': 987318,  # 󱂶 U+F10B6
    'folder-information': 987319,  # 󱂷 U+F10B7
    'folder-information-outline': 987320,  # 󱂸 U+F10B8
    'iv-bag': 987321,  # 󱂹 U+F10B9
    'link-lock': 987322,  # 󱂺 U+F10BA
    'message-plus-outline': 987323,  # 󱂻 U+F10BB
    'phone-cancel': 987324,  # 󱂼 U+F10BC
    'smart-card': 987325,  # 󱂽 U+F10BD
    'smart-card-outline': 987326,  # 󱂾 U+F10BE
    'smart-card-reader': 987327,  # 󱂿 U+F10BF
    'smart-card-reader-outline': 987328,  # 󱃀 U+F10C0
    'storefront-outline': 987329,  # 󱃁 U+F10C1
    'thermometer-high#1': 987330,  # 󱃂 U+F10C2
    'thermometer-low#1': 987331,  # 󱃃 U+F10C3
    'ufo': 987332,  # 󱃄 U+F10C4
    'ufo-outline': 987333,  # 󱃅 U+F10C5
    'upload-off': 987334,  # 󱃆 U+F10C6
    'upload-off-outline': 987335,  # 󱃇 U+F10C7
    'account-child-outline': 987336,  # 󱃈 U+F10C8
    'account-settings-outline': 987337,  # 󱃉 U+F10C9
    'account-tie-outline': 987338,  # 󱃊 U+F10CA
    'alien-outline': 987339,  # 󱃋 U+F10CB
    'battery-alert-variant': 987340,  # 󱃌 U+F10CC
    'battery-alert-variant-outline': 987341,  # 󱃍 U+F10CD
    'beehive-outline': 987342,  # 󱃎 U+F10CE
    'boomerang': 987343,  # 󱃏 U+F10CF
    'briefcase-clock': 987344,  # 󱃐 U+F10D0
    'briefcase-clock-outline': 987345,  # 󱃑 U+F10D1
    'cellphone-message-off': 987346,  # 󱃒 U+F10D2
    'circle-off-outline': 987347,  # 󱃓 U+F10D3
    'clipboard-list': 987348,  # 󱃔 U+F10D4
    'clipboard-list-outline': 987349,  # 󱃕 U+F10D5
    'code-braces-box': 987350,  # 󱃖 U+F10D6
    'code-parentheses-box': 987351,  # 󱃗 U+F10D7
    'consolidate': 987352,  # 󱃘 U+F10D8
    'electric-switch-closed': 987353,  # 󱃙 U+F10D9
    'email-receive': 987354,  # 󱃚 U+F10DA
    'email-receive-outline': 987355,  # 󱃛 U+F10DB
    'email-send': 987356,  # 󱃜 U+F10DC
    'email-send-outline': 987357,  # 󱃝 U+F10DD
    'emoticon-confused': 987358,  # 󱃞 U+F10DE
    'emoticon-confused-outline': 987359,  # 󱃟 U+F10DF
    'epsilon': 987360,  # 󱃠 U+F10E0
    'file-table-box': 987361,  # 󱃡 U+F10E1
    'file-table-box-multiple': 987362,  # 󱃢 U+F10E2
    'file-table-box-multiple-outline': 987363,  # 󱃣 U+F10E3
    'file-table-box-outline': 987364,  # 󱃤 U+F10E4
    'filter-menu': 987365,  # 󱃥 U+F10E5
    'filter-menu-outline': 987366,  # 󱃦 U+F10E6
    'flip-horizontal': 987367,  # 󱃧 U+F10E7
    'flip-vertical': 987368,  # 󱃨 U+F10E8
    'folder-download-outline': 987369,  # 󱃩 U+F10E9
    'folder-heart': 987370,  # 󱃪 U+F10EA
    'folder-heart-outline': 987371,  # 󱃫 U+F10EB
    'folder-key-outline': 987372,  # 󱃬 U+F10EC
    'folder-upload-outline': 987373,  # 󱃭 U+F10ED
    'gamma': 987374,  # 󱃮 U+F10EE
    'hair-dryer': 987375,  # 󱃯 U+F10EF
    'hair-dryer-outline': 987376,  # 󱃰 U+F10F0
    'hand-heart': 987377,  # 󱃱 U+F10F1
    'hexagon-multiple-outline': 987378,  # 󱃲 U+F10F2
    'horizontal-rotate-clockwise': 987379,  # 󱃳 U+F10F3
    'horizontal-rotate-counterclockwise': 987380,  # 󱃴 U+F10F4
    'application-array': 987381,  # 󱃵 U+F10F5
    'application-array-outline': 987382,  # 󱃶 U+F10F6
    'application-braces': 987383,  # 󱃷 U+F10F7
    'application-braces-outline': 987384,  # 󱃸 U+F10F8
    'application-parentheses': 987385,  # 󱃹 U+F10F9
    'application-parentheses-outline': 987386,  # 󱃺 U+F10FA
    'application-variable': 987387,  # 󱃻 U+F10FB
    'application-variable-outline': 987388,  # 󱃼 U+F10FC
    'khanda#1': 987389,  # 󱃽 U+F10FD
    'kubernetes': 987390,  # 󱃾 U+F10FE
    'link-variant-minus': 987391,  # 󱃿 U+F10FF
    'link-variant-plus': 987392,  # 󱄀 U+F1100
    'link-variant-remove': 987393,  # 󱄁 U+F1101
    'map-marker-down': 987394,  # 󱄂 U+F1102
    'map-marker-up': 987395,  # 󱄃 U+F1103
    'monitor-shimmer': 987396,  # 󱄄 U+F1104
    'nix': 987397,  # 󱄅 U+F1105
    'nuxt': 987398,  # 󱄆 U+F1106
    'power-socket-de': 987399,  # 󱄇 U+F1107
    'power-socket-fr': 987400,  # 󱄈 U+F1108
    'power-socket-jp': 987401,  # 󱄉 U+F1109
    'progress-close': 987402,  # 󱄊 U+F110A
    'reload-alert': 987403,  # 󱄋 U+F110B
    'restart-alert': 987404,  # 󱄌 U+F110C
    'restore-alert': 987405,  # 󱄍 U+F110D
    'shaker': 987406,  # 󱄎 U+F110E
    'shaker-outline': 987407,  # 󱄏 U+F110F
    'television-shimmer': 987408,  # 󱄐 U+F1110
    'variable-box': 987409,  # 󱄑 U+F1111
    'filter-variant-minus': 987410,  # 󱄒 U+F1112
    'filter-variant-plus': 987411,  # 󱄓 U+F1113
    'slot-machine': 987412,  # 󱄔 U+F1114
    'slot-machine-outline': 987413,  # 󱄕 U+F1115
    'glass-mug-variant': 987414,  # 󱄖 U+F1116
    'clipboard-flow-outline': 987415,  # 󱄗 U+F1117
    'sign-real-estate': 987416,  # 󱄘 U+F1118
    'antenna': 987417,  # 󱄙 U+F1119
    'centos#1': 987418,  # 󱄚 U+F111A
    'redhat#1': 987419,  # 󱄛 U+F111B
    'window-shutter': 987420,  # 󱄜 U+F111C
    'window-shutter-alert': 987421,  # 󱄝 U+F111D
    'window-shutter-open': 987422,  # 󱄞 U+F111E
    'bike-fast': 987423,  # 󱄟 U+F111F
    'volume-source': 987424,  # 󱄠 U+F1120
    'volume-vibrate': 987425,  # 󱄡 U+F1121
    'movie-edit': 987426,  # 󱄢 U+F1122
    'movie-edit-outline': 987427,  # 󱄣 U+F1123
    'movie-filter': 987428,  # 󱄤 U+F1124
    'movie-filter-outline': 987429,  # 󱄥 U+F1125
    'diabetes': 987430,  # 󱄦 U+F1126
    'cursor-default-gesture': 987431,  # 󱄧 U+F1127
    'cursor-default-gesture-outline': 987432,  # 󱄨 U+F1128
    'toothbrush': 987433,  # 󱄩 U+F1129
    'toothbrush-paste': 987434,  # 󱄪 U+F112A
    'home-roof': 987435,  # 󱄫 U+F112B
    'toothbrush-electric': 987436,  # 󱄬 U+F112C
    'account-supervisor-outline': 987437,  # 󱄭 U+F112D
    'bottle-tonic': 987438,  # 󱄮 U+F112E
    'bottle-tonic-outline': 987439,  # 󱄯 U+F112F
    'bottle-tonic-plus': 987440,  # 󱄰 U+F1130
    'bottle-tonic-plus-outline': 987441,  # 󱄱 U+F1131
    'bottle-tonic-skull': 987442,  # 󱄲 U+F1132
    'bottle-tonic-skull-outline': 987443,  # 󱄳 U+F1133
    'calendar-arrow-left': 987444,  # 󱄴 U+F1134
    'calendar-arrow-right': 987445,  # 󱄵 U+F1135
    'crosshairs-question': 987446,  # 󱄶 U+F1136
    'fire-hydrant': 987447,  # 󱄷 U+F1137
    'fire-hydrant-alert': 987448,  # 󱄸 U+F1138
    'fire-hydrant-off': 987449,  # 󱄹 U+F1139
    'ocr': 987450,  # 󱄺 U+F113A
    'shield-star': 987451,  # 󱄻 U+F113B
    'shield-star-outline': 987452,  # 󱄼 U+F113C
    'text-recognition': 987453,  # 󱄽 U+F113D
    'handcuffs': 987454,  # 󱄾 U+F113E
    'gender-male-female-variant': 987455,  # 󱄿 U+F113F
    'gender-non-binary': 987456,  # 󱅀 U+F1140
    'minus-box-multiple': 987457,  # 󱅁 U+F1141
    'minus-box-multiple-outline': 987458,  # 󱅂 U+F1142
    'plus-box-multiple-outline': 987459,  # 󱅃 U+F1143
    'pencil-box-multiple': 987460,  # 󱅄 U+F1144
    'pencil-box-multiple-outline': 987461,  # 󱅅 U+F1145
    'printer-check': 987462,  # 󱅆 U+F1146
    'sort-variant-remove': 987463,  # 󱅇 U+F1147
    'sort-alphabetical-ascending-variant': 987464,  # 󱅈 U+F1148
    'sort-alphabetical-descending-variant': 987465,  # 󱅉 U+F1149
    'dice-1-outline': 987466,  # 󱅊 U+F114A
    'dice-2-outline': 987467,  # 󱅋 U+F114B
    'dice-3-outline': 987468,  # 󱅌 U+F114C
    'dice-4-outline': 987469,  # 󱅍 U+F114D
    'dice-5-outline': 987470,  # 󱅎 U+F114E
    'dice-6-outline': 987471,  # 󱅏 U+F114F
    'dice-d4': 987472,  # 󱅐 U+F1150
    'dice-d6': 987473,  # 󱅑 U+F1151
    'dice-d8': 987474,  # 󱅒 U+F1152
    'dice-d10': 987475,  # 󱅓 U+F1153
    'dice-d12': 987476,  # 󱅔 U+F1154
    'dice-d20': 987477,  # 󱅕 U+F1155
    'dice-multiple-outline': 987478,  # 󱅖 U+F1156
    'paper-roll': 987479,  # 󱅗 U+F1157
    'paper-roll-outline': 987480,  # 󱅘 U+F1158
    'home-edit': 987481,  # 󱅙 U+F1159
    'home-edit-outline': 987482,  # 󱅚 U+F115A
    'arrow-horizontal-lock': 987483,  # 󱅛 U+F115B
    'arrow-vertical-lock': 987484,  # 󱅜 U+F115C
    'weight-lifter': 987485,  # 󱅝 U+F115D
    'account-lock': 987486,  # 󱅞 U+F115E
    'account-lock-outline': 987487,  # 󱅟 U+F115F
    'pasta': 987488,  # 󱅠 U+F1160
    'send-check': 987489,  # 󱅡 U+F1161
    'send-check-outline': 987490,  # 󱅢 U+F1162
    'send-clock': 987491,  # 󱅣 U+F1163
    'send-clock-outline': 987492,  # 󱅤 U+F1164
    'send-outline': 987493,  # 󱅥 U+F1165
    'send-lock-outline': 987494,  # 󱅦 U+F1166
    'police-badge': 987495,  # 󱅧 U+F1167
    'police-badge-outline': 987496,  # 󱅨 U+F1168
    'gate-arrow-right': 987497,  # 󱅩 U+F1169
    'gate-open': 987498,  # 󱅪 U+F116A
    'bell-badge': 987499,  # 󱅫 U+F116B
    'message-image-outline': 987500,  # 󱅬 U+F116C
    'message-lock-outline': 987501,  # 󱅭 U+F116D
    'message-minus': 987502,  # 󱅮 U+F116E
    'message-minus-outline': 987503,  # 󱅯 U+F116F
    'message-processing-outline': 987504,  # 󱅰 U+F1170
    'message-settings-outline': 987505,  # 󱅱 U+F1171
    'message-cog-outline': 987506,  # 󱅲 U+F1172
    'message-text-clock': 987507,  # 󱅳 U+F1173
    'message-text-clock-outline': 987508,  # 󱅴 U+F1174
    'message-text-lock-outline': 987509,  # 󱅵 U+F1175
    'checkbox-blank-badge': 987510,  # 󱅶 U+F1176
    'file-link': 987511,  # 󱅷 U+F1177
    'file-link-outline': 987512,  # 󱅸 U+F1178
    'file-phone': 987513,  # 󱅹 U+F1179
    'file-phone-outline': 987514,  # 󱅺 U+F117A
    'meditation': 987515,  # 󱅻 U+F117B
    'yoga': 987516,  # 󱅼 U+F117C
    'leek': 987517,  # 󱅽 U+F117D
    'noodles': 987518,  # 󱅾 U+F117E
    'pound-box-outline': 987519,  # 󱅿 U+F117F
    'school-outline': 987520,  # 󱆀 U+F1180
    'basket-outline': 987521,  # 󱆁 U+F1181
    'phone-in-talk-outline': 987522,  # 󱆂 U+F1182
    'bash': 987523,  # 󱆃 U+F1183
    'file-key': 987524,  # 󱆄 U+F1184
    'file-key-outline': 987525,  # 󱆅 U+F1185
    'file-certificate': 987526,  # 󱆆 U+F1186
    'file-certificate-outline': 987527,  # 󱆇 U+F1187
    'certificate-outline': 987528,  # 󱆈 U+F1188
    'cigar': 987529,  # 󱆉 U+F1189
    'grill-outline': 987530,  # 󱆊 U+F118A
    'qrcode-plus': 987531,  # 󱆋 U+F118B
    'qrcode-minus': 987532,  # 󱆌 U+F118C
    'qrcode-remove': 987533,  # 󱆍 U+F118D
    'phone-alert-outline': 987534,  # 󱆎 U+F118E
    'phone-bluetooth-outline': 987535,  # 󱆏 U+F118F
    'phone-cancel-outline': 987536,  # 󱆐 U+F1190
    'phone-forward-outline': 987537,  # 󱆑 U+F1191
    'phone-hangup-outline': 987538,  # 󱆒 U+F1192
    'phone-incoming-outline': 987539,  # 󱆓 U+F1193
    'phone-lock-outline': 987540,  # 󱆔 U+F1194
    'phone-log-outline': 987541,  # 󱆕 U+F1195
    'phone-message': 987542,  # 󱆖 U+F1196
    'phone-message-outline': 987543,  # 󱆗 U+F1197
    'phone-minus-outline': 987544,  # 󱆘 U+F1198
    'phone-outgoing-outline': 987545,  # 󱆙 U+F1199
    'phone-paused-outline': 987546,  # 󱆚 U+F119A
    'phone-plus-outline': 987547,  # 󱆛 U+F119B
    'phone-return-outline': 987548,  # 󱆜 U+F119C
    'phone-settings-outline': 987549,  # 󱆝 U+F119D
    'key-star': 987550,  # 󱆞 U+F119E
    'key-link': 987551,  # 󱆟 U+F119F
    'shield-edit': 987552,  # 󱆠 U+F11A0
    'shield-edit-outline': 987553,  # 󱆡 U+F11A1
    'shield-sync': 987554,  # 󱆢 U+F11A2
    'shield-sync-outline': 987555,  # 󱆣 U+F11A3
    'golf-cart': 987556,  # 󱆤 U+F11A4
    'phone-missed-outline': 987557,  # 󱆥 U+F11A5
    'phone-off-outline': 987558,  # 󱆦 U+F11A6
    'format-quote-open-outline': 987559,  # 󱆧 U+F11A7
    'format-quote-close-outline': 987560,  # 󱆨 U+F11A8
    'phone-check': 987561,  # 󱆩 U+F11A9
    'phone-check-outline': 987562,  # 󱆪 U+F11AA
    'phone-ring': 987563,  # 󱆫 U+F11AB
    'phone-ring-outline': 987564,  # 󱆬 U+F11AC
    'share-circle': 987565,  # 󱆭 U+F11AD
    'reply-circle': 987566,  # 󱆮 U+F11AE
    'fridge-off': 987567,  # 󱆯 U+F11AF
    'fridge-off-outline': 987568,  # 󱆰 U+F11B0
    'fridge-alert': 987569,  # 󱆱 U+F11B1
    'fridge-alert-outline': 987570,  # 󱆲 U+F11B2
    'water-boiler-alert': 987571,  # 󱆳 U+F11B3
    'water-boiler-off': 987572,  # 󱆴 U+F11B4
    'amplifier-off': 987573,  # 󱆵 U+F11B5
    'audio-video-off': 987574,  # 󱆶 U+F11B6
    'toaster-off': 987575,  # 󱆷 U+F11B7
    'dishwasher-alert': 987576,  # 󱆸 U+F11B8
    'dishwasher-off': 987577,  # 󱆹 U+F11B9
    'tumble-dryer-alert': 987578,  # 󱆺 U+F11BA
    'tumble-dryer-off': 987579,  # 󱆻 U+F11BB
    'washing-machine-alert': 987580,  # 󱆼 U+F11BC
    'washing-machine-off': 987581,  # 󱆽 U+F11BD
    'car-info': 987582,  # 󱆾 U+F11BE
    'comment-edit': 987583,  # 󱆿 U+F11BF
    'printer-3d-nozzle-alert': 987584,  # 󱇀 U+F11C0
    'printer-3d-nozzle-alert-outline': 987585,  # 󱇁 U+F11C1
    'align-horizontal-left': 987586,  # 󱇂 U+F11C2
    'align-horizontal-center': 987587,  # 󱇃 U+F11C3
    'align-horizontal-right': 987588,  # 󱇄 U+F11C4
    'align-vertical-bottom': 987589,  # 󱇅 U+F11C5
    'align-vertical-center': 987590,  # 󱇆 U+F11C6
    'align-vertical-top': 987591,  # 󱇇 U+F11C7
    'distribute-horizontal-left': 987592,  # 󱇈 U+F11C8
    'distribute-horizontal-center': 987593,  # 󱇉 U+F11C9
    'distribute-horizontal-right': 987594,  # 󱇊 U+F11CA
    'distribute-vertical-bottom': 987595,  # 󱇋 U+F11CB
    'distribute-vertical-center': 987596,  # 󱇌 U+F11CC
    'distribute-vertical-top': 987597,  # 󱇍 U+F11CD
    'alert-rhombus': 987598,  # 󱇎 U+F11CE
    'alert-rhombus-outline': 987599,  # 󱇏 U+F11CF
    'crown-outline': 987600,  # 󱇐 U+F11D0
    'image-off-outline': 987601,  # 󱇑 U+F11D1
    'movie-search': 987602,  # 󱇒 U+F11D2
    'movie-search-outline': 987603,  # 󱇓 U+F11D3
    'rv-truck': 987604,  # 󱇔 U+F11D4
    'shopping-outline': 987605,  # 󱇕 U+F11D5
    'strategy': 987606,  # 󱇖 U+F11D6
    'note-text-outline': 987607,  # 󱇗 U+F11D7
    'view-agenda-outline': 987608,  # 󱇘 U+F11D8
    'view-grid-outline': 987609,  # 󱇙 U+F11D9
    'view-grid-plus-outline': 987610,  # 󱇚 U+F11DA
    'window-closed-variant': 987611,  # 󱇛 U+F11DB
    'window-open-variant': 987612,  # 󱇜 U+F11DC
    'cog-clockwise': 987613,  # 󱇝 U+F11DD
    'cog-counterclockwise': 987614,  # 󱇞 U+F11DE
    'chart-sankey': 987615,  # 󱇟 U+F11DF
    'chart-sankey-variant': 987616,  # 󱇠 U+F11E0
    'vanity-light': 987617,  # 󱇡 U+F11E1
    'router': 987618,  # 󱇢 U+F11E2
    'image-edit': 987619,  # 󱇣 U+F11E3
    'image-edit-outline': 987620,  # 󱇤 U+F11E4
    'bell-check': 987621,  # 󱇥 U+F11E5
    'bell-check-outline': 987622,  # 󱇦 U+F11E6
    'file-edit': 987623,  # 󱇧 U+F11E7
    'file-edit-outline': 987624,  # 󱇨 U+F11E8
    'human-scooter': 987625,  # 󱇩 U+F11E9
    'spider#1': 987626,  # 󱇪 U+F11EA
    'spider-thread': 987627,  # 󱇫 U+F11EB
    'plus-thick': 987628,  # 󱇬 U+F11EC
    'alert-circle-check': 987629,  # 󱇭 U+F11ED
    'alert-circle-check-outline': 987630,  # 󱇮 U+F11EE
    'state-machine': 987631,  # 󱇯 U+F11EF
    'usb-port': 987632,  # 󱇰 U+F11F0
    'cloud-lock': 987633,  # 󱇱 U+F11F1
    'cloud-lock-outline': 987634,  # 󱇲 U+F11F2
    'robot-mower-outline': 987635,  # 󱇳 U+F11F3
    'share-all': 987636,  # 󱇴 U+F11F4
    'share-all-outline': 987637,  # 󱇵 U+F11F5
    'google-cloud': 987638,  # 󱇶 U+F11F6
    'robot-mower': 987639,  # 󱇷 U+F11F7
    'fast-forward-5': 987640,  # 󱇸 U+F11F8
    'rewind-5': 987641,  # 󱇹 U+F11F9
    'shape-oval-plus': 987642,  # 󱇺 U+F11FA
    'timeline-clock': 987643,  # 󱇻 U+F11FB
    'timeline-clock-outline': 987644,  # 󱇼 U+F11FC
    'mirror#2': 987645,  # 󱇽 U+F11FD
    'account-multiple-check-outline': 987646,  # 󱇾 U+F11FE
    'card-plus': 987647,  # 󱇿 U+F11FF
    'card-plus-outline': 987648,  # 󱈀 U+F1200
    'checkerboard-plus': 987649,  # 󱈁 U+F1201
    'checkerboard-minus': 987650,  # 󱈂 U+F1202
    'checkerboard-remove': 987651,  # 󱈃 U+F1203
    'select-search': 987652,  # 󱈄 U+F1204
    'selection-search': 987653,  # 󱈅 U+F1205
    'layers-search': 987654,  # 󱈆 U+F1206
    'layers-search-outline': 987655,  # 󱈇 U+F1207
    'lightbulb-cfl': 987656,  # 󱈈 U+F1208
    'lightbulb-cfl-off': 987657,  # 󱈉 U+F1209
    'account-multiple-remove': 987658,  # 󱈊 U+F120A
    'account-multiple-remove-outline': 987659,  # 󱈋 U+F120B
    'magnify-remove-cursor': 987660,  # 󱈌 U+F120C
    'magnify-remove-outline': 987661,  # 󱈍 U+F120D
    'archive-outline': 987662,  # 󱈎 U+F120E
    'battery-heart': 987663,  # 󱈏 U+F120F
    'battery-heart-outline': 987664,  # 󱈐 U+F1210
    'battery-heart-variant': 987665,  # 󱈑 U+F1211
    'bus-marker': 987666,  # 󱈒 U+F1212
    'chart-multiple': 987667,  # 󱈓 U+F1213
    'emoticon-lol': 987668,  # 󱈔 U+F1214
    'emoticon-lol-outline': 987669,  # 󱈕 U+F1215
    'file-sync': 987670,  # 󱈖 U+F1216
    'file-sync-outline': 987671,  # 󱈗 U+F1217
    'handshake#1': 987672,  # 󱈘 U+F1218
    'language-kotlin': 987673,  # 󱈙 U+F1219
    'language-fortran': 987674,  # 󱈚 U+F121A
    'offer': 987675,  # 󱈛 U+F121B
    'radio-off': 987676,  # 󱈜 U+F121C
    'table-headers-eye': 987677,  # 󱈝 U+F121D
    'table-headers-eye-off': 987678,  # 󱈞 U+F121E
    'tag-minus-outline': 987679,  # 󱈟 U+F121F
    'tag-off': 987680,  # 󱈠 U+F1220
    'tag-off-outline': 987681,  # 󱈡 U+F1221
    'tag-plus-outline': 987682,  # 󱈢 U+F1222
    'tag-remove-outline': 987683,  # 󱈣 U+F1223
    'tag-text': 987684,  # 󱈤 U+F1224
    'vector-polyline-edit': 987685,  # 󱈥 U+F1225
    'vector-polyline-minus': 987686,  # 󱈦 U+F1226
    'vector-polyline-plus': 987687,  # 󱈧 U+F1227
    'vector-polyline-remove': 987688,  # 󱈨 U+F1228
    'beaker-alert': 987689,  # 󱈩 U+F1229
    'beaker-alert-outline': 987690,  # 󱈪 U+F122A
    'beaker-check': 987691,  # 󱈫 U+F122B
    'beaker-check-outline': 987692,  # 󱈬 U+F122C
    'beaker-minus': 987693,  # 󱈭 U+F122D
    'beaker-minus-outline': 987694,  # 󱈮 U+F122E
    'beaker-plus': 987695,  # 󱈯 U+F122F
    'beaker-plus-outline': 987696,  # 󱈰 U+F1230
    'beaker-question': 987697,  # 󱈱 U+F1231
    'beaker-question-outline': 987698,  # 󱈲 U+F1232
    'beaker-remove': 987699,  # 󱈳 U+F1233
    'beaker-remove-outline': 987700,  # 󱈴 U+F1234
    'bicycle-basket': 987701,  # 󱈵 U+F1235
    'barcode-off': 987702,  # 󱈶 U+F1236
    'digital-ocean': 987703,  # 󱈷 U+F1237
    'exclamation-thick': 987704,  # 󱈸 U+F1238
    'desk': 987705,  # 󱈹 U+F1239
    'flask-empty-minus': 987706,  # 󱈺 U+F123A
    'flask-empty-minus-outline': 987707,  # 󱈻 U+F123B
    'flask-empty-plus': 987708,  # 󱈼 U+F123C
    'flask-empty-plus-outline': 987709,  # 󱈽 U+F123D
    'flask-empty-remove': 987710,  # 󱈾 U+F123E
    'flask-empty-remove-outline': 987711,  # 󱈿 U+F123F
    'flask-minus': 987712,  # 󱉀 U+F1240
    'flask-minus-outline': 987713,  # 󱉁 U+F1241
    'flask-plus': 987714,  # 󱉂 U+F1242
    'flask-plus-outline': 987715,  # 󱉃 U+F1243
    'flask-remove': 987716,  # 󱉄 U+F1244
    'flask-remove-outline': 987717,  # 󱉅 U+F1245
    'folder-move-outline': 987718,  # 󱉆 U+F1246
    'home-remove': 987719,  # 󱉇 U+F1247
    'webrtc': 987720,  # 󱉈 U+F1248
    'seat-passenger': 987721,  # 󱉉 U+F1249
    'web-clock': 987722,  # 󱉊 U+F124A
    'flask-round-bottom': 987723,  # 󱉋 U+F124B
    'flask-round-bottom-empty': 987724,  # 󱉌 U+F124C
    'flask-round-bottom-empty-outline': 987725,  # 󱉍 U+F124D
    'flask-round-bottom-outline': 987726,  # 󱉎 U+F124E
    'gold': 987727,  # 󱉏 U+F124F
    'message-star-outline': 987728,  # 󱉐 U+F1250
    'home-lightbulb': 987729,  # 󱉑 U+F1251
    'home-lightbulb-outline': 987730,  # 󱉒 U+F1252
    'lightbulb-group': 987731,  # 󱉓 U+F1253
    'lightbulb-group-outline': 987732,  # 󱉔 U+F1254
    'lightbulb-multiple': 987733,  # 󱉕 U+F1255
    'lightbulb-multiple-outline': 987734,  # 󱉖 U+F1256
    'api-off': 987735,  # 󱉗 U+F1257
    'allergy': 987736,  # 󱉘 U+F1258
    'archive-arrow-down': 987737,  # 󱉙 U+F1259
    'archive-arrow-down-outline': 987738,  # 󱉚 U+F125A
    'archive-arrow-up': 987739,  # 󱉛 U+F125B
    'archive-arrow-up-outline': 987740,  # 󱉜 U+F125C
    'battery-off': 987741,  # 󱉝 U+F125D
    'battery-off-outline': 987742,  # 󱉞 U+F125E
    'bookshelf': 987743,  # 󱉟 U+F125F
    'cash-minus': 987744,  # 󱉠 U+F1260
    'cash-plus': 987745,  # 󱉡 U+F1261
    'cash-remove': 987746,  # 󱉢 U+F1262
    'clipboard-check-multiple': 987747,  # 󱉣 U+F1263
    'clipboard-check-multiple-outline': 987748,  # 󱉤 U+F1264
    'clipboard-file': 987749,  # 󱉥 U+F1265
    'clipboard-file-outline': 987750,  # 󱉦 U+F1266
    'clipboard-multiple': 987751,  # 󱉧 U+F1267
    'clipboard-multiple-outline': 987752,  # 󱉨 U+F1268
    'clipboard-play-multiple': 987753,  # 󱉩 U+F1269
    'clipboard-play-multiple-outline': 987754,  # 󱉪 U+F126A
    'clipboard-text-multiple': 987755,  # 󱉫 U+F126B
    'clipboard-text-multiple-outline': 987756,  # 󱉬 U+F126C
    'folder-marker': 987757,  # 󱉭 U+F126D
    'folder-marker-outline': 987758,  # 󱉮 U+F126E
    'format-list-text': 987759,  # 󱉯 U+F126F
    'inbox-arrow-down-outline': 987760,  # 󱉰 U+F1270
    'inbox-arrow-up-outline': 987761,  # 󱉱 U+F1271
    'inbox-full': 987762,  # 󱉲 U+F1272
    'inbox-full-outline': 987763,  # 󱉳 U+F1273
    'inbox-outline': 987764,  # 󱉴 U+F1274
    'lightbulb-cfl-spiral': 987765,  # 󱉵 U+F1275
    'magnify-scan': 987766,  # 󱉶 U+F1276
    'map-marker-multiple-outline': 987767,  # 󱉷 U+F1277
    'percent-outline': 987768,  # 󱉸 U+F1278
    'phone-classic-off': 987769,  # 󱉹 U+F1279
    'play-box': 987770,  # 󱉺 U+F127A
    'account-eye-outline': 987771,  # 󱉻 U+F127B
    'safe-square': 987772,  # 󱉼 U+F127C
    'safe-square-outline': 987773,  # 󱉽 U+F127D
    'scoreboard': 987774,  # 󱉾 U+F127E
    'scoreboard-outline': 987775,  # 󱉿 U+F127F
    'select-marker': 987776,  # 󱊀 U+F1280
    'select-multiple': 987777,  # 󱊁 U+F1281
    'select-multiple-marker': 987778,  # 󱊂 U+F1282
    'selection-marker': 987779,  # 󱊃 U+F1283
    'selection-multiple-marker': 987780,  # 󱊄 U+F1284
    'selection-multiple': 987781,  # 󱊅 U+F1285
    'star-box-multiple': 987782,  # 󱊆 U+F1286
    'star-box-multiple-outline': 987783,  # 󱊇 U+F1287
    'toy-brick': 987784,  # 󱊈 U+F1288
    'toy-brick-marker': 987785,  # 󱊉 U+F1289
    'toy-brick-marker-outline': 987786,  # 󱊊 U+F128A
    'toy-brick-minus': 987787,  # 󱊋 U+F128B
    'toy-brick-minus-outline': 987788,  # 󱊌 U+F128C
    'toy-brick-outline': 987789,  # 󱊍 U+F128D
    'toy-brick-plus': 987790,  # 󱊎 U+F128E
    'toy-brick-plus-outline': 987791,  # 󱊏 U+F128F
    'toy-brick-remove': 987792,  # 󱊐 U+F1290
    'toy-brick-remove-outline': 987793,  # 󱊑 U+F1291
    'toy-brick-search': 987794,  # 󱊒 U+F1292
    'toy-brick-search-outline': 987795,  # 󱊓 U+F1293
    'tray': 987796,  # 󱊔 U+F1294
    'tray-alert': 987797,  # 󱊕 U+F1295
    'tray-full': 987798,  # 󱊖 U+F1296
    'tray-minus': 987799,  # 󱊗 U+F1297
    'tray-plus': 987800,  # 󱊘 U+F1298
    'tray-remove': 987801,  # 󱊙 U+F1299
    'truck-check-outline': 987802,  # 󱊚 U+F129A
    'truck-delivery-outline': 987803,  # 󱊛 U+F129B
    'truck-fast-outline': 987804,  # 󱊜 U+F129C
    'truck-outline': 987805,  # 󱊝 U+F129D
    'usb-flash-drive': 987806,  # 󱊞 U+F129E
    'usb-flash-drive-outline': 987807,  # 󱊟 U+F129F
    'water-polo': 987808,  # 󱊠 U+F12A0
    'battery-low': 987809,  # 󱊡 U+F12A1
    'battery-medium': 987810,  # 󱊢 U+F12A2
    'battery-high': 987811,  # 󱊣 U+F12A3
    'battery-charging-low': 987812,  # 󱊤 U+F12A4
    'battery-charging-medium': 987813,  # 󱊥 U+F12A5
    'battery-charging-high': 987814,  # 󱊦 U+F12A6
    'hexadecimal': 987815,  # 󱊧 U+F12A7
    'gesture-tap-button': 987816,  # 󱊨 U+F12A8
    'gesture-tap-box': 987817,  # 󱊩 U+F12A9
    'lan-check': 987818,  # 󱊪 U+F12AA
    'keyboard-f1': 987819,  # 󱊫 U+F12AB
    'keyboard-f2': 987820,  # 󱊬 U+F12AC
    'keyboard-f3': 987821,  # 󱊭 U+F12AD
    'keyboard-f4': 987822,  # 󱊮 U+F12AE
    'keyboard-f5': 987823,  # 󱊯 U+F12AF
    'keyboard-f6': 987824,  # 󱊰 U+F12B0
    'keyboard-f7': 987825,  # 󱊱 U+F12B1
    'keyboard-f8': 987826,  # 󱊲 U+F12B2
    'keyboard-f9': 987827,  # 󱊳 U+F12B3
    'keyboard-f10': 987828,  # 󱊴 U+F12B4
    'keyboard-f11': 987829,  # 󱊵 U+F12B5
    'keyboard-f12': 987830,  # 󱊶 U+F12B6
    'keyboard-esc': 987831,  # 󱊷 U+F12B7
    'toslink': 987832,  # 󱊸 U+F12B8
    'cheese#1': 987833,  # 󱊹 U+F12B9
    'string-lights': 987834,  # 󱊺 U+F12BA
    'string-lights-off': 987835,  # 󱊻 U+F12BB
    'whistle-outline': 987836,  # 󱊼 U+F12BC
    'stairs-up': 987837,  # 󱊽 U+F12BD
    'stairs-down': 987838,  # 󱊾 U+F12BE
    'escalator-up': 987839,  # 󱊿 U+F12BF
    'escalator-down': 987840,  # 󱋀 U+F12C0
    'elevator-up': 987841,  # 󱋁 U+F12C1
    'elevator-down': 987842,  # 󱋂 U+F12C2
    'lightbulb-cfl-spiral-off': 987843,  # 󱋃 U+F12C3
    'comment-edit-outline': 987844,  # 󱋄 U+F12C4
    'tooltip-edit-outline': 987845,  # 󱋅 U+F12C5
    'monitor-edit': 987846,  # 󱋆 U+F12C6
    'email-sync': 987847,  # 󱋇 U+F12C7
    'email-sync-outline': 987848,  # 󱋈 U+F12C8
    'chat-alert-outline': 987849,  # 󱋉 U+F12C9
    'chat-processing-outline': 987850,  # 󱋊 U+F12CA
    'snowflake-melt': 987851,  # 󱋋 U+F12CB
    'cloud-check-outline': 987852,  # 󱋌 U+F12CC
    'lightbulb-group-off': 987853,  # 󱋍 U+F12CD
    'lightbulb-group-off-outline': 987854,  # 󱋎 U+F12CE
    'lightbulb-multiple-off': 987855,  # 󱋏 U+F12CF
    'lightbulb-multiple-off-outline': 987856,  # 󱋐 U+F12D0
    'chat-sleep': 987857,  # 󱋑 U+F12D1
    'chat-sleep-outline': 987858,  # 󱋒 U+F12D2
    'garage-variant': 987859,  # 󱋓 U+F12D3
    'garage-open-variant': 987860,  # 󱋔 U+F12D4
    'garage-alert-variant': 987861,  # 󱋕 U+F12D5
    'cloud-sync-outline': 987862,  # 󱋖 U+F12D6
    'globe-light': 987863,  # 󱋗 U+F12D7
    'cellphone-nfc-off': 987864,  # 󱋘 U+F12D8
    'leaf-off': 987865,  # 󱋙 U+F12D9
    'leaf-maple-off': 987866,  # 󱋚 U+F12DA
    'map-marker-left': 987867,  # 󱋛 U+F12DB
    'map-marker-right': 987868,  # 󱋜 U+F12DC
    'map-marker-left-outline': 987869,  # 󱋝 U+F12DD
    'map-marker-right-outline': 987870,  # 󱋞 U+F12DE
    'account-cancel': 987871,  # 󱋟 U+F12DF
    'account-cancel-outline': 987872,  # 󱋠 U+F12E0
    'file-clock': 987873,  # 󱋡 U+F12E1
    'file-clock-outline': 987874,  # 󱋢 U+F12E2
    'folder-table': 987875,  # 󱋣 U+F12E3
    'folder-table-outline': 987876,  # 󱋤 U+F12E4
    'hydro-power': 987877,  # 󱋥 U+F12E5
    'doorbell': 987878,  # 󱋦 U+F12E6
    'bulma': 987879,  # 󱋧 U+F12E7
    'iobroker': 987880,  # 󱋨 U+F12E8
    'oci': 987881,  # 󱋩 U+F12E9
    'label-percent': 987882,  # 󱋪 U+F12EA
    'label-percent-outline': 987883,  # 󱋫 U+F12EB
    'checkbox-blank-off': 987884,  # 󱋬 U+F12EC
    'checkbox-blank-off-outline': 987885,  # 󱋭 U+F12ED
    'square-off': 987886,  # 󱋮 U+F12EE
    'square-off-outline': 987887,  # 󱋯 U+F12EF
    'drag-horizontal-variant': 987888,  # 󱋰 U+F12F0
    'drag-vertical-variant': 987889,  # 󱋱 U+F12F1
    'message-arrow-left': 987890,  # 󱋲 U+F12F2
    'message-arrow-left-outline': 987891,  # 󱋳 U+F12F3
    'message-arrow-right': 987892,  # 󱋴 U+F12F4
    'message-arrow-right-outline': 987893,  # 󱋵 U+F12F5
    'database-marker': 987894,  # 󱋶 U+F12F6
    'tag-multiple-outline': 987895,  # 󱋷 U+F12F7
    'map-marker-plus-outline': 987896,  # 󱋸 U+F12F8
    'map-marker-minus-outline': 987897,  # 󱋹 U+F12F9
    'map-marker-remove-outline': 987898,  # 󱋺 U+F12FA
    'map-marker-check-outline': 987899,  # 󱋻 U+F12FB
    'map-marker-radius-outline': 987900,  # 󱋼 U+F12FC
    'map-marker-off-outline': 987901,  # 󱋽 U+F12FD
    'molecule-co': 987902,  # 󱋾 U+F12FE
    'jump-rope': 987903,  # 󱋿 U+F12FF
    'kettlebell': 987904,  # 󱌀 U+F1300
    'account-convert-outline': 987905,  # 󱌁 U+F1301
    'bunk-bed': 987906,  # 󱌂 U+F1302
    'fleur-de-lis': 987907,  # 󱌃 U+F1303
    'ski': 987908,  # 󱌄 U+F1304
    'ski-cross-country': 987909,  # 󱌅 U+F1305
    'ski-water': 987910,  # 󱌆 U+F1306
    'snowboard': 987911,  # 󱌇 U+F1307
    'account-tie-voice': 987912,  # 󱌈 U+F1308
    'account-tie-voice-outline': 987913,  # 󱌉 U+F1309
    'account-tie-voice-off': 987914,  # 󱌊 U+F130A
    'account-tie-voice-off-outline': 987915,  # 󱌋 U+F130B
    'beer-outline': 987916,  # 󱌌 U+F130C
    'glass-pint-outline': 987917,  # 󱌍 U+F130D
    'coffee-to-go-outline': 987918,  # 󱌎 U+F130E
    'cup-outline': 987919,  # 󱌏 U+F130F
    'bottle-wine-outline': 987920,  # 󱌐 U+F1310
    'earth-arrow-right': 987921,  # 󱌑 U+F1311
    'key-arrow-right': 987922,  # 󱌒 U+F1312
    'format-color-marker-cancel': 987923,  # 󱌓 U+F1313
    'mother-heart': 987924,  # 󱌔 U+F1314
    'currency-eur-off': 987925,  # 󱌕 U+F1315
    'semantic-web': 987926,  # 󱌖 U+F1316
    'kettle-alert': 987927,  # 󱌗 U+F1317
    'kettle-alert-outline': 987928,  # 󱌘 U+F1318
    'kettle-steam': 987929,  # 󱌙 U+F1319
    'kettle-steam-outline': 987930,  # 󱌚 U+F131A
    'kettle-off': 987931,  # 󱌛 U+F131B
    'kettle-off-outline': 987932,  # 󱌜 U+F131C
    'simple-icons': 987933,  # 󱌝 U+F131D
    'briefcase-check-outline': 987934,  # 󱌞 U+F131E
    'clipboard-plus-outline': 987935,  # 󱌟 U+F131F
    'download-lock': 987936,  # 󱌠 U+F1320
    'download-lock-outline': 987937,  # 󱌡 U+F1321
    'hammer-screwdriver': 987938,  # 󱌢 U+F1322
    'hammer-wrench': 987939,  # 󱌣 U+F1323
    'hydraulic-oil-level': 987940,  # 󱌤 U+F1324
    'hydraulic-oil-temperature': 987941,  # 󱌥 U+F1325
    'medal-outline': 987942,  # 󱌦 U+F1326
    'rodent': 987943,  # 󱌧 U+F1327
    'abjad-arabic': 987944,  # 󱌨 U+F1328
    'abjad-hebrew': 987945,  # 󱌩 U+F1329
    'abugida-devanagari': 987946,  # 󱌪 U+F132A
    'abugida-thai': 987947,  # 󱌫 U+F132B
    'alphabet-aurebesh': 987948,  # 󱌬 U+F132C
    'alphabet-cyrillic': 987949,  # 󱌭 U+F132D
    'alphabet-greek': 987950,  # 󱌮 U+F132E
    'alphabet-latin': 987951,  # 󱌯 U+F132F
    'alphabet-piqad': 987952,  # 󱌰 U+F1330
    'ideogram-cjk': 987953,  # 󱌱 U+F1331
    'ideogram-cjk-variant': 987954,  # 󱌲 U+F1332
    'syllabary-hangul': 987955,  # 󱌳 U+F1333
    'syllabary-hiragana': 987956,  # 󱌴 U+F1334
    'syllabary-katakana': 987957,  # 󱌵 U+F1335
    'syllabary-katakana-halfwidth': 987958,  # 󱌶 U+F1336
    'alphabet-tengwar': 987959,  # 󱌷 U+F1337
    'head-alert': 987960,  # 󱌸 U+F1338
    'head-alert-outline': 987961,  # 󱌹 U+F1339
    'head-check': 987962,  # 󱌺 U+F133A
    'head-check-outline': 987963,  # 󱌻 U+F133B
    'head-cog': 987964,  # 󱌼 U+F133C
    'head-cog-outline': 987965,  # 󱌽 U+F133D
    'head-dots-horizontal': 987966,  # 󱌾 U+F133E
    'head-dots-horizontal-outline': 987967,  # 󱌿 U+F133F
    'head-flash': 987968,  # 󱍀 U+F1340
    'head-flash-outline': 987969,  # 󱍁 U+F1341
    'head-heart': 987970,  # 󱍂 U+F1342
    'head-heart-outline': 987971,  # 󱍃 U+F1343
    'head-lightbulb': 987972,  # 󱍄 U+F1344
    'head-lightbulb-outline': 987973,  # 󱍅 U+F1345
    'head-minus': 987974,  # 󱍆 U+F1346
    'head-minus-outline': 987975,  # 󱍇 U+F1347
    'head-plus': 987976,  # 󱍈 U+F1348
    'head-plus-outline': 987977,  # 󱍉 U+F1349
    'head-question': 987978,  # 󱍊 U+F134A
    'head-question-outline': 987979,  # 󱍋 U+F134B
    'head-remove': 987980,  # 󱍌 U+F134C
    'head-remove-outline': 987981,  # 󱍍 U+F134D
    'head-snowflake': 987982,  # 󱍎 U+F134E
    'head-snowflake-outline': 987983,  # 󱍏 U+F134F
    'head-sync': 987984,  # 󱍐 U+F1350
    'head-sync-outline': 987985,  # 󱍑 U+F1351
    'hvac': 987986,  # 󱍒 U+F1352
    'pencil-ruler': 987987,  # 󱍓 U+F1353
    'pipe-wrench': 987988,  # 󱍔 U+F1354
    'widgets-outline': 987989,  # 󱍕 U+F1355
    'television-ambient-light': 987990,  # 󱍖 U+F1356
    'propane-tank': 987991,  # 󱍗 U+F1357
    'propane-tank-outline': 987992,  # 󱍘 U+F1358
    'folder-music': 987993,  # 󱍙 U+F1359
    'folder-music-outline': 987994,  # 󱍚 U+F135A
    'klingon': 987995,  # 󱍛 U+F135B
    'palette-swatch-outline': 987996,  # 󱍜 U+F135C
    'form-textbox-lock': 987997,  # 󱍝 U+F135D
    'head': 987998,  # 󱍞 U+F135E
    'head-outline': 987999,  # 󱍟 U+F135F
    'shield-half': 988000,  # 󱍠 U+F1360
    'store-outline': 988001,  # 󱍡 U+F1361
    'google-downasaur': 988002,  # 󱍢 U+F1362
    'bottle-soda-classic-outline': 988003,  # 󱍣 U+F1363
    'sticker': 988004,  # 󱍤 U+F1364
    'sticker-alert': 988005,  # 󱍥 U+F1365
    'sticker-alert-outline': 988006,  # 󱍦 U+F1366
    'sticker-check': 988007,  # 󱍧 U+F1367
    'sticker-check-outline': 988008,  # 󱍨 U+F1368
    'sticker-minus': 988009,  # 󱍩 U+F1369
    'sticker-minus-outline': 988010,  # 󱍪 U+F136A
    'sticker-outline': 988011,  # 󱍫 U+F136B
    'sticker-plus': 988012,  # 󱍬 U+F136C
    'sticker-plus-outline': 988013,  # 󱍭 U+F136D
    'sticker-remove': 988014,  # 󱍮 U+F136E
    'sticker-remove-outline': 988015,  # 󱍯 U+F136F
    'account-cog': 988016,  # 󱍰 U+F1370
    'account-cog-outline': 988017,  # 󱍱 U+F1371
    'account-details-outline': 988018,  # 󱍲 U+F1372
    'upload-lock': 988019,  # 󱍳 U+F1373
    'upload-lock-outline': 988020,  # 󱍴 U+F1374
    'label-multiple': 988021,  # 󱍵 U+F1375
    'label-multiple-outline': 988022,  # 󱍶 U+F1376
    'refresh-circle': 988023,  # 󱍷 U+F1377
    'sync-circle': 988024,  # 󱍸 U+F1378
    'bookmark-music-outline': 988025,  # 󱍹 U+F1379
    'bookmark-remove-outline': 988026,  # 󱍺 U+F137A
    'bookmark-check-outline': 988027,  # 󱍻 U+F137B
    'traffic-cone': 988028,  # 󱍼 U+F137C
    'cup-off-outline': 988029,  # 󱍽 U+F137D
    'auto-download': 988030,  # 󱍾 U+F137E
    'shuriken': 988031,  # 󱍿 U+F137F
    'chart-ppf': 988032,  # 󱎀 U+F1380
    'elevator-passenger': 988033,  # 󱎁 U+F1381
    'compass-rose': 988034,  # 󱎂 U+F1382
    'space-station': 988035,  # 󱎃 U+F1383
    'order-bool-descending': 988036,  # 󱎄 U+F1384
    'sort-bool-ascending': 988037,  # 󱎅 U+F1385
    'sort-bool-ascending-variant': 988038,  # 󱎆 U+F1386
    'sort-bool-descending': 988039,  # 󱎇 U+F1387
    'sort-bool-descending-variant': 988040,  # 󱎈 U+F1388
    'sort-numeric-ascending': 988041,  # 󱎉 U+F1389
    'sort-numeric-descending': 988042,  # 󱎊 U+F138A
    'human-baby-changing-table': 988043,  # 󱎋 U+F138B
    'human-male-child': 988044,  # 󱎌 U+F138C
    'human-wheelchair': 988045,  # 󱎍 U+F138D
    'microsoft-access': 988046,  # 󱎎 U+F138E
    'microsoft-excel': 988047,  # 󱎏 U+F138F
    'microsoft-powerpoint': 988048,  # 󱎐 U+F1390
    'microsoft-sharepoint': 988049,  # 󱎑 U+F1391
    'microsoft-word': 988050,  # 󱎒 U+F1392
    'nintendo-game-boy': 988051,  # 󱎓 U+F1393
    'cable-data': 988052,  # 󱎔 U+F1394
    'circle-half': 988053,  # 󱎕 U+F1395
    'circle-half-full': 988054,  # 󱎖 U+F1396
    'cellphone-charging': 988055,  # 󱎗 U+F1397
    'close-thick': 988056,  # 󱎘 U+F1398
    'escalator-box': 988057,  # 󱎙 U+F1399
    'lock-check': 988058,  # 󱎚 U+F139A
    'lock-open-alert': 988059,  # 󱎛 U+F139B
    'lock-open-check': 988060,  # 󱎜 U+F139C
    'recycle-variant': 988061,  # 󱎝 U+F139D
    'stairs-box': 988062,  # 󱎞 U+F139E
    'hand-water': 988063,  # 󱎟 U+F139F
    'table-refresh': 988064,  # 󱎠 U+F13A0
    'table-sync': 988065,  # 󱎡 U+F13A1
    'size-xxs': 988066,  # 󱎢 U+F13A2
    'size-xs': 988067,  # 󱎣 U+F13A3
    'size-s': 988068,  # 󱎤 U+F13A4
    'size-m': 988069,  # 󱎥 U+F13A5
    'alpha-l#1': 988070,  # 󱎦 U+F13A6
    'size-xl': 988071,  # 󱎧 U+F13A7
    'size-xxl': 988072,  # 󱎨 U+F13A8
    'size-xxxl': 988073,  # 󱎩 U+F13A9
    'ticket-confirmation-outline': 988074,  # 󱎪 U+F13AA
    'timer': 988075,  # 󱎫 U+F13AB
    'timer-off': 988076,  # 󱎬 U+F13AC
    'book-account': 988077,  # 󱎭 U+F13AD
    'book-account-outline': 988078,  # 󱎮 U+F13AE
    'rocket-outline': 988079,  # 󱎯 U+F13AF
    'home-search': 988080,  # 󱎰 U+F13B0
    'home-search-outline': 988081,  # 󱎱 U+F13B1
    'car-arrow-left': 988082,  # 󱎲 U+F13B2
    'car-arrow-right': 988083,  # 󱎳 U+F13B3
    'monitor-eye': 988084,  # 󱎴 U+F13B4
    'lipstick#1': 988085,  # 󱎵 U+F13B5
    'virus#1': 988086,  # 󱎶 U+F13B6
    'virus-outline': 988087,  # 󱎷 U+F13B7
    'text-search': 988088,  # 󱎸 U+F13B8
    'table-account': 988089,  # 󱎹 U+F13B9
    'table-alert': 988090,  # 󱎺 U+F13BA
    'table-arrow-down': 988091,  # 󱎻 U+F13BB
    'table-arrow-left': 988092,  # 󱎼 U+F13BC
    'table-arrow-right': 988093,  # 󱎽 U+F13BD
    'table-arrow-up': 988094,  # 󱎾 U+F13BE
    'table-cancel': 988095,  # 󱎿 U+F13BF
    'table-check': 988096,  # 󱏀 U+F13C0
    'table-clock': 988097,  # 󱏁 U+F13C1
    'table-cog': 988098,  # 󱏂 U+F13C2
    'table-eye-off': 988099,  # 󱏃 U+F13C3
    'table-heart': 988100,  # 󱏄 U+F13C4
    'table-key': 988101,  # 󱏅 U+F13C5
    'table-lock': 988102,  # 󱏆 U+F13C6
    'table-minus': 988103,  # 󱏇 U+F13C7
    'table-multiple': 988104,  # 󱏈 U+F13C8
    'table-network': 988105,  # 󱏉 U+F13C9
    'table-off': 988106,  # 󱏊 U+F13CA
    'table-star': 988107,  # 󱏋 U+F13CB
    'car-cog': 988108,  # 󱏌 U+F13CC
    'car-settings': 988109,  # 󱏍 U+F13CD
    'cog-off': 988110,  # 󱏎 U+F13CE
    'cog-off-outline': 988111,  # 󱏏 U+F13CF
    'credit-card-check': 988112,  # 󱏐 U+F13D0
    'credit-card-check-outline': 988113,  # 󱏑 U+F13D1
    'file-tree-outline': 988114,  # 󱏒 U+F13D2
    'folder-star-multiple': 988115,  # 󱏓 U+F13D3
    'folder-star-multiple-outline': 988116,  # 󱏔 U+F13D4
    'home-minus-outline': 988117,  # 󱏕 U+F13D5
    'home-plus-outline': 988118,  # 󱏖 U+F13D6
    'home-remove-outline': 988119,  # 󱏗 U+F13D7
    'scan-helper': 988120,  # 󱏘 U+F13D8
    'video-3d-off': 988121,  # 󱏙 U+F13D9
    'shield-bug': 988122,  # 󱏚 U+F13DA
    'shield-bug-outline': 988123,  # 󱏛 U+F13DB
    'eyedropper-plus': 988124,  # 󱏜 U+F13DC
    'eyedropper-minus': 988125,  # 󱏝 U+F13DD
    'eyedropper-remove': 988126,  # 󱏞 U+F13DE
    'eyedropper-off': 988127,  # 󱏟 U+F13DF
    'baby-buggy': 988128,  # 󱏠 U+F13E0
    'umbrella-closed-variant': 988129,  # 󱏡 U+F13E1
    'umbrella-closed-outline': 988130,  # 󱏢 U+F13E2
    'email-off': 988131,  # 󱏣 U+F13E3
    'email-off-outline': 988132,  # 󱏤 U+F13E4
    'food-variant-off': 988133,  # 󱏥 U+F13E5
    'play-box-multiple-outline': 988134,  # 󱏦 U+F13E6
    'bell-cancel': 988135,  # 󱏧 U+F13E7
    'bell-cancel-outline': 988136,  # 󱏨 U+F13E8
    'bell-minus': 988137,  # 󱏩 U+F13E9
    'bell-minus-outline': 988138,  # 󱏪 U+F13EA
    'bell-remove': 988139,  # 󱏫 U+F13EB
    'bell-remove-outline': 988140,  # 󱏬 U+F13EC
    'beehive-off-outline': 988141,  # 󱏭 U+F13ED
    'cheese-off': 988142,  # 󱏮 U+F13EE
    'corn-off': 988143,  # 󱏯 U+F13EF
    'egg-off': 988144,  # 󱏰 U+F13F0
    'egg-off-outline': 988145,  # 󱏱 U+F13F1
    'egg-outline': 988146,  # 󱏲 U+F13F2
    'fish-off': 988147,  # 󱏳 U+F13F3
    'flask-empty-off': 988148,  # 󱏴 U+F13F4
    'flask-empty-off-outline': 988149,  # 󱏵 U+F13F5
    'flask-off': 988150,  # 󱏶 U+F13F6
    'flask-off-outline': 988151,  # 󱏷 U+F13F7
    'fruit-cherries-off': 988152,  # 󱏸 U+F13F8
    'fruit-citrus-off': 988153,  # 󱏹 U+F13F9
    'mushroom-off': 988154,  # 󱏺 U+F13FA
    'mushroom-off-outline': 988155,  # 󱏻 U+F13FB
    'soy-sauce-off': 988156,  # 󱏼 U+F13FC
    'seed-off': 988157,  # 󱏽 U+F13FD
    'seed-off-outline': 988158,  # 󱏾 U+F13FE
    'tailwind': 988159,  # 󱏿 U+F13FF
    'form-dropdown': 988160,  # 󱐀 U+F1400
    'form-select': 988161,  # 󱐁 U+F1401
    'pump': 988162,  # 󱐂 U+F1402
    'earth-plus': 988163,  # 󱐃 U+F1403
    'earth-minus': 988164,  # 󱐄 U+F1404
    'earth-remove': 988165,  # 󱐅 U+F1405
    'earth-box-plus': 988166,  # 󱐆 U+F1406
    'earth-box-minus': 988167,  # 󱐇 U+F1407
    'earth-box-remove': 988168,  # 󱐈 U+F1408
    'gas-station-off': 988169,  # 󱐉 U+F1409
    'gas-station-off-outline': 988170,  # 󱐊 U+F140A
    'lightning-bolt': 988171,  # 󱐋 U+F140B
    'lightning-bolt-outline': 988172,  # 󱐌 U+F140C
    'smoking-pipe': 988173,  # 󱐍 U+F140D
    'axis-arrow-info': 988174,  # 󱐎 U+F140E
    'chat-plus': 988175,  # 󱐏 U+F140F
    'chat-minus': 988176,  # 󱐐 U+F1410
    'chat-remove': 988177,  # 󱐑 U+F1411
    'chat-plus-outline': 988178,  # 󱐒 U+F1412
    'chat-minus-outline': 988179,  # 󱐓 U+F1413
    'chat-remove-outline': 988180,  # 󱐔 U+F1414
    'bucket': 988181,  # 󱐕 U+F1415
    'bucket-outline': 988182,  # 󱐖 U+F1416
    'pail': 988183,  # 󱐗 U+F1417
    'image-remove': 988184,  # 󱐘 U+F1418
    'image-minus': 988185,  # 󱐙 U+F1419
    'pine-tree-fire': 988186,  # 󱐚 U+F141A
    'cigar-off': 988187,  # 󱐛 U+F141B
    'cube-off': 988188,  # 󱐜 U+F141C
    'cube-off-outline': 988189,  # 󱐝 U+F141D
    'dome-light': 988190,  # 󱐞 U+F141E
    'food-drumstick': 988191,  # 󱐟 U+F141F
    'food-drumstick-outline': 988192,  # 󱐠 U+F1420
    'incognito-circle': 988193,  # 󱐡 U+F1421
    'incognito-circle-off': 988194,  # 󱐢 U+F1422
    'microwave-off': 988195,  # 󱐣 U+F1423
    'power-plug-off-outline': 988196,  # 󱐤 U+F1424
    'power-plug-outline': 988197,  # 󱐥 U+F1425
    'puzzle-check': 988198,  # 󱐦 U+F1426
    'puzzle-check-outline': 988199,  # 󱐧 U+F1427
    'smoking-pipe-off': 988200,  # 󱐨 U+F1428
    'spoon-sugar': 988201,  # 󱐩 U+F1429
    'table-split-cell': 988202,  # 󱐪 U+F142A
    'ticket-percent-outline': 988203,  # 󱐫 U+F142B
    'fuse-off': 988204,  # 󱐬 U+F142C
    'fuse-alert': 988205,  # 󱐭 U+F142D
    'heart-plus': 988206,  # 󱐮 U+F142E
    'heart-minus': 988207,  # 󱐯 U+F142F
    'heart-remove': 988208,  # 󱐰 U+F1430
    'heart-plus-outline': 988209,  # 󱐱 U+F1431
    'heart-minus-outline': 988210,  # 󱐲 U+F1432
    'heart-remove-outline': 988211,  # 󱐳 U+F1433
    'heart-off-outline': 988212,  # 󱐴 U+F1434
    'motion-sensor-off': 988213,  # 󱐵 U+F1435
    'pail-plus': 988214,  # 󱐶 U+F1436
    'pail-minus': 988215,  # 󱐷 U+F1437
    'pail-remove': 988216,  # 󱐸 U+F1438
    'pail-off': 988217,  # 󱐹 U+F1439
    'pail-outline': 988218,  # 󱐺 U+F143A
    'pail-plus-outline': 988219,  # 󱐻 U+F143B
    'pail-minus-outline': 988220,  # 󱐼 U+F143C
    'pail-remove-outline': 988221,  # 󱐽 U+F143D
    'pail-off-outline': 988222,  # 󱐾 U+F143E
    'clock-time-one': 988223,  # 󱐿 U+F143F
    'clock-time-two': 988224,  # 󱑀 U+F1440
    'clock-time-three': 988225,  # 󱑁 U+F1441
    'clock-time-four': 988226,  # 󱑂 U+F1442
    'clock-time-five': 988227,  # 󱑃 U+F1443
    'clock-time-six': 988228,  # 󱑄 U+F1444
    'clock-time-seven': 988229,  # 󱑅 U+F1445
    'clock-time-eight': 988230,  # 󱑆 U+F1446
    'clock-time-nine': 988231,  # 󱑇 U+F1447
    'clock-time-ten': 988232,  # 󱑈 U+F1448
    'clock-time-eleven': 988233,  # 󱑉 U+F1449
    'clock-time-twelve': 988234,  # 󱑊 U+F144A
    'clock-time-one-outline': 988235,  # 󱑋 U+F144B
    'clock-time-two-outline': 988236,  # 󱑌 U+F144C
    'clock-time-three-outline': 988237,  # 󱑍 U+F144D
    'clock-time-four-outline': 988238,  # 󱑎 U+F144E
    'clock-time-five-outline': 988239,  # 󱑏 U+F144F
    'clock-time-six-outline': 988240,  # 󱑐 U+F1450
    'clock-time-seven-outline': 988241,  # 󱑑 U+F1451
    'clock-time-eight-outline': 988242,  # 󱑒 U+F1452
    'clock-time-nine-outline': 988243,  # 󱑓 U+F1453
    'clock-time-ten-outline': 988244,  # 󱑔 U+F1454
    'clock-time-eleven-outline': 988245,  # 󱑕 U+F1455
    'clock-time-twelve-outline': 988246,  # 󱑖 U+F1456
    'printer-search': 988247,  # 󱑗 U+F1457
    'printer-eye': 988248,  # 󱑘 U+F1458
    'minus-circle-off': 988249,  # 󱑙 U+F1459
    'minus-circle-off-outline': 988250,  # 󱑚 U+F145A
    'content-save-cog': 988251,  # 󱑛 U+F145B
    'content-save-cog-outline': 988252,  # 󱑜 U+F145C
    'set-square': 988253,  # 󱑝 U+F145D
    'cog-refresh': 988254,  # 󱑞 U+F145E
    'cog-refresh-outline': 988255,  # 󱑟 U+F145F
    'cog-sync': 988256,  # 󱑠 U+F1460
    'cog-sync-outline': 988257,  # 󱑡 U+F1461
    'download-box': 988258,  # 󱑢 U+F1462
    'download-box-outline': 988259,  # 󱑣 U+F1463
    'download-circle': 988260,  # 󱑤 U+F1464
    'download-circle-outline': 988261,  # 󱑥 U+F1465
    'air-humidifier-off': 988262,  # 󱑦 U+F1466
    'chili-off': 988263,  # 󱑧 U+F1467
    'food-drumstick-off': 988264,  # 󱑨 U+F1468
    'food-drumstick-off-outline': 988265,  # 󱑩 U+F1469
    'food-steak': 988266,  # 󱑪 U+F146A
    'food-steak-off': 988267,  # 󱑫 U+F146B
    'fan-alert': 988268,  # 󱑬 U+F146C
    'fan-chevron-down': 988269,  # 󱑭 U+F146D
    'fan-chevron-up': 988270,  # 󱑮 U+F146E
    'fan-plus': 988271,  # 󱑯 U+F146F
    'fan-minus': 988272,  # 󱑰 U+F1470
    'fan-remove': 988273,  # 󱑱 U+F1471
    'fan-speed-1': 988274,  # 󱑲 U+F1472
    'fan-speed-2': 988275,  # 󱑳 U+F1473
    'fan-speed-3': 988276,  # 󱑴 U+F1474
    'rug': 988277,  # 󱑵 U+F1475
    'lingerie': 988278,  # 󱑶 U+F1476
    'wizard-hat': 988279,  # 󱑷 U+F1477
    'hours-24': 988280,  # 󱑸 U+F1478
    'cosine-wave': 988281,  # 󱑹 U+F1479
    'sawtooth-wave': 988282,  # 󱑺 U+F147A
    'square-wave': 988283,  # 󱑻 U+F147B
    'triangle-wave': 988284,  # 󱑼 U+F147C
    'waveform': 988285,  # 󱑽 U+F147D
    'folder-multiple-plus': 988286,  # 󱑾 U+F147E
    'folder-multiple-plus-outline': 988287,  # 󱑿 U+F147F
    'current-ac': 988288,  # 󱒀 U+F1480
    'watering-can': 988289,  # 󱒁 U+F1481
    'watering-can-outline': 988290,  # 󱒂 U+F1482
    'monitor-share': 988291,  # 󱒃 U+F1483
    'laser-pointer': 988292,  # 󱒄 U+F1484
    'view-array-outline': 988293,  # 󱒅 U+F1485
    'view-carousel-outline': 988294,  # 󱒆 U+F1486
    'view-column-outline': 988295,  # 󱒇 U+F1487
    'view-comfy-outline': 988296,  # 󱒈 U+F1488
    'view-dashboard-variant-outline': 988297,  # 󱒉 U+F1489
    'view-day-outline': 988298,  # 󱒊 U+F148A
    'view-list-outline': 988299,  # 󱒋 U+F148B
    'view-module-outline': 988300,  # 󱒌 U+F148C
    'view-parallel-outline': 988301,  # 󱒍 U+F148D
    'view-quilt-outline': 988302,  # 󱒎 U+F148E
    'view-sequential-outline': 988303,  # 󱒏 U+F148F
    'view-stream-outline': 988304,  # 󱒐 U+F1490
    'view-week-outline': 988305,  # 󱒑 U+F1491
    'compare-horizontal': 988306,  # 󱒒 U+F1492
    'compare-vertical': 988307,  # 󱒓 U+F1493
    'briefcase-variant': 988308,  # 󱒔 U+F1494
    'briefcase-variant-outline': 988309,  # 󱒕 U+F1495
    'relation-many-to-many': 988310,  # 󱒖 U+F1496
    'relation-many-to-one': 988311,  # 󱒗 U+F1497
    'relation-many-to-one-or-many': 988312,  # 󱒘 U+F1498
    'relation-many-to-only-one': 988313,  # 󱒙 U+F1499
    'relation-many-to-zero-or-many': 988314,  # 󱒚 U+F149A
    'relation-many-to-zero-or-one': 988315,  # 󱒛 U+F149B
    'relation-one-or-many-to-many': 988316,  # 󱒜 U+F149C
    'relation-one-or-many-to-one': 988317,  # 󱒝 U+F149D
    'relation-one-or-many-to-one-or-many': 988318,  # 󱒞 U+F149E
    'relation-one-or-many-to-only-one': 988319,  # 󱒟 U+F149F
    'relation-one-or-many-to-zero-or-many': 988320,  # 󱒠 U+F14A0
    'relation-one-or-many-to-zero-or-one': 988321,  # 󱒡 U+F14A1
    'relation-one-to-many': 988322,  # 󱒢 U+F14A2
    'relation-one-to-one': 988323,  # 󱒣 U+F14A3
    'relation-one-to-one-or-many': 988324,  # 󱒤 U+F14A4
    'relation-one-to-only-one': 988325,  # 󱒥 U+F14A5
    'relation-one-to-zero-or-many': 988326,  # 󱒦 U+F14A6
    'relation-one-to-zero-or-one': 988327,  # 󱒧 U+F14A7
    'relation-only-one-to-many': 988328,  # 󱒨 U+F14A8
    'relation-only-one-to-one': 988329,  # 󱒩 U+F14A9
    'relation-only-one-to-one-or-many': 988330,  # 󱒪 U+F14AA
    'relation-only-one-to-only-one': 988331,  # 󱒫 U+F14AB
    'relation-only-one-to-zero-or-many': 988332,  # 󱒬 U+F14AC
    'relation-only-one-to-zero-or-one': 988333,  # 󱒭 U+F14AD
    'relation-zero-or-many-to-many': 988334,  # 󱒮 U+F14AE
    'relation-zero-or-many-to-one': 988335,  # 󱒯 U+F14AF
    'relation-zero-or-many-to-one-or-many': 988336,  # 󱒰 U+F14B0
    'relation-zero-or-many-to-only-one': 988337,  # 󱒱 U+F14B1
    'relation-zero-or-many-to-zero-or-many': 988338,  # 󱒲 U+F14B2
    'relation-zero-or-many-to-zero-or-one': 988339,  # 󱒳 U+F14B3
    'relation-zero-or-one-to-many': 988340,  # 󱒴 U+F14B4
    'relation-zero-or-one-to-one': 988341,  # 󱒵 U+F14B5
    'relation-zero-or-one-to-one-or-many': 988342,  # 󱒶 U+F14B6
    'relation-zero-or-one-to-only-one': 988343,  # 󱒷 U+F14B7
    'relation-zero-or-one-to-zero-or-many': 988344,  # 󱒸 U+F14B8
    'relation-zero-or-one-to-zero-or-one': 988345,  # 󱒹 U+F14B9
    'alert-plus': 988346,  # 󱒺 U+F14BA
    'alert-minus': 988347,  # 󱒻 U+F14BB
    'alert-remove': 988348,  # 󱒼 U+F14BC
    'alert-plus-outline': 988349,  # 󱒽 U+F14BD
    'alert-minus-outline': 988350,  # 󱒾 U+F14BE
    'alert-remove-outline': 988351,  # 󱒿 U+F14BF
    'carabiner': 988352,  # 󱓀 U+F14C0
    'fencing': 988353,  # 󱓁 U+F14C1
    'skateboard': 988354,  # 󱓂 U+F14C2
    'polo': 988355,  # 󱓃 U+F14C3
    'tractor-variant': 988356,  # 󱓄 U+F14C4
    'radiology-box': 988357,  # 󱓅 U+F14C5
    'radiology-box-outline': 988358,  # 󱓆 U+F14C6
    'skull-scan': 988359,  # 󱓇 U+F14C7
    'skull-scan-outline': 988360,  # 󱓈 U+F14C8
    'plus-minus-variant': 988361,  # 󱓉 U+F14C9
    'source-branch-plus': 988362,  # 󱓊 U+F14CA
    'source-branch-minus': 988363,  # 󱓋 U+F14CB
    'source-branch-remove': 988364,  # 󱓌 U+F14CC
    'source-branch-refresh': 988365,  # 󱓍 U+F14CD
    'source-branch-sync': 988366,  # 󱓎 U+F14CE
    'source-branch-check': 988367,  # 󱓏 U+F14CF
    'puzzle-plus': 988368,  # 󱓐 U+F14D0
    'puzzle-minus': 988369,  # 󱓑 U+F14D1
    'puzzle-remove': 988370,  # 󱓒 U+F14D2
    'puzzle-edit': 988371,  # 󱓓 U+F14D3
    'puzzle-heart': 988372,  # 󱓔 U+F14D4
    'puzzle-star': 988373,  # 󱓕 U+F14D5
    'puzzle-plus-outline': 988374,  # 󱓖 U+F14D6
    'puzzle-minus-outline': 988375,  # 󱓗 U+F14D7
    'puzzle-remove-outline': 988376,  # 󱓘 U+F14D8
    'puzzle-edit-outline': 988377,  # 󱓙 U+F14D9
    'puzzle-heart-outline': 988378,  # 󱓚 U+F14DA
    'puzzle-star-outline': 988379,  # 󱓛 U+F14DB
    'rhombus-medium-outline': 988380,  # 󱓜 U+F14DC
    'rhombus-split-outline': 988381,  # 󱓝 U+F14DD
    'rocket-launch': 988382,  # 󱓞 U+F14DE
    'rocket-launch-outline': 988383,  # 󱓟 U+F14DF
    'set-merge': 988384,  # 󱓠 U+F14E0
    'set-split': 988385,  # 󱓡 U+F14E1
    'beekeeper': 988386,  # 󱓢 U+F14E2
    'snowflake-off': 988387,  # 󱓣 U+F14E3
    'weather-sunny-off': 988388,  # 󱓤 U+F14E4
    'clipboard-edit': 988389,  # 󱓥 U+F14E5
    'clipboard-edit-outline': 988390,  # 󱓦 U+F14E6
    'notebook-edit': 988391,  # 󱓧 U+F14E7
    'human-edit': 988392,  # 󱓨 U+F14E8
    'notebook-edit-outline': 988393,  # 󱓩 U+F14E9
    'cash-lock': 988394,  # 󱓪 U+F14EA
    'cash-lock-open': 988395,  # 󱓫 U+F14EB
    'account-supervisor-circle-outline': 988396,  # 󱓬 U+F14EC
    'car-outline': 988397,  # 󱓭 U+F14ED
    'cash-check': 988398,  # 󱓮 U+F14EE
    'filter-off': 988399,  # 󱓯 U+F14EF
    'filter-off-outline': 988400,  # 󱓰 U+F14F0
    'spirit-level': 988401,  # 󱓱 U+F14F1
    'wheel-barrow': 988402,  # 󱓲 U+F14F2
    'book-check': 988403,  # 󱓳 U+F14F3
    'book-check-outline': 988404,  # 󱓴 U+F14F4
    'notebook-check': 988405,  # 󱓵 U+F14F5
    'notebook-check-outline': 988406,  # 󱓶 U+F14F6
    'book-open-variant': 988407,  # 󱓷 U+F14F7
    'sign-pole': 988408,  # 󱓸 U+F14F8
    'shore': 988409,  # 󱓹 U+F14F9
    'shape-square-rounded-plus': 988410,  # 󱓺 U+F14FA
    'square-rounded': 988411,  # 󱓻 U+F14FB
    'square-rounded-outline': 988412,  # 󱓼 U+F14FC
    'archive-alert': 988413,  # 󱓽 U+F14FD
    'archive-alert-outline': 988414,  # 󱓾 U+F14FE
    'power-socket-it': 988415,  # 󱓿 U+F14FF
    'square-circle': 988416,  # 󱔀 U+F1500
    'symbol': 988417,  # 󱔁 U+F1501
    'water-alert': 988418,  # 󱔂 U+F1502
    'water-alert-outline': 988419,  # 󱔃 U+F1503
    'water-check': 988420,  # 󱔄 U+F1504
    'water-check-outline': 988421,  # 󱔅 U+F1505
    'water-minus': 988422,  # 󱔆 U+F1506
    'water-minus-outline': 988423,  # 󱔇 U+F1507
    'water-off-outline': 988424,  # 󱔈 U+F1508
    'water-percent-alert': 988425,  # 󱔉 U+F1509
    'water-plus': 988426,  # 󱔊 U+F150A
    'water-plus-outline': 988427,  # 󱔋 U+F150B
    'water-remove': 988428,  # 󱔌 U+F150C
    'water-remove-outline': 988429,  # 󱔍 U+F150D
    'snake#1': 988430,  # 󱔎 U+F150E
    'format-text-variant-outline': 988431,  # 󱔏 U+F150F
    'grass': 988432,  # 󱔐 U+F1510
    'access-point-off': 988433,  # 󱔑 U+F1511
    'currency-mnt': 988434,  # 󱔒 U+F1512
    'dock-top': 988435,  # 󱔓 U+F1513
    'share-variant-outline': 988436,  # 󱔔 U+F1514
    'transit-skip': 988437,  # 󱔕 U+F1515
    'yurt': 988438,  # 󱔖 U+F1516
    'file-document-multiple': 988439,  # 󱔗 U+F1517
    'file-document-multiple-outline': 988440,  # 󱔘 U+F1518
    'ev-plug-ccs1': 988441,  # 󱔙 U+F1519
    'ev-plug-ccs2': 988442,  # 󱔚 U+F151A
    'ev-plug-chademo': 988443,  # 󱔛 U+F151B
    'ev-plug-tesla': 988444,  # 󱔜 U+F151C
    'ev-plug-type1': 988445,  # 󱔝 U+F151D
    'ev-plug-type2': 988446,  # 󱔞 U+F151E
    'office-building-outline': 988447,  # 󱔟 U+F151F
    'office-building-marker': 988448,  # 󱔠 U+F1520
    'office-building-marker-outline': 988449,  # 󱔡 U+F1521
    'progress-question': 988450,  # 󱔢 U+F1522
    'basket-minus': 988451,  # 󱔣 U+F1523
    'basket-minus-outline': 988452,  # 󱔤 U+F1524
    'basket-off': 988453,  # 󱔥 U+F1525
    'basket-off-outline': 988454,  # 󱔦 U+F1526
    'basket-plus': 988455,  # 󱔧 U+F1527
    'basket-plus-outline': 988456,  # 󱔨 U+F1528
    'basket-remove': 988457,  # 󱔩 U+F1529
    'basket-remove-outline': 988458,  # 󱔪 U+F152A
    'account-reactivate': 988459,  # 󱔫 U+F152B
    'account-reactivate-outline': 988460,  # 󱔬 U+F152C
    'car-lifted-pickup': 988461,  # 󱔭 U+F152D
    'video-high-definition': 988462,  # 󱔮 U+F152E
    'phone-remove': 988463,  # 󱔯 U+F152F
    'phone-remove-outline': 988464,  # 󱔰 U+F1530
    'thermometer-off': 988465,  # 󱔱 U+F1531
    'timeline-check': 988466,  # 󱔲 U+F1532
    'timeline-check-outline': 988467,  # 󱔳 U+F1533
    'timeline-minus': 988468,  # 󱔴 U+F1534
    'timeline-minus-outline': 988469,  # 󱔵 U+F1535
    'timeline-remove': 988470,  # 󱔶 U+F1536
    'timeline-remove-outline': 988471,  # 󱔷 U+F1537
    'access-point-check': 988472,  # 󱔸 U+F1538
    'access-point-minus': 988473,  # 󱔹 U+F1539
    'access-point-plus': 988474,  # 󱔺 U+F153A
    'access-point-remove': 988475,  # 󱔻 U+F153B
    'data-matrix': 988476,  # 󱔼 U+F153C
    'data-matrix-edit': 988477,  # 󱔽 U+F153D
    'data-matrix-minus': 988478,  # 󱔾 U+F153E
    'data-matrix-plus': 988479,  # 󱔿 U+F153F
    'data-matrix-remove': 988480,  # 󱕀 U+F1540
    'data-matrix-scan': 988481,  # 󱕁 U+F1541
    'tune-variant': 988482,  # 󱕂 U+F1542
    'tune-vertical-variant': 988483,  # 󱕃 U+F1543
    'rake': 988484,  # 󱕄 U+F1544
    'shimmer': 988485,  # 󱕅 U+F1545
    'transit-connection-horizontal': 988486,  # 󱕆 U+F1546
    'sort-calendar-ascending': 988487,  # 󱕇 U+F1547
    'sort-calendar-descending': 988488,  # 󱕈 U+F1548
    'sort-clock-ascending': 988489,  # 󱕉 U+F1549
    'sort-clock-ascending-outline': 988490,  # 󱕊 U+F154A
    'sort-clock-descending': 988491,  # 󱕋 U+F154B
    'sort-clock-descending-outline': 988492,  # 󱕌 U+F154C
    'chart-box': 988493,  # 󱕍 U+F154D
    'chart-box-outline': 988494,  # 󱕎 U+F154E
    'chart-box-plus-outline': 988495,  # 󱕏 U+F154F
    'mouse-move-down': 988496,  # 󱕐 U+F1550
    'mouse-move-up': 988497,  # 󱕑 U+F1551
    'mouse-move-vertical': 988498,  # 󱕒 U+F1552
    'pitchfork': 988499,  # 󱕓 U+F1553
    'vanish-quarter': 988500,  # 󱕔 U+F1554
    'application-settings-outline': 988501,  # 󱕕 U+F1555
    'delete-clock': 988502,  # 󱕖 U+F1556
    'delete-clock-outline': 988503,  # 󱕗 U+F1557
    'kangaroo': 988504,  # 󱕘 U+F1558
    'phone-dial': 988505,  # 󱕙 U+F1559
    'phone-dial-outline': 988506,  # 󱕚 U+F155A
    'star-off-outline': 988507,  # 󱕛 U+F155B
    'tooltip-check': 988508,  # 󱕜 U+F155C
    'tooltip-check-outline': 988509,  # 󱕝 U+F155D
    'tooltip-minus': 988510,  # 󱕞 U+F155E
    'tooltip-minus-outline': 988511,  # 󱕟 U+F155F
    'tooltip-remove': 988512,  # 󱕠 U+F1560
    'tooltip-remove-outline': 988513,  # 󱕡 U+F1561
    'pretzel': 988514,  # 󱕢 U+F1562
    'star-plus': 988515,  # 󱕣 U+F1563
    'star-minus': 988516,  # 󱕤 U+F1564
    'star-remove': 988517,  # 󱕥 U+F1565
    'star-check': 988518,  # 󱕦 U+F1566
    'star-plus-outline': 988519,  # 󱕧 U+F1567
    'star-minus-outline': 988520,  # 󱕨 U+F1568
    'star-remove-outline': 988521,  # 󱕩 U+F1569
    'star-check-outline': 988522,  # 󱕪 U+F156A
    'eiffel-tower': 988523,  # 󱕫 U+F156B
    'submarine': 988524,  # 󱕬 U+F156C
    'sofa-outline': 988525,  # 󱕭 U+F156D
    'sofa-single': 988526,  # 󱕮 U+F156E
    'sofa-single-outline': 988527,  # 󱕯 U+F156F
    'text-account': 988528,  # 󱕰 U+F1570
    'human-queue': 988529,  # 󱕱 U+F1571
    'food-halal': 988530,  # 󱕲 U+F1572
    'food-kosher': 988531,  # 󱕳 U+F1573
    'key-chain': 988532,  # 󱕴 U+F1574
    'key-chain-variant': 988533,  # 󱕵 U+F1575
    'lamps': 988534,  # 󱕶 U+F1576
    'application-cog-outline': 988535,  # 󱕷 U+F1577
    'dance-pole': 988536,  # 󱕸 U+F1578
    'social-distance-2-meters': 988537,  # 󱕹 U+F1579
    'social-distance-6-feet': 988538,  # 󱕺 U+F157A
    'calendar-cursor': 988539,  # 󱕻 U+F157B
    'emoticon-sick': 988540,  # 󱕼 U+F157C
    'emoticon-sick-outline': 988541,  # 󱕽 U+F157D
    'hand-heart-outline': 988542,  # 󱕾 U+F157E
    'hand-wash': 988543,  # 󱕿 U+F157F
    'hand-wash-outline': 988544,  # 󱖀 U+F1580
    'human-cane': 988545,  # 󱖁 U+F1581
    'lotion': 988546,  # 󱖂 U+F1582
    'lotion-outline': 988547,  # 󱖃 U+F1583
    'lotion-plus': 988548,  # 󱖄 U+F1584
    'lotion-plus-outline': 988549,  # 󱖅 U+F1585
    'face-mask': 988550,  # 󱖆 U+F1586
    'face-mask-outline': 988551,  # 󱖇 U+F1587
    'reiterate': 988552,  # 󱖈 U+F1588
    'butterfly#1': 988553,  # 󱖉 U+F1589
    'butterfly-outline': 988554,  # 󱖊 U+F158A
    'bag-suitcase': 988555,  # 󱖋 U+F158B
    'bag-suitcase-outline': 988556,  # 󱖌 U+F158C
    'bag-suitcase-off': 988557,  # 󱖍 U+F158D
    'bag-suitcase-off-outline': 988558,  # 󱖎 U+F158E
    'motion-play': 988559,  # 󱖏 U+F158F
    'motion-pause': 988560,  # 󱖐 U+F1590
    'motion-play-outline': 988561,  # 󱖑 U+F1591
    'motion-pause-outline': 988562,  # 󱖒 U+F1592
    'arrow-top-left-thin-circle-outline': 988563,  # 󱖓 U+F1593
    'arrow-top-right-thin-circle-outline': 988564,  # 󱖔 U+F1594
    'arrow-bottom-right-thin-circle-outline': 988565,  # 󱖕 U+F1595
    'arrow-bottom-left-thin-circle-outline': 988566,  # 󱖖 U+F1596
    'arrow-up-thin-circle-outline': 988567,  # 󱖗 U+F1597
    'arrow-right-thin-circle-outline': 988568,  # 󱖘 U+F1598
    'arrow-down-thin-circle-outline': 988569,  # 󱖙 U+F1599
    'arrow-left-thin-circle-outline': 988570,  # 󱖚 U+F159A
    'human-capacity-decrease': 988571,  # 󱖛 U+F159B
    'human-capacity-increase': 988572,  # 󱖜 U+F159C
    'human-greeting-proximity': 988573,  # 󱖝 U+F159D
    'hvac-off': 988574,  # 󱖞 U+F159E
    'inbox-remove': 988575,  # 󱖟 U+F159F
    'inbox-remove-outline': 988576,  # 󱖠 U+F15A0
    'handshake-outline': 988577,  # 󱖡 U+F15A1
    'ladder': 988578,  # 󱖢 U+F15A2
    'router-wireless-off': 988579,  # 󱖣 U+F15A3
    'seesaw': 988580,  # 󱖤 U+F15A4
    'slide': 988581,  # 󱖥 U+F15A5
    'calculator-variant-outline': 988582,  # 󱖦 U+F15A6
    'shield-account-variant': 988583,  # 󱖧 U+F15A7
    'shield-account-variant-outline': 988584,  # 󱖨 U+F15A8
    'message-flash': 988585,  # 󱖩 U+F15A9
    'message-flash-outline': 988586,  # 󱖪 U+F15AA
    'list-status': 988587,  # 󱖫 U+F15AB
    'message-bookmark': 988588,  # 󱖬 U+F15AC
    'message-bookmark-outline': 988589,  # 󱖭 U+F15AD
    'comment-bookmark': 988590,  # 󱖮 U+F15AE
    'comment-bookmark-outline': 988591,  # 󱖯 U+F15AF
    'comment-flash': 988592,  # 󱖰 U+F15B0
    'comment-flash-outline': 988593,  # 󱖱 U+F15B1
    'motion': 988594,  # 󱖲 U+F15B2
    'motion-outline': 988595,  # 󱖳 U+F15B3
    'bicycle-electric': 988596,  # 󱖴 U+F15B4
    'car-electric-outline': 988597,  # 󱖵 U+F15B5
    'chart-timeline-variant-shimmer': 988598,  # 󱖶 U+F15B6
    'moped-electric': 988599,  # 󱖷 U+F15B7
    'moped-electric-outline': 988600,  # 󱖸 U+F15B8
    'moped-outline': 988601,  # 󱖹 U+F15B9
    'motorbike-electric': 988602,  # 󱖺 U+F15BA
    'rickshaw': 988603,  # 󱖻 U+F15BB
    'rickshaw-electric': 988604,  # 󱖼 U+F15BC
    'scooter': 988605,  # 󱖽 U+F15BD
    'scooter-electric': 988606,  # 󱖾 U+F15BE
    'horse#1': 988607,  # 󱖿 U+F15BF
    'horse-human': 988608,  # 󱗀 U+F15C0
    'horse-variant': 988609,  # 󱗁 U+F15C1
    'unicorn': 988610,  # 󱗂 U+F15C2
    'unicorn-variant': 988611,  # 󱗃 U+F15C3
    'alarm-panel': 988612,  # 󱗄 U+F15C4
    'alarm-panel-outline': 988613,  # 󱗅 U+F15C5
    'bird': 988614,  # 󱗆 U+F15C6
    'shoe-cleat': 988615,  # 󱗇 U+F15C7
    'shoe-sneaker': 988616,  # 󱗈 U+F15C8
    'human-female-dance': 988617,  # 󱗉 U+F15C9
    'shoe-ballet': 988618,  # 󱗊 U+F15CA
    'numeric-positive-1': 988619,  # 󱗋 U+F15CB
    'face-man-shimmer': 988620,  # 󱗌 U+F15CC
    'face-man-shimmer-outline': 988621,  # 󱗍 U+F15CD
    'face-woman-shimmer': 988622,  # 󱗎 U+F15CE
    'face-woman-shimmer-outline': 988623,  # 󱗏 U+F15CF
    'home-alert-outline': 988624,  # 󱗐 U+F15D0
    'lock-alert-outline': 988625,  # 󱗑 U+F15D1
    'lock-open-alert-outline': 988626,  # 󱗒 U+F15D2
    'sim-alert-outline': 988627,  # 󱗓 U+F15D3
    'sim-off-outline': 988628,  # 󱗔 U+F15D4
    'sim-outline': 988629,  # 󱗕 U+F15D5
    'book-open-page-variant-outline': 988630,  # 󱗖 U+F15D6
    'fire-alert': 988631,  # 󱗗 U+F15D7
    'ray-start-vertex-end': 988632,  # 󱗘 U+F15D8
    'camera-flip': 988633,  # 󱗙 U+F15D9
    'camera-flip-outline': 988634,  # 󱗚 U+F15DA
    'orbit-variant': 988635,  # 󱗛 U+F15DB
    'circle-box': 988636,  # 󱗜 U+F15DC
    'circle-box-outline': 988637,  # 󱗝 U+F15DD
    'mustache#1': 988638,  # 󱗞 U+F15DE
    'comment-minus': 988639,  # 󱗟 U+F15DF
    'comment-minus-outline': 988640,  # 󱗠 U+F15E0
    'comment-off': 988641,  # 󱗡 U+F15E1
    'comment-off-outline': 988642,  # 󱗢 U+F15E2
    'eye-remove': 988643,  # 󱗣 U+F15E3
    'eye-remove-outline': 988644,  # 󱗤 U+F15E4
    'unicycle': 988645,  # 󱗥 U+F15E5
    'glass-cocktail-off': 988646,  # 󱗦 U+F15E6
    'glass-mug-off': 988647,  # 󱗧 U+F15E7
    'glass-mug-variant-off': 988648,  # 󱗨 U+F15E8
    'bicycle-penny-farthing': 988649,  # 󱗩 U+F15E9
    'cart-check': 988650,  # 󱗪 U+F15EA
    'cart-variant': 988651,  # 󱗫 U+F15EB
    'baseball-diamond': 988652,  # 󱗬 U+F15EC
    'baseball-diamond-outline': 988653,  # 󱗭 U+F15ED
    'fridge-industrial': 988654,  # 󱗮 U+F15EE
    'fridge-industrial-alert': 988655,  # 󱗯 U+F15EF
    'fridge-industrial-alert-outline': 988656,  # 󱗰 U+F15F0
    'fridge-industrial-off': 988657,  # 󱗱 U+F15F1
    'fridge-industrial-off-outline': 988658,  # 󱗲 U+F15F2
    'fridge-industrial-outline': 988659,  # 󱗳 U+F15F3
    'fridge-variant': 988660,  # 󱗴 U+F15F4
    'fridge-variant-alert': 988661,  # 󱗵 U+F15F5
    'fridge-variant-alert-outline': 988662,  # 󱗶 U+F15F6
    'fridge-variant-off': 988663,  # 󱗷 U+F15F7
    'fridge-variant-off-outline': 988664,  # 󱗸 U+F15F8
    'fridge-variant-outline': 988665,  # 󱗹 U+F15F9
    'windsock': 988666,  # 󱗺 U+F15FA
    'dance-ballroom': 988667,  # 󱗻 U+F15FB
    'dots-grid': 988668,  # 󱗼 U+F15FC
    'dots-square': 988669,  # 󱗽 U+F15FD
    'dots-triangle': 988670,  # 󱗾 U+F15FE
    'dots-hexagon': 988671,  # 󱗿 U+F15FF
    'card-minus': 988672,  # 󱘀 U+F1600
    'card-minus-outline': 988673,  # 󱘁 U+F1601
    'card-off': 988674,  # 󱘂 U+F1602
    'card-off-outline': 988675,  # 󱘃 U+F1603
    'card-remove': 988676,  # 󱘄 U+F1604
    'card-remove-outline': 988677,  # 󱘅 U+F1605
    'torch': 988678,  # 󱘆 U+F1606
    'navigation-outline': 988679,  # 󱘇 U+F1607
    'map-marker-star': 988680,  # 󱘈 U+F1608
    'map-marker-star-outline': 988681,  # 󱘉 U+F1609
    'manjaro': 988682,  # 󱘊 U+F160A
    'fast-forward-60': 988683,  # 󱘋 U+F160B
    'rewind-60': 988684,  # 󱘌 U+F160C
    'image-text': 988685,  # 󱘍 U+F160D
    'family-tree': 988686,  # 󱘎 U+F160E
    'car-emergency': 988687,  # 󱘏 U+F160F
    'notebook-minus': 988688,  # 󱘐 U+F1610
    'notebook-minus-outline': 988689,  # 󱘑 U+F1611
    'notebook-plus': 988690,  # 󱘒 U+F1612
    'notebook-plus-outline': 988691,  # 󱘓 U+F1613
    'notebook-remove': 988692,  # 󱘔 U+F1614
    'notebook-remove-outline': 988693,  # 󱘕 U+F1615
    'connection': 988694,  # 󱘖 U+F1616
    'language-rust': 988695,  # 󱘗 U+F1617
    'clipboard-minus': 988696,  # 󱘘 U+F1618
    'clipboard-minus-outline': 988697,  # 󱘙 U+F1619
    'clipboard-off': 988698,  # 󱘚 U+F161A
    'clipboard-off-outline': 988699,  # 󱘛 U+F161B
    'clipboard-remove': 988700,  # 󱘜 U+F161C
    'clipboard-remove-outline': 988701,  # 󱘝 U+F161D
    'clipboard-search': 988702,  # 󱘞 U+F161E
    'clipboard-search-outline': 988703,  # 󱘟 U+F161F
    'clipboard-text-off': 988704,  # 󱘠 U+F1620
    'clipboard-text-off-outline': 988705,  # 󱘡 U+F1621
    'clipboard-text-search': 988706,  # 󱘢 U+F1622
    'clipboard-text-search-outline': 988707,  # 󱘣 U+F1623
    'database-alert-outline': 988708,  # 󱘤 U+F1624
    'database-arrow-down-outline': 988709,  # 󱘥 U+F1625
    'database-arrow-left-outline': 988710,  # 󱘦 U+F1626
    'database-arrow-right-outline': 988711,  # 󱘧 U+F1627
    'database-arrow-up-outline': 988712,  # 󱘨 U+F1628
    'database-check-outline': 988713,  # 󱘩 U+F1629
    'database-clock-outline': 988714,  # 󱘪 U+F162A
    'database-edit-outline': 988715,  # 󱘫 U+F162B
    'database-export-outline': 988716,  # 󱘬 U+F162C
    'database-import-outline': 988717,  # 󱘭 U+F162D
    'database-lock-outline': 988718,  # 󱘮 U+F162E
    'database-marker-outline': 988719,  # 󱘯 U+F162F
    'database-minus-outline': 988720,  # 󱘰 U+F1630
    'database-off-outline': 988721,  # 󱘱 U+F1631
    'database-outline': 988722,  # 󱘲 U+F1632
    'database-plus-outline': 988723,  # 󱘳 U+F1633
    'database-refresh-outline': 988724,  # 󱘴 U+F1634
    'database-remove-outline': 988725,  # 󱘵 U+F1635
    'database-search-outline': 988726,  # 󱘶 U+F1636
    'database-settings-outline': 988727,  # 󱘷 U+F1637
    'database-sync-outline': 988728,  # 󱘸 U+F1638
    'minus-thick': 988729,  # 󱘹 U+F1639
    'database-alert': 988730,  # 󱘺 U+F163A
    'database-arrow-down': 988731,  # 󱘻 U+F163B
    'database-arrow-left': 988732,  # 󱘼 U+F163C
    'database-arrow-right': 988733,  # 󱘽 U+F163D
    'database-arrow-up': 988734,  # 󱘾 U+F163E
    'database-clock': 988735,  # 󱘿 U+F163F
    'database-off': 988736,  # 󱙀 U+F1640
    'calendar-lock': 988737,  # 󱙁 U+F1641
    'calendar-lock-outline': 988738,  # 󱙂 U+F1642
    'content-save-off': 988739,  # 󱙃 U+F1643
    'content-save-off-outline': 988740,  # 󱙄 U+F1644
    'credit-card-refresh': 988741,  # 󱙅 U+F1645
    'credit-card-refresh-outline': 988742,  # 󱙆 U+F1646
    'credit-card-search': 988743,  # 󱙇 U+F1647
    'credit-card-search-outline': 988744,  # 󱙈 U+F1648
    'credit-card-sync': 988745,  # 󱙉 U+F1649
    'credit-card-sync-outline': 988746,  # 󱙊 U+F164A
    'database-cog': 988747,  # 󱙋 U+F164B
    'database-cog-outline': 988748,  # 󱙌 U+F164C
    'message-off': 988749,  # 󱙍 U+F164D
    'message-off-outline': 988750,  # 󱙎 U+F164E
    'note-minus': 988751,  # 󱙏 U+F164F
    'note-minus-outline': 988752,  # 󱙐 U+F1650
    'note-remove': 988753,  # 󱙑 U+F1651
    'note-remove-outline': 988754,  # 󱙒 U+F1652
    'note-search': 988755,  # 󱙓 U+F1653
    'note-search-outline': 988756,  # 󱙔 U+F1654
    'bank-check': 988757,  # 󱙕 U+F1655
    'bank-off': 988758,  # 󱙖 U+F1656
    'bank-off-outline': 988759,  # 󱙗 U+F1657
    'briefcase-off': 988760,  # 󱙘 U+F1658
    'briefcase-off-outline': 988761,  # 󱙙 U+F1659
    'briefcase-variant-off': 988762,  # 󱙚 U+F165A
    'briefcase-variant-off-outline': 988763,  # 󱙛 U+F165B
    'ghost-off-outline': 988764,  # 󱙜 U+F165C
    'ghost-outline': 988765,  # 󱙝 U+F165D
    'store-minus': 988766,  # 󱙞 U+F165E
    'store-plus': 988767,  # 󱙟 U+F165F
    'store-remove': 988768,  # 󱙠 U+F1660
    'email-remove': 988769,  # 󱙡 U+F1661
    'email-remove-outline': 988770,  # 󱙢 U+F1662
    'heart-cog': 988771,  # 󱙣 U+F1663
    'heart-cog-outline': 988772,  # 󱙤 U+F1664
    'heart-settings': 988773,  # 󱙥 U+F1665
    'heart-settings-outline': 988774,  # 󱙦 U+F1666
    'pentagram': 988775,  # 󱙧 U+F1667
    'star-cog': 988776,  # 󱙨 U+F1668
    'star-cog-outline': 988777,  # 󱙩 U+F1669
    'star-settings': 988778,  # 󱙪 U+F166A
    'star-settings-outline': 988779,  # 󱙫 U+F166B
    'calendar-end': 988780,  # 󱙬 U+F166C
    'calendar-start': 988781,  # 󱙭 U+F166D
    'cannabis-off': 988782,  # 󱙮 U+F166E
    'mower': 988783,  # 󱙯 U+F166F
    'mower-bag': 988784,  # 󱙰 U+F1670
    'lock-off': 988785,  # 󱙱 U+F1671
    'lock-off-outline': 988786,  # 󱙲 U+F1672
    'shark-fin': 988787,  # 󱙳 U+F1673
    'shark-fin-outline': 988788,  # 󱙴 U+F1674
    'paw-outline': 988789,  # 󱙵 U+F1675
    'paw-off-outline': 988790,  # 󱙶 U+F1676
    'snail': 988791,  # 󱙷 U+F1677
    'pig-variant-outline': 988792,  # 󱙸 U+F1678
    'piggy-bank-outline': 988793,  # 󱙹 U+F1679
    'robot-outline': 988794,  # 󱙺 U+F167A
    'robot-off-outline': 988795,  # 󱙻 U+F167B
    'book-alert': 988796,  # 󱙼 U+F167C
    'book-alert-outline': 988797,  # 󱙽 U+F167D
    'book-arrow-down': 988798,  # 󱙾 U+F167E
    'book-arrow-down-outline': 988799,  # 󱙿 U+F167F
    'book-arrow-left': 988800,  # 󱚀 U+F1680
    'book-arrow-left-outline': 988801,  # 󱚁 U+F1681
    'book-arrow-right': 988802,  # 󱚂 U+F1682
    'book-arrow-right-outline': 988803,  # 󱚃 U+F1683
    'book-arrow-up': 988804,  # 󱚄 U+F1684
    'book-arrow-up-outline': 988805,  # 󱚅 U+F1685
    'book-cancel': 988806,  # 󱚆 U+F1686
    'book-cancel-outline': 988807,  # 󱚇 U+F1687
    'book-clock': 988808,  # 󱚈 U+F1688
    'book-clock-outline': 988809,  # 󱚉 U+F1689
    'book-cog': 988810,  # 󱚊 U+F168A
    'book-cog-outline': 988811,  # 󱚋 U+F168B
    'book-edit': 988812,  # 󱚌 U+F168C
    'book-edit-outline': 988813,  # 󱚍 U+F168D
    'book-lock-open-outline': 988814,  # 󱚎 U+F168E
    'book-lock-outline': 988815,  # 󱚏 U+F168F
    'book-marker': 988816,  # 󱚐 U+F1690
    'book-marker-outline': 988817,  # 󱚑 U+F1691
    'book-minus-outline': 988818,  # 󱚒 U+F1692
    'book-music-outline': 988819,  # 󱚓 U+F1693
    'book-off': 988820,  # 󱚔 U+F1694
    'book-off-outline': 988821,  # 󱚕 U+F1695
    'book-plus-outline': 988822,  # 󱚖 U+F1696
    'book-refresh': 988823,  # 󱚗 U+F1697
    'book-refresh-outline': 988824,  # 󱚘 U+F1698
    'book-remove-outline': 988825,  # 󱚙 U+F1699
    'book-settings': 988826,  # 󱚚 U+F169A
    'book-settings-outline': 988827,  # 󱚛 U+F169B
    'book-sync': 988828,  # 󱚜 U+F169C
    'robot-angry': 988829,  # 󱚝 U+F169D
    'robot-angry-outline': 988830,  # 󱚞 U+F169E
    'robot-confused': 988831,  # 󱚟 U+F169F
    'robot-confused-outline': 988832,  # 󱚠 U+F16A0
    'robot-dead': 988833,  # 󱚡 U+F16A1
    'robot-dead-outline': 988834,  # 󱚢 U+F16A2
    'robot-excited': 988835,  # 󱚣 U+F16A3
    'robot-excited-outline': 988836,  # 󱚤 U+F16A4
    'robot-love': 988837,  # 󱚥 U+F16A5
    'robot-love-outline': 988838,  # 󱚦 U+F16A6
    'robot-off': 988839,  # 󱚧 U+F16A7
    'lock-check-outline': 988840,  # 󱚨 U+F16A8
    'lock-minus': 988841,  # 󱚩 U+F16A9
    'lock-minus-outline': 988842,  # 󱚪 U+F16AA
    'lock-open-check-outline': 988843,  # 󱚫 U+F16AB
    'lock-open-minus': 988844,  # 󱚬 U+F16AC
    'lock-open-minus-outline': 988845,  # 󱚭 U+F16AD
    'lock-open-plus': 988846,  # 󱚮 U+F16AE
    'lock-open-plus-outline': 988847,  # 󱚯 U+F16AF
    'lock-open-remove': 988848,  # 󱚰 U+F16B0
    'lock-open-remove-outline': 988849,  # 󱚱 U+F16B1
    'lock-plus-outline': 988850,  # 󱚲 U+F16B2
    'lock-remove': 988851,  # 󱚳 U+F16B3
    'lock-remove-outline': 988852,  # 󱚴 U+F16B4
    'wifi-alert': 988853,  # 󱚵 U+F16B5
    'wifi-arrow-down': 988854,  # 󱚶 U+F16B6
    'wifi-arrow-left': 988855,  # 󱚷 U+F16B7
    'wifi-arrow-left-right': 988856,  # 󱚸 U+F16B8
    'wifi-arrow-right': 988857,  # 󱚹 U+F16B9
    'wifi-arrow-up': 988858,  # 󱚺 U+F16BA
    'wifi-arrow-up-down': 988859,  # 󱚻 U+F16BB
    'wifi-cancel': 988860,  # 󱚼 U+F16BC
    'wifi-check': 988861,  # 󱚽 U+F16BD
    'wifi-cog': 988862,  # 󱚾 U+F16BE
    'wifi-lock': 988863,  # 󱚿 U+F16BF
    'wifi-lock-open': 988864,  # 󱛀 U+F16C0
    'wifi-marker': 988865,  # 󱛁 U+F16C1
    'wifi-minus': 988866,  # 󱛂 U+F16C2
    'wifi-plus': 988867,  # 󱛃 U+F16C3
    'wifi-refresh': 988868,  # 󱛄 U+F16C4
    'wifi-remove': 988869,  # 󱛅 U+F16C5
    'wifi-settings': 988870,  # 󱛆 U+F16C6
    'wifi-sync': 988871,  # 󱛇 U+F16C7
    'book-sync-outline': 988872,  # 󱛈 U+F16C8
    'book-education': 988873,  # 󱛉 U+F16C9
    'book-education-outline': 988874,  # 󱛊 U+F16CA
    'wifi-strength-1-lock-open': 988875,  # 󱛋 U+F16CB
    'wifi-strength-2-lock-open': 988876,  # 󱛌 U+F16CC
    'wifi-strength-3-lock-open': 988877,  # 󱛍 U+F16CD
    'wifi-strength-4-lock-open': 988878,  # 󱛎 U+F16CE
    'wifi-strength-lock-open-outline': 988879,  # 󱛏 U+F16CF
    'cookie-alert': 988880,  # 󱛐 U+F16D0
    'cookie-alert-outline': 988881,  # 󱛑 U+F16D1
    'cookie-check': 988882,  # 󱛒 U+F16D2
    'cookie-check-outline': 988883,  # 󱛓 U+F16D3
    'cookie-cog': 988884,  # 󱛔 U+F16D4
    'cookie-cog-outline': 988885,  # 󱛕 U+F16D5
    'cookie-plus': 988886,  # 󱛖 U+F16D6
    'cookie-plus-outline': 988887,  # 󱛗 U+F16D7
    'cookie-remove': 988888,  # 󱛘 U+F16D8
    'cookie-remove-outline': 988889,  # 󱛙 U+F16D9
    'cookie-minus': 988890,  # 󱛚 U+F16DA
    'cookie-minus-outline': 988891,  # 󱛛 U+F16DB
    'cookie-settings': 988892,  # 󱛜 U+F16DC
    'cookie-settings-outline': 988893,  # 󱛝 U+F16DD
    'cookie-outline': 988894,  # 󱛞 U+F16DE
    'tape-drive': 988895,  # 󱛟 U+F16DF
    'abacus': 988896,  # 󱛠 U+F16E0
    'calendar-clock-outline': 988897,  # 󱛡 U+F16E1
    'clipboard-clock': 988898,  # 󱛢 U+F16E2
    'clipboard-clock-outline': 988899,  # 󱛣 U+F16E3
    'cookie-clock': 988900,  # 󱛤 U+F16E4
    'cookie-clock-outline': 988901,  # 󱛥 U+F16E5
    'cookie-edit': 988902,  # 󱛦 U+F16E6
    'cookie-edit-outline': 988903,  # 󱛧 U+F16E7
    'cookie-lock': 988904,  # 󱛨 U+F16E8
    'cookie-lock-outline': 988905,  # 󱛩 U+F16E9
    'cookie-off': 988906,  # 󱛪 U+F16EA
    'cookie-off-outline': 988907,  # 󱛫 U+F16EB
    'cookie-refresh': 988908,  # 󱛬 U+F16EC
    'cookie-refresh-outline': 988909,  # 󱛭 U+F16ED
    'dog-side-off': 988910,  # 󱛮 U+F16EE
    'gift-off': 988911,  # 󱛯 U+F16EF
    'gift-off-outline': 988912,  # 󱛰 U+F16F0
    'gift-open': 988913,  # 󱛱 U+F16F1
    'gift-open-outline': 988914,  # 󱛲 U+F16F2
    'movie-check': 988915,  # 󱛳 U+F16F3
    'movie-check-outline': 988916,  # 󱛴 U+F16F4
    'movie-cog': 988917,  # 󱛵 U+F16F5
    'movie-cog-outline': 988918,  # 󱛶 U+F16F6
    'movie-minus': 988919,  # 󱛷 U+F16F7
    'movie-minus-outline': 988920,  # 󱛸 U+F16F8
    'movie-off': 988921,  # 󱛹 U+F16F9
    'movie-off-outline': 988922,  # 󱛺 U+F16FA
    'movie-open-check': 988923,  # 󱛻 U+F16FB
    'movie-open-check-outline': 988924,  # 󱛼 U+F16FC
    'movie-open-cog': 988925,  # 󱛽 U+F16FD
    'movie-open-cog-outline': 988926,  # 󱛾 U+F16FE
    'movie-open-edit': 988927,  # 󱛿 U+F16FF
    'movie-open-edit-outline': 988928,  # 󱜀 U+F1700
    'movie-open-minus': 988929,  # 󱜁 U+F1701
    'movie-open-minus-outline': 988930,  # 󱜂 U+F1702
    'movie-open-off': 988931,  # 󱜃 U+F1703
    'movie-open-off-outline': 988932,  # 󱜄 U+F1704
    'movie-open-play': 988933,  # 󱜅 U+F1705
    'movie-open-play-outline': 988934,  # 󱜆 U+F1706
    'movie-open-plus': 988935,  # 󱜇 U+F1707
    'movie-open-plus-outline': 988936,  # 󱜈 U+F1708
    'movie-open-remove': 988937,  # 󱜉 U+F1709
    'movie-open-remove-outline': 988938,  # 󱜊 U+F170A
    'movie-open-settings': 988939,  # 󱜋 U+F170B
    'movie-open-settings-outline': 988940,  # 󱜌 U+F170C
    'movie-open-star': 988941,  # 󱜍 U+F170D
    'movie-open-star-outline': 988942,  # 󱜎 U+F170E
    'movie-play': 988943,  # 󱜏 U+F170F
    'movie-play-outline': 988944,  # 󱜐 U+F1710
    'movie-plus': 988945,  # 󱜑 U+F1711
    'movie-plus-outline': 988946,  # 󱜒 U+F1712
    'movie-remove': 988947,  # 󱜓 U+F1713
    'movie-remove-outline': 988948,  # 󱜔 U+F1714
    'movie-settings': 988949,  # 󱜕 U+F1715
    'movie-settings-outline': 988950,  # 󱜖 U+F1716
    'movie-star': 988951,  # 󱜗 U+F1717
    'movie-star-outline': 988952,  # 󱜘 U+F1718
    'robot-happy': 988953,  # 󱜙 U+F1719
    'robot-happy-outline': 988954,  # 󱜚 U+F171A
    'turkey': 988955,  # 󱜛 U+F171B
    'food-turkey': 988956,  # 󱜜 U+F171C
    'fan-auto': 988957,  # 󱜝 U+F171D
    'alarm-light-off': 988958,  # 󱜞 U+F171E
    'alarm-light-off-outline': 988959,  # 󱜟 U+F171F
    'broadcast#2': 988960,  # 󱜠 U+F1720
    'broadcast-off': 988961,  # 󱜡 U+F1721
    'fire-off': 988962,  # 󱜢 U+F1722
    'firework-off': 988963,  # 󱜣 U+F1723
    'projector-screen-outline': 988964,  # 󱜤 U+F1724
    'script-text-key': 988965,  # 󱜥 U+F1725
    'script-text-key-outline': 988966,  # 󱜦 U+F1726
    'script-text-play': 988967,  # 󱜧 U+F1727
    'script-text-play-outline': 988968,  # 󱜨 U+F1728
    'surround-sound-2-1': 988969,  # 󱜩 U+F1729
    'surround-sound-5-1-2': 988970,  # 󱜪 U+F172A
    'tag-arrow-down': 988971,  # 󱜫 U+F172B
    'tag-arrow-down-outline': 988972,  # 󱜬 U+F172C
    'tag-arrow-left': 988973,  # 󱜭 U+F172D
    'tag-arrow-left-outline': 988974,  # 󱜮 U+F172E
    'tag-arrow-right': 988975,  # 󱜯 U+F172F
    'tag-arrow-right-outline': 988976,  # 󱜰 U+F1730
    'tag-arrow-up': 988977,  # 󱜱 U+F1731
    'tag-arrow-up-outline': 988978,  # 󱜲 U+F1732
    'train-car-passenger': 988979,  # 󱜳 U+F1733
    'train-car-passenger-door': 988980,  # 󱜴 U+F1734
    'train-car-passenger-door-open': 988981,  # 󱜵 U+F1735
    'train-car-passenger-variant': 988982,  # 󱜶 U+F1736
    'webcam-off': 988983,  # 󱜷 U+F1737
    'chat-question': 988984,  # 󱜸 U+F1738
    'chat-question-outline': 988985,  # 󱜹 U+F1739
    'message-question': 988986,  # 󱜺 U+F173A
    'message-question-outline': 988987,  # 󱜻 U+F173B
    'kettle-pour-over': 988988,  # 󱜼 U+F173C
    'message-reply-outline': 988989,  # 󱜽 U+F173D
    'message-reply-text-outline': 988990,  # 󱜾 U+F173E
    'koala': 988991,  # 󱜿 U+F173F
    'check-decagram-outline': 988992,  # 󱝀 U+F1740
    'star-shooting': 988993,  # 󱝁 U+F1741
    'star-shooting-outline': 988994,  # 󱝂 U+F1742
    'table-picnic': 988995,  # 󱝃 U+F1743
    'kitesurfing': 988996,  # 󱝄 U+F1744
    'paragliding': 988997,  # 󱝅 U+F1745
    'surfing': 988998,  # 󱝆 U+F1746
    'floor-lamp-torchiere': 988999,  # 󱝇 U+F1747
    'mortar-pestle': 989000,  # 󱝈 U+F1748
    'cast-audio-variant': 989001,  # 󱝉 U+F1749
    'gradient-horizontal': 989002,  # 󱝊 U+F174A
    'archive-cancel': 989003,  # 󱝋 U+F174B
    'archive-cancel-outline': 989004,  # 󱝌 U+F174C
    'archive-check': 989005,  # 󱝍 U+F174D
    'archive-check-outline': 989006,  # 󱝎 U+F174E
    'archive-clock': 989007,  # 󱝏 U+F174F
    'archive-clock-outline': 989008,  # 󱝐 U+F1750
    'archive-cog': 989009,  # 󱝑 U+F1751
    'archive-cog-outline': 989010,  # 󱝒 U+F1752
    'archive-edit': 989011,  # 󱝓 U+F1753
    'archive-edit-outline': 989012,  # 󱝔 U+F1754
    'archive-eye': 989013,  # 󱝕 U+F1755
    'archive-eye-outline': 989014,  # 󱝖 U+F1756
    'archive-lock': 989015,  # 󱝗 U+F1757
    'archive-lock-open': 989016,  # 󱝘 U+F1758
    'archive-lock-open-outline': 989017,  # 󱝙 U+F1759
    'archive-lock-outline': 989018,  # 󱝚 U+F175A
    'archive-marker': 989019,  # 󱝛 U+F175B
    'archive-marker-outline': 989020,  # 󱝜 U+F175C
    'archive-minus': 989021,  # 󱝝 U+F175D
    'archive-minus-outline': 989022,  # 󱝞 U+F175E
    'archive-music': 989023,  # 󱝟 U+F175F
    'archive-music-outline': 989024,  # 󱝠 U+F1760
    'archive-off': 989025,  # 󱝡 U+F1761
    'archive-off-outline': 989026,  # 󱝢 U+F1762
    'archive-plus': 989027,  # 󱝣 U+F1763
    'archive-plus-outline': 989028,  # 󱝤 U+F1764
    'archive-refresh': 989029,  # 󱝥 U+F1765
    'archive-refresh-outline': 989030,  # 󱝦 U+F1766
    'archive-remove': 989031,  # 󱝧 U+F1767
    'archive-remove-outline': 989032,  # 󱝨 U+F1768
    'archive-search': 989033,  # 󱝩 U+F1769
    'archive-search-outline': 989034,  # 󱝪 U+F176A
    'archive-settings': 989035,  # 󱝫 U+F176B
    'archive-settings-outline': 989036,  # 󱝬 U+F176C
    'archive-star': 989037,  # 󱝭 U+F176D
    'archive-star-outline': 989038,  # 󱝮 U+F176E
    'archive-sync': 989039,  # 󱝯 U+F176F
    'archive-sync-outline': 989040,  # 󱝰 U+F1770
    'brush-off': 989041,  # 󱝱 U+F1771
    'file-image-marker': 989042,  # 󱝲 U+F1772
    'file-image-marker-outline': 989043,  # 󱝳 U+F1773
    'file-marker': 989044,  # 󱝴 U+F1774
    'file-marker-outline': 989045,  # 󱝵 U+F1775
    'hamburger-check': 989046,  # 󱝶 U+F1776
    'hamburger-minus': 989047,  # 󱝷 U+F1777
    'hamburger-off': 989048,  # 󱝸 U+F1778
    'hamburger-plus': 989049,  # 󱝹 U+F1779
    'hamburger-remove': 989050,  # 󱝺 U+F177A
    'image-marker': 989051,  # 󱝻 U+F177B
    'image-marker-outline': 989052,  # 󱝼 U+F177C
    'note-alert': 989053,  # 󱝽 U+F177D
    'note-alert-outline': 989054,  # 󱝾 U+F177E
    'note-check': 989055,  # 󱝿 U+F177F
    'note-check-outline': 989056,  # 󱞀 U+F1780
    'note-edit': 989057,  # 󱞁 U+F1781
    'note-edit-outline': 989058,  # 󱞂 U+F1782
    'note-off': 989059,  # 󱞃 U+F1783
    'note-off-outline': 989060,  # 󱞄 U+F1784
    'printer-off-outline': 989061,  # 󱞅 U+F1785
    'printer-outline': 989062,  # 󱞆 U+F1786
    'progress-pencil': 989063,  # 󱞇 U+F1787
    'progress-star': 989064,  # 󱞈 U+F1788
    'sausage-off': 989065,  # 󱞉 U+F1789
    'folder-eye': 989066,  # 󱞊 U+F178A
    'folder-eye-outline': 989067,  # 󱞋 U+F178B
    'information-off': 989068,  # 󱞌 U+F178C
    'information-off-outline': 989069,  # 󱞍 U+F178D
    'sticker-text': 989070,  # 󱞎 U+F178E
    'sticker-text-outline': 989071,  # 󱞏 U+F178F
    'web-cancel': 989072,  # 󱞐 U+F1790
    'web-refresh': 989073,  # 󱞑 U+F1791
    'web-sync': 989074,  # 󱞒 U+F1792
    'chandelier': 989075,  # 󱞓 U+F1793
    'home-switch': 989076,  # 󱞔 U+F1794
    'home-switch-outline': 989077,  # 󱞕 U+F1795
    'sun-snowflake': 989078,  # 󱞖 U+F1796
    'ceiling-fan': 989079,  # 󱞗 U+F1797
    'ceiling-fan-light': 989080,  # 󱞘 U+F1798
    'smoke': 989081,  # 󱞙 U+F1799
    'fence': 989082,  # 󱞚 U+F179A
    'light-recessed': 989083,  # 󱞛 U+F179B
    'battery-lock': 989084,  # 󱞜 U+F179C
    'battery-lock-open': 989085,  # 󱞝 U+F179D
    'folder-hidden': 989086,  # 󱞞 U+F179E
    'mirror-rectangle': 989087,  # 󱞟 U+F179F
    'mirror-variant': 989088,  # 󱞠 U+F17A0
    'arrow-down-left#1': 989089,  # 󱞡 U+F17A1
    'arrow-down-left-bold': 989090,  # 󱞢 U+F17A2
    'arrow-down-right#1': 989091,  # 󱞣 U+F17A3
    'arrow-down-right-bold': 989092,  # 󱞤 U+F17A4
    'arrow-left-bottom': 989093,  # 󱞥 U+F17A5
    'arrow-left-bottom-bold': 989094,  # 󱞦 U+F17A6
    'arrow-left-top': 989095,  # 󱞧 U+F17A7
    'arrow-left-top-bold': 989096,  # 󱞨 U+F17A8
    'arrow-right-bottom': 989097,  # 󱞩 U+F17A9
    'arrow-right-bottom-bold': 989098,  # 󱞪 U+F17AA
    'arrow-right-top': 989099,  # 󱞫 U+F17AB
    'arrow-right-top-bold': 989100,  # 󱞬 U+F17AC
    'arrow-u-down-left': 989101,  # 󱞭 U+F17AD
    'arrow-u-down-left-bold': 989102,  # 󱞮 U+F17AE
    'arrow-u-down-right': 989103,  # 󱞯 U+F17AF
    'arrow-u-down-right-bold': 989104,  # 󱞰 U+F17B0
    'arrow-u-left-bottom': 989105,  # 󱞱 U+F17B1
    'arrow-u-left-bottom-bold': 989106,  # 󱞲 U+F17B2
    'arrow-u-left-top': 989107,  # 󱞳 U+F17B3
    'arrow-u-left-top-bold': 989108,  # 󱞴 U+F17B4
    'arrow-u-right-bottom': 989109,  # 󱞵 U+F17B5
    'arrow-u-right-bottom-bold': 989110,  # 󱞶 U+F17B6
    'arrow-u-right-top': 989111,  # 󱞷 U+F17B7
    'arrow-u-right-top-bold': 989112,  # 󱞸 U+F17B8
    'arrow-u-up-left': 989113,  # 󱞹 U+F17B9
    'arrow-u-up-left-bold': 989114,  # 󱞺 U+F17BA
    'arrow-u-up-right': 989115,  # 󱞻 U+F17BB
    'arrow-u-up-right-bold': 989116,  # 󱞼 U+F17BC
    'arrow-up-left#1': 989117,  # 󱞽 U+F17BD
    'arrow-up-left-bold': 989118,  # 󱞾 U+F17BE
    'arrow-up-right#1': 989119,  # 󱞿 U+F17BF
    'arrow-up-right-bold': 989120,  # 󱟀 U+F17C0
    'select-remove': 989121,  # 󱟁 U+F17C1
    'selection-ellipse-remove': 989122,  # 󱟂 U+F17C2
    'selection-remove': 989123,  # 󱟃 U+F17C3
    'human-greeting': 989124,  # 󱟄 U+F17C4
    'ph': 989125,  # 󱟅 U+F17C5
    'water-sync': 989126,  # 󱟆 U+F17C6
    'ceiling-light-outline': 989127,  # 󱟇 U+F17C7
    'floor-lamp-outline': 989128,  # 󱟈 U+F17C8
    'wall-sconce-flat-outline': 989129,  # 󱟉 U+F17C9
    'wall-sconce-flat-variant-outline': 989130,  # 󱟊 U+F17CA
    'wall-sconce-outline': 989131,  # 󱟋 U+F17CB
    'wall-sconce-round-outline': 989132,  # 󱟌 U+F17CC
    'wall-sconce-round-variant-outline': 989133,  # 󱟍 U+F17CD
    'floor-lamp-dual-outline': 989134,  # 󱟎 U+F17CE
    'floor-lamp-torchiere-variant-outline': 989135,  # 󱟏 U+F17CF
    'lamp-outline': 989136,  # 󱟐 U+F17D0
    'lamps-outline': 989137,  # 󱟑 U+F17D1
    'candelabra': 989138,  # 󱟒 U+F17D2
    'candelabra-fire': 989139,  # 󱟓 U+F17D3
    'menorah#1': 989140,  # 󱟔 U+F17D4
    'menorah-fire': 989141,  # 󱟕 U+F17D5
    'floor-lamp-torchiere-outline': 989142,  # 󱟖 U+F17D6
    'credit-card-edit': 989143,  # 󱟗 U+F17D7
    'credit-card-edit-outline': 989144,  # 󱟘 U+F17D8
    'briefcase-eye': 989145,  # 󱟙 U+F17D9
    'briefcase-eye-outline': 989146,  # 󱟚 U+F17DA
    'soundbar': 989147,  # 󱟛 U+F17DB
    'crown-circle': 989148,  # 󱟜 U+F17DC
    'crown-circle-outline': 989149,  # 󱟝 U+F17DD
    'battery-arrow-down': 989150,  # 󱟞 U+F17DE
    'battery-arrow-down-outline': 989151,  # 󱟟 U+F17DF
    'battery-arrow-up': 989152,  # 󱟠 U+F17E0
    'battery-arrow-up-outline': 989153,  # 󱟡 U+F17E1
    'battery-check': 989154,  # 󱟢 U+F17E2
    'battery-check-outline': 989155,  # 󱟣 U+F17E3
    'battery-minus': 989156,  # 󱟤 U+F17E4
    'battery-minus-outline': 989157,  # 󱟥 U+F17E5
    'battery-plus': 989158,  # 󱟦 U+F17E6
    'battery-plus-outline': 989159,  # 󱟧 U+F17E7
    'battery-remove': 989160,  # 󱟨 U+F17E8
    'battery-remove-outline': 989161,  # 󱟩 U+F17E9
    'chili-alert': 989162,  # 󱟪 U+F17EA
    'chili-alert-outline': 989163,  # 󱟫 U+F17EB
    'chili-hot-outline': 989164,  # 󱟬 U+F17EC
    'chili-medium-outline': 989165,  # 󱟭 U+F17ED
    'chili-mild-outline': 989166,  # 󱟮 U+F17EE
    'chili-off-outline': 989167,  # 󱟯 U+F17EF
    'cake-variant-outline': 989168,  # 󱟰 U+F17F0
    'card-multiple': 989169,  # 󱟱 U+F17F1
    'card-multiple-outline': 989170,  # 󱟲 U+F17F2
    'account-cowboy-hat-outline': 989171,  # 󱟳 U+F17F3
    'lightbulb-spot': 989172,  # 󱟴 U+F17F4
    'lightbulb-spot-off': 989173,  # 󱟵 U+F17F5
    'fence-electric': 989174,  # 󱟶 U+F17F6
    'gate-arrow-left': 989175,  # 󱟷 U+F17F7
    'gate-alert': 989176,  # 󱟸 U+F17F8
    'boom-gate-up': 989177,  # 󱟹 U+F17F9
    'boom-gate-up-outline': 989178,  # 󱟺 U+F17FA
    'garage-lock': 989179,  # 󱟻 U+F17FB
    'garage-variant-lock': 989180,  # 󱟼 U+F17FC
    'cellphone-check': 989181,  # 󱟽 U+F17FD
    'sun-wireless': 989182,  # 󱟾 U+F17FE
    'sun-wireless-outline': 989183,  # 󱟿 U+F17FF
    'lightbulb-auto': 989184,  # 󱠀 U+F1800
    'lightbulb-auto-outline': 989185,  # 󱠁 U+F1801
    'lightbulb-variant': 989186,  # 󱠂 U+F1802
    'lightbulb-variant-outline': 989187,  # 󱠃 U+F1803
    'lightbulb-fluorescent-tube': 989188,  # 󱠄 U+F1804
    'lightbulb-fluorescent-tube-outline': 989189,  # 󱠅 U+F1805
    'water-circle': 989190,  # 󱠆 U+F1806
    'fire-circle': 989191,  # 󱠇 U+F1807
    'smoke-detector-outline': 989192,  # 󱠈 U+F1808
    'smoke-detector-off': 989193,  # 󱠉 U+F1809
    'smoke-detector-off-outline': 989194,  # 󱠊 U+F180A
    'smoke-detector-variant': 989195,  # 󱠋 U+F180B
    'smoke-detector-variant-off': 989196,  # 󱠌 U+F180C
    'projector-screen-off': 989197,  # 󱠍 U+F180D
    'projector-screen-off-outline': 989198,  # 󱠎 U+F180E
    'projector-screen-variant': 989199,  # 󱠏 U+F180F
    'projector-screen-variant-off': 989200,  # 󱠐 U+F1810
    'projector-screen-variant-off-outline': 989201,  # 󱠑 U+F1811
    'projector-screen-variant-outline': 989202,  # 󱠒 U+F1812
    'brush-variant': 989203,  # 󱠓 U+F1813
    'car-wrench': 989204,  # 󱠔 U+F1814
    'account-injury': 989205,  # 󱠕 U+F1815
    'account-injury-outline': 989206,  # 󱠖 U+F1816
    'balcony': 989207,  # 󱠗 U+F1817
    'bathtub': 989208,  # 󱠘 U+F1818
    'bathtub-outline': 989209,  # 󱠙 U+F1819
    'blender-outline': 989210,  # 󱠚 U+F181A
    'coffee-maker-outline': 989211,  # 󱠛 U+F181B
    'countertop': 989212,  # 󱠜 U+F181C
    'countertop-outline': 989213,  # 󱠝 U+F181D
    'door-sliding': 989214,  # 󱠞 U+F181E
    'door-sliding-lock': 989215,  # 󱠟 U+F181F
    'door-sliding-open': 989216,  # 󱠠 U+F1820
    'hand-wave': 989217,  # 󱠡 U+F1821
    'hand-wave-outline': 989218,  # 󱠢 U+F1822
    'human-male-female-child': 989219,  # 󱠣 U+F1823
    'iron': 989220,  # 󱠤 U+F1824
    'iron-outline': 989221,  # 󱠥 U+F1825
    'liquid-spot': 989222,  # 󱠦 U+F1826
    'mosque#1': 989223,  # 󱠧 U+F1827
    'shield-moon': 989224,  # 󱠨 U+F1828
    'shield-moon-outline': 989225,  # 󱠩 U+F1829
    'traffic-light-outline': 989226,  # 󱠪 U+F182A
    'hand-front-left': 989227,  # 󱠫 U+F182B
    'hand-back-left-outline': 989228,  # 󱠬 U+F182C
    'hand-back-right-outline': 989229,  # 󱠭 U+F182D
    'hand-front-left-outline': 989230,  # 󱠮 U+F182E
    'hand-front-right-outline': 989231,  # 󱠯 U+F182F
    'hand-back-left-off': 989232,  # 󱠰 U+F1830
    'hand-back-right-off': 989233,  # 󱠱 U+F1831
    'hand-back-left-off-outline': 989234,  # 󱠲 U+F1832
    'hand-back-right-off-outline': 989235,  # 󱠳 U+F1833
    'battery-sync': 989236,  # 󱠴 U+F1834
    'battery-sync-outline': 989237,  # 󱠵 U+F1835
    'food-takeout-box': 989238,  # 󱠶 U+F1836
    'food-takeout-box-outline': 989239,  # 󱠷 U+F1837
    'iron-board': 989240,  # 󱠸 U+F1838
    'police-station': 989241,  # 󱠹 U+F1839
    'cellphone-marker': 989242,  # 󱠺 U+F183A
    'tooltip-cellphone': 989243,  # 󱠻 U+F183B
    'table-pivot': 989244,  # 󱠼 U+F183C
    'tunnel': 989245,  # 󱠽 U+F183D
    'tunnel-outline': 989246,  # 󱠾 U+F183E
    'arrow-projectile-multiple': 989247,  # 󱠿 U+F183F
    'arrow-projectile': 989248,  # 󱡀 U+F1840
    'bow-arrow': 989249,  # 󱡁 U+F1841
    'axe-battle': 989250,  # 󱡂 U+F1842
    'mace': 989251,  # 󱡃 U+F1843
    'magic-staff': 989252,  # 󱡄 U+F1844
    'spear': 989253,  # 󱡅 U+F1845
    'curtains': 989254,  # 󱡆 U+F1846
    'curtains-closed': 989255,  # 󱡇 U+F1847
    'human-non-binary': 989256,  # 󱡈 U+F1848
    'waterfall': 989257,  # 󱡉 U+F1849
    'egg-fried': 989258,  # 󱡊 U+F184A
    'food-hot-dog': 989259,  # 󱡋 U+F184B
    'induction': 989260,  # 󱡌 U+F184C
    'pipe-valve': 989261,  # 󱡍 U+F184D
    'shipping-pallet': 989262,  # 󱡎 U+F184E
    'earbuds': 989263,  # 󱡏 U+F184F
    'earbuds-off': 989264,  # 󱡐 U+F1850
    'earbuds-off-outline': 989265,  # 󱡑 U+F1851
    'earbuds-outline': 989266,  # 󱡒 U+F1852
    'circle-opacity': 989267,  # 󱡓 U+F1853
    'square-opacity': 989268,  # 󱡔 U+F1854
    'water-opacity': 989269,  # 󱡕 U+F1855
    'vector-polygon-variant': 989270,  # 󱡖 U+F1856
    'vector-square-close': 989271,  # 󱡗 U+F1857
    'vector-square-open': 989272,  # 󱡘 U+F1858
    'waves-arrow-left': 989273,  # 󱡙 U+F1859
    'waves-arrow-right': 989274,  # 󱡚 U+F185A
    'waves-arrow-up': 989275,  # 󱡛 U+F185B
    'cash-fast': 989276,  # 󱡜 U+F185C
    'radioactive-circle': 989277,  # 󱡝 U+F185D
    'radioactive-circle-outline': 989278,  # 󱡞 U+F185E
    'cctv-off': 989279,  # 󱡟 U+F185F
    'format-list-group': 989280,  # 󱡠 U+F1860
    'clock-plus': 989281,  # 󱡡 U+F1861
    'clock-plus-outline': 989282,  # 󱡢 U+F1862
    'clock-minus': 989283,  # 󱡣 U+F1863
    'clock-minus-outline': 989284,  # 󱡤 U+F1864
    'clock-remove': 989285,  # 󱡥 U+F1865
    'clock-remove-outline': 989286,  # 󱡦 U+F1866
    'account-arrow-up': 989287,  # 󱡧 U+F1867
    'account-arrow-down': 989288,  # 󱡨 U+F1868
    'account-arrow-down-outline': 989289,  # 󱡩 U+F1869
    'account-arrow-up-outline': 989290,  # 󱡪 U+F186A
    'audio-input-rca': 989291,  # 󱡫 U+F186B
    'audio-input-stereo-minijack': 989292,  # 󱡬 U+F186C
    'audio-input-xlr': 989293,  # 󱡭 U+F186D
    'horse-variant-fast': 989294,  # 󱡮 U+F186E
    'email-fast': 989295,  # 󱡯 U+F186F
    'email-fast-outline': 989296,  # 󱡰 U+F1870
    'camera-document': 989297,  # 󱡱 U+F1871
    'camera-document-off': 989298,  # 󱡲 U+F1872
    'glass-fragile': 989299,  # 󱡳 U+F1873
    'magnify-expand': 989300,  # 󱡴 U+F1874
    'town-hall': 989301,  # 󱡵 U+F1875
    'monitor-small': 989302,  # 󱡶 U+F1876
    'diversify': 989303,  # 󱡷 U+F1877
    'car-wireless': 989304,  # 󱡸 U+F1878
    'car-select': 989305,  # 󱡹 U+F1879
    'airplane-alert': 989306,  # 󱡺 U+F187A
    'airplane-check': 989307,  # 󱡻 U+F187B
    'airplane-clock': 989308,  # 󱡼 U+F187C
    'airplane-cog': 989309,  # 󱡽 U+F187D
    'airplane-edit': 989310,  # 󱡾 U+F187E
    'airplane-marker': 989311,  # 󱡿 U+F187F
    'airplane-minus': 989312,  # 󱢀 U+F1880
    'airplane-plus': 989313,  # 󱢁 U+F1881
    'airplane-remove': 989314,  # 󱢂 U+F1882
    'airplane-search': 989315,  # 󱢃 U+F1883
    'airplane-settings': 989316,  # 󱢄 U+F1884
    'flower-pollen': 989317,  # 󱢅 U+F1885
    'flower-pollen-outline': 989318,  # 󱢆 U+F1886
    'hammer-sickle': 989319,  # 󱢇 U+F1887
    'view-gallery': 989320,  # 󱢈 U+F1888
    'view-gallery-outline': 989321,  # 󱢉 U+F1889
    'umbrella-beach': 989322,  # 󱢊 U+F188A
    'umbrella-beach-outline': 989323,  # 󱢋 U+F188B
    'cabin-a-frame': 989324,  # 󱢌 U+F188C
    'all-inclusive-box': 989325,  # 󱢍 U+F188D
    'all-inclusive-box-outline': 989326,  # 󱢎 U+F188E
    'hand-coin': 989327,  # 󱢏 U+F188F
    'hand-coin-outline': 989328,  # 󱢐 U+F1890
    'truck-flatbed': 989329,  # 󱢑 U+F1891
    'layers-edit': 989330,  # 󱢒 U+F1892
    'multicast': 989331,  # 󱢓 U+F1893
    'hydrogen-station': 989332,  # 󱢔 U+F1894
    'thermometer-bluetooth': 989333,  # 󱢕 U+F1895
    'tire': 989334,  # 󱢖 U+F1896
    'forest': 989335,  # 󱢗 U+F1897
    'account-tie-hat': 989336,  # 󱢘 U+F1898
    'account-tie-hat-outline': 989337,  # 󱢙 U+F1899
    'account-wrench': 989338,  # 󱢚 U+F189A
    'account-wrench-outline': 989339,  # 󱢛 U+F189B
    'bicycle-cargo': 989340,  # 󱢜 U+F189C
    'calendar-collapse-horizontal': 989341,  # 󱢝 U+F189D
    'calendar-expand-horizontal': 989342,  # 󱢞 U+F189E
    'cards-club-outline': 989343,  # 󱢟 U+F189F
    'heart-outline#1': 989344,  # 󱢠 U+F18A0
    'cards-playing': 989345,  # 󱢡 U+F18A1
    'cards-playing-club': 989346,  # 󱢢 U+F18A2
    'cards-playing-club-multiple': 989347,  # 󱢣 U+F18A3
    'cards-playing-club-multiple-outline': 989348,  # 󱢤 U+F18A4
    'cards-playing-club-outline': 989349,  # 󱢥 U+F18A5
    'cards-playing-diamond': 989350,  # 󱢦 U+F18A6
    'cards-playing-diamond-multiple': 989351,  # 󱢧 U+F18A7
    'cards-playing-diamond-multiple-outline': 989352,  # 󱢨 U+F18A8
    'cards-playing-diamond-outline': 989353,  # 󱢩 U+F18A9
    'cards-playing-heart': 989354,  # 󱢪 U+F18AA
    'cards-playing-heart-multiple': 989355,  # 󱢫 U+F18AB
    'cards-playing-heart-multiple-outline': 989356,  # 󱢬 U+F18AC
    'cards-playing-heart-outline': 989357,  # 󱢭 U+F18AD
    'cards-playing-spade': 989358,  # 󱢮 U+F18AE
    'cards-playing-spade-multiple': 989359,  # 󱢯 U+F18AF
    'cards-playing-spade-multiple-outline': 989360,  # 󱢰 U+F18B0
    'cards-playing-spade-outline': 989361,  # 󱢱 U+F18B1
    'cards-spade-outline': 989362,  # 󱢲 U+F18B2
    'compare-remove': 989363,  # 󱢳 U+F18B3
    'dolphin': 989364,  # 󱢴 U+F18B4
    'fuel-cell': 989365,  # 󱢵 U+F18B5
    'hand-extended': 989366,  # 󱢶 U+F18B6
    'hand-extended-outline': 989367,  # 󱢷 U+F18B7
    'printer-3d-nozzle-heat': 989368,  # 󱢸 U+F18B8
    'printer-3d-nozzle-heat-outline': 989369,  # 󱢹 U+F18B9
    'shark': 989370,  # 󱢺 U+F18BA
    'shark-off': 989371,  # 󱢻 U+F18BB
    'shield-crown': 989372,  # 󱢼 U+F18BC
    'shield-crown-outline': 989373,  # 󱢽 U+F18BD
    'shield-sword': 989374,  # 󱢾 U+F18BE
    'shield-sword-outline': 989375,  # 󱢿 U+F18BF
    'sickle': 989376,  # 󱣀 U+F18C0
    'store-alert': 989377,  # 󱣁 U+F18C1
    'store-alert-outline': 989378,  # 󱣂 U+F18C2
    'store-check': 989379,  # 󱣃 U+F18C3
    'store-check-outline': 989380,  # 󱣄 U+F18C4
    'store-clock': 989381,  # 󱣅 U+F18C5
    'store-clock-outline': 989382,  # 󱣆 U+F18C6
    'store-cog': 989383,  # 󱣇 U+F18C7
    'store-cog-outline': 989384,  # 󱣈 U+F18C8
    'store-edit': 989385,  # 󱣉 U+F18C9
    'store-edit-outline': 989386,  # 󱣊 U+F18CA
    'store-marker': 989387,  # 󱣋 U+F18CB
    'store-marker-outline': 989388,  # 󱣌 U+F18CC
    'store-minus-outline': 989389,  # 󱣍 U+F18CD
    'store-off': 989390,  # 󱣎 U+F18CE
    'store-off-outline': 989391,  # 󱣏 U+F18CF
    'store-plus-outline': 989392,  # 󱣐 U+F18D0
    'store-remove-outline': 989393,  # 󱣑 U+F18D1
    'store-search': 989394,  # 󱣒 U+F18D2
    'store-search-outline': 989395,  # 󱣓 U+F18D3
    'store-settings': 989396,  # 󱣔 U+F18D4
    'store-settings-outline': 989397,  # 󱣕 U+F18D5
    'sun-thermometer': 989398,  # 󱣖 U+F18D6
    'sun-thermometer-outline': 989399,  # 󱣗 U+F18D7
    'truck-cargo-container': 989400,  # 󱣘 U+F18D8
    'vector-square-edit': 989401,  # 󱣙 U+F18D9
    'vector-square-minus': 989402,  # 󱣚 U+F18DA
    'vector-square-plus': 989403,  # 󱣛 U+F18DB
    'vector-square-remove': 989404,  # 󱣜 U+F18DC
    'ceiling-light-multiple': 989405,  # 󱣝 U+F18DD
    'ceiling-light-multiple-outline': 989406,  # 󱣞 U+F18DE
    'wiper-wash-alert': 989407,  # 󱣟 U+F18DF
    'cart-heart': 989408,  # 󱣠 U+F18E0
    'virus-off': 989409,  # 󱣡 U+F18E1
    'virus-off-outline': 989410,  # 󱣢 U+F18E2
    'map-marker-account': 989411,  # 󱣣 U+F18E3
    'map-marker-account-outline': 989412,  # 󱣤 U+F18E4
    'basket-check': 989413,  # 󱣥 U+F18E5
    'basket-check-outline': 989414,  # 󱣦 U+F18E6
    'credit-card-lock': 989415,  # 󱣧 U+F18E7
    'credit-card-lock-outline': 989416,  # 󱣨 U+F18E8
    'format-underline-wavy': 989417,  # 󱣩 U+F18E9
    'content-save-check': 989418,  # 󱣪 U+F18EA
    'content-save-check-outline': 989419,  # 󱣫 U+F18EB
    'filter-check': 989420,  # 󱣬 U+F18EC
    'filter-check-outline': 989421,  # 󱣭 U+F18ED
    'flag-off': 989422,  # 󱣮 U+F18EE
    'flag-off-outline': 989423,  # 󱣯 U+F18EF
    'near-me#1': 989424,  # 󱣰 U+F18F0
    'navigation-variant-outline': 989425,  # 󱣱 U+F18F1
    'refresh-auto': 989426,  # 󱣲 U+F18F2
    'tilde-off': 989427,  # 󱣳 U+F18F3
    'fit-to-screen': 989428,  # 󱣴 U+F18F4
    'fit-to-screen-outline': 989429,  # 󱣵 U+F18F5
    'weather-cloudy-clock': 989430,  # 󱣶 U+F18F6
    'smart-card-off': 989431,  # 󱣷 U+F18F7
    'smart-card-off-outline': 989432,  # 󱣸 U+F18F8
    'clipboard-text-clock': 989433,  # 󱣹 U+F18F9
    'clipboard-text-clock-outline': 989434,  # 󱣺 U+F18FA
    'teddy-bear': 989435,  # 󱣻 U+F18FB
    'cow-off': 989436,  # 󱣼 U+F18FC
    'eye-arrow-left': 989437,  # 󱣽 U+F18FD
    'eye-arrow-left-outline': 989438,  # 󱣾 U+F18FE
    'eye-arrow-right': 989439,  # 󱣿 U+F18FF
    'eye-arrow-right-outline': 989440,  # 󱤀 U+F1900
    'home-battery': 989441,  # 󱤁 U+F1901
    'home-battery-outline': 989442,  # 󱤂 U+F1902
    'home-lightning-bolt': 989443,  # 󱤃 U+F1903
    'home-lightning-bolt-outline': 989444,  # 󱤄 U+F1904
    'leaf-circle': 989445,  # 󱤅 U+F1905
    'leaf-circle-outline': 989446,  # 󱤆 U+F1906
    'tag-search': 989447,  # 󱤇 U+F1907
    'tag-search-outline': 989448,  # 󱤈 U+F1908
    'car-brake-fluid-level': 989449,  # 󱤉 U+F1909
    'car-brake-low-pressure': 989450,  # 󱤊 U+F190A
    'car-brake-temperature': 989451,  # 󱤋 U+F190B
    'car-brake-worn-linings': 989452,  # 󱤌 U+F190C
    'car-light-alert': 989453,  # 󱤍 U+F190D
    'car-speed-limiter': 989454,  # 󱤎 U+F190E
    'credit-card-chip': 989455,  # 󱤏 U+F190F
    'credit-card-chip-outline': 989456,  # 󱤐 U+F1910
    'credit-card-fast': 989457,  # 󱤑 U+F1911
    'credit-card-fast-outline': 989458,  # 󱤒 U+F1912
    'integrated-circuit-chip': 989459,  # 󱤓 U+F1913
    'thumbs-up-down-outline': 989460,  # 󱤔 U+F1914
    'food-off-outline': 989461,  # 󱤕 U+F1915
    'food-outline': 989462,  # 󱤖 U+F1916
    'format-page-split': 989463,  # 󱤗 U+F1917
    'chart-waterfall': 989464,  # 󱤘 U+F1918
    'gamepad-outline': 989465,  # 󱤙 U+F1919
    'network-strength-4-cog': 989466,  # 󱤚 U+F191A
    'account-sync': 989467,  # 󱤛 U+F191B
    'account-sync-outline': 989468,  # 󱤜 U+F191C
    'bus-electric': 989469,  # 󱤝 U+F191D
    'liquor': 989470,  # 󱤞 U+F191E
    'database-eye': 989471,  # 󱤟 U+F191F
    'database-eye-off': 989472,  # 󱤠 U+F1920
    'database-eye-off-outline': 989473,  # 󱤡 U+F1921
    'database-eye-outline': 989474,  # 󱤢 U+F1922
    'timer-settings': 989475,  # 󱤣 U+F1923
    'timer-settings-outline': 989476,  # 󱤤 U+F1924
    'timer-cog': 989477,  # 󱤥 U+F1925
    'timer-cog-outline': 989478,  # 󱤦 U+F1926
    'checkbox-marked-circle-plus-outline': 989479,  # 󱤧 U+F1927
    'panorama-horizontal': 989480,  # 󱤨 U+F1928
    'panorama-vertical': 989481,  # 󱤩 U+F1929
    'advertisements': 989482,  # 󱤪 U+F192A
    'advertisements-off': 989483,  # 󱤫 U+F192B
    'transmission-tower-export': 989484,  # 󱤬 U+F192C
    'transmission-tower-import': 989485,  # 󱤭 U+F192D
    'smoke-detector-alert': 989486,  # 󱤮 U+F192E
    'smoke-detector-alert-outline': 989487,  # 󱤯 U+F192F
    'smoke-detector-variant-alert': 989488,  # 󱤰 U+F1930
    'coffee-maker-check': 989489,  # 󱤱 U+F1931
    'coffee-maker-check-outline': 989490,  # 󱤲 U+F1932
    'cog-pause': 989491,  # 󱤳 U+F1933
    'cog-pause-outline': 989492,  # 󱤴 U+F1934
    'cog-play': 989493,  # 󱤵 U+F1935
    'cog-play-outline': 989494,  # 󱤶 U+F1936
    'cog-stop': 989495,  # 󱤷 U+F1937
    'cog-stop-outline': 989496,  # 󱤸 U+F1938
    'copyleft': 989497,  # 󱤹 U+F1939
    'fast-forward-15': 989498,  # 󱤺 U+F193A
    'file-image-minus': 989499,  # 󱤻 U+F193B
    'file-image-minus-outline': 989500,  # 󱤼 U+F193C
    'file-image-plus': 989501,  # 󱤽 U+F193D
    'file-image-plus-outline': 989502,  # 󱤾 U+F193E
    'file-image-remove': 989503,  # 󱤿 U+F193F
    'file-image-remove-outline': 989504,  # 󱥀 U+F1940
    'message-badge': 989505,  # 󱥁 U+F1941
    'message-badge-outline': 989506,  # 󱥂 U+F1942
    'newspaper-check': 989507,  # 󱥃 U+F1943
    'newspaper-remove': 989508,  # 󱥄 U+F1944
    'publish-off': 989509,  # 󱥅 U+F1945
    'rewind-15': 989510,  # 󱥆 U+F1946
    'view-dashboard-edit': 989511,  # 󱥇 U+F1947
    'view-dashboard-edit-outline': 989512,  # 󱥈 U+F1948
    'office-building-cog': 989513,  # 󱥉 U+F1949
    'office-building-cog-outline': 989514,  # 󱥊 U+F194A
    'hand-clap': 989515,  # 󱥋 U+F194B
    'cone': 989516,  # 󱥌 U+F194C
    'cone-off': 989517,  # 󱥍 U+F194D
    'cylinder': 989518,  # 󱥎 U+F194E
    'cylinder-off': 989519,  # 󱥏 U+F194F
    'octahedron': 989520,  # 󱥐 U+F1950
    'octahedron-off': 989521,  # 󱥑 U+F1951
    'pyramid': 989522,  # 󱥒 U+F1952
    'pyramid-off': 989523,  # 󱥓 U+F1953
    'sphere': 989524,  # 󱥔 U+F1954
    'sphere-off': 989525,  # 󱥕 U+F1955
    'format-letter-spacing': 989526,  # 󱥖 U+F1956
    'french-fries': 989527,  # 󱥗 U+F1957
    'scent': 989528,  # 󱥘 U+F1958
    'scent-off': 989529,  # 󱥙 U+F1959
    'palette-swatch-variant': 989530,  # 󱥚 U+F195A
    'email-seal': 989531,  # 󱥛 U+F195B
    'email-seal-outline': 989532,  # 󱥜 U+F195C
    'stool': 989533,  # 󱥝 U+F195D
    'stool-outline': 989534,  # 󱥞 U+F195E
    'panorama-wide-angle': 989535,  # 󱥟 U+F195F
    'account-lock-open': 989536,  # 󱥠 U+F1960
    'account-lock-open-outline': 989537,  # 󱥡 U+F1961
    'align-horizontal-distribute': 989538,  # 󱥢 U+F1962
    'align-vertical-distribute': 989539,  # 󱥣 U+F1963
    'arrow-bottom-left-bold-box': 989540,  # 󱥤 U+F1964
    'arrow-bottom-left-bold-box-outline': 989541,  # 󱥥 U+F1965
    'arrow-bottom-right-bold-box': 989542,  # 󱥦 U+F1966
    'arrow-bottom-right-bold-box-outline': 989543,  # 󱥧 U+F1967
    'arrow-top-left-bold-box': 989544,  # 󱥨 U+F1968
    'arrow-top-left-bold-box-outline': 989545,  # 󱥩 U+F1969
    'arrow-top-right-bold-box': 989546,  # 󱥪 U+F196A
    'arrow-top-right-bold-box-outline': 989547,  # 󱥫 U+F196B
    'bookmark-box-multiple': 989548,  # 󱥬 U+F196C
    'bookmark-box-multiple-outline': 989549,  # 󱥭 U+F196D
    'bullhorn-variant': 989550,  # 󱥮 U+F196E
    'bullhorn-variant-outline': 989551,  # 󱥯 U+F196F
    'candy': 989552,  # 󱥰 U+F1970
    'candy-off': 989553,  # 󱥱 U+F1971
    'candy-off-outline': 989554,  # 󱥲 U+F1972
    'candy-outline': 989555,  # 󱥳 U+F1973
    'car-clock': 989556,  # 󱥴 U+F1974
    'crowd': 989557,  # 󱥵 U+F1975
    'currency-rupee': 989558,  # 󱥶 U+F1976
    'diving': 989559,  # 󱥷 U+F1977
    'dots-circle': 989560,  # 󱥸 U+F1978
    'elevator-passenger-off': 989561,  # 󱥹 U+F1979
    'elevator-passenger-off-outline': 989562,  # 󱥺 U+F197A
    'elevator-passenger-outline': 989563,  # 󱥻 U+F197B
    'eye-refresh': 989564,  # 󱥼 U+F197C
    'eye-refresh-outline': 989565,  # 󱥽 U+F197D
    'folder-check': 989566,  # 󱥾 U+F197E
    'folder-check-outline': 989567,  # 󱥿 U+F197F
    'human-dolly': 989568,  # 󱦀 U+F1980
    'human-white-cane': 989569,  # 󱦁 U+F1981
    'ip-outline': 989570,  # 󱦂 U+F1982
    'key-alert': 989571,  # 󱦃 U+F1983
    'key-alert-outline': 989572,  # 󱦄 U+F1984
    'kite': 989573,  # 󱦅 U+F1985
    'kite-outline': 989574,  # 󱦆 U+F1986
    'light-flood-down': 989575,  # 󱦇 U+F1987
    'light-flood-up': 989576,  # 󱦈 U+F1988
    'microphone-question': 989577,  # 󱦉 U+F1989
    'microphone-question-outline': 989578,  # 󱦊 U+F198A
    'cradle': 989579,  # 󱦋 U+F198B
    'panorama-outline': 989580,  # 󱦌 U+F198C
    'panorama-sphere': 989581,  # 󱦍 U+F198D
    'panorama-sphere-outline': 989582,  # 󱦎 U+F198E
    'panorama-variant': 989583,  # 󱦏 U+F198F
    'panorama-variant-outline': 989584,  # 󱦐 U+F1990
    'cradle-outline': 989585,  # 󱦑 U+F1991
    'fraction-one-half': 989586,  # 󱦒 U+F1992
    'phone-refresh': 989587,  # 󱦓 U+F1993
    'phone-refresh-outline': 989588,  # 󱦔 U+F1994
    'phone-sync': 989589,  # 󱦕 U+F1995
    'phone-sync-outline': 989590,  # 󱦖 U+F1996
    'razor-double-edge': 989591,  # 󱦗 U+F1997
    'razor-single-edge': 989592,  # 󱦘 U+F1998
    'rotate-360': 989593,  # 󱦙 U+F1999
    'shield-lock-open': 989594,  # 󱦚 U+F199A
    'shield-lock-open-outline': 989595,  # 󱦛 U+F199B
    'sitemap-outline': 989596,  # 󱦜 U+F199C
    'sprinkler-fire': 989597,  # 󱦝 U+F199D
    'tab-search': 989598,  # 󱦞 U+F199E
    'timer-sand-complete': 989599,  # 󱦟 U+F199F
    'timer-sand-paused': 989600,  # 󱦠 U+F19A0
    'vacuum': 989601,  # 󱦡 U+F19A1
    'vacuum-outline': 989602,  # 󱦢 U+F19A2
    'wrench-clock': 989603,  # 󱦣 U+F19A3
    'pliers': 989604,  # 󱦤 U+F19A4
    'sun-compass': 989605,  # 󱦥 U+F19A5
    'truck-snowflake': 989606,  # 󱦦 U+F19A6
    'camera-marker': 989607,  # 󱦧 U+F19A7
    'camera-marker-outline': 989608,  # 󱦨 U+F19A8
    'video-marker': 989609,  # 󱦩 U+F19A9
    'video-marker-outline': 989610,  # 󱦪 U+F19AA
    'wind-turbine-alert': 989611,  # 󱦫 U+F19AB
    'wind-turbine-check': 989612,  # 󱦬 U+F19AC
    'truck-plus': 989613,  # 󱦭 U+F19AD
    'truck-minus': 989614,  # 󱦮 U+F19AE
    'truck-remove': 989615,  # 󱦯 U+F19AF
    'arrow-right-thin': 989616,  # 󱦰 U+F19B0
    'arrow-left-thin': 989617,  # 󱦱 U+F19B1
    'arrow-up-thin': 989618,  # 󱦲 U+F19B2
    'arrow-down-thin': 989619,  # 󱦳 U+F19B3
    'arrow-top-right-thin': 989620,  # 󱦴 U+F19B4
    'arrow-top-left-thin': 989621,  # 󱦵 U+F19B5
    'arrow-bottom-left-thin': 989622,  # 󱦶 U+F19B6
    'arrow-bottom-right-thin': 989623,  # 󱦷 U+F19B7
    'scale-unbalanced': 989624,  # 󱦸 U+F19B8
    'draw-pen': 989625,  # 󱦹 U+F19B9
    'clock-edit': 989626,  # 󱦺 U+F19BA
    'clock-edit-outline': 989627,  # 󱦻 U+F19BB
    'truck-plus-outline': 989628,  # 󱦼 U+F19BC
    'truck-minus-outline': 989629,  # 󱦽 U+F19BD
    'truck-remove-outline': 989630,  # 󱦾 U+F19BE
    'camera-off-outline': 989631,  # 󱦿 U+F19BF
    'home-group-plus': 989632,  # 󱧀 U+F19C0
    'home-group-minus': 989633,  # 󱧁 U+F19C1
    'home-group-remove': 989634,  # 󱧂 U+F19C2
    'file-sign': 989635,  # 󱧃 U+F19C3
    'attachment-lock': 989636,  # 󱧄 U+F19C4
    'cellphone-arrow-down-variant': 989637,  # 󱧅 U+F19C5
    'file-chart-check': 989638,  # 󱧆 U+F19C6
    'file-chart-check-outline': 989639,  # 󱧇 U+F19C7
    'file-lock-open': 989640,  # 󱧈 U+F19C8
    'file-lock-open-outline': 989641,  # 󱧉 U+F19C9
    'folder-question': 989642,  # 󱧊 U+F19CA
    'folder-question-outline': 989643,  # 󱧋 U+F19CB
    'message-fast': 989644,  # 󱧌 U+F19CC
    'message-fast-outline': 989645,  # 󱧍 U+F19CD
    'message-text-fast': 989646,  # 󱧎 U+F19CE
    'message-text-fast-outline': 989647,  # 󱧏 U+F19CF
    'monitor-arrow-down': 989648,  # 󱧐 U+F19D0
    'monitor-arrow-down-variant': 989649,  # 󱧑 U+F19D1
    'needle-off': 989650,  # 󱧒 U+F19D2
    'numeric-off': 989651,  # 󱧓 U+F19D3
    'package-variant-closed-minus': 989652,  # 󱧔 U+F19D4
    'package-variant-closed-plus': 989653,  # 󱧕 U+F19D5
    'package-variant-closed-remove': 989654,  # 󱧖 U+F19D6
    'package-variant-minus': 989655,  # 󱧗 U+F19D7
    'package-variant-plus': 989656,  # 󱧘 U+F19D8
    'package-variant-remove': 989657,  # 󱧙 U+F19D9
    'paperclip-lock': 989658,  # 󱧚 U+F19DA
    'phone-clock': 989659,  # 󱧛 U+F19DB
    'receipt-outline': 989660,  # 󱧜 U+F19DC
    'transmission-tower-off': 989661,  # 󱧝 U+F19DD
    'truck-alert': 989662,  # 󱧞 U+F19DE
    'truck-alert-outline': 989663,  # 󱧟 U+F19DF
    'bone-off': 989664,  # 󱧠 U+F19E0
    'lightbulb-alert': 989665,  # 󱧡 U+F19E1
    'lightbulb-alert-outline': 989666,  # 󱧢 U+F19E2
    'lightbulb-question': 989667,  # 󱧣 U+F19E3
    'lightbulb-question-outline': 989668,  # 󱧤 U+F19E4
    'battery-clock': 989669,  # 󱧥 U+F19E5
    'battery-clock-outline': 989670,  # 󱧦 U+F19E6
    'autorenew-off': 989671,  # 󱧧 U+F19E7
    'folder-arrow-down': 989672,  # 󱧨 U+F19E8
    'folder-arrow-down-outline': 989673,  # 󱧩 U+F19E9
    'folder-arrow-left': 989674,  # 󱧪 U+F19EA
    'folder-arrow-left-outline': 989675,  # 󱧫 U+F19EB
    'folder-arrow-left-right': 989676,  # 󱧬 U+F19EC
    'folder-arrow-left-right-outline': 989677,  # 󱧭 U+F19ED
    'folder-arrow-right': 989678,  # 󱧮 U+F19EE
    'folder-arrow-right-outline': 989679,  # 󱧯 U+F19EF
    'folder-arrow-up': 989680,  # 󱧰 U+F19F0
    'folder-arrow-up-down': 989681,  # 󱧱 U+F19F1
    'folder-arrow-up-down-outline': 989682,  # 󱧲 U+F19F2
    'folder-arrow-up-outline': 989683,  # 󱧳 U+F19F3
    'folder-cancel': 989684,  # 󱧴 U+F19F4
    'folder-cancel-outline': 989685,  # 󱧵 U+F19F5
    'folder-file': 989686,  # 󱧶 U+F19F6
    'folder-file-outline': 989687,  # 󱧷 U+F19F7
    'folder-off': 989688,  # 󱧸 U+F19F8
    'folder-off-outline': 989689,  # 󱧹 U+F19F9
    'folder-play': 989690,  # 󱧺 U+F19FA
    'folder-play-outline': 989691,  # 󱧻 U+F19FB
    'folder-wrench': 989692,  # 󱧼 U+F19FC
    'folder-wrench-outline': 989693,  # 󱧽 U+F19FD
    'image-refresh': 989694,  # 󱧾 U+F19FE
    'image-refresh-outline': 989695,  # 󱧿 U+F19FF
    'image-sync': 989696,  # 󱨀 U+F1A00
    'image-sync-outline': 989697,  # 󱨁 U+F1A01
    'percent-box': 989698,  # 󱨂 U+F1A02
    'percent-box-outline': 989699,  # 󱨃 U+F1A03
    'percent-circle': 989700,  # 󱨄 U+F1A04
    'percent-circle-outline': 989701,  # 󱨅 U+F1A05
    'sale-outline': 989702,  # 󱨆 U+F1A06
    'square-rounded-badge': 989703,  # 󱨇 U+F1A07
    'square-rounded-badge-outline': 989704,  # 󱨈 U+F1A08
    'triangle-small-down': 989705,  # 󱨉 U+F1A09
    'triangle-small-up': 989706,  # 󱨊 U+F1A0A
    'notebook-heart': 989707,  # 󱨋 U+F1A0B
    'notebook-heart-outline': 989708,  # 󱨌 U+F1A0C
    'brush-outline': 989709,  # 󱨍 U+F1A0D
    'fruit-pear': 989710,  # 󱨎 U+F1A0E
    'raw': 989711,  # 󱨏 U+F1A0F
    'raw-off': 989712,  # 󱨐 U+F1A10
    'wall-fire': 989713,  # 󱨑 U+F1A11
    'home-clock': 989714,  # 󱨒 U+F1A12
    'home-clock-outline': 989715,  # 󱨓 U+F1A13
    'camera-lock': 989716,  # 󱨔 U+F1A14
    'camera-lock-outline': 989717,  # 󱨕 U+F1A15
    'play-box-lock': 989718,  # 󱨖 U+F1A16
    'play-box-lock-open': 989719,  # 󱨗 U+F1A17
    'play-box-lock-open-outline': 989720,  # 󱨘 U+F1A18
    'play-box-lock-outline': 989721,  # 󱨙 U+F1A19
    'robot-industrial-outline': 989722,  # 󱨚 U+F1A1A
    'gas-burner': 989723,  # 󱨛 U+F1A1B
    'video-2d': 989724,  # 󱨜 U+F1A1C
    'book-heart': 989725,  # 󱨝 U+F1A1D
    'book-heart-outline': 989726,  # 󱨞 U+F1A1E
    'account-hard-hat-outline': 989727,  # 󱨟 U+F1A1F
    'account-school': 989728,  # 󱨠 U+F1A20
    'account-school-outline': 989729,  # 󱨡 U+F1A21
    'library-outline': 989730,  # 󱨢 U+F1A22
    'projector-off': 989731,  # 󱨣 U+F1A23
    'light-switch-off': 989732,  # 󱨤 U+F1A24
    'toggle-switch-variant': 989733,  # 󱨥 U+F1A25
    'toggle-switch-variant-off': 989734,  # 󱨦 U+F1A26
    'asterisk-circle-outline': 989735,  # 󱨧 U+F1A27
    'barrel-outline': 989736,  # 󱨨 U+F1A28
    'bell-cog': 989737,  # 󱨩 U+F1A29
    'bell-cog-outline': 989738,  # 󱨪 U+F1A2A
    'blinds-horizontal': 989739,  # 󱨫 U+F1A2B
    'blinds-horizontal-closed': 989740,  # 󱨬 U+F1A2C
    'blinds-vertical': 989741,  # 󱨭 U+F1A2D
    'blinds-vertical-closed': 989742,  # 󱨮 U+F1A2E
    'bulkhead-light': 989743,  # 󱨯 U+F1A2F
    'calendar-today-outline': 989744,  # 󱨰 U+F1A30
    'calendar-week-begin-outline': 989745,  # 󱨱 U+F1A31
    'calendar-week-end': 989746,  # 󱨲 U+F1A32
    'calendar-week-end-outline': 989747,  # 󱨳 U+F1A33
    'calendar-week-outline': 989748,  # 󱨴 U+F1A34
    'cloud-percent': 989749,  # 󱨵 U+F1A35
    'cloud-percent-outline': 989750,  # 󱨶 U+F1A36
    'coach-lamp-variant': 989751,  # 󱨷 U+F1A37
    'compost': 989752,  # 󱨸 U+F1A38
    'currency-fra': 989753,  # 󱨹 U+F1A39
    'fan-clock': 989754,  # 󱨺 U+F1A3A
    'file-rotate-left': 989755,  # 󱨻 U+F1A3B
    'file-rotate-left-outline': 989756,  # 󱨼 U+F1A3C
    'file-rotate-right': 989757,  # 󱨽 U+F1A3D
    'file-rotate-right-outline': 989758,  # 󱨾 U+F1A3E
    'filter-multiple': 989759,  # 󱨿 U+F1A3F
    'filter-multiple-outline': 989760,  # 󱩀 U+F1A40
    'gymnastics': 989761,  # 󱩁 U+F1A41
    'hand-clap-off': 989762,  # 󱩂 U+F1A42
    'heat-pump': 989763,  # 󱩃 U+F1A43
    'heat-pump-outline': 989764,  # 󱩄 U+F1A44
    'heat-wave': 989765,  # 󱩅 U+F1A45
    'home-off': 989766,  # 󱩆 U+F1A46
    'home-off-outline': 989767,  # 󱩇 U+F1A47
    'landslide': 989768,  # 󱩈 U+F1A48
    'landslide-outline': 989769,  # 󱩉 U+F1A49
    'laptop-account': 989770,  # 󱩊 U+F1A4A
    'led-strip-variant-off': 989771,  # 󱩋 U+F1A4B
    'lightbulb-night': 989772,  # 󱩌 U+F1A4C
    'lightbulb-night-outline': 989773,  # 󱩍 U+F1A4D
    'lightbulb-on-10': 989774,  # 󱩎 U+F1A4E
    'lightbulb-on-20': 989775,  # 󱩏 U+F1A4F
    'lightbulb-on-30': 989776,  # 󱩐 U+F1A50
    'lightbulb-on-40': 989777,  # 󱩑 U+F1A51
    'lightbulb-on-50': 989778,  # 󱩒 U+F1A52
    'lightbulb-on-60': 989779,  # 󱩓 U+F1A53
    'lightbulb-on-70': 989780,  # 󱩔 U+F1A54
    'lightbulb-on-80': 989781,  # 󱩕 U+F1A55
    'lightbulb-on-90': 989782,  # 󱩖 U+F1A56
    'meter-electric': 989783,  # 󱩗 U+F1A57
    'meter-electric-outline': 989784,  # 󱩘 U+F1A58
    'meter-gas': 989785,  # 󱩙 U+F1A59
    'meter-gas-outline': 989786,  # 󱩚 U+F1A5A
    'monitor-account': 989787,  # 󱩛 U+F1A5B
    'pill-off': 989788,  # 󱩜 U+F1A5C
    'plus-lock': 989789,  # 󱩝 U+F1A5D
    'plus-lock-open': 989790,  # 󱩞 U+F1A5E
    'pool-thermometer': 989791,  # 󱩟 U+F1A5F
    'post-lamp': 989792,  # 󱩠 U+F1A60
    'rabbit-variant': 989793,  # 󱩡 U+F1A61
    'rabbit-variant-outline': 989794,  # 󱩢 U+F1A62
    'receipt-text-check': 989795,  # 󱩣 U+F1A63
    'receipt-text-check-outline': 989796,  # 󱩤 U+F1A64
    'receipt-text-minus': 989797,  # 󱩥 U+F1A65
    'receipt-text-minus-outline': 989798,  # 󱩦 U+F1A66
    'receipt-text-plus': 989799,  # 󱩧 U+F1A67
    'receipt-text-plus-outline': 989800,  # 󱩨 U+F1A68
    'receipt-text-remove': 989801,  # 󱩩 U+F1A69
    'receipt-text-remove-outline': 989802,  # 󱩪 U+F1A6A
    'roller-shade': 989803,  # 󱩫 U+F1A6B
    'roller-shade-closed': 989804,  # 󱩬 U+F1A6C
    'seed-plus': 989805,  # 󱩭 U+F1A6D
    'seed-plus-outline': 989806,  # 󱩮 U+F1A6E
    'shopping-search-outline': 989807,  # 󱩯 U+F1A6F
    'snowflake-check': 989808,  # 󱩰 U+F1A70
    'snowflake-thermometer': 989809,  # 󱩱 U+F1A71
    'snowshoeing': 989810,  # 󱩲 U+F1A72
    'solar-power-variant': 989811,  # 󱩳 U+F1A73
    'solar-power-variant-outline': 989812,  # 󱩴 U+F1A74
    'storage-tank': 989813,  # 󱩵 U+F1A75
    'storage-tank-outline': 989814,  # 󱩶 U+F1A76
    'sun-clock': 989815,  # 󱩷 U+F1A77
    'sun-clock-outline': 989816,  # 󱩸 U+F1A78
    'sun-snowflake-variant': 989817,  # 󱩹 U+F1A79
    'tag-check': 989818,  # 󱩺 U+F1A7A
    'tag-check-outline': 989819,  # 󱩻 U+F1A7B
    'text-box-edit': 989820,  # 󱩼 U+F1A7C
    'text-box-edit-outline': 989821,  # 󱩽 U+F1A7D
    'text-search-variant': 989822,  # 󱩾 U+F1A7E
    'thermometer-check': 989823,  # 󱩿 U+F1A7F
    'thermometer-water': 989824,  # 󱪀 U+F1A80
    'tsunami': 989825,  # 󱪁 U+F1A81
    'turbine': 989826,  # 󱪂 U+F1A82
    'volcano#1': 989827,  # 󱪃 U+F1A83
    'volcano-outline': 989828,  # 󱪄 U+F1A84
    'water-thermometer': 989829,  # 󱪅 U+F1A85
    'water-thermometer-outline': 989830,  # 󱪆 U+F1A86
    'wheelchair#1': 989831,  # 󱪇 U+F1A87
    'wind-power': 989832,  # 󱪈 U+F1A88
    'wind-power-outline': 989833,  # 󱪉 U+F1A89
    'window-shutter-cog': 989834,  # 󱪊 U+F1A8A
    'window-shutter-settings': 989835,  # 󱪋 U+F1A8B
    'account-tie-woman': 989836,  # 󱪌 U+F1A8C
    'briefcase-arrow-left-right': 989837,  # 󱪍 U+F1A8D
    'briefcase-arrow-left-right-outline': 989838,  # 󱪎 U+F1A8E
    'briefcase-arrow-up-down': 989839,  # 󱪏 U+F1A8F
    'briefcase-arrow-up-down-outline': 989840,  # 󱪐 U+F1A90
    'cash-clock': 989841,  # 󱪑 U+F1A91
    'cash-sync': 989842,  # 󱪒 U+F1A92
    'file-arrow-left-right': 989843,  # 󱪓 U+F1A93
    'file-arrow-left-right-outline': 989844,  # 󱪔 U+F1A94
    'file-arrow-up-down': 989845,  # 󱪕 U+F1A95
    'file-arrow-up-down-outline': 989846,  # 󱪖 U+F1A96
    'file-document-alert': 989847,  # 󱪗 U+F1A97
    'file-document-alert-outline': 989848,  # 󱪘 U+F1A98
    'file-document-check': 989849,  # 󱪙 U+F1A99
    'file-document-check-outline': 989850,  # 󱪚 U+F1A9A
    'file-document-minus': 989851,  # 󱪛 U+F1A9B
    'file-document-minus-outline': 989852,  # 󱪜 U+F1A9C
    'file-document-plus': 989853,  # 󱪝 U+F1A9D
    'file-document-plus-outline': 989854,  # 󱪞 U+F1A9E
    'file-document-remove': 989855,  # 󱪟 U+F1A9F
    'file-document-remove-outline': 989856,  # 󱪠 U+F1AA0
    'file-minus': 989857,  # 󱪡 U+F1AA1
    'file-minus-outline': 989858,  # 󱪢 U+F1AA2
    'filter-cog': 989859,  # 󱪣 U+F1AA3
    'filter-cog-outline': 989860,  # 󱪤 U+F1AA4
    'filter-settings': 989861,  # 󱪥 U+F1AA5
    'filter-settings-outline': 989862,  # 󱪦 U+F1AA6
    'folder-lock-open-outline': 989863,  # 󱪧 U+F1AA7
    'folder-lock-outline': 989864,  # 󱪨 U+F1AA8
    'forum-minus': 989865,  # 󱪩 U+F1AA9
    'forum-minus-outline': 989866,  # 󱪪 U+F1AAA
    'forum-plus': 989867,  # 󱪫 U+F1AAB
    'forum-plus-outline': 989868,  # 󱪬 U+F1AAC
    'forum-remove': 989869,  # 󱪭 U+F1AAD
    'forum-remove-outline': 989870,  # 󱪮 U+F1AAE
    'heating-coil': 989871,  # 󱪯 U+F1AAF
    'image-lock': 989872,  # 󱪰 U+F1AB0
    'image-lock-outline': 989873,  # 󱪱 U+F1AB1
    'land-fields': 989874,  # 󱪲 U+F1AB2
    'land-plots': 989875,  # 󱪳 U+F1AB3
    'land-plots-circle': 989876,  # 󱪴 U+F1AB4
    'land-plots-circle-variant': 989877,  # 󱪵 U+F1AB5
    'land-rows-horizontal': 989878,  # 󱪶 U+F1AB6
    'land-rows-vertical': 989879,  # 󱪷 U+F1AB7
    'medical-cotton-swab': 989880,  # 󱪸 U+F1AB8
    'rolodex': 989881,  # 󱪹 U+F1AB9
    'rolodex-outline': 989882,  # 󱪺 U+F1ABA
    'sort-variant-off': 989883,  # 󱪻 U+F1ABB
    'tally-mark-1': 989884,  # 󱪼 U+F1ABC
    'tally-mark-2': 989885,  # 󱪽 U+F1ABD
    'tally-mark-3': 989886,  # 󱪾 U+F1ABE
    'tally-mark-4': 989887,  # 󱪿 U+F1ABF
    'tally-mark-5': 989888,  # 󱫀 U+F1AC0
    'attachment-check': 989889,  # 󱫁 U+F1AC1
    'attachment-minus': 989890,  # 󱫂 U+F1AC2
    'attachment-off': 989891,  # 󱫃 U+F1AC3
    'attachment-plus': 989892,  # 󱫄 U+F1AC4
    'attachment-remove': 989893,  # 󱫅 U+F1AC5
    'paperclip-check': 989894,  # 󱫆 U+F1AC6
    'paperclip-minus': 989895,  # 󱫇 U+F1AC7
    'paperclip-off': 989896,  # 󱫈 U+F1AC8
    'paperclip-plus': 989897,  # 󱫉 U+F1AC9
    'paperclip-remove': 989898,  # 󱫊 U+F1ACA
    'network-pos': 989899,  # 󱫋 U+F1ACB
    'timer-alert': 989900,  # 󱫌 U+F1ACC
    'timer-alert-outline': 989901,  # 󱫍 U+F1ACD
    'timer-cancel': 989902,  # 󱫎 U+F1ACE
    'timer-cancel-outline': 989903,  # 󱫏 U+F1ACF
    'timer-check': 989904,  # 󱫐 U+F1AD0
    'timer-check-outline': 989905,  # 󱫑 U+F1AD1
    'timer-edit': 989906,  # 󱫒 U+F1AD2
    'timer-edit-outline': 989907,  # 󱫓 U+F1AD3
    'timer-lock': 989908,  # 󱫔 U+F1AD4
    'timer-lock-open': 989909,  # 󱫕 U+F1AD5
    'timer-lock-open-outline': 989910,  # 󱫖 U+F1AD6
    'timer-lock-outline': 989911,  # 󱫗 U+F1AD7
    'timer-marker': 989912,  # 󱫘 U+F1AD8
    'timer-marker-outline': 989913,  # 󱫙 U+F1AD9
    'timer-minus': 989914,  # 󱫚 U+F1ADA
    'timer-minus-outline': 989915,  # 󱫛 U+F1ADB
    'timer-music': 989916,  # 󱫜 U+F1ADC
    'timer-music-outline': 989917,  # 󱫝 U+F1ADD
    'timer-pause': 989918,  # 󱫞 U+F1ADE
    'timer-pause-outline': 989919,  # 󱫟 U+F1ADF
    'timer-play': 989920,  # 󱫠 U+F1AE0
    'timer-play-outline': 989921,  # 󱫡 U+F1AE1
    'timer-plus': 989922,  # 󱫢 U+F1AE2
    'timer-plus-outline': 989923,  # 󱫣 U+F1AE3
    'timer-refresh': 989924,  # 󱫤 U+F1AE4
    'timer-refresh-outline': 989925,  # 󱫥 U+F1AE5
    'timer-remove': 989926,  # 󱫦 U+F1AE6
    'timer-remove-outline': 989927,  # 󱫧 U+F1AE7
    'timer-star': 989928,  # 󱫨 U+F1AE8
    'timer-star-outline': 989929,  # 󱫩 U+F1AE9
    'timer-stop': 989930,  # 󱫪 U+F1AEA
    'timer-stop-outline': 989931,  # 󱫫 U+F1AEB
    'timer-sync': 989932,  # 󱫬 U+F1AEC
    'timer-sync-outline': 989933,  # 󱫭 U+F1AED
    'ear-hearing-loop': 989934,  # 󱫮 U+F1AEE
    'sail-boat-sink': 989935,  # 󱫯 U+F1AEF
    'lecturn': 989936,  # 󱫰 U+F1AF0
}


class NerdfontIconUtils(ColorThemeABC, FontABC):
    default_icon_color: str = None

    def symbol(self, symbol: int | str, **kwargs):
        """
        Returns a symbol from the Nerdfont icon set.

        :param symbol:  The symbol to retrieve. This can be either an integer representing the Unicode code point or a
                        string representing the symbol name.
        :param kwargs: additional keyword arguments to customize the symbol's appearance. (passed to the manim `Text`
                       function)
        :return: a Manim Text object representing the requested symbol.
        """
        color = kwargs.pop("font_color", self.default_icon_color)  # for consistency with math_text
        if color is None:
            color = self.yellow
        default_params = {
            "font": "Symbols Nerd Font Mono",
            "color": color,
            "font_size": self.font_size_normal,
        }
        params = {**default_params, **kwargs}
        if isinstance(symbol, str):
            if symbol in SYMBOLS_UNICODE.keys():
                return m.Text(chr(SYMBOLS_UNICODE[symbol]), **params)
            else:
                return m.Text(symbol, **params)
        elif isinstance(symbol, int):
            return m.Text(chr(symbol), **params)
        else:
            raise ValueError("symbol must be int, str or chr")

    @functools.lru_cache(maxsize=10)
    def symbol_t2c(self, color=None, **kwargs) -> dict:
        """
        a util function to maps symbols to colors. It can be used as the `t2c` argument for functions like
        `m.Text` and `m.MathText` or manta util functions like `term_text`.

        `color` and `font_color` serve the same purpose. If `color` is None, `font_color` will be used.
        This funtion uses `**kwargs` to avoid errors when unspecified arguments are passed.

        :param color: the color to use for the symbols
        :param kwargs: may contain the key `font_color` to specify the color
        :return:
        """
        if color is None:
            color = kwargs.pop("font_color", self.yellow)  # for consistency with math_text
        return {chr(SYMBOLS_UNICODE[icon]): color for icon in SYMBOLS_UNICODE.keys()}
